﻿////
/// Copyright(C) 2020 Micronet Co. All Rights Reserved.
//
using System;

namespace RTedge
{
    /// <summary>
    /// 基底クラス
    /// </summary>
    public class egBaseClass : IDisposable
    {
        // Dispose済みかの判別用
        bool disposed = false;

        // SafeHandleインスタンスをインスタンス化する
        public void Dispose()
        {
            Dispose(true);
            GC.SuppressFinalize(this);
        }

        /// <summary>
        /// Disposeパターンの保護された実装
        /// </summary>
        /// <param name="disposing">true:管理対象オブジェクトを解放する、false:管理対象オブジェクトを解放しない</param>
        protected virtual void Dispose(bool disposing)
        {
            if (disposed)
                return;

            if (disposing)
            {
                // ここで管理対象オブジェクトを解放する
                fnReleaseManagedObject();
            }

            // ここでは管理されていないオブジェクトを解放する
            disposed = true;
        }

        /// <summary>
        /// 管理対象オブジェクトを解放する
        /// </summary>
        protected virtual void fnReleaseManagedObject()
        {

        }
    }
}
