﻿////
/// Copyright(C) 2020 Micronet Co. All Rights Reserved.
//
using System;
using System.Text;
using System.Diagnostics;           // Debug.WriteLine出力用
using System.Runtime.InteropServices;
namespace RTedge
{
    public static class egCommonFunc
    {

        /// <summary>
        /// unicodeからshiftjisに変換
        /// </summary>
        /// <param name="unicodeStrings"></param>
        /// <returns></returns>
        static public string ToShiftJis(string unicodeStrings)
        {
            try
            {
                Encoding unicode = Encoding.Unicode;                                                //Unicode
                byte[] unicodeByte = unicode.GetBytes(unicodeStrings);                              //Unicodeの文字列をバイトに
                Encoding s_jis = Encoding.GetEncoding("shift_jis");                                 //shift_jis
                byte[] s_jisByte = Encoding.Convert(unicode, s_jis, unicodeByte);                   //unicodeのbyte配列をsjisのバイト配列に
                char[] s_jisChars = new char[s_jis.GetCharCount(s_jisByte, 0, s_jisByte.Length)];   //sjisのbyte配列からsjisの文字列配列に
                s_jis.GetChars(s_jisByte, 0, s_jisByte.Length, s_jisChars, 0);                      //最後string型に
                return new string(s_jisChars);
            }
            catch (Exception ex)
        {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return "";
            }
        }
        /// <summary>
        /// バイト配列を型にあったデータに変換しobject型で返す
        /// </summary>
        /// <param name="srcByte"></param>
        /// <param name="type"></param>
        /// <param name="byBitNo"></param>
        /// <returns></returns>
        public static object GetTagAnyData(byte[] srcByte, EGDEFINE.egTagDataType type, byte byBitNo = 0)
        {
            try
            {
                object obj = new object();
                EGDEFINE.egTagDataType enType = type;
                switch (enType)
                {
                    case EGDEFINE.egTagDataType.Boolean:
                        {
                            bool bIsOn = false;
                            if (fnBitChek(srcByte[0], byBitNo, ref bIsOn) == 0)
                            {
                                obj = bIsOn;
                            }
                            else
                            {
                                obj = false;
                            }
                        }
                        break;

                    case EGDEFINE.egTagDataType.SByte:
                        {
                            sbyte tmpSbyte = unchecked((sbyte)srcByte[0]);
                            obj = (object)tmpSbyte;
                        }
                        break;

                    case EGDEFINE.egTagDataType.Byte:
                        obj = srcByte[0];
                        break;

                    case EGDEFINE.egTagDataType.Int16:
                        obj = BitConverter.ToInt16(srcByte, 0);
                        break;

                    case EGDEFINE.egTagDataType.UInt16:
                        obj = BitConverter.ToUInt16(srcByte, 0);
                        break;

                    case EGDEFINE.egTagDataType.Int32:
                        obj = BitConverter.ToInt32(srcByte, 0);
                        break;

                    case EGDEFINE.egTagDataType.UInt32:
                        obj = BitConverter.ToUInt32(srcByte, 0);
                        break;

                    case EGDEFINE.egTagDataType.Int64:
                        obj = BitConverter.ToInt64(srcByte, 0);
                        break;

                    case EGDEFINE.egTagDataType.UInt64:
                        obj = BitConverter.ToUInt64(srcByte, 0);
                        break;

                    case EGDEFINE.egTagDataType.Float:
                        obj = BitConverter.ToSingle(srcByte, 0);
                        break;

                    case EGDEFINE.egTagDataType.Double:
                        obj = BitConverter.ToDouble(srcByte, 0);
                        break;

                    case EGDEFINE.egTagDataType.String:
                        {
                            string stValue = System.Text.Encoding.GetEncoding("shift_jis").GetString(srcByte);
                            // \0以降を削除
                            string outStr = "";
                            int p = stValue.IndexOf('\0');
                            if (p < 0) p = stValue.Length;
                            outStr = stValue.Substring(0, p);
                            obj = outStr.TrimEnd('\0');
                        }
                        break;

                    case EGDEFINE.egTagDataType.Segment:
                    case EGDEFINE.egTagDataType.FIFO:
                        obj = srcByte;
                        break;

                    default:
                        obj = null;
                        break;
                }
                return obj;
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return null;
            }
        }
        ///// <summary>
        ///// バイト配列を型にあったデータに変換しobject型で返す
        ///// </summary>
        ///// <param name="srcByte"></param>
        ///// <param name="type"></param>
        ///// <param name="byBitNo"></param>
        ///// <returns></returns>
        public static object GetTagAnyDataFromString(string srcStr, EGDEFINE.egTagDataType type)
        {
            try
            {
                object obj = new object();
                EGDEFINE.egTagDataType enType = type;
                switch (enType)
                {
                    case EGDEFINE.egTagDataType.Boolean:
                        {
                            //文字列の指定がある場合はそれに則る
                            if (srcStr.Equals("true", StringComparison.OrdinalIgnoreCase))
                            {
                                obj = true;
                            }
                            else if (srcStr.Equals("false", StringComparison.OrdinalIgnoreCase))
                            {
                                obj = false;
                            }
                            else//数値の場合は0の場合false, 0ではない場合trueとする
                            {
                                int inData = Convert.ToInt32(srcStr);
                                if (inData == 0)
                                    obj = false;
                                else
                                    obj = true;
                            }
                        }
                        break;

                    case EGDEFINE.egTagDataType.SByte:
                        {
                            SByte tmpData = 0;
                            if (SByte.TryParse(srcStr, out tmpData))
                                obj = (object)tmpData;
                            else
                                obj = null;
                        }
                        break;

                    case EGDEFINE.egTagDataType.Byte:
                        {
                            Byte tmpData = 0;
                            if (Byte.TryParse(srcStr, out tmpData))
                                obj = (object)tmpData;
                            else
                                obj = null;
                        }
                        break;

                    case EGDEFINE.egTagDataType.Int16:
                        {
                            Int16 tmpData = 0;
                            if (Int16.TryParse(srcStr, out tmpData))
                                obj = (object)tmpData;
                            else
                                obj = null;
                        }
                        break;

                    case EGDEFINE.egTagDataType.UInt16:
                        {
                            UInt16 tmpData = 0;
                            if (UInt16.TryParse(srcStr, out tmpData))
                                obj = (object)tmpData;
                            else
                                obj = null;
                        }
                        break;

                    case EGDEFINE.egTagDataType.Int32:
                        {
                            Int32 tmpData = 0;
                            if (Int32.TryParse(srcStr, out tmpData))
                                obj = (object)tmpData;
                            else
                                obj = null;
                        }
                        break;

                    case EGDEFINE.egTagDataType.UInt32:
                        {
                            UInt32 tmpData = 0;
                            if (UInt32.TryParse(srcStr, out tmpData))
                                obj = (object)tmpData;
                            else
                                obj = null;
                        }
                        break;

                    case EGDEFINE.egTagDataType.Int64:
                        {
                            Int64 tmpData = 0;
                            if (Int64.TryParse(srcStr, out tmpData))
                                obj = (object)tmpData;
                            else
                                obj = null;
                        }
                        break;

                    case EGDEFINE.egTagDataType.UInt64:
                        {
                            UInt64 tmpData = 0;
                            if (UInt64.TryParse(srcStr, out tmpData))
                                obj = (object)tmpData;
                            else
                                obj = null;
                        }
                        break;

                    case EGDEFINE.egTagDataType.Float:
                        {
                            float tmpData = 0;
                            if (float.TryParse(srcStr, out tmpData))
                                obj = (object)tmpData;
                            else
                                obj = null;
                        }
                        break;

                    case EGDEFINE.egTagDataType.Double:
                        {
                            double tmpData = 0;
                            if (double.TryParse(srcStr, out tmpData))
                                obj = (object)tmpData;
                            else
                                obj = null;
                        }
                        break;

                    case EGDEFINE.egTagDataType.String:
                        {
                            obj = (object)srcStr;
                        }
                        break;

                    case EGDEFINE.egTagDataType.Segment:
                        {
                            //ASCII エンコード
                            byte[] tmpData = System.Text.Encoding.ASCII.GetBytes(srcStr);
                            obj = (object)tmpData;
                        }
                        break;

                    default:
                        obj = null;
                        break;
                }
                return obj;
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return null;
            }
        }

        /// <summary>
        /// 型にあったデータをobject型としてnewして返す
        /// </summary>
        /// <param name="type"></param>
        /// <param name="size"></param>
        /// <returns></returns>
        public static object SetTagType2Objct(ushort type, ushort size)
        {
            try
            {
                
                EGDEFINE.egTagDataType enType = (EGDEFINE.egTagDataType)type;
                switch (enType)
                {
                    case EGDEFINE.egTagDataType.Boolean:
                        return new bool();

                    case EGDEFINE.egTagDataType.SByte:
                        return new sbyte();

                    case EGDEFINE.egTagDataType.Byte:
                        return new byte();

                    case EGDEFINE.egTagDataType.Int16:
                        return new Int16();

                    case EGDEFINE.egTagDataType.UInt16:
                        return new UInt16();

                    case EGDEFINE.egTagDataType.Int32:
                        return new Int32();

                    case EGDEFINE.egTagDataType.UInt32:
                        return new UInt32();

                    case EGDEFINE.egTagDataType.Int64:
                        return new Int64();

                    case EGDEFINE.egTagDataType.UInt64:
                        return new UInt64();

                    case EGDEFINE.egTagDataType.Float:
                        return new float();

                    case EGDEFINE.egTagDataType.Double:
                        return new double();

                    case EGDEFINE.egTagDataType.String:
                        return new byte[size];

                    case EGDEFINE.egTagDataType.Segment:
                        return new byte[size];

                    default:
                        break;
                }
                return null;
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return null;
            }
        }
        /*━━━━━━━━━━━━━━━━━━━━━━━━━━
        * 機能
        * 　指定された位置のビットをチェックします
        * 引数
        * 　bySrc   : 入力データ
        * 　byBitNo : チェックするビット位置
        * 　byDes　 : 計算結果
        * 戻り値
        *   0                   : 成功
        *   0x0000000300000001  : ビット位置指定範囲外
        *   上記以外  : システムエラー
         * 備考
         *   00000010(社内技術情報)から流用
        *━━━━━━━━━━━━━━━━━━━━━━━━━━*/
        private static int fnBitChek(byte bySrc, byte byBitNo, ref bool bIsON)
        {
            byte by1 = 0x01;     　　           //論理積計算用の変数

            //引数の範囲チェック（0から7）範囲外はエラー
            if ((byBitNo < 0) || (byBitNo > 7))
            {
                return -1;
            }

            try
            {
                by1 <<= byBitNo;                //引数分だけシフトさせる
                bySrc &= by1;                   //引数と変数の論理積を計算
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return -1;
            }

            //処理結果をセット
            if (bySrc == 0x00)
            {
                bIsON = false;
            }
            else
            {
                bIsON = true;
            }
            return 0;       //正常終了

        }
        /*━━━━━━━━━━━━━━━━━━━━━━━━━━
         * 機能
         * 　バイト配列から構造体へ内容のコピーを行う
         * 引数
         * 　bySrc   : コピー元になるバイト配列 
         * 　objDes  : コピー先の構造体 
         * 戻り値 
         *   0                   : 成功
         *   0x0000001600000001  : 入力と出力のサイズアンマッチ 
         *   上記以外   : システムエラー
         * 備考
         *   00000010(社内技術情報)から流用
        *━━━━━━━━━━━━━━━━━━━━━━━━━━*/
        public static int fnByToObj(byte[] bySrc, ref object objDes)
        {
            IntPtr ptrDestination = IntPtr.Zero;        //作業領域のポインタ

            int iobjSize = Marshal.SizeOf(objDes);      //オブジェクトのサイズを取得
                                                        //オブジェクトの中身が空でもゼロでなくて１がかえる

            // 処理の試行
            try
            {
                // 作業領域の作成
                //ptrDestination = Marshal.AllocHGlobal(Marshal.SizeOf(objDes));
                ptrDestination = Marshal.AllocHGlobal(bySrc.Length);

                // バイト配列の内容を作業領域へコピー
                //Marshal.Copy(bySrc, 0, ptrDestination, Marshal.SizeOf(objDes));
                Marshal.Copy(bySrc, 0, ptrDestination, bySrc.Length);

                // 作業領域の内容をオブジェクトへコピー
                objDes = Marshal.PtrToStructure(ptrDestination, objDes.GetType());
            }
            catch (Exception)
            {
                return -1;          //エラーコード編集
            }
            finally
            {
                // 作業領域の解放
                Marshal.FreeHGlobal(ptrDestination);
            }

            // 処理の終了
            return 0;       //正常終了
        }
        /*━━━━━━━━━━━━━━━━━━━━━━━━━━
         * 機能
         * 　構造体からバイト配列へ内容のコピーを行う
         * 引数
         * 　objSrc   : コピー元になる構造体 
         * 　byDes    : コピー先のバイト配列 
         * 戻り値 
         *   0                   : 成功
         *   0x0000001700000001  : 入力と出力のサイズアンマッチ 
         *   上記以外   : システムエラー
         * 備考
         *   00000010(社内技術情報)から流用
        *━━━━━━━━━━━━━━━━━━━━━━━━━━*/
        public static int fnObjToBy(object objSrc, ref byte[] byDes)
        {
            IntPtr ptrSrc = IntPtr.Zero;                //作業領域のポインタ

            int iobjSize = Marshal.SizeOf(objSrc);      //オブジェクトのサイズを取得

            // 処理の試行
            try
            {
                // 作業領域の作成
                ptrSrc = Marshal.AllocHGlobal(Marshal.SizeOf(objSrc));

                // オブジェクト内容を作業領域へコピー
                Marshal.StructureToPtr(objSrc, ptrSrc, false);

                // 作業領域の内容をバイト配列へコピー
                Marshal.Copy(ptrSrc, byDes, 0, Marshal.SizeOf(objSrc));
            }
            catch (Exception ex)
            {
                // 例外の格納
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return -1;
            }
            finally
            {
                // 作業領域の解放
                Marshal.FreeHGlobal(ptrSrc);
            }

            // 処理の終了
            return 0;       //正常終了
        }
    }
}
