﻿////
/// Copyright(C) 2020 Micronet Co. All Rights Reserved.
//
using System;
using System.Collections.Generic;
using System.Runtime.InteropServices;
namespace RTedge
{
    //RT-edge内オブジェクト
    public class EgHgObject
    {
        public string Name;
        public EgHgObject()
        {
            Name = "";
        }
    }
    /// <summary>
    /// Serviceで保持するTagの参照
    /// </summary>
    public class EgHgTagRef : EgHgObject
    {
        public ushort Size;
        public ushort Type;
        public string Address;
        public IntPtr Entry;
        public object OldVal;
        public EgHgTagRef()
        {
            Name = "";
            Size = 0;
            Type = 0;
            Address = "";
            Entry = IntPtr.Zero;
            OldVal = null;
        }
        public EgHgTagRef(string stName, ushort shtype, ushort shsize, string stAddress, IntPtr pEntry, byte[] byOldVal)
        {
            Name = stName;
            Type = shtype;
            Size = shsize;
            Address = stAddress;
            Entry = pEntry;
            OldVal = egCommonFunc.SetTagType2Objct(shtype,shsize);
            if ((EGDEFINE.egTagDataType)shtype != EGDEFINE.egTagDataType.String &&
                (EGDEFINE.egTagDataType)shtype != EGDEFINE.egTagDataType.Segment)//string, segmentは対象外
            egCommonFunc.fnByToObj(byOldVal, ref OldVal);
        }
    }
    public class EgHgEdgeIOPort
    {
        public IntPtr Entry;
        public string SrcAddr;
        public string AddrName;
        //public ushort CategoryNum;
        //public uint[] Category;
        public int Index;
        public ulong Offset;
        public byte BitUse;
        public ushort BitPos;
        public ushort MaxBit;
        public byte IsValied;
        public EgHgEdgeIOPort()
        {
            Entry = IntPtr.Zero;
            SrcAddr = "";
            AddrName = "";
            Offset = 0;
            BitUse = 0;
            BitPos = 0;
            MaxBit = 0;
            IsValied = 0;
        }
        public EgHgEdgeIOPort(IntPtr pEntry, string stSrcAddr, string stAddrName, int iIndex, ulong ulOffset, byte byBitUse, ushort woBitPos, ushort woMaxBit, byte byIsValied)
        {
            Entry = pEntry;
            SrcAddr = stSrcAddr;
            AddrName = stAddrName;
            Index = iIndex;
            Offset = ulOffset;
            BitUse = byBitUse;
            BitPos = woBitPos;
            MaxBit = woMaxBit;
            IsValied = byIsValied;
        }

    }
    /// <summary>
    /// tagrefの集まり
    /// </summary>
    public class EgHgDataset : EgHgObject
    {
        public List<EgHgTagRef> TagRefList;
        public uint HashValue;
        public EgHgDataset()
        {
            Name = "";
            TagRefList = new List<EgHgTagRef>();
            HashValue = 0;
        }
        public EgHgDataset(string stName, uint uiHash)
        {
            Name = stName;
            HashValue = uiHash;
            TagRefList = new List<EgHgTagRef>();
        }

        public void AddTagRef(EgHgTagRef tagref)
        {
            TagRefList.Add(tagref);
        }

    }
    /// <summary>
    /// EgHgMailBox
    /// </summary>
    public class EgHgMailBox : EgHgObject
    {
        public uint ColSize;
        public uint RowSize;
        public edge_API.RTCDMEMINFO stMemInfo;
        public EgHgMailBox()
        {
            Name = "";
            ColSize = 2048; //ﾃﾞﾌｫﾙﾄ値
            RowSize = 1024; //ﾃﾞﾌｫﾙﾄ値
            stMemInfo = new edge_API.RTCDMEMINFO();
        }
        public EgHgMailBox(string stName, uint uiColSize, uint uiRowSize, edge_API.RTCDMEMINFO MemInfo)
        {
            Name = stName;
            ColSize = uiColSize;
            RowSize = uiRowSize;
            stMemInfo = MemInfo;
        }
    }
    /// <summary>
    /// EgHgCollectorRef
    /// </summary>
    public class EgHgCollectorRef : EgHgObject
    {
        public uint ColSize;
        public uint RowSize;
        public string DatasetName;
        public EgHgCollectorRef()
        {
            Name = "";
            ColSize = 0; //ﾃﾞﾌｫﾙﾄ値
            RowSize = 0; //ﾃﾞﾌｫﾙﾄ値
            DatasetName = "";
        }
        public EgHgCollectorRef(string stName, uint uiColSize, uint uiRowSize, string stDatasetName)
        {
            Name = stName;
            ColSize = uiColSize;
            RowSize = uiRowSize;
            DatasetName = stDatasetName;
        }
    }
    /// <summary>
    /// EgHgTagTrigger
    /// </summary>
    public class EgHgTagTrigger : EgHgObject
    {
        public ushort Size;
        public ushort Type;
        public string Address;
        public IntPtr Entry;
        public byte Count;
        public EgHgTagTrigger()
        {
            Name = "";
            Size = 0;
            Type = 0;
            Address = "";
            Entry = IntPtr.Zero;
            Count = 0;
        }
        public EgHgTagTrigger(string stName, ushort shtype, ushort shsize, string stAddress, IntPtr pEntry, byte byCount)
        {
            Name = stName;
            Type = shtype;
            Size = shsize;
            Address = stAddress;
            Entry = pEntry;
            Count = byCount;
        }
    }
    //------------------------------------------------------------------------------
    // TagRef オブジェクト
    //------------------------------------------------------------------------------
    [StructLayout(LayoutKind.Sequential, CharSet = CharSet.Ansi, Pack = 1)]
    public struct EGTAGREF
    {
        [MarshalAs(UnmanagedType.ByValTStr, SizeConst = EGDEFINE.EG_MAX_TAGNAME)]
        public string   Name;
        public byte _nil00;
        [MarshalAs(UnmanagedType.ByValTStr, SizeConst = EGDEFINE.EG_MAX_TAGADDRESS)]
        public string Address;
        public byte _nil01;
        public ushort   Type;
        public ushort   Size;
        public IntPtr   Entry;
        [MarshalAs(UnmanagedType.ByValArray, SizeConst = EGDEFINE.EG_DIFF_VALUE_SIZE)]
        public byte[] byOldVal;
        public EGTAGREF(string stName, ushort usType, ushort usSize, string stAddress, IntPtr pEntry, byte[] byOldVal)
        {
            this.Name = stName;
            this.Type = usType;
            this.Size = usSize;
            this.Address = stAddress;
            this.Entry = pEntry;
            this._nil00 = 0;
            this._nil01 = 0;
            this.byOldVal = byOldVal;
        }
    }

    //------------------------------------------------------------------------------
    // CollectorRef オブジェクト
    //------------------------------------------------------------------------------
    [StructLayout(LayoutKind.Sequential, CharSet = CharSet.Ansi, Pack = 1)]
    public struct EGCOLLECTORREF
    {
        [MarshalAs(UnmanagedType.ByValTStr, SizeConst = EGDEFINE.EG_MAX_CLCTNAME)]
        public string CollectorName;
        public byte _nil00;
        [MarshalAs(UnmanagedType.ByValTStr, SizeConst = EGDEFINE.EG_MAX_DSETNAME)]
        public string DatasetName;
        public byte _nil01;
        public uint dwColSize;
        public uint dwRowSize;
        public EGCOLLECTORREF(string stCollectorName, uint dwColSize, uint dwRowSize, string stDatasetName)
        {
            this.CollectorName = stCollectorName;
            this.dwColSize = dwColSize;
            this.dwRowSize = dwRowSize;
            this.DatasetName = stDatasetName;
            this._nil00 = 0;
            this._nil01 = 0;
        }
    }

    //------------------------------------------------------------------------------
    // Mailbox オブジェクト
    //------------------------------------------------------------------------------
    [StructLayout(LayoutKind.Sequential, CharSet = CharSet.Ansi, Pack = 1)]
    public struct EGMAILBOX
    {
        [MarshalAs(UnmanagedType.ByValTStr, SizeConst = EGDEFINE.EG_MAX_MAILBOXNAME)]
        public string MailBoxName;
        public byte _nil00;
        public uint dwColSize;
        public uint dwRowSize;
        public edge_API.RTCDMEMINFO MemInfo;
        public EGMAILBOX(string stMailBoxName, uint dwColSize, uint dwRowSize, edge_API.RTCDMEMINFO stMemInfo)
        {
            this.MailBoxName = stMailBoxName;
            this.dwColSize = dwColSize;
            this.dwRowSize = dwRowSize;
            this.MemInfo = stMemInfo;
            this._nil00 = 0;
        }
    }
    //------------------------------------------------------------------------------
    // EdgeIOポート オブジェクト
    //------------------------------------------------------------------------------
    [StructLayout(LayoutKind.Sequential, CharSet = CharSet.Ansi, Pack = 1)]
    public struct EGIOPORT
    {
        public IntPtr Entry;
        [MarshalAs(UnmanagedType.ByValTStr, SizeConst = EGDEFINE.EG_MAX_TAGADDRESS)]
        public string SrcAddr;
        public byte _nil00;
        [MarshalAs(UnmanagedType.ByValTStr, SizeConst = EGDEFINE.EG_MAX_CATALOGNAME)]
        public string AddrName;
        public byte _nil01;
        public int Index;
        public uint Offset;
        public byte BitUse;
        public ushort BitPos;
        public ushort MaxBit;
        public byte IsValied;
    }
    //------------------------------------------------------------------------------
    // TagTrigger オブジェクト
    //------------------------------------------------------------------------------
    [StructLayout(LayoutKind.Sequential, CharSet = CharSet.Ansi, Pack = 1)]
    public struct EGTAGTRIGGER
    {
        [MarshalAs(UnmanagedType.ByValTStr, SizeConst = EGDEFINE.EG_MAX_TAGNAME)]
        public string Name;
        public byte _nil00;
        [MarshalAs(UnmanagedType.ByValTStr, SizeConst = EGDEFINE.EG_MAX_TAGADDRESS)]
        public string Address;
        public byte _nil01;
        public IntPtr Entry;
        public ushort Type;
        public ushort Size;
        public byte Count;
    
        public EGTAGTRIGGER(string stName, ushort usType, ushort usSize, string stAddress, IntPtr pEntry, byte byCount)
        {
            this.Name = stName;
            this.Type = usType;
            this.Size = usSize;
            this.Address = stAddress;
            this.Entry = pEntry;
            this._nil00 = 0;
            this._nil01 = 0;
            this.Count = byCount;
        }
    }
    //------------------------------------------------------------------------------
    // TagTrigger スレッド間データ受け渡し用　1Tag情報
    //------------------------------------------------------------------------------
    [StructLayout(LayoutKind.Sequential, CharSet = CharSet.Ansi, Pack = 1)]
    public struct EGTAGTRIGINFO
    {
        public ushort arrNo;    //更新タグトリガー配列Index番号
        public ulong var;       //更新データ
    }
    //------------------------------------------------------------------------------
    // TagTrigger スレッド間データ受け渡し用 1メッセージ
    //------------------------------------------------------------------------------
    [StructLayout(LayoutKind.Sequential, CharSet = CharSet.Ansi, Pack = 1)]
    public struct EGTAGTRIGINFOS
    {
        public ushort TblCount;              // テーブルの配列数
        public IntPtr pTblTagTriggers;       // タグトリガー構造体テーブル
    }
}
