﻿////
/// Copyright(C) 2020 Micronet Co. All Rights Reserved.
//
using System;
using System.IO;
using System.Collections.Generic;
using System.Text;
using System.Diagnostics;           // Debug.WriteLine出力用
using System.Reflection;
using System.Runtime.InteropServices;
namespace RTedge
{
    /// <summary>
    /// edge Framework C# Wrapper
    /// </summary>
    public partial class edge_FW : egBaseClass
    {
        /// <summary>
        /// パスの参照先設定
        /// </summary>
        /// <param name="lpPathName"></param>
        /// <returns></returns>
        [DllImport("Kernel32.dll")]
        static extern bool SetDllDirectory(string lpPathName);
        /// <summary>
        /// setDllLocation
        /// </summary>
        /// <returns></returns>
        static bool setDllLocation()
        {
            string dllDir;
            string filePath;
            dllDir = Path.GetDirectoryName(Assembly.GetExecutingAssembly().Location);        // step1: 現在のdllの位置
            filePath = dllDir + "\\eghgapiWin.dll";
            if (Directory.Exists(dllDir) && File.Exists(filePath))
            {
                SetDllDirectory(dllDir);
                return true;
            }
            dllDir = Environment.GetEnvironmentVariable("RTEDGE"); ;                    // step2: 環境変数RTEDGE位置
            filePath = dllDir + "eghgapiWin.dll";
            if (Directory.Exists(dllDir))
            {
                SetDllDirectory(dllDir);
                return true;
            }

            return false;
        }

        /// <summary>
        /// 関数追加関連
        /// </summary>
        public delegate int InstanceMessageHandler(string senderName, int messNo, byte[] paramArray);
        public edge_API.InstanceMessageHandler mInstMessFunc;
        public edge_API.CallbackDelegateEx mCallbackMessFuncEx;

        /// <summary>
        /// Initialize終了時のコールバック関数・関数追加関連
        /// </summary>
        [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
        public delegate void CallbackInitEndFuncDelegate();
        public delegate void InstanceInitEndFunc();
        public static InstanceInitEndFunc mInstInitEndFunc;
        public CallbackInitEndFuncDelegate mCallbackInitEndFunc;

        /// <summary>
        /// デバッグ出力用コールバック関数・関数追加関連
        /// </summary>
        [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
        public delegate void CallbackDebugFuncDelegate();
        public delegate void InstanceDebugFunc();
        public static InstanceDebugFunc mInstDebugFunc;
        public CallbackDebugFuncDelegate mCallbackDebugFunc;

        public string ServiceName;                //サービス名(メールボックス名)
        public string ServiceMboxName;            //メールボックス名
        public string ServiceRealName;            //サービス名

        public List<EgHgTagRef> tagRef_OUT_List;  //xml記載済みの参照Tagリスト
        public List<EgHgTagRef> tagRef_IN_List;   //xml記載済みの参照Tagリスト
        public List<EgHgTagRef> tagRefList;       //xml記載済みの参照Tagリスト
        public List<EgHgEdgeIOPort> tagRef_OUT_IOPortList;  //TagRefs_OUTに対応するEdgeIOポートリスト
        public List<EgHgEdgeIOPort> tagRef_IN_IOPortList;   //TagRefs_INに対応するEdgeIOポートリスト
        public List<EgHgEdgeIOPort> tagRef_IOPortList;      //TagRefsに対応するEdgeIOポートリスト
        public List<EgHgCollectorRef> clctRefList; //xml記載済みの参照コレクタ
        public List<EgHgMailBox> mailBoxList;     //xml記載済みのメールボックスリスト
        public List<EgHgTagTrigger> tagTriggerList;     //xml記載済みのTagTriggerリスト
        
        bool bIsEndCoreReadXmlF;
        /// <summary>
        /// Constructors
        /// </summary>
        public edge_FW()
        {
            setDllLocation();                           //Dllフォルダの設定(anycpu用)
            mInstMessFunc = null;                       //メッセージ用コールバック関数 instance
            mCallbackMessFuncEx = null;                 //メッセージ用コールバック関数 delegate用 サイズ可変用
            mInstInitEndFunc = null;                    //イニシャライズ終了時の処理用コールバック関数 instance
            mCallbackInitEndFunc = null;                //イニシャライズ終了時の処理用コールバック関数 delegate用
            mInstDebugFunc = null;                      //debug処理用コールバック関数 instance
            mCallbackDebugFunc = null;                  //debug処理用コールバック関数 delegate用

            tagRef_OUT_List = new List<EgHgTagRef>();   //tagRef_OUTのリスト Initialize終了時に格納
            tagRef_IN_List = new List<EgHgTagRef>();    //tagRef_INのリスト Initialize終了時に格納
            tagRefList = new List<EgHgTagRef>();        //tagRefのリスト Initialize終了時に格納
            tagRef_OUT_IOPortList = new List<EgHgEdgeIOPort>();     //TagRefs_OUTに対応するEdgeIOポートリスト
            tagRef_IN_IOPortList = new List<EgHgEdgeIOPort>();      //TagRefs_INに対応するEdgeIOポートリスト
            tagRef_IOPortList = new List<EgHgEdgeIOPort>();         //TagRefsに対応するEdgeIOポートリスト
            clctRefList = new List<EgHgCollectorRef>(); //clctRefのリスト Initialize終了時に格納
            mailBoxList = new List<EgHgMailBox>();      //mailBoxのリスト Initialize終了時に格納
            tagTriggerList = new List<EgHgTagTrigger>();    //tagtriggerのリスト Initialize終了時に格納
            bIsEndCoreReadXmlF = false;                 //
            
            AddUserInitEndFunc(InitMember);             //関数の追加
            AddDebugFunc(DebugFunc);                    //デバッグ用コールバック関数追加
        }
        /// <summary>
        /// 初期化処理
        /// </summary>
        public int Initialize(edge_API._EDGECONFIG _config)
        {
            int ret = -1;
            try
            {
                int count = 0;
                while(true)
                {
                    // xmlの読み込みがない または EgBootがxmlを読み込んだフラグが立った または 5秒経過したら抜ける
                    if (_config.bReadXmlF == 0 || bIsEndCoreReadXmlF || count == 10)
                    {
                        ret = 0;
                        break;
                    }
                    count++;
                    System.Threading.Thread.Sleep(500);
                }
                ServiceName = GetServiceName(); //C側のフレームワークからサービスメールボックス名の取得
                ServiceMboxName = GetServiceMboxName(); //C側のフレームワークからサービスメールボックス名の取得
                ServiceRealName = GetServiceRealName(); //C側のフレームワークからサービス名の取得
                return ret;
            }
            catch (Exception ex)
            {
                PrintSysLog(EG_LOG_FER, "FATAL ERROR : {0}", System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return ret;
            }
        }
        


        /// <summary>
        /// 管理対象オブジェクトを解放する
        /// </summary>
        protected override void fnReleaseManagedObject()
        {
            PrintSysLog(EG_LOG_DBG, "Start Win ReleaseManagedObject");
            System.Threading.Thread.Sleep(500); //mailbox等close用
            base.fnReleaseManagedObject();
            //ログ出力
            PrintSysLog(EG_LOG_DBG, "End Win ReleaseManagedObject");
        }
        /// <summary>
        /// UserMessageWrap
        /// </summary>
        /// <param name="senderName"></param>
        /// <param name="inMessageNo"></param>
        /// <param name="Data"></param>
        /// <returns></returns>
        public int UserMessageWrap(string senderName, int inMessageNo, IntPtr Data, int inSize)
        {
            try
            {
                //Wrap用タグトリガーRef更新メッセージ処理
                if(inMessageNo == (int)EGDEFINE.EGMSG.EM_WRAPONLY_UPDATE_TAGTRIGREF)
                {
                    UpdateMemger_TagTrigger();
                    return 0;
                }
                if (mInstMessFunc != null)
                {
                    //↓これをやらないと渡せなかったためわざわざコールバック関数のラップを作った
                    byte[] bytes = new byte[inSize];
                    Marshal.Copy(Data, bytes, 0, inSize);
                    return mInstMessFunc(senderName, inMessageNo, bytes);
                }
                return -1;
                ;
            }
            catch (Exception ex)
            {
                edge_FW._PrintSysLog("",EG_LOG_FER, "FATAL ERROR : {0}", System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return -1;
            }
        }


        /// <summary>
        /// xmlでサービス内のオブジェクトが登録された場合に呼び出す関数の追加 ラッパー
        /// </summary>
        static public void UserInitEndMessageWrap()
        {
            try
            {
                if (mInstInitEndFunc != null)
                {
                    mInstInitEndFunc(); //コールバック関数呼び出し
                }

            }
            catch (Exception ex)
            {
                edge_FW._PrintSysLog("", EG_LOG_FER, "FATAL ERROR : {0}", System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
            }
        }
        /// <summary>
        /// xmlでサービス内のオブジェクトが登録された場合に呼び出す関数の追加
        /// </summary>
        /// <param name="b"></param>
        public void AddUserInitEndFunc(InstanceInitEndFunc b)
        {
            try
            {
                mInstInitEndFunc = new InstanceInitEndFunc(b);
                mCallbackInitEndFunc = new CallbackInitEndFuncDelegate(UserInitEndMessageWrap);
                _C_EgFW_AddInitEndFunc(mCallbackInitEndFunc);
            }
            catch (Exception ex)
            {
                PrintSysLog(EG_LOG_FER, "FATAL ERROR : {0}", System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
            }

        }
        /// <summary>
        /// デバッグ用関数の追加 ラッパー
        /// </summary>
        static public void DebugFuncMessageWrap()
        {
            try
            {
                if (mInstDebugFunc != null)
                {
                    mInstDebugFunc();   //コールバック関数呼び出し
                }

            }
            catch (Exception ex)
            {
                edge_FW._PrintSysLog("", EG_LOG_FER, "FATAL ERROR : {0}", System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
            }
        }
        /// <summary>
        /// デバッグ用関数の追加
        /// </summary>
        /// <param name="b"></param>
        public void AddDebugFunc(InstanceDebugFunc b)
        {
            try
            {
                mInstDebugFunc = new InstanceDebugFunc(b);
                mCallbackDebugFunc = new CallbackDebugFuncDelegate(DebugFuncMessageWrap);
                _C_EgFW_AddDebugFunc(mCallbackDebugFunc);
            }
            catch (Exception ex)
            {
                PrintSysLog(EG_LOG_FER, "FATAL ERROR : {0}", System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
            }

        }
        /// <summary>
        /// サービス名の取得
        /// </summary>
        /// <returns></returns>
        public string GetServiceName()
        {
            try
            {
                StringBuilder sbServiceName = new StringBuilder(EGDEFINE.EG_MAX_SERVICEMAILBOXNAME + 1);//サービスメールボックス名最大文字数を指定 null考慮しないと内部でエラーになる
                if(_C__EgFW_GetServiceName(sbServiceName) != 0)
                    return "";
                return sbServiceName.ToString();
            }
            catch(Exception ex)
            {
                PrintSysLog(EG_LOG_FER, "FATAL ERROR : {0}", System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return "";
            }
        }
        /// <summary>
        /// xmlで記載のサービス名取得
        /// </summary>
        /// <returns></returns>
        public string GetServiceRealName()
        {
            try
            {
                StringBuilder sbServiceRealName = new StringBuilder(EGDEFINE.EG_MAX_SERVICENAME + 1);//サービス名最大文字数を指定 null考慮しないと内部でエラーになる
                if (_C__EgFW_GetServiceRealName(sbServiceRealName) != 0)
                    return "";
                return sbServiceRealName.ToString();
            }
            catch (Exception ex)
            {
                PrintSysLog(EG_LOG_FER, "FATAL ERROR : {0}", System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return "";
            }
        }
        /// <summary>
        /// サービスメールボックスの取得
        /// </summary>
        /// <returns></returns>
        public string GetServiceMboxName()
        {
            try
            {
                StringBuilder sbServiceMboxName = new StringBuilder(EGDEFINE.EG_MAX_SERVICEMAILBOXNAME + 1);//サービスメールボックス名最大文字数を指定 null考慮しないと内部でエラーになる
                if (_C__EgFW_GetServiceMboxName(sbServiceMboxName) != 0)
                    return "";
                return sbServiceMboxName.ToString();
            }
            catch (Exception ex)
            {
                PrintSysLog(EG_LOG_FER, "FATAL ERROR : {0}", System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return "";
            }
        }
        /// <summary>
        /// C#用 Wrpparクラスで保持するメンバの更新(xmlからサービス内のオブジェクトの登録が終わったら毎回この関数が呼ばれる)
        /// </summary>
        protected void InitMember()
        {
            try
            {
                PrintSysLog(EG_LOG_DBG, "InitMember Start");
                ServiceName = GetServiceName();             //C側のframeworkからサービスメールボックス名の取得
                ServiceMboxName = GetServiceMboxName();     //C側のframeworkからサービスメールボックス名の取得
                ServiceRealName = GetServiceRealName();     //C側のframeworkからサービス名の取得
                _GetTagRefs_OUT();                          //C側のframeworkからTagRefOUTを取得
                _GetTagRefs_IN();                           //C側のframeworkからTagRefINを取得
                _GetTagRefs();                              //C側のframeworkからTagRefを取得
                _GetTagRefs_OUT_PortInfo();                 //C側のframeworkからTagRefOUTに対応するEdgeIOポート情報を取得
                _GetTagRefs_IN_PortInfo();                  //C側のframeworkからTagRefINに対応するEdgeIOポート情報を取得
                _GetTagRefs_PortInfo();                     //C側のframeworkからTagRefに対応するEdgeIOポート情報を取得
                _GetClctRefs();                             //C側のframeworkからCollectorRefを取得
                _GetMailbox();                              //C側のframeworkからMailboxを取得
                _GetTagTrigger();                           //C側のframeworkからTagTriggerを取得

                bIsEndCoreReadXmlF = true;                  //xml読み終わったよフラグをtrueに
                PrintSysLog(EG_LOG_DBG, "InitMember End");
            }
            catch (Exception ex)
            {
                PrintSysLog(EG_LOG_FER, "FATAL ERROR : {0}", System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
            }
        }
        /// <summary>
        /// タグトリガー更新 (何度も更新されるため
        /// </summary>
        protected void UpdateMemger_TagTrigger()
        {
            _GetTagTrigger();
        }

        //-------------------------------------------------------
        // private function
        //-------------------------------------------------------
        /// <summary>
        /// TagRefs_OUTをCのフレームワークから取得(全)
        /// </summary>
        private void _GetTagRefs_OUT()
        {
            try
            {
                //EGTAGREF[] tempTagRefs;
                int count = _C__EgFW_GetEgTagRef_OUT_Count();
                if (count < 1)
                    return;

                tagRef_OUT_List.Clear();
                for (int i = 0; i < count; i++)
                {
                    _GetTagRefs_OUT(i);
                }
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
            }
        }
        /// <summary>
        /// TagRefs_OUTをCのフレームワークから取得
        /// </summary>
        /// <param name="index"></param>
        /// <returns></returns>
        private int _GetTagRefs_OUT(int index)
        {
            int ret = -1;
            IntPtr pPtr = IntPtr.Zero;
            try
            {
                //メモリリークをさせないためのおまじない
                System.Runtime.CompilerServices.RuntimeHelpers.PrepareConstrainedRegions();
                try { }
                finally
                {
                    //メモリ確保
                    pPtr = Marshal.AllocCoTaskMem(Marshal.SizeOf(typeof(EGTAGREF)));
                }
                int rtn = _C__EgFW_GetTagRef_OUT(pPtr, index);
                if (rtn == 0)
                {
                    EGTAGREF tempTagRef = (EGTAGREF)Marshal.PtrToStructure(pPtr, typeof(EGTAGREF));
                    if (tempTagRef.Name.Length > 0) //名前があればとってこれたも同然！
                    {
                        EgHgTagRef item = new EgHgTagRef(tempTagRef.Name, tempTagRef.Type, tempTagRef.Size, tempTagRef.Address, tempTagRef.Entry, tempTagRef.byOldVal);
                        tagRef_OUT_List.Add(item);
                    }
                }
                else
                {
                    PrintSysLog(EG_LOG_ERR, "Could not be added TagRef because the number of TagRef exceeded.");
                    EgHgTagRef item = new EgHgTagRef("__ERROR__", 0, 0,"", IntPtr.Zero, null);
                    tagRef_OUT_List.Add(item);  //一応エラー用のタグを追加
                    ret = -1;
                }
                return ret;
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return ret;
            }
            finally
            {
                if (pPtr != IntPtr.Zero)
                    Marshal.FreeCoTaskMem(pPtr);
            }
        }
        /// <summary>
        /// TagRefs_INをCのフレームワークから取得(全)
        /// </summary>
        private void _GetTagRefs_IN()
        {
            try
            {
                int count = _C__EgFW_GetEgTagRef_IN_Count();
                if (count < 1)
                    return;

                tagRef_IN_List.Clear();
                for (int i = 0; i < count; i++)
                {
                    _GetTagRefs_IN(i);
                }
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
            }
        }
        /// <summary>
        /// TagRefs_INをCのフレームワークから取得
        /// </summary>
        /// <param name="index"></param>
        /// <returns></returns>
        private int _GetTagRefs_IN(int index)
        {
            int ret = -1;
            IntPtr pPtr = IntPtr.Zero;
            try
            {
                //メモリリークをさせないためのおまじない
                System.Runtime.CompilerServices.RuntimeHelpers.PrepareConstrainedRegions();
                try { }
                finally
                {
                    //メモリ確保
                    pPtr = Marshal.AllocCoTaskMem(Marshal.SizeOf(typeof(EGTAGREF)));
                }
                int rtn = _C__EgFW_GetTagRef_IN(pPtr, index);
                if (rtn == 0)
                {
                    EGTAGREF tempTagRef = (EGTAGREF)Marshal.PtrToStructure(pPtr, typeof(EGTAGREF));
                    if (tempTagRef.Name.Length > 0)//名前があればとってこれたも同然！
                    {
                        EgHgTagRef item = new EgHgTagRef(tempTagRef.Name, tempTagRef.Type, tempTagRef.Size, tempTagRef.Address, tempTagRef.Entry, tempTagRef.byOldVal);
                        tagRef_IN_List.Add(item);
                    }
                }
                else
                {
                    PrintSysLog(EG_LOG_ERR, "Could not be added TagRef because the number of TagRef exceeded.");
                    EgHgTagRef item = new EgHgTagRef("__ERROR__", 0, 0, "", IntPtr.Zero, null);
                    tagRef_IN_List.Add(item);//一応エラー用のタグを追加
                    ret = -1;
                }
                return ret;
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return ret;
            }
            finally
            {
                if (pPtr != IntPtr.Zero)
                    Marshal.FreeCoTaskMem(pPtr);
            }
        }
        /// <summary>
        /// TagRefをCのフレームワークから取得
        /// </summary>
        private void _GetTagRefs()
        {
            IntPtr pPtr = IntPtr.Zero;
            try
            {
                int count = _C__EgFW_GetEgTagRef_Count();
                if (count < 1)
                    return;

                tagRefList.Clear();
                for (int i = 0; i < count; i++)
                {
                    _GetTagRefs(i);
                }
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
            }

        }
        /// <summary>
        /// TagRefをCのフレームワークから取得
        /// </summary>
        /// <param name="index"></param>
        /// <returns></returns>
        private int _GetTagRefs(int index)
        {
            int ret = -1;
            IntPtr pPtr = IntPtr.Zero;
            try
            {
                //メモリリークをさせないためのおまじない
                System.Runtime.CompilerServices.RuntimeHelpers.PrepareConstrainedRegions();
                try { }
                finally
                {
                    //メモリ確保
                    pPtr = Marshal.AllocCoTaskMem(Marshal.SizeOf(typeof(EGTAGREF)));
                }
                
                int rtn = _C__EgFW_GetTagRef(pPtr, index);
                if (rtn == 0)
                {
                    EGTAGREF tempTagRef = (EGTAGREF)Marshal.PtrToStructure(pPtr, typeof(EGTAGREF));
                    if (tempTagRef.Name.Length > 0)//名前があればとってこれたも同然！
                    {
                        EgHgTagRef item = new EgHgTagRef(tempTagRef.Name, tempTagRef.Type, tempTagRef.Size, tempTagRef.Address, tempTagRef.Entry, tempTagRef.byOldVal);
                        tagRefList.Add(item);
                    }
                }
                else
                {
                    PrintSysLog(EG_LOG_ERR, "Could not be added TagRef because the number of TagRef exceeded.");
                    EgHgTagRef item = new EgHgTagRef("__ERROR__", 0, 0, "", IntPtr.Zero, null);
                    tagRefList.Add(item);//一応エラー用のタグを追加
                    ret = -1;
                }
                
                return ret;
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return ret;
            }
            finally
            {
                if (pPtr != IntPtr.Zero)
                    Marshal.FreeCoTaskMem(pPtr);
            }
        }


        /// <summary>
        /// TagRefs_OUT_PortInfoをCのフレームワークから取得(全)
        /// </summary>
        private void _GetTagRefs_OUT_PortInfo()
        {
            try
            {
                int count = _C__EgFW_GetEgTagRef_OUT_PortInfo_Count();
                if (count < 1)
                    return;

                tagRef_OUT_IOPortList.Clear();
                for (int i = 0; i < count; i++)
                {
                    _GetTagRefs_OUT_PortInfo(i);
                }
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
            }
        }
        /// <summary>
        /// TagRefs_OUT_PortInfoをCのフレームワークから取得
        /// </summary>
        /// <param name="index"></param>
        /// <returns></returns>
        private int _GetTagRefs_OUT_PortInfo(int index)
        {
            int ret = -1;
            IntPtr pPtr = IntPtr.Zero;
            try
            {
                //メモリリークをさせないためのおまじない
                System.Runtime.CompilerServices.RuntimeHelpers.PrepareConstrainedRegions();
                try { }
                finally
                {
                    //メモリ確保
                    pPtr = Marshal.AllocCoTaskMem(Marshal.SizeOf(typeof(EGIOPORT)));
                }
                int rtn = _C__EgFW_GetTagRef_OUT_PortInfo(pPtr, index);
                if (rtn == 0)
                {
                    EGIOPORT temp = (EGIOPORT)Marshal.PtrToStructure(pPtr, typeof(EGIOPORT));
                    if (temp.Entry != IntPtr.Zero)
                    {
                        EgHgEdgeIOPort item = new EgHgEdgeIOPort(temp.Entry, temp.SrcAddr, temp.AddrName, temp.Index, temp.Offset, temp.BitUse, temp.BitPos, temp.MaxBit, temp.IsValied);
                        tagRef_OUT_IOPortList.Add(item);
                    }
                }
                else
                {
                    PrintSysLog(EG_LOG_ERR, "Could not be added TagRef because the number of TagRef exceeded.");
                    EgHgEdgeIOPort item = new EgHgEdgeIOPort();
                    tagRef_OUT_IOPortList.Add(item);  //一応エラー用のタグを追加
                    ret = -1;
                }
                return ret;
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return ret;
            }
            finally
            {
                if (pPtr != IntPtr.Zero)
                    Marshal.FreeCoTaskMem(pPtr);
            }
        }
        /// <summary>
        /// TagRefs_IN_PortInfoをCのフレームワークから取得(全)
        /// </summary>
        private void _GetTagRefs_IN_PortInfo()
        {
            try
            {
                int count = _C__EgFW_GetEgTagRef_IN_PortInfo_Count();
                if (count < 1)
                    return;

                tagRef_IN_IOPortList.Clear();
                for (int i = 0; i < count; i++)
                {
                    _GetTagRefs_IN_PortInfo(i);
                }
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
            }
        }
        /// <summary>
        /// TagRefs_IN_PortInfoをCのフレームワークから取得
        /// </summary>
        /// <param name="index"></param>
        /// <returns></returns>
        private int _GetTagRefs_IN_PortInfo(int index)
        {
            int ret = -1;
            IntPtr pPtr = IntPtr.Zero;
            try
            {
                //メモリリークをさせないためのおまじない
                System.Runtime.CompilerServices.RuntimeHelpers.PrepareConstrainedRegions();
                try { }
                finally
                {
                    //メモリ確保
                    pPtr = Marshal.AllocCoTaskMem(Marshal.SizeOf(typeof(EGIOPORT)));
                }
                int rtn = _C__EgFW_GetTagRef_IN_PortInfo(pPtr, index);
                if (rtn == 0)
                {
                    EGIOPORT temp = (EGIOPORT)Marshal.PtrToStructure(pPtr, typeof(EGIOPORT));
                    if (temp.Entry != IntPtr.Zero)
                    {
                        EgHgEdgeIOPort item = new EgHgEdgeIOPort(temp.Entry, temp.SrcAddr, temp.AddrName, temp.Index, temp.Offset, temp.BitUse, temp.BitPos, temp.MaxBit, temp.IsValied);
                        tagRef_IN_IOPortList.Add(item);
                    }
                }
                else
                {
                    PrintSysLog(EG_LOG_ERR, "Could not be added TagRef because the number of TagRef exceeded.");
                    EgHgEdgeIOPort item = new EgHgEdgeIOPort();
                    tagRef_IN_IOPortList.Add(item);  //一応エラー用のタグを追加
                    ret = -1;
                }
                return ret;
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return ret;
            }
            finally
            {
                if (pPtr != IntPtr.Zero)
                    Marshal.FreeCoTaskMem(pPtr);
            }
        }
        /// <summary>
        /// TagRefs_PortInfoをCのフレームワークから取得(全)
        /// </summary>
        private void _GetTagRefs_PortInfo()
        {
            try
            {
                int count = _C__EgFW_GetEgTagRef_PortInfo_Count();
                if (count < 1)
                    return;

                tagRef_IOPortList.Clear();
                for (int i = 0; i < count; i++)
                {
                    _GetTagRefs_PortInfo(i);
                }
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
            }
        }
        /// <summary>
        /// TagRefs_PortInfoをCのフレームワークから取得
        /// </summary>
        /// <param name="index"></param>
        /// <returns></returns>
        private int _GetTagRefs_PortInfo(int index)
        {
            int ret = -1;
            IntPtr pPtr = IntPtr.Zero;
            try
            {
                //メモリリークをさせないためのおまじない
                System.Runtime.CompilerServices.RuntimeHelpers.PrepareConstrainedRegions();
                try { }
                finally
                {
                    //メモリ確保
                    pPtr = Marshal.AllocCoTaskMem(Marshal.SizeOf(typeof(EGIOPORT)));
                }
                int rtn = _C__EgFW_GetTagRef_PortInfo(pPtr, index);
                if (rtn == 0)
                {
                    EGIOPORT temp = (EGIOPORT)Marshal.PtrToStructure(pPtr, typeof(EGIOPORT));
                    if (temp.Entry != IntPtr.Zero)
                    {
                        EgHgEdgeIOPort item = new EgHgEdgeIOPort(temp.Entry, temp.SrcAddr, temp.AddrName, temp.Index, temp.Offset, temp.BitUse, temp.BitPos, temp.MaxBit, temp.IsValied);
                        tagRef_IOPortList.Add(item);
                    }
                }
                else
                {
                    PrintSysLog(EG_LOG_ERR, "Could not be added TagRef because the number of TagRef exceeded.");
                    EgHgEdgeIOPort item = new EgHgEdgeIOPort();
                    tagRef_IOPortList.Add(item);  //一応エラー用のタグを追加
                    ret = -1;
                }
                return ret;
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return ret;
            }
            finally
            {
                if (pPtr != IntPtr.Zero)
                    Marshal.FreeCoTaskMem(pPtr);
            }
        }
        /// <summary>
        /// CollectorRefをCのフレームワークから取得
        /// </summary>
        private void _GetClctRefs()
        {
            try
            {
                int count = _C__EgFW_GetCollectorRef_Count();
                if (count < 1)
                    return;

                clctRefList.Clear();
                for (int i = 0; i < count; i++)
                {
                    _GetClctRefs(i);
                }
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
            }
        }
        /// <summary>
        /// CollectorRefをCのフレームワークから取得
        /// </summary>
        /// <param name="index"></param>
        /// <returns></returns>
        private int _GetClctRefs(int index)
        {
            int ret = -1;
            IntPtr pPtr = IntPtr.Zero;
            try
            {
                //メモリリークをさせないためのおまじない
                System.Runtime.CompilerServices.RuntimeHelpers.PrepareConstrainedRegions();
                try { }
                finally
                {
                    //メモリ確保
                    pPtr = Marshal.AllocCoTaskMem(Marshal.SizeOf(typeof(EGCOLLECTORREF)));
                }
                int rtn = _C__EgFW_GetCollectorRef(pPtr, index);
                if (rtn == 0)
                {
                    EGCOLLECTORREF tempClctRef = (EGCOLLECTORREF)Marshal.PtrToStructure(pPtr, typeof(EGCOLLECTORREF));
                    if (tempClctRef.CollectorName.Length > 0)//名前をとってこれたら存在するも同然！
                    {
                        EgHgCollectorRef item = new EgHgCollectorRef(tempClctRef.CollectorName, tempClctRef.dwColSize, tempClctRef.dwRowSize, tempClctRef.DatasetName);
                        clctRefList.Add(item);
                    }
                }
                else
                {
                    PrintSysLog(EG_LOG_ERR, "Could not be added CollectorRef because the number of CollectorRef exceeded.");
                    EgHgCollectorRef item = new EgHgCollectorRef("__ERROR__", 0, 0, "");
                    clctRefList.Add(item);//一応エラー用のcollectorRefを追加
                    ret = -1;
                }
                return ret;
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return ret;
            }
            finally
            {
                if (pPtr != IntPtr.Zero)
                    Marshal.FreeCoTaskMem(pPtr);
            }
        }
        /// <summary>
        /// メールボックスをCのフレームワークから取得（全）
        /// </summary>
        private void _GetMailbox()
        {
            try
            {
                int count = _C__EgFW_GetMailbox_Count();
                if (count < 1)
                    return;

                mailBoxList.Clear();
                for (int i = 0; i < count; i++)
                {
                    _GetMailbox(i);
                }
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
            }
        }
        /// <summary>
        /// メールボックスのCのフレームワークから取得
        /// </summary>
        /// <param name="index"></param>
        /// <returns></returns>
        private int _GetMailbox(int index)
        {
            int ret = -1;
            IntPtr pPtr = IntPtr.Zero;
            try
            {
                //メモリリークをさせないためのおまじない
                System.Runtime.CompilerServices.RuntimeHelpers.PrepareConstrainedRegions();
                try { }
                finally
                {
                    //メモリ確保
                    pPtr = Marshal.AllocCoTaskMem(Marshal.SizeOf(typeof(EGMAILBOX)));
                }
                int rtn = _C__EgFW_GetMailBox(pPtr, index);
                if (rtn == 0)
                {
                    EGMAILBOX tempMailbox = (EGMAILBOX)Marshal.PtrToStructure(pPtr, typeof(EGMAILBOX));
                    if (tempMailbox.MailBoxName.Length > 0)//名前をとってこれたら成功したも同然!
                    {
                        edge_API.RTCDMEMINFO memInfo = tempMailbox.MemInfo;
                        EgHgMailBox item = new EgHgMailBox(tempMailbox.MailBoxName, tempMailbox.dwColSize, tempMailbox.dwRowSize, memInfo);
                        mailBoxList.Add(item);
                    }
                }
                else
                {
                    PrintSysLog(EG_LOG_ERR, "Could not be added Mailbox because the number of Mailbox exceeded.");
                    edge_API.RTCDMEMINFO memInfo = new edge_API.RTCDMEMINFO();
                    EgHgMailBox item = new EgHgMailBox("__ERROR__", 0, 0, memInfo);
                    mailBoxList.Add(item);//一応エラー用のcollectorRefを追加
                    ret = -1;
                }
                return ret;
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return ret;
            }
            finally
            {
                if (pPtr != IntPtr.Zero)
                    Marshal.FreeCoTaskMem(pPtr);
            }
        }
        /// <summary>
        /// TagTriggersをCのフレームワークから取得(全)
        /// </summary>
        private void _GetTagTrigger()
        {
            try
            {
                int count = _C__EgFW_GetTagTrigger_Count();
                if (count < 1)
                    return;

                tagTriggerList.Clear();
                for (int i = 0; i < count; i++)
                {
                    _GetTagTrigger(i);
                }
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
            }
        }
        /// <summary>
        /// TagTriggerをCのフレームワークから取得
        /// </summary>
        /// <param name="index"></param>
        /// <returns></returns>
        private int _GetTagTrigger(int index)
        {
            int ret = -1;
            IntPtr pPtr = IntPtr.Zero;
            try
            {
                //メモリリークをさせないためのおまじない
                System.Runtime.CompilerServices.RuntimeHelpers.PrepareConstrainedRegions();
                try { }
                finally
                {
                    //メモリ確保
                    pPtr = Marshal.AllocCoTaskMem(Marshal.SizeOf(typeof(EGTAGREF)));
                }
                int rtn = _C__EgFW_GetTagTrigger(pPtr, index);
                if (rtn == 0)
                {
                    EGTAGTRIGGER tempTagTrig = (EGTAGTRIGGER)Marshal.PtrToStructure(pPtr, typeof(EGTAGTRIGGER));
                    if (tempTagTrig.Name.Length > 0)//名前があればとってこれたも同然！
                    {
                        EgHgTagTrigger item = new EgHgTagTrigger(tempTagTrig.Name, tempTagTrig.Type, tempTagTrig.Size, tempTagTrig.Address, tempTagTrig.Entry, tempTagTrig.Count);
                        tagTriggerList.Add(item);
                    }
                }
                else
                {
                    PrintSysLog(EG_LOG_ERR, "Could not be added TagTrigger because the number of TagTrigger exceeded.");
                    EgHgTagTrigger item = new EgHgTagTrigger("__ERROR__", 0, 0, "", IntPtr.Zero, 0);
                    tagTriggerList.Add(item);//一応エラー用のタグを追加
                    ret = -1;
                }
                return ret;
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return ret;
            }
            finally
            {
                if (pPtr != IntPtr.Zero)
                    Marshal.FreeCoTaskMem(pPtr);
            }
        }
        /// <summary>
        /// ログ出力(サービス名指定) 非公開関数
        /// </summary>
        /// <param name="ServiceName"></param>
        /// <param name="Mode"></param>
        /// <param name="format"></param>
        /// <param name="param"></param>
        public void _PrintLog(string ServiceName, string Mode, string format, params string[] param)
        {
            try
            {
                _C_EgFW_PrintLog(ServiceName, Mode, format, param);
            }
            catch (Exception ex)
            {
                PrintSysLog(EG_LOG_FER, "FATAL ERROR : {0}", System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
            }
        }
        /// <summary>
        /// デバッグ出力
        /// </summary>
        public void DebugFunc()
        {
            try
            {
                PrintSysLog(EG_LOG_DBG, "#============================================================");
                PrintSysLog(EG_LOG_DBG, "#          DEBUG WRITE C# WRAPPER SERVICE OBJECT            =");
                PrintSysLog(EG_LOG_DBG, "#============================================================");
                PrintSysLog(EG_LOG_DBG, "# ServiceRealName       : {0}", ServiceRealName);
                PrintSysLog(EG_LOG_DBG, "# ServiceMboxName       : {0}", ServiceMboxName);
                PrintSysLog(EG_LOG_DBG, "# ServiceName           : {0}", ServiceName);
                PrintSysLog(EG_LOG_DBG, "# egTagRef       Count  : {0}", tagRefList.Count.ToString());
                PrintSysLog(EG_LOG_DBG, "# egTagRef OUT   Count  : {0}", tagRef_OUT_List.Count.ToString());
                PrintSysLog(EG_LOG_DBG, "# egTagRef IN    Count  : {0}", tagRef_IN_List.Count.ToString());
                PrintSysLog(EG_LOG_DBG, "# egCollectorRef Count  : {0}", clctRefList.Count.ToString());
                PrintSysLog(EG_LOG_DBG, "# egMailBox      Count  : {0}", mailBoxList.Count.ToString());
                PrintSysLog(EG_LOG_DBG, "# egTagTrigger   Count  : {0}", tagTriggerList.Count.ToString());

                //egTagRef
                if(tagRefList.Count == 0)
                {
                    PrintSysLog(EG_LOG_DBG, "# egTagRef No Data");
                }
                else
                {
                    PrintSysLog(EG_LOG_DBG, "# egTagRef Data");
                    foreach(EgHgTagRef t in tagRefList)
                    {
                        PrintSysLog(EG_LOG_DBG, "#    Name[{0}], Size[{1}], Type[{2}]", t.Name, t.Size.ToString(), t.Type.ToString());
                    }
                }

                //egTagRef_OUT
                if (tagRef_OUT_List.Count == 0)
                {
                    PrintSysLog(EG_LOG_DBG, "# egTagRef_OUT No Data");
                }
                else
                {
                    PrintSysLog(EG_LOG_DBG, "# egTagRef_OUT Data");
                    foreach (EgHgTagRef t in tagRef_OUT_List)
                    {
                        PrintSysLog(EG_LOG_DBG, "#    Name[{0}], Size[{1}], Type[{2}]", t.Name, t.Size.ToString(), t.Type.ToString());
                    }
                }

                //egTagRef_IN
                if (tagRef_IN_List.Count == 0)
                {
                    PrintSysLog(EG_LOG_DBG, "# egTagRef_IN No Data");
                }
                else
                {
                    PrintSysLog(EG_LOG_DBG, "# egTagRef_IN Data");
                    foreach (EgHgTagRef t in tagRef_IN_List)
                    {
                        PrintSysLog(EG_LOG_DBG, "#    Name[{0}], Size[{1}], Type[{2}]", t.Name, t.Size.ToString(), t.Type.ToString());
                    }
                }

                //clctRef
                if (clctRefList.Count == 0)
                {
                    PrintSysLog(EG_LOG_DBG, "# egCollectorRefs No Data");
                }
                else
                {
                    PrintSysLog(EG_LOG_DBG, "# egCollectorRefs Data");
                    foreach (EgHgCollectorRef t in clctRefList)
                    {
                        PrintSysLog(EG_LOG_DBG, "#    Name[{0}], ColSize[{1}], RowSize[{2}]", t.Name, t.ColSize.ToString(), t.RowSize.ToString());
                        if(t.DatasetName.Length == 0)
                            PrintSysLog(EG_LOG_DBG, "#    -- egCollectorRef No Item -- ");
                        else
                            PrintSysLog(EG_LOG_DBG, "#    -- egCollectorRef Item Name [{0}] -- ", t.DatasetName);
                    }
                }
                //MailBox
                if (mailBoxList.Count == 0)
                {
                    PrintSysLog(EG_LOG_DBG, "# egMailBox No Data");
                }
                else
                {
                    PrintSysLog(EG_LOG_DBG, "# egMailBox Data");
                    foreach (EgHgMailBox t in mailBoxList)
                    {
                        PrintSysLog(EG_LOG_DBG, "#    Name[{0}], ColSize[{1}], RowSize[{2}]", t.Name, t.ColSize.ToString(), t.RowSize.ToString());                    }
                }

                //TagTrigger tagTriggerList
                if (tagTriggerList.Count == 0)
                {
                    PrintSysLog(EG_LOG_DBG, "# egTagTrigger No Data");
                }
                else
                {
                    PrintSysLog(EG_LOG_DBG, "# egTagTrigger Data");
                    foreach (EgHgTagTrigger t in tagTriggerList)
                    {
                        PrintSysLog(EG_LOG_DBG, "#    Name[{0}], Size[{1}], Type[{2}]", t.Name, t.Size.ToString(), t.Type.ToString());
                    }
                }
                PrintSysLog(EG_LOG_DBG, "#============================================================");
                PrintSysLog(EG_LOG_DBG, "#          END DEBUG WRITE C# WRAPPER SERVICE OBJECT         =");
                PrintSysLog(EG_LOG_DBG, "#============================================================");

            }
            catch (Exception ex)
            {
                PrintSysLog(EG_LOG_FER, "FATAL ERROR : {0}", System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
            }
        }
    }
}
