﻿////
/// Copyright(C) 2020 Micronet Co. All Rights Reserved.
//
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Diagnostics;           // Debug.WriteLine出力用
using System.Runtime.InteropServices;
using System.IO;
using System.Text.RegularExpressions;

namespace RTedge
{
    public partial class edge_API
    {
        //-------------------------------------------------------------------------------
        //
        // ETC C# Only
        //
        //-------------------------------------------------------------------------------
        /// <summary>
        /// DiskInfoが出力したファイル情報を読み込む
        /// </summary>
        /// <param name="diskExecInfo"></param>
        /// <param name="filePath"></param>
        private void _ParseDiskInfoFile(ref DiskExecInfo diskExecInfo, string filePath)
        {
            try
            {
                //子インスタンスがnullだったら作る
                if (diskExecInfo.diskInfos == null)
                    diskExecInfo.diskInfos = new Dictionary<int, DiskInfo>();
                else//既にある場合はクリアする
                    diskExecInfo.diskInfos.Clear();

                //値取得用
                DiskInfo currentDiskInfo = new DiskInfo();
                bool isInModelBlock = false;                //モデル情報取得フラグ
                bool isInSmartBlock = false;                //SMART情報取得フラグ
                int inNo = 0;   //disk no
                foreach (var line in File.ReadLines(filePath))  //一行読み込む
                {
                    if (line.StartsWith("#ModelStart"))         //モデル情報の開始
                    {
                        isInModelBlock = true;                  //フラグを立てる
                        currentDiskInfo = new DiskInfo();       //変数の初期化
                        currentDiskInfo.SmartAttributes = new Dictionary<int, SmartAttribute>();
                        continue;
                    }

                    if (line.StartsWith("#ModelEnd"))           //モデル情報の終了
                    {
                        isInModelBlock = false;                 //フラグを落とす
                        diskExecInfo.diskInfos.Add(inNo, currentDiskInfo);  //ディスクアレイに追加する
                        continue;
                    }

                    if (line.StartsWith("#SMART_START"))        //SMART情報の開始
                    {
                        isInSmartBlock = true;                  //フラグを立てる
                        continue;
                    }

                    if (line.StartsWith("#SMART_END"))          //SMART情報の終了
                    {
                        isInSmartBlock = false;                 //フラグを落とす
                        continue;
                    }

                    if (isInModelBlock && !isInSmartBlock)      //モデル情報の場合
                    {
                        // : で区切った値を分解
                        var keyValue = line.Split(new[] { " : " }, StringSplitOptions.None);
                        if (keyValue.Length == 2)   //2このはず
                        {
                            //各値を取得
                            var key = keyValue[0].Trim();
                            var value = keyValue[1].Trim();
                            switch (key)    //各値を取得 名前をキーに値を格納
                            {
                                case "No":
                                    currentDiskInfo.No = value;
                                    inNo = int.Parse(value);
                                    break;
                                case "Model":
                                    currentDiskInfo.Model = value;
                                    break;
                                case "Firmware":
                                    currentDiskInfo.Firmware = value;
                                    break;
                                case "Serial Number":
                                    currentDiskInfo.SerialNumber = value;
                                    break;
                                case "Disk Size":
                                    currentDiskInfo.DiskSize = value;
                                    break;
                                case "Buffer Size":
                                    currentDiskInfo.BufferSize = value;
                                    break;
                                case "Queue Depth":
                                    currentDiskInfo.QueueDepth = value;
                                    break;
                                case "# of Sectors":
                                    currentDiskInfo.NumberOfSectors = value;
                                    break;
                                case "Rotation Rate":
                                    currentDiskInfo.RotationRate = value;
                                    break;
                                case "Interface":
                                    currentDiskInfo.Interface = value;
                                    break;
                                case "Major Version":
                                    currentDiskInfo.MajorVersion = value;
                                    break;
                                case "Minor Version":
                                    currentDiskInfo.MinorVersion = value;
                                    break;
                                case "Transfer Mode":
                                    currentDiskInfo.TransferMode = value;
                                    break;
                                case "Power On Hours":
                                    currentDiskInfo.PowerOnHours = value;
                                    break;
                                case "Power On Count":
                                    currentDiskInfo.PowerOnCount = value;
                                    break;
                                case "Host Writes":
                                    currentDiskInfo.HostWrites = value;
                                    break;
                                case "Temperature":
                                    currentDiskInfo.Temperature = value;
                                    break;
                                case "Health Status":
                                    currentDiskInfo.HealthStatus = value;
                                    break;
                                case "Features":
                                    currentDiskInfo.Features = value;
                                    break;
                                case "APM Level":
                                    currentDiskInfo.APMLevel = value;
                                    break;
                                case "AAM Level":
                                    currentDiskInfo.AAMLevel = value;
                                    break;
                                case "Drive Letter":
                                    currentDiskInfo.DriveLetter = value;
                                    break;
                            }
                        }
                    }
                    else if (isInSmartBlock)                        //SMART情報の場合
                    {
                        // スペース区切りで分解
                        var smartParts = Regex.Split(line.Trim(), @"\s+");
                        if (smartParts.Length >= 6) //6個以上あるはず
                        {
                            // _ がついていたら取り除く
                            string stID = smartParts[0].TrimStart('_');
                            if (stID != "ID")   //IDという文字でない場合は全部SMART情報の為格納する
                            {
                                //各値を保持する 追加する際は _ を取り除いた値 IDは 16進数の文字列値をintとして変換する
                                var smartAttribute = new SmartAttribute
                                {
                                    ID = int.Parse(stID, System.Globalization.NumberStyles.HexNumber),
                                    Status = smartParts[1].TrimStart('_'),
                                    Cur = smartParts[2].TrimStart('_'),
                                    Wor = smartParts[3].TrimStart('_'),
                                    Thr = smartParts[4].TrimStart('_'),
                                    RawValues = smartParts[5],
                                    AttributeName = string.Join(" ", smartParts.Skip(6))
                                };
                                //アレイに追加する
                                currentDiskInfo.SmartAttributes.Add(smartAttribute.ID, smartAttribute);
                            }
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
            }
        }
        /// <summary>
        /// SMART Current値 , Status 取得
        /// </summary>
        /// <param name="DiskNo"></param>
        /// <param name="Id"></param>
        /// <param name="Value"></param>
        /// <param name="Status"></param>
        /// <returns></returns>
        public bool EgMDiagGetSMARTValue(DiskExecInfo diskExecInfo, int diskNo, int Id, ref byte value, ref byte status)
        {
            value = 0;
            status = (byte)EGDEFINE.SMART_STATUS.UNKNOWN;
            try
            {
                //diskInfosが存在し、DiskNoが含まれている場合のみ実行
                if (diskExecInfo.diskInfos != null && diskExecInfo.diskInfos.ContainsKey(diskNo))
                {
                    //対象の配列情報を取り出す
                    edge_API.DiskInfo dInfo = diskExecInfo.diskInfos[diskNo];
                    //指定されたId情報が格納されている場合のみ値を設定する
                    if (dInfo.SmartAttributes.ContainsKey(Id))
                    {
                        value = byte.Parse(dInfo.SmartAttributes[Id].Cur);
                        status = byte.Parse(dInfo.SmartAttributes[Id].Status);
                        return true;
                    }


                }
                return false;
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return false;
            }
        }
        /// <summary>
        /// モデル名の取得
        /// </summary>
        /// <param name="DiskNo"></param>
        /// <param name="Value"></param>
        /// <returns></returns>
        public bool EgMDiagGetSMARTDiskName(DiskExecInfo diskExecInfo, int diskNo, ref string value)
        {
            value = "N/A";
            try
            {
                // diskInfosが存在し、DiskNoが存在する場合のみ実行
                if (diskExecInfo.diskInfos != null && diskExecInfo.diskInfos.ContainsKey(diskNo))
                {
                    value = diskExecInfo.diskInfos[diskNo].Model;
                    return true;
                }
                return false;
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return false;
            }
        }
        /// <summary>
        /// ログ収集 実行
        /// </summary>
        /// <returns></returns>
        public int EgMDiagExecute_DiskInfo(ref DiskExecInfo diskExecInfo)
        {
            try
            {
                string argFilePath = @"DiskInfo.txt";           //ファイルパス
                
                // ProcessStartInfoオブジェクトを作成
                ProcessStartInfo startInfo = new ProcessStartInfo
                {
                    FileName = @"DiskInfo64.exe",
                    // 必要に応じて引数を追加
                    Arguments = "/CopyExit",
                    Verb = "RunAs",
                    // 必要に応じて他のオプションも設定
                    UseShellExecute = false,
                };

                //プログラム実行
                // Processオブジェクトを作成してプログラムを開始
                using (Process process = Process.Start(startInfo))
                {
                    if (process == null)
                    {
                        //プロセスの起動に失敗しました
                        return (int)EGDEFINE.EDGE_FAILED_TO_START_PROCESS;
                    }

                    // プログラムが終了するまで待つ
                    process.WaitForExit();

                    // プログラムの終了コードを取得
                    int exitCode = process.ExitCode;

                    // ファイル読み込み・格納
                    _ParseDiskInfoFile(ref diskExecInfo, argFilePath);
#if DEBUG
                    // テスト出力
                    foreach (KeyValuePair<int, DiskInfo> disk in diskExecInfo.diskInfos)
                    {
                        Debug.WriteLine($"Model: {disk.Value.Model}, No: {disk.Key}, Firmware: {disk.Value.Firmware}, Serial Number: {disk.Value.SerialNumber}");
                        Debug.WriteLine($"ID ST Cur Wor Thr RawValues(6) Attribute Name");
                        foreach (KeyValuePair<int, SmartAttribute> smartAttr in disk.Value.SmartAttributes)
                        {
                            Debug.WriteLine($"{smartAttr.Value.ID.ToString("x2")} {smartAttr.Value.Status} {smartAttr.Value.Cur} {smartAttr.Value.Wor} {smartAttr.Value.Thr} {smartAttr.Value.RawValues} {smartAttr.Value.AttributeName}");
                        }
                    }
#else
#endif
                }
                return 0;
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return -1;
            }
        }
        //内部で保持するカウンタのディクショナリ
        private Dictionary<string, PerformanceCounter> PCounterDic = new Dictionary<string, PerformanceCounter>();
        /// <summary>
        /// 3つの名前からカウンタ名(全体)を取得
        /// </summary>
        /// <param name="categoryName"></param>
        /// <param name="counterName"></param>
        /// <param name="instanceName"></param>
        /// <returns></returns>
        private string _GetOfmCntName(string categoryName, string counterName, string instanceName)
        {
            //カテゴリ名が無い、またはカウンタ名が無い場合は空　(インスタンス名は無い場合がある)
            if (categoryName.Length == 0 || counterName.Length == 0)
                return "";

            string counterName_All = "";
            if (instanceName.Length == 0)
                counterName_All = categoryName + "\\" + counterName;
            else
                counterName_All = categoryName + "(" + instanceName + ")" + "\\" + counterName;
            return counterName_All;
        }
        /// <summary>
        /// 診断・通知 パフォーマンスカウンタ追加
        /// </summary>
        /// <param name="categoryName"></param>
        /// <param name="counterName"></param>
        /// <param name="instanceName"></param>
        /// <returns></returns>
        public int EgMDiagExecute_PfmCntAdd(string categoryName, string counterName, string instanceName)
        {
            try
            {
                //カテゴリ名が無い、またはカウンタ名が無い場合はエラー　(インスタンス名は無い場合がある)
                if (categoryName.Length == 0 || counterName.Length == 0)
                    return (int)EGDEFINE.EDGE_WRAP_PARAM_INVALID; //パラメータが間違っているエラー

                string counterName_All = _GetOfmCntName(categoryName, counterName, instanceName);
                return EgMDiagExecute_PfmCntAdd(counterName_All, categoryName, counterName, instanceName);
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return -1;
            }
        }
        /// <summary>
        /// 診断・通知 パフォーマンスカウンタ追加
        /// </summary>
        /// <param name="categoryName"></param>
        /// <param name="counterName"></param>
        /// <param name="instanceName"></param>
        /// <returns></returns>
        public int EgMDiagExecute_PfmCntAdd(string counterName_All, string categoryName, string counterName, string instanceName)
        {
            try
            {
                //カウンタ名(全体)の名前が無い、カテゴリ名が無い、またはカウンタ名が無い場合はエラー　(インスタンス名は無い場合がある)
                if(counterName_All.Length == 0 || categoryName.Length == 0 || counterName.Length == 0)
                    return (int)EGDEFINE.EDGE_WRAP_PARAM_INVALID; //パラメータが間違っているエラー

                PerformanceCounter PCounter = new PerformanceCounter(); //パフォーマンスカウンタ インスタンス作成
                PCounter.CategoryName = categoryName;                   //カテゴリ名
                PCounter.CounterName = counterName;                     //カウンタ名
                PCounter.InstanceName = instanceName;                   //インスタンス名
                PCounterDic.Add(counterName_All, PCounter);             //診断用に追加

                return 0;
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return -1;
            }
        }
        /// <summary>
        /// 診断・通知 パフォーマンスカウンタ追加
        /// </summary>
        /// <param name="counterName_All"></param>
        /// <returns></returns>
        public int EgMDiagExecute_PfmCntAdd(string counterName_All)
        {
            try
            {
                //カウンタ名(全体)の名前が無い場合はエラー　(インスタンス名は無い場合がある)
                if (counterName_All.Length == 0)
                    return (int)EGDEFINE.EDGE_WRAP_PARAM_INVALID; //パラメータが間違っているエラー
                
                // 最初の \ を無視
                string trimmedInput = counterName_All.TrimStart('\\');

                // \ で分割
                string[] parts = trimmedInput.Split(new[] { '\\' }, 2);

                if (parts.Length != 2)
                    return (int)EGDEFINE.EDGE_WRAP_PARAM_INVALID; //パラメータが間違っているエラー
                
                string part1 = parts[0]; // 1つ目の文字列
                string part2 = parts[1]; // 2つ目の文字列

                // 1つ目の文字列をかっこで分割
                int openParenIndex = part1.IndexOf('(');
                int closeParenIndex = part1.IndexOf(')');

                //カッコが存在する場合は、カッコが存在する前までの文字列を一つ目の文字列とする
                //カッコが存在しない場合は、そのまま一つ目も文字列とする
                string categoryName = openParenIndex > -1 ? part1.Substring(0, openParenIndex) : part1;

                // カッコ"("が存在する場合でかつ。かっこ ) が ( の後に存在する場合は、間の文字列を三つ目の文字列に
                // かっこが存在しない場合、または正しい順序で閉じていない場合は、instanceName としてアンダーバー "_ " を設定
                string instanceName = (openParenIndex > -1 && closeParenIndex > openParenIndex)
                                    ? part1.Substring(openParenIndex + 1, closeParenIndex - openParenIndex - 1)
                                    : "_";

                PerformanceCounter PCounter = new PerformanceCounter(); //パフォーマンスカウンタ インスタンス作成
                PCounter.CategoryName = categoryName;                   //カテゴリ名
                PCounter.CounterName = part2;                           //カウンタ名
                PCounter.InstanceName = instanceName;                   //インスタンス名
                PCounterDic.Add(trimmedInput, PCounter);                //診断用に追加

                return 0;
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return -1;
            }
        }
        /// <summary>
        /// 診断・通知 パフォーマンスカウンタ値取得API
        /// </summary>
        /// <param name="CategoryName"></param>
        /// <param name="CounterName"></param>
        /// <param name="InstanceName"></param>
        /// <param name="refValue"></param>
        /// <returns></returns>
        public int EgMDiagExecute_PfmCntGet(string categoryName, string counterName, string instanceName, ref float refValue)
        {
            try
            {
                //カテゴリ名が無い、またはカウンタ名が無い場合はエラー　(インスタンス名は無い場合がある)
                if (categoryName.Length == 0 || counterName.Length == 0)
                    return (int)EGDEFINE.EDGE_WRAP_PARAM_INVALID; //パラメータが間違っているエラー

                string counterName_All = _GetOfmCntName(categoryName, counterName, instanceName);
                return EgMDiagExecute_PfmCntGet(counterName_All, ref refValue);
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return -1;
            }
        }
        /// <summary>
        /// 診断・通知 パフォーマンスカウンタ値取得API
        /// </summary>
        /// <param name="PCounter"></param>
        /// <param name="refValue"></param>
        /// <returns></returns>
        public int EgMDiagExecute_PfmCntGet(string counterName_All, ref float refValue)
        {
            try
            {
                //カウンタ名(全体)の名前が無い場合はエラー　(インスタンス名は無い場合がある)
                if (counterName_All.Length == 0)
                    return (int)EGDEFINE.EDGE_WRAP_PARAM_INVALID; //パラメータが間違っているエラー

                // 最初の \ を無視
                string trimmedInput = counterName_All.TrimStart('\\');
                if(PCounterDic.ContainsKey(trimmedInput) == false)
                    return (int)EGDEFINE.EDGE_WRAP_PARAM_INVALID; //パラメータが間違っているエラー

                // 値を取得
                refValue = PCounterDic[trimmedInput].NextValue();
                return 0;
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return -1;
            }
        }
        /// <summary>
        /// 内部で保持しているパフォーマンスカウンタをクリア
        /// </summary>
        public void EgMDiagExecute_PfmCntClear()
        {
            PCounterDic.Clear();
        }
        /// <summary>
        /// ログ収集 実行
        /// </summary>
        /// <returns></returns>
        public int EgGatherExecute(string outputDirPath, string listPath)
        {
            try
            {
                string argFolderPath = "";                      //フォルダパス
                string argListPath = "";                        //リストパス

                if (listPath.Length == 0 || outputDirPath.Length == 0)
                    return (int)EGDEFINE.EDGE_WRAP_PARAM_INVALID; //パラメータが間違っているエラー

                argFolderPath = @"-d=" + outputDirPath; //引数を追加
                argListPath = @"-s=" + @listPath;       //引数を追加


                var proc = new System.Diagnostics.Process();
                proc.StartInfo.FileName = @"MtnGather.exe";
                proc.StartInfo.Arguments = argFolderPath + " " + argListPath;   //引数
                bool bRet = proc.Start();
                if (bRet == false)
                    return (int)EGDEFINE.EDGE_FAILED_TO_START_PROCESS;
                return 0;
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return -1;
            }
        }
        //
        // S.M.A.R.T.情報全般の構造体
        //
        public struct DiskExecInfo
        {
            public Dictionary<int, DiskInfo> diskInfos;//No ディスク情報 個別
        }
        //
        // ディスク情報の構造体
        //
        public struct DiskInfo
        {
            public string No;               //No
            public string Model;            //Model名
            public string Firmware;         //Firmware名
            public string SerialNumber;     //SerialNumber名
            public string DiskSize;         //Diskサイズ
            public string BufferSize;       //Bufferサイズ
            public string QueueDepth;       //-
            public string NumberOfSectors;  //-
            public string RotationRate;     //回転数
            public string Interface;        //インタフェース
            public string MajorVersion;     //メジャーバージョン
            public string MinorVersion;     //マイナーバージョン
            public string TransferMode;     //転送モード
            public string PowerOnHours;     //使用時間
            public string PowerOnCount;     //電源投入回数
            public string HostWrites;       //総書込量(ホスト)
            public string Temperature;      //温度
            public string HealthStatus;     //健康状態
            public string Features;         //-
            public string APMLevel;         //-
            public string AAMLevel;         //-
            public string DriveLetter;      //ドライブレター
            public Dictionary<int, SmartAttribute> SmartAttributes;    //S.M.A.R.T.情報
        }

        // S.M.A.R.T.属性の構造体
        public struct SmartAttribute
        {
            public int ID;                  //S.M.A.R.T.情報のID キーとなる
            public string Status;           //CurとThrからチェックされたStatus情報
            public string Cur;              //カレント値
            public string Wor;              //Worst値
            public string Thr;              //Threshold値
            public string RawValues;        //生値
            public string AttributeName;    //名前・文字列
        }
        /// <summary>
        /// デバイスマネージャの情報取得
        /// </summary>
        public class MDiagDeviceMgr
        {
            [DllImport("setupapi.dll", SetLastError = true)]
            private static extern Int32 SetupDiDestroyDeviceInfoList(IntPtr DeviceInfoSet);

            [DllImport("setupapi.dll", SetLastError = true)]
            private static extern bool SetupDiEnumDeviceInfo(IntPtr DeviceInfoSet,
                Int32 MemberIndex,
                ref SP_DEVINFO_DATA DeviceInterfaceData);

            [DllImport("setupapi.dll", SetLastError = true)]
            private static extern IntPtr SetupDiGetClassDevs(ref Guid ClassGuid,
                [MarshalAs(UnmanagedType.LPStr)]String enumerator,
                IntPtr hwndParent,
                Int32 Flags);
            [DllImport("setupapi.dll", SetLastError = true)]
            static extern bool SetupDiGetDeviceRegistryPropertyW(
                IntPtr DeviceInfoSet,
                [In] ref SP_DEVINFO_DATA DeviceInfoData,
                UInt32 Property,
                [Out] out UInt32 PropertyRegDataType,
                IntPtr PropertyBuffer,
                UInt32 PropertyBufferSize,
                [In, Out] ref UInt32 RequiredSize);

            private enum DIGCF
            {
                DEFAULT = 0x1,
                PRESENT = 0x2,
                ALLCLASSES = 0x4,
                PROFILE = 0x8,
                DEVICEINTERFACE = 0x10,
            }

            [StructLayout(LayoutKind.Sequential)]
            private struct SP_DEVINFO_DATA
            {
                public int cbSize;
                public Guid ClassGuid;
                public uint DevInst;
                public IntPtr Reserved;
            }
            //ID
            private const uint SPDRP_DEVICEDESC = (0x00000000);  // DeviceDesc (R/W)
            private const uint SPDRP_HARDWAREID = (0x000000001);  // HardwareID (R/W)
            private const uint SPDRP_COMPATIBLEIDS = (0x000000002);  // CompatibleIDs (R/W)
            private const uint SPDRP_UNUSED0 = (0x000000003);  // unused
            private const uint SPDRP_SERVICE = (0x000000004);  // Service (R/W)
            private const uint SPDRP_UNUSED1 = (0x000000005);  // unused
            private const uint SPDRP_UNUSED2 = (0x000000006);  // unused
            private const uint SPDRP_CLASS = (0x000000007);  // Class (R--tied to ClassGUID)
            private const uint SPDRP_CLASSGUID = (0x000000008);  // ClassGUID (R/W)
            private const uint SPDRP_DRIVER = (0x000000009);  // Driver (R/W)
            private const uint SPDRP_CONFIGFLAGS = (0x00000000A);  // ConfigFlags (R/W)
            private const uint SPDRP_MFG = (0x00000000B);  // Mfg (R/W)
            private const uint SPDRP_FRIENDLYNAME = (0x00000000C);  // FriendlyName (R/W)
            private const uint SPDRP_LOCATION_INFORMATION = (0x00000000D);  // LocationInformation (R/W)
            private const uint SPDRP_PHYSICAL_DEVICE_OBJECT_NAME = (0x00000000E);  // PhysicalDeviceObjectName (R)
            private const uint SPDRP_CAPABILITIES = (0x00000000F);  // Capabilities (R)
            private const uint SPDRP_UI_NUMBER = (0x000000010);  // UiNumber (R)
            private const uint SPDRP_UPPERFILTERS = (0x000000011);  // UpperFilters (R/W)
            private const uint SPDRP_LOWERFILTERS = (0x000000012);  // LowerFilters (R/W)
            private const uint SPDRP_BUSTYPEGUID = (0x000000013);  // BusTypeGUID (R)
            private const uint SPDRP_LEGACYBUSTYPE = (0x000000014);  // LegacyBusType (R)
            private const uint SPDRP_BUSNUMBER = (0x000000015);  // BusNumber (R)
            private const uint SPDRP_ENUMERATOR_NAME = (0x000000016);  // Enumerator Name (R)
            private const uint SPDRP_SECURITY = (0x000000017);  // Security (R/W, binary form)
            private const uint SPDRP_SECURITY_SDS = (0x000000018);  // Security (W, SDS form)
            private const uint SPDRP_DEVTYPE = (0x000000019);  // Device Type (R/W)
            private const uint SPDRP_EXCLUSIVE = (0x00000001A);  // Device is exclusive-access (R/W)
            private const uint SPDRP_CHARACTERISTICS = (0x00000001B);  // Device Characteristics (R/W)
            private const uint SPDRP_ADDRESS = (0x00000001C);  // Device Address (R)
            private const uint SPDRP_UI_NUMBER_DESC_FORMAT = (0x00000001D);  // UiNumberDescFormat (R/W)
            private const uint SPDRP_DEVICE_POWER_DATA = (0x00000001E);  // Device Power Data (R)
            private const uint SPDRP_REMOVAL_POLICY = (0x00000001F);  // Removal Policy (R)
            private const uint SPDRP_REMOVAL_POLICY_HW_DEFAULT = (0x000000020);  // Hardware Removal Policy (R)
            private const uint SPDRP_REMOVAL_POLICY_OVERRIDE = (0x000000021);  // Removal Policy Override (RW)
            private const uint SPDRP_INSTALL_STATE = (0x000000022);  // Device Install State (R)
            private const uint SPDRP_LOCATION_PATHS = (0x000000023);  // Device Location Paths (R)
            private const uint SPDRP_BASE_CONTAINERID = (0x000000024);  // Base ContainerID (R)

            private const uint SPDRP_MAXIMUM_PROPERTY = (0x000000025);  // Upper bound on ordinals

            //
            // Class registry property codes
            // (Codes marked as read-only (R) may only be used for
            // SetupDiGetClassRegistryProperty)
            //
            // These values should cover the same set of registry properties
            // as defined by the CM_CRP codes in cfgmgr32.h.
            // they should also have a 1:1 correspondence with Device registers, where applicable
            // but no overlap otherwise
            //
            private const uint SPCRP_UPPERFILTERS = (0x000000011);  // UpperFilters (R/W)
            private const uint SPCRP_LOWERFILTERS = (0x000000012);  // LowerFilters (R/W)
            private const uint SPCRP_SECURITY = (0x000000017);  // Security (R/W, binary form)
            private const uint SPCRP_SECURITY_SDS = (0x000000018);  // Security (W, SDS form)
            private const uint SPCRP_DEVTYPE = (0x000000019);  // Device Type (R/W)
            private const uint SPCRP_EXCLUSIVE = (0x00000001A);  // Device is exclusive-access (R/W)
            private const uint SPCRP_CHARACTERISTICS = (0x00000001B);  // Device Characteristics (R/W)
            private const uint SPCRP_MAXIMUM_PROPERTY = (0x00000001C);  // Upper bound on ordinals


            //
            // DevInst problem values, returned by call to CM_Get_DevInst_Status
            //
            private const uint CM_PROB_NOT_CONFIGURED = (0x00000001);  // no config for device
            private const uint CM_PROB_DEVLOADER_FAILED = (0x00000002);   // service load failed
            private const uint CM_PROB_OUT_OF_MEMORY = (0x00000003);   // out of memory
            private const uint CM_PROB_ENTRY_IS_WRONG_TYPE = (0x00000004);  // 
            private const uint CM_PROB_LACKED_ARBITRATOR = (0x00000005);  // 
            private const uint CM_PROB_BOOT_CONFIG_CONFLICT = (0x00000006);   // boot config conflict
            private const uint CM_PROB_FAILED_FILTER = (0x00000007);  // 
            private const uint CM_PROB_DEVLOADER_NOT_FOUND = (0x00000008);   // Devloader not found
            private const uint CM_PROB_INVALID_DATA = (0x00000009);   // Invalid ID
            private const uint CM_PROB_FAILED_START = (0x0000000A);  // 
            private const uint CM_PROB_LIAR = (0x0000000B);  // 
            private const uint CM_PROB_NORMAL_CONFLICT = (0x0000000C);   // config conflict
            private const uint CM_PROB_NOT_VERIFIED = (0x0000000D);  // 
            private const uint CM_PROB_NEED_RESTART = (0x0000000E);   // requires restart
            private const uint CM_PROB_REENUMERATION = (0x0000000F);  // 
            private const uint CM_PROB_PARTIAL_LOG_CONF = (0x00000010);  // 
            private const uint CM_PROB_UNKNOWN_RESOURCE = (0x00000011);   // unknown res type
            private const uint CM_PROB_REINSTALL = (0x00000012);  // 
            private const uint CM_PROB_REGISTRY = (0x00000013);  // 
            private const uint CM_PROB_VXDLDR = (0x00000014);   // WINDOWS 95 ONLY
            private const uint CM_PROB_WILL_BE_REMOVED = (0x00000015);   // devinst will remove
            private const uint CM_PROB_DISABLED = (0x00000016);   // devinst is disabled
            private const uint CM_PROB_DEVLOADER_NOT_READY = (0x00000017);   // Devloader not ready
            private const uint CM_PROB_DEVICE_NOT_THERE = (0x00000018);   // device doesn't exist
            private const uint CM_PROB_MOVED = (0x00000019);  // 
            private const uint CM_PROB_TOO_EARLY = (0x0000001A);  // 
            private const uint CM_PROB_NO_VALID_LOG_CONF = (0x0000001B);   // no valid log config
            private const uint CM_PROB_FAILED_INSTALL = (0x0000001C);   // install failed
            private const uint CM_PROB_HARDWARE_DISABLED = (0x0000001D);   // device disabled
            private const uint CM_PROB_CANT_SHARE_IRQ = (0x0000001E);   // can't share IRQ
            private const uint CM_PROB_FAILED_ADD = (0x0000001F);   // driver failed add
            private const uint CM_PROB_DISABLED_SERVICE = (0x00000020);   // service's Start = 4
            private const uint CM_PROB_TRANSLATION_FAILED = (0x00000021);   // resource translation failed
            private const uint CM_PROB_NO_SOFTCONFIG = (0x00000022);   // no soft config
            private const uint CM_PROB_BIOS_TABLE = (0x00000023);   // device missing in BIOS table
            private const uint CM_PROB_IRQ_TRANSLATION_FAILED = (0x00000024);   // IRQ translator failed
            private const uint CM_PROB_FAILED_DRIVER_ENTRY = (0x00000025);   // DriverEntry() failed.
            private const uint CM_PROB_DRIVER_FAILED_PRIOR_UNLOAD = (0x00000026);   // Driver should have unloaded.
            private const uint CM_PROB_DRIVER_FAILED_LOAD = (0x00000027);   // Driver load unsuccessful.
            private const uint CM_PROB_DRIVER_SERVICE_KEY_INVALID = (0x00000028);   // Error accessing driver's service key
            private const uint CM_PROB_LEGACY_SERVICE_NO_DEVICES = (0x00000029);   // Loaded legacy service created no devices
            private const uint CM_PROB_DUPLICATE_DEVICE = (0x0000002A);   // Two devices were discovered with the same name
            private const uint CM_PROB_FAILED_POST_START = (0x0000002B);   // The drivers set the device state to failed
            private const uint CM_PROB_HALTED = (0x0000002C);   // This device was failed post start via usermode
            private const uint CM_PROB_PHANTOM = (0x0000002D);   // The devinst currently exists only in the registry
            private const uint CM_PROB_SYSTEM_SHUTDOWN = (0x0000002E);   // The system is shutting down
            private const uint CM_PROB_HELD_FOR_EJECT = (0x0000002F);   // The device is offline awaiting removal
            private const uint CM_PROB_DRIVER_BLOCKED = (0x00000030);   // One or more drivers is blocked from loading
            private const uint CM_PROB_REGISTRY_TOO_LARGE = (0x00000031);   // System hive has grown too large
            private const uint CM_PROB_SETPROPERTIES_FAILED = (0x00000032);   // Failed to apply one or more registry properties  
            private const uint CM_PROB_WAITING_ON_DEPENDENCY = (0x00000033);   // Device is stalled waiting on a dependency to start
            private const uint CM_PROB_UNSIGNED_DRIVER = (0x00000034);   // Failed load driver due to unsigned image.
            private const uint CM_PROB_USED_BY_DEBUGGER = (0x00000035);   // Device is being used by kernel debugger
            private const uint CM_PROB_DEVICE_RESET = (0x00000036);   // Device is being reset
            private const uint CM_PROB_CONSOLE_LOCKED = (0x00000037);   // Device is blocked while console is locked
            private const uint CM_PROB_NEED_CLASS_CONFIG = (0x00000038);   // Device needs extended class configuration to start
            private const uint CM_PROB_GUEST_ASSIGNMENT_FAILED = (0x00000039);   // Assignment to guest partition failed

            private const uint NUM_CM_PROB_V1 = (0x00000025);
            private const uint NUM_CM_PROB_V2 = (0x00000032);
            private const uint NUM_CM_PROB_V3 = (0x00000033);
            private const uint NUM_CM_PROB_V4 = (0x00000034);
            private const uint NUM_CM_PROB_V5 = (0x00000035);
            private const uint NUM_CM_PROB_V6 = (0x00000036);
            private const uint NUM_CM_PROB_V7 = (0x00000037);
            private const uint NUM_CM_PROB_V8 = (0x00000039);
            private const uint NUM_CM_PROB_V9 = (0x0000003A);


            // エラー
            private const uint NO_ERROR = 0;
            private const uint ERROR_INVALID_DATA = 13;
            private const uint ERROR_NOT_CONTAINER = 1207;
            private const uint ERROR_INVALID_PARAMETER = 87;
            private const uint ERROR_NO_NETWORK = 1222;
            private const uint ERROR_EXTENDED_ERROR = 1208;
            private const uint ERROR_NO_MORE_ITEMS = 259;
            //エラー情報格納構造体
            public struct DevErrST
            {
                public string DeviceName;  //デバイス名
                public uint problemCode; //エラーコード
                public string ErrMessage;  //その他エラーメッセージ
            }
            public bool GetDiEnumDeviceInfoErr(ref List<MDiagDeviceMgr.DevErrST> err_List)
            {
                bool bRet = false;              //戻り値
                UInt32 dwIndex = 0;             //デバイスマネージャ値取得時のインデックス
                IntPtr info = IntPtr.Zero;      //デバイス情報のポインタ
                                                // GUID for processor classid
                                                //Guid processorGuid = new Guid("{50127dc3-0f36-415e-a6cc-4cb3be910b65}");
                Guid NullGuid = Guid.Empty;     //GUID
                IntPtr buffer = IntPtr.Zero;    //バッファ
                uint buflen = 512;
                buffer = Marshal.AllocHGlobal((int)buflen);

                err_List.Clear();                   //エラークリア
                try
                {
                    // get a list of all processor devices
                    info = SetupDiGetClassDevs(ref NullGuid, null, IntPtr.Zero, (int)DIGCF.ALLCLASSES);

                    SP_DEVINFO_DATA devdata = new SP_DEVINFO_DATA();
                    devdata.cbSize = (int)Marshal.SizeOf(devdata);


                    uint proptype = 0;
                    uint outsize = 0;
                    dwIndex = 0;
                    uint problemCode = 0;
                    //全てのデバイス情報を取得するまで継続
                    while (true)
                    {
                        //次のインデックス
                        dwIndex++;

                        //デバイス情報セット内のデバイス情報要素を指定するSP_DEVINFO_DATA構造体を返す
                        bRet = SetupDiEnumDeviceInfo(info,
                                                     (int)dwIndex,
                                                     ref devdata);
                        if (bRet == false)  //失敗時は終了
                            break;

                        //指定されたプラグ アンド プレイ デバイス プロパティを取得 最初にサイズを取得
                        bRet = SetupDiGetDeviceRegistryPropertyW(
                                                    info,                   //デバイス 情報セット へのハンドル
                                                    ref devdata,            //DeviceInfoSet のデバイス情報要素の構造体
                                                    SPDRP_DEVICEDESC,       //取得するプロパティ  SPDRP_DEVICEDESC=デバイスの説明を含むREG_SZ文字列を取得
                                                    out proptype,           //取得されるプロパティのデータ型を受け取る変数へのポインター
                                                    IntPtr.Zero,            //取得されるプロパティを受け取るバッファーへのポインター
                                                    0,                      //上記Zeroと左記0を指定することでoutsizeに必要なサイズを返す
                                                    ref outsize);

                        byte[] lbuffer = new byte[outsize];                 //必要サイズ分バッファを作成

                        //サイズが分かったためサイズを指定して実行
                        bRet = SetupDiGetDeviceRegistryPropertyW(
                                info,
                                ref devdata,
                                SPDRP_DEVICEDESC,
                                out proptype,
                                buffer,
                                outsize,
                                ref outsize);

                        //失敗時は次を実行
                        if (bRet == false)
                            continue;

                        //データのコピー
                        Marshal.Copy(buffer, lbuffer, 0, (int)outsize);
                        int errcode = Marshal.GetLastWin32Error();
                        if (errcode == ERROR_INVALID_DATA) break;           //データが無い場合は終了
                        string name = Encoding.Unicode.GetString(lbuffer);  //デバイス名を取得
                        name = name.TrimEnd('\0');
                        byte[] dwbuffer = new byte[sizeof(UInt32)];         //バッファを確保

                        //SPDRP_CONFIGFLAGSは,DWORD 値内のデバイスの構成フラグのビットごとの OR を取得
                        //ONFIGFLAG_Xxx ビットマスクによって理由がわかる
                        bRet = SetupDiGetDeviceRegistryPropertyW(
                                info,
                                ref devdata,
                                SPDRP_CONFIGFLAGS,
                                out proptype,
                                buffer,
                                sizeof(UInt32),
                                ref outsize);

                        if (bRet == true)   //正常に取得できた場合
                        {
                            //問題のコードを取得
                            Marshal.Copy(buffer, dwbuffer, 0, sizeof(UInt32));
                            problemCode = BitConverter.ToUInt32(dwbuffer, 0);
                            if (problemCode != 0)//0以外はエラー エラーを出力すると同時にエラーカウントをあげる
                            {
                                DevErrST errST = new DevErrST();
                                errST.DeviceName = name;
                                errST.problemCode = problemCode;
                                errST.ErrMessage = "ERR : " + name + " , STATUE = " + problemCode.ToString();
                                err_List.Add(errST);
                            }
                        }

                    }
                    return true;
                }
                catch (Exception ex)
                {
                    DevErrST errST = new DevErrST();
                    errST.DeviceName = "FATAL";
                    errST.problemCode = 0;
                    errST.ErrMessage = "FATAL ERROR : " + System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message;
                    err_List.Add(errST);
                    return bRet;
                }
                finally
                {
                    if (buffer != IntPtr.Zero)
                        Marshal.FreeHGlobal(buffer);
                    if (info != IntPtr.Zero) //意図的に開放しなければリークする
                        SetupDiDestroyDeviceInfoList(info);
                }

            }
        }
    }
}
