﻿////
/// Copyright(C) 2020 Micronet Co. All Rights Reserved.
//
using System;
using System.Text;
using System.Runtime.InteropServices;

namespace RTedge
{
    public partial class edge_API
    {
        // RTCDヘッダー
        public struct RTCDHEADER
        {
            public uint record_num;                 // レコード数
            public uint record_size;                // レコードサイズ(バイト数)
            public uint read_pos;                   // 読込み位置
            public uint write_pos;                  // 書込み位置
            public uint write_mode;                 // 書込みモード 1:シングル 2:マルチ
            public uint read_open_count;            // 読込みモードオープンカウンタ
            public uint write_open_count;           // 書込みモードオープンカウンタ
            public uint lost_count;                 // 欠落カウンタ
            public ulong mem_handle;                 // RTCDメモリハンドル値
            public ulong lock_sem_handle;            // 排他用セマフォハンドル値
            public ulong notify_sem_handle;          // 書込通知用セマフォハンドル値
            [MarshalAs(UnmanagedType.ByValTStr, SizeConst = EGDEFINE.EG_MAX_RTCD_NAME)]
            public string name;                     // RTCD名称(最大10文字まで)
            [MarshalAs(UnmanagedType.ByValArray, SizeConst = EGDEFINE.EG_RTCD_HEADER_RESERVE)]
            public byte[] reserve;                  // 予備
        }
        /// <summary>
        /// RTCDメモリ情報
        /// </summary>
        public struct RTCDMEMINFO
        {
            public IntPtr pRtcd;				    // RTCDメモリポインタ(本体)
            public ulong mem_handle;                 // RTCDメモリハンドル
            public ulong sem1_handle;                // 排他用セマフォハンドル
            public ulong sem2_handle;			    // 書込通知用セマフォハンドル
            [MarshalAs(UnmanagedType.ByValTStr, SizeConst = EGDEFINE.EG_MAX_RTCD_NAME)]
            public string name;
        }
        /// <summary>
        /// RTCDハンドル
        /// </summary>
        public struct RTCDHANDLE
        {
            public uint code;					   // RTCDハンドル識別コード
            public IntPtr pRtcdHeader;             // RTCDヘッダーのポインタ
            public IntPtr pRtcdData;               // RTCDデータのポインタ
            public IntPtr pMem_handle;             // RTCDメモリハンドル
            public IntPtr pSem1_handle;            // 排他用セマフォハンドル
            public IntPtr pSem2_handle;            // 書込通知用セマフォハンドル
            public uint access;					   // アクセスモード
        }
        /// <summary>
        /// Egオブジェクト参照インデックス
        /// </summary>
        [StructLayout(LayoutKind.Sequential, CharSet = CharSet.Ansi, Pack = 1)]
        public struct EGOBJINDEX
        {
            public UInt32 reserve;                      // 予約(システム内部で使用)
            public UInt32 offset;						// ヒープ空間内アイテムへのバイトオフセットポインタ
        }
        /// Dataset参照情報
        /// </summary>
        [StructLayout(LayoutKind.Sequential, CharSet = CharSet.Ansi)]
        public struct TDSLINK
        {
            [MarshalAs(UnmanagedType.LPStr)]
            public string pTagName;  // タグ名
            public IntPtr pTag;      // TagDesc*
            public IntPtr pds;       // DsetDesc*
            public ushort inst;      // FindFirst/Next
            public ushort offset;
        }
        /// <summary>
        /// タイムスタンプ情報 (12バイト) 
        /// </summary>
        public struct EG_TIME_STAMP
        {
            public ulong time;          // 現在日時 1970年1月1日からの経過秒
            public ushort millisecond;  // 現在日時 ミリ秒
            public ushort microsecond;  // 現在日時 マイクロ秒
        };
        /// <summary>
        /// タイムスタンプ情報 (12バイト) 
        /// </summary>
        public struct EG_TIME_VAL
        {
            public ulong time;          // 現在日時 1970年1月1日からの経過秒
            public uint microsecond;  // 現在日時 マイクロ秒
        };
        /// <summary>
        /// FIFO情報
        /// </summary>
        public struct RTCDHANDLEANDINFO
        {
            public RTCDHEADER stHeader;		   // RTCDHEADER
            public RTCDHANDLE stHandle;          // RTCDHANDLE
        }
        /// <summary>
        /// FIFOのハンドル情報
        /// </summary>
        public class HIGHRTCDHANDLE
        {
            public string FIFOName;
            public bool IsActive;
            public uint AccessMode;
            public RTCDHANDLEANDINFO HandleInfo = new RTCDHANDLEANDINFO();
            public HIGHRTCDHANDLE(string stName, bool bActive, uint uinAccessMode, RTCDHANDLEANDINFO tHandleInfo)
            {
                FIFOName = stName;
                IsActive = bActive;
                AccessMode = uinAccessMode;
                HandleInfo = tHandleInfo;
            }
        }
        //
        // RT-edge system 動作設定
        //
        public delegate int InstanceMessageHandler(string senderName, int messNo, byte[] paramArray);

        [StructLayout(LayoutKind.Sequential, Pack = 1)]
        /// <summary>
        /// RT-edge system 各オブジェクトデータサイズ設定値 (ファイル読み込み値)
        /// </summary>
        /// 
        public struct EDGESYSTEMCONFIG
        {
            public uint MaxDsetEntry;      // データセットエントリ最大数（テーブルのインデックス数制限）
            public uint MaxDsetLink;       // データセット データリンクタグ最大数(TagRef数)
            public uint MaxClctEntry;      // コレクタ エントリ最大数
            public uint MaxServiceEntry;   // サービスコンテナエントリ最大数
            public uint MaxMailboxEntry;   // メールボックス エントリ最大数
            public uint MaxC2CMailboxCol;  // RT-edgeフレームワーク コンテナ間通信メールボックス最大データサイズ(ColSize) 
            public uint MaxC2CMailboxRow;  // RT-edgeフレームワーク コンテナ間通信メールボックス 最大保持件数(RowSize)
            public uint MaxFwClctEntry;        // RT-edgeフレームワーク  コレクタオブジェクト参照 エントリ最大数
            public uint MaxFwMailboxEntry; // RT-edgeフレームワーク メールボックスオブジェクト エントリ最大数
            public uint MaxFwTagRefEntry;  // RT-edgeフレームワーク TagRefオブジェクト エントリ最大数
            public uint MaxTagPtrNum;           // タグポインタ最大数
            public uint MaxTagPtrSize;          // タグポインタ最大データサイズ(バイト単位)
            public uint MaxTagPtrRetry;         // タグポインタロックを試みる回数
            public uint MaxTagTriggerNum;       // タグトリガー最大数
        };
        public EDGESYSTEMCONFIG EDGE_SYSTEMCONFIG_DEFAULT = new EDGESYSTEMCONFIG() { MaxDsetEntry = 512,
                                                                                     MaxDsetLink = 2048,
                                                                                     MaxClctEntry = 64,
                                                                                     MaxServiceEntry = 32,
                                                                                     MaxMailboxEntry = 64,
                                                                                     MaxC2CMailboxCol = 100352,
                                                                                     MaxC2CMailboxRow = 4096,
                                                                                     MaxFwClctEntry = 16,
                                                                                     MaxFwMailboxEntry = 16,
                                                                                     MaxFwTagRefEntry = 1024,
                                                                                     MaxTagPtrNum = 256,
                                                                                     MaxTagPtrSize = (1024 * 1024),
                                                                                     MaxTagPtrRetry = 3,
                                                                                     MaxTagTriggerNum = 32
        };
        [StructLayout(LayoutKind.Sequential, CharSet = CharSet.Ansi, Pack = 1)]
        /// <summary>
        /// サービス側設定用 Configファイル
        /// </summary>
        public struct EDGECONFIG
        {
            public byte bDebug;     // デバッグフラグ
            public byte bReadXmlF;  // xmlを読むか読まないかのフラグ 0:読まない, 1:読む (Core用)　デフォルト 1:読む 
            public ushort shPriority; // メールボックススレッド　プライオリティ 
            public InstanceMessageHandler mInstMessFunc;
            [MarshalAs(UnmanagedType.ByValTStr, SizeConst = EGDEFINE.EG_MAX_SERVICEPATH+1)]
            public string Path; // サービスプロセス/DLLファイルパス(内部でサービス名・xmlのパスに変換するために使用)
            public byte reserve0;
            public int reserve1;
        };
        public EDGECONFIG EDGE_CONFIG_DEFAULT = new EDGECONFIG() { bDebug = 0, bReadXmlF = 1, shPriority = 65535, mInstMessFunc = null, Path = "" };

        /// <summary>
        /// サービス起動設定用 Configファイル
        /// </summary>
        [StructLayout(LayoutKind.Sequential, CharSet = CharSet.Ansi, Pack = 1)]
        public struct EDGESERVICECONFIG
        {
            [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 64 + 1)]
            public string NodeName;     // サービスを起動するノード名を指定
            public ushort Class;        // サービスクラス
        };
        public EDGESERVICECONFIG EDGE_SERVICECONFIG_DEFAULT = new EDGESERVICECONFIG() { NodeName = "" };
        /// <summary>
        /// 実際フレームワーク(C)に渡す用
        /// </summary>

        /// <summary>
        /// サービス間通信コールバック関数・関数追加関連
        /// </summary>
        /// <param name="senderName"></param>
        /// <param name="inMessageNo"></param>
        /// <param name="Data"></param>
        /// <returns></returns>
        [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
        public delegate int CallbackDelegate(/*[MarshalAs(UnmanagedType.AnsiBStr)]*/string senderName, int inMessageNo, IntPtr Data);
        [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
        public delegate int CallbackDelegateEx(/*[MarshalAs(UnmanagedType.AnsiBStr)]*/string senderName, int inMessageNo, IntPtr Data, int inSize);
        [StructLayout(LayoutKind.Sequential, Pack = 1)]
        public struct _EDGECONFIG
        {
            public byte bDebug;     // デバッグフラグ
            public byte bReadXmlF;  // xmlを読むか読まないかのフラグ 0:読まない, 1:読む (Core用)　デフォルト 1:読む 
            public ushort shPriority; // メールボックススレッド　プライオリティ 
            public CallbackDelegate mCallbackMessFunc;  //■未使用■ サービスコンテナ間通信ユーザーメッセージファンクションコールバック関数 
            [MarshalAs(UnmanagedType.ByValTStr, SizeConst = EGDEFINE.EG_MAX_SERVICEPATH+1)]
            public string Path; // サービスプロセス/DLLファイルパス(内部でサービス名・xmlのパスに変換するために使用)
            [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 1)]
            public string _nill; // 
            [MarshalAs(UnmanagedType.ByValArray, SizeConst = 2)]    //予備
            public byte[] reserve;                  // 予備
            public CallbackDelegateEx mCallbackMessFuncEx;//サービスコンテナ間通信ユーザーメッセージファンクションコールバック関数 サイズ可変
        };
        public _EDGECONFIG _EDGE_CONFIG_DEFAULT = new _EDGECONFIG() { bDebug = 0, bReadXmlF = 1, shPriority = 65535, mCallbackMessFunc = null, Path = "", mCallbackMessFuncEx = null };


        /// <summary>
        /// ハッシュコード取得用構造体
        /// </summary>
        [StructLayout(LayoutKind.Explicit, CharSet = CharSet.Ansi)]
        public struct HASHCODE
        {
            [FieldOffset(0)]
            [MarshalAs(UnmanagedType.U4)]
            public uint code1;

            [FieldOffset(4)]
            [MarshalAs(UnmanagedType.U4)]
            public uint code2;

            [FieldOffset(8)]
            [MarshalAs(UnmanagedType.U4)]
            public uint code3;

            [FieldOffset(12)]
            [MarshalAs(UnmanagedType.U4)]
            public uint code4;

            [FieldOffset(0)]
            [MarshalAs(UnmanagedType.U8)]
            public ulong code;

            [FieldOffset(8)]
            [MarshalAs(UnmanagedType.U8)]
            public ulong codeL;
        }
        //------------------------------------------------------------------------------------------------
        // Common
        //------------------------------------------------------------------------------------------------


        /// <summary>
        /// イニシャライズ ライブラリ呼び出し部分
        /// </summary>
        /// <returns></returns>
        [DllImport("eghgapiWin.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "EgInit")]
        public static extern int _C_EgInit(IntPtr ptr);

        /// <summary>
        /// ファイナライズ ライブラリ呼び出し部分
        /// </summary>
        /// <returns></returns>
        [DllImport("eghgapiWin.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "EgFinalize")]
        public static extern int _C_EgFinalize();

        /// <summary>
        /// RTCD初期化
        /// </summary>
        /// <param name="stName"></param>
        /// <param name="NumMax"></param>
        /// <returns></returns>
        [DllImport("eghgapiWin.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "EgSetRTCD")]
        public static extern int _C_EgSetRTCD(string stName);
        /// <summary>
        /// 各オブジェクト最大値設定
        /// </summary>
        /// <returns></returns>
        [DllImport("eghgapiWin.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "_EgSysDefSetting")]
        public static extern int _C__EgSysDefSetting(IntPtr ptr);

        /// <summary>
        /// システム用タグ設定
        /// </summary>
        /// <returns></returns>
        [DllImport("eghgapiWin.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "_EgSysTagCreate")]
        public static extern bool _C__EgSysTagCreate();

        //------------------------------------------------------------------------------------------------
        // Tag
        //------------------------------------------------------------------------------------------------
        /// <summary>
        /// _C_EgTagInit
        /// </summary>
        /// <param name="NumMax"></param>
        /// <returns></returns>
        [DllImport("eghgapiWin.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "EgTagInit")]
        public static extern int _C_EgTagInit(uint NumMax);

        /// <summary>
        /// _C_EgTagCreate
        /// </summary>
        /// <param name="stName"></param>
        /// <param name="Type"></param>
        /// <param name="Size"></param>
        /// <param name="stSource"></param>
        /// <returns></returns>
        [DllImport("eghgapiWin.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "EgTagCreate")]
        public static extern int _C_EgTagCreate(string stName, ushort Type, ushort Size, string stSource, string stComment);

        /// <summary>
        /// _C_EgTagCreateEx
        /// </summary>
        /// <param name="stName"></param>
        /// <param name="Type"></param>
        /// <param name="stSource"></param>
        /// <param name="stComment"></param>
        /// <returns></returns>
        [DllImport("eghgapiWin.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "EgTagCreateEx")]
        public static extern int _C_EgTagCreateEx(string stName, ushort Type, string stSource, string stComment);

        /// <summary>
        /// _C_EgTagCreateSp
        /// </summary>
        /// <param name="stName"></param>
        /// <param name="Type"></param>
        /// <param name="stSource"></param>
        /// <param name="stComment"></param>
        /// <param name="pBuffer"></param>
        /// <param name="dwSize"></param>
        /// <returns></returns>
        [DllImport("eghgapiWin.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "EgTagCreateSp")]
        public static extern int _C_EgTagCreateSp(string stName, ushort Type, string stSource, string stComment, IntPtr pBuffer, uint dwSize, ref ushort isLink);

        /// <summary>
        /// _C_EgTagWrite
        /// </summary>
        /// <param name="stName"></param>
        /// <param name="pBuffer"></param>
        /// <param name="wSize"></param>
        /// <returns></returns>
        [DllImport("eghgapiWin.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "EgTagWrite")]
        public static extern int _C_EgTagWrite(string stName, IntPtr pBuffer, ushort wSize);

        /// <summary>
        /// _C_EgTagWriteByIndex
        /// </summary>
        /// <param name="index"></param>
        /// <param name="pBuffer"></param>
        /// <param name="wSize"></param>
        /// <returns></returns>
        [DllImport("eghgapiWin.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "EgTagWriteByIndex")]
        public static extern int _C_EgTagWriteByIndex(IntPtr index, IntPtr pBuffer, ushort wSize);

        /// <summary>
        /// _C_EgTagWriteByEntry
        /// </summary>
        /// <param name="entry"></param>
        /// <param name="pBuffer"></param>
        /// <param name="wSize"></param>
        /// <returns></returns>
        [DllImport("eghgapiWin.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "EgTagWriteByEntry")]
        public static extern int _C_EgTagWriteByEntry(IntPtr entry, IntPtr pBuffer, ushort wSize);

        /// <summary>
        /// _C_EgTagRead
        /// </summary>
        /// <param name="stName"></param>
        /// <param name="pBuffer"></param>
        /// <param name="wSize"></param>
        /// <returns></returns>
        [DllImport("eghgapiWin.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "EgTagRead")]
        public static extern int _C_EgTagRead(string stName, IntPtr pBuffer, ushort wSize);

        /// <summary>
        /// _C_EgTagReadByIndex
        /// </summary>
        /// <param name="index"></param>
        /// <param name="pBuffer"></param>
        /// <param name="wSize"></param>
        /// <returns></returns>
        [DllImport("eghgapiWin.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "EgTagReadByIndex")]
        public static extern int _C_EgTagReadByIndex(IntPtr index, IntPtr pBuffer, ushort wSize);

        /// <summary>
        /// _C_EgTagReadByEntry
        /// </summary>
        /// <param name="entry"></param>
        /// <param name="pBuffer"></param>
        /// <param name="wSize"></param>
        /// <returns></returns>
        [DllImport("eghgapiWin.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "EgTagReadByEntry")]
        public static extern int _C_EgTagReadByEntry(IntPtr entry, IntPtr pBuffer, ushort wSize);

        /// <summary>
        /// _C_EgTagGetProperty
        /// </summary>
        /// <param name="stName"></param>
        /// <param name="propName"></param>
        /// <param name="pBuffer"></param>
        /// <param name="dwSize"></param>
        /// <returns></returns>
        [DllImport("eghgapiWin.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "EgTagGetProperty")]
        public static extern int _C_EgTagGetProperty(string stName, string propName, IntPtr pBuffer, uint dwSize);

        /// <summary>
        /// _C_EgTagGetPropertyByIndex
        /// </summary>
        /// <param name="index"></param>
        /// <param name="propName"></param>
        /// <param name="pBuffer"></param>
        /// <param name="dwSize"></param>
        /// <returns></returns>
        [DllImport("eghgapiWin.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "EgTagGetPropertyByIndex")]
        public static extern int _C_EgTagGetPropertyByIndex(IntPtr index, string propName, IntPtr pBuffer, uint dwSize);

        /// <summary>
        /// _C_EgTagGetPropertyByEntry
        /// </summary>
        /// <param name="entry"></param>
        /// <param name="propName"></param>
        /// <param name="pBuffer"></param>
        /// <param name="dwSize"></param>
        /// <returns></returns>
        [DllImport("eghgapiWin.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "EgTagGetPropertyByEntry")]
        public static extern int _C_EgTagGetPropertyByEntry(IntPtr entry, string propName, IntPtr pBuffer, uint dwSize);

        /// <summary>
        /// _C_EgTagGetCount
        /// </summary>
        /// <param name="itemNum"></param>
        /// <returns></returns>
        [DllImport("eghgapiWin.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "EgTagGetCount")]
        public static extern int _C_EgTagGetCount(ref uint itemNum);

        /// <summary>
        /// _C_EgTagGetEnum
        /// </summary>
        /// <param name="pBuffer"></param>
        /// <param name="dwSize"></param>
        /// <param name="itemNum"></param>
        /// <returns></returns>
        [DllImport("eghgapiWin.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "EgTagGetEnum")]
        public static extern int _C_EgTagGetEnum(IntPtr pBuffer, uint dwSize, ref uint itemNum);

        /// <summary>
        /// _C_EgTagInitSegment
        /// </summary>
        /// <returns></returns>
        [DllImport("eghgapiWin.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "EgTagInitSegment")]
        public static extern int _C_EgTagInitSegment();

        /// <summary>
        /// _C_EgTagReadSegment
        /// </summary>
        /// <param name="stName"></param>
        /// <param name="wOffset"></param>
        /// <param name="pBuffer"></param>
        /// <param name="wSize"></param>
        /// <returns></returns>
        [DllImport("eghgapiWin.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "EgTagReadSegment")]
        public static extern int _C_EgTagReadSegment(string stName, ushort wOffset, IntPtr pBuffer, ushort wSize);

        /// <summary>
        /// _C_EgTagWriteSegment
        /// </summary>
        /// <param name="stName"></param>
        /// <param name="wOffset"></param>
        /// <param name="pBuffer"></param>
        /// <param name="wSize"></param>
        /// <returns></returns>
        [DllImport("eghgapiWin.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "EgTagWriteSegment")]
        public static extern int _C_EgTagWriteSegment(string stName, ushort wOffset, IntPtr pBuffer, ushort wSize);

        /// <summary>
        /// _C_EgTagIsAliveSegment
        /// </summary>
        /// <param name="stName"></param>
        /// <returns></returns>
        [DllImport("eghgapiWin.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "EgTagIsAliveSegment")]
        public static extern int _C_EgTagIsAliveSegment(string stName);

        /// <summary>
        /// _C_EgTagCreateSegment
        /// </summary>
        /// <param name="stName"></param>
        /// <param name="Type"></param>
        /// <param name="Size"></param>
        /// <returns></returns>
        [DllImport("eghgapiWin.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "EgTagCreateSegment")]
        public static extern int _C_EgTagCreateSegment(string stName, ushort Type, ushort Size, string stSource, string stComment, IntPtr pBuffer, uint dwSize, ref ushort isLink);

        /// <summary>
        /// _C_EgTagReadString
        /// </summary>
        /// <param name="stName"></param>
        /// <param name="pString"></param>
        /// <param name="wSize"></param>
        /// <returns></returns>
        [DllImport("eghgapiWin.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "EgTagReadString")]
        public static extern int _C_EgTagReadString(string stName, StringBuilder pString, ushort wSize);

        /// <summary>
        /// _C_EgTagWriteString
        /// </summary>
        /// <param name="stName"></param>
        /// <param name="pString"></param>
        /// <returns></returns>
        [DllImport("eghgapiWin.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "EgTagWriteString")]
        public static extern int _C_EgTagWriteString(string stName, string pString);

        /// <summary>
        /// _C_EgTagReadSegmentByIndex
        /// </summary>
        /// <param name="index"></param>
        /// <param name="wOffset"></param>
        /// <param name="pBuffer"></param>
        /// <param name="wSize"></param>
        /// <returns></returns>
        [DllImport("eghgapiWin.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "EgTagReadSegmentByIndex")]
        public static extern int _C_EgTagReadSegmentByIndex(IntPtr index, ushort wOffset, IntPtr pBuffer, ushort wSize);

        /// <summary>
        /// _C_EgTagWriteSegmentByIndex
        /// </summary>
        /// <param name="index"></param>
        /// <param name="wOffset"></param>
        /// <param name="pBuffer"></param>
        /// <param name="wSize"></param>
        /// <returns></returns>
        [DllImport("eghgapiWin.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "EgTagWriteSegmentByIndex")]
        public static extern int _C_EgTagWriteSegmentByIndex(IntPtr index, ushort wOffset, IntPtr pBuffer, ushort wSize);

        /// <summary>
        /// _C_EgTagReadSegmentByEntry
        /// </summary>
        /// <param name="entry"></param>
        /// <param name="wOffset"></param>
        /// <param name="pBuffer"></param>
        /// <param name="wSize"></param>
        /// <returns></returns>
        [DllImport("eghgapiWin.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "EgTagReadSegmentByEntry")]
        public static extern int _C_EgTagReadSegmentByEntry(IntPtr entry, ushort wOffset, IntPtr pBuffer, ushort wSize);

        /// <summary>
        /// _C_EgTagWriteSegmentByEntry
        /// </summary>
        /// <param name="entry"></param>
        /// <param name="wOffset"></param>
        /// <param name="pBuffer"></param>
        /// <param name="wSize"></param>
        /// <returns></returns>
        [DllImport("eghgapiWin.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "EgTagWriteSegmentByEntry")]
        public static extern int _C_EgTagWriteSegmentByEntry(IntPtr entry, ushort wOffset, IntPtr pBuffer, ushort wSize);

        /// <summary>
        /// _C_EgTagSetDefaultValue
        /// </summary>
        /// <returns></returns>
        [DllImport("eghgapiWin.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "EgTagSetDefaultValue")]
        public static extern int _C_EgTagSetDefaultValue();

        /// <summary>
        /// _C_EgTagSetDefaultValueByGroup
        /// </summary>
        /// <param name="wGroup"></param>
        /// <returns></returns>
        [DllImport("eghgapiWin.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "EgTagSetDefaultValueByGroup")]
        public static extern int _C_EgTagSetDefaultValueByGroup(UInt16 wGroup);

        /// <summary>
        /// _C_EgTagCreateFIFO
        /// </summary>
        /// <param name="stName"></param>
        /// <param name="RecordSize"></param>
        /// <param name="RecordCount"></param>
        /// <param name="stComment"></param>
        /// <param name="regName"></param>
        /// <param name="regNameSize"></param>
        /// <param name="isLink"></param>
        /// <returns></returns>
        [DllImport("eghgapiWin.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "EgTagCreateFIFO")]
        public static extern int _C_EgTagCreateFIFO(string stName, ushort RecordSize, ushort RecordCount, bool Reserved1_LetFalse, string stComment, StringBuilder regName, ushort regNameSize, out ushort isLink);

        /// <summary>
        /// _C_EgTagDequeueFIFOByEntry
        /// </summary>
        /// <param name="entry"></param>
        /// <param name="pBuffer"></param>
        /// <param name="wSize"></param>
        /// <param name="wRemain"></param>
        /// <returns></returns>
        [DllImport("eghgapiWin.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "EgTagDequeueFIFOByEntry")]
        public static extern int _C_EgTagDequeueFIFOByEntry(IntPtr entry, IntPtr pBuffer, ushort wSize, out ushort wRemain);
        
        /// <summary>
        /// _C_EgTagDequeueFIFOByIndex
        /// </summary>
        /// <param name="index"></param>
        /// <param name="pBuffer"></param>
        /// <param name="wSize"></param>
        /// <param name="wRemain"></param>
        /// <returns></returns>
        [DllImport("eghgapiWin.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "EgTagDequeueFIFOByIndex")]
        public static extern int _C_EgTagDequeueFIFOByIndex(ref EGOBJINDEX index, IntPtr pBuffer, ushort wSize, out ushort wRemain);
        
        /// <summary>
        /// _C_EgTagDequeueFIFO
        /// </summary>
        /// <param name="stName"></param>
        /// <param name="pBuffer"></param>
        /// <param name="wSize"></param>
        /// <param name=""></param>
        /// <param name="wRemain"></param>
        /// <returns></returns>
        [DllImport("eghgapiWin.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "EgTagDequeueFIFO")]
        public static extern int _C_EgTagDequeueFIFO(string stName, IntPtr pBuffer, ushort wSize, out ushort wRemain);
        
        /// <summary>
        /// _C_EgTagEnqueueFIFOByEntry
        /// </summary>
        /// <param name="entry"></param>
        /// <param name="pBuffer"></param>
        /// <param name="wSize"></param>
        /// <param name="wRemain"></param>
        /// <returns></returns>
        [DllImport("eghgapiWin.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "EgTagEnqueueFIFOByEntry")]
        public static extern int _C_EgTagEnqueueFIFOByEntry(IntPtr entry, IntPtr pBuffer, ushort wSize, out ushort wRemain);
        
        /// <summary>
        /// _C_EgTagEnqueueFIFOByIndex
        /// </summary>
        /// <param name="index"></param>
        /// <param name="pBuffer"></param>
        /// <param name="wSize"></param>
        /// <param name="wRemain"></param>
        /// <returns></returns>
        [DllImport("eghgapiWin.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "EgTagEnqueueFIFOByIndex")]
        public static extern int _C_EgTagEnqueueFIFOByIndex(ref EGOBJINDEX index, IntPtr pBuffer, ushort wSize, out ushort wRemain);
        
        /// <summary>
        /// _C_EgTagEnqueueFIFO
        /// </summary>
        /// <param name="stName"></param>
        /// <param name="pBuffer"></param>
        /// <param name="wSize"></param>
        /// <param name="wRemain"></param>
        /// <returns></returns>
        [DllImport("eghgapiWin.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "EgTagEnqueueFIFO")]
        public static extern int _C_EgTagEnqueueFIFO(string stName, IntPtr pBuffer, ushort wSize, out ushort wRemain);
        
        /// <summary>
        /// _C_EgTagGetRemainFIFOByEntry
        /// </summary>
        /// <param name="entry"></param>
        /// <param name="wRemain"></param>
        /// <returns></returns>
        [DllImport("eghgapiWin.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "EgTagGetRemainFIFOByEntry")]
        public static extern int _C_EgTagGetRemainFIFOByEntry(IntPtr entry, out ushort wRemain);
        
        /// <summary>
        /// _C_EgTagGetRemainFIFOByIndex
        /// </summary>
        /// <param name="index"></param>
        /// <param name="wRemain"></param>
        /// <returns></returns>
        [DllImport("eghgapiWin.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "EgTagGetRemainFIFOByIndex")]
        public static extern int _C_EgTagGetRemainFIFOByIndex(ref EGOBJINDEX index, out ushort wRemain);
        
        /// <summary>
        /// _C_EgTagGetRemainFIFO
        /// </summary>
        /// <param name="stName"></param>
        /// <param name="wRemain"></param>
        /// <returns></returns>
        [DllImport("eghgapiWin.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "EgTagGetRemainFIFO")]
        public static extern int _C_EgTagGetRemainFIFO(string stName, out ushort wRemain);
        
        /// <summary>
        /// _C_EgTagClearFIFOByEntry
        /// </summary>
        /// <param name="entry"></param>
        /// <returns></returns>
        [DllImport("eghgapiWin.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "EgTagClearFIFOByEntry")]
        public static extern int _C_EgTagClearFIFOByEntry(IntPtr entry);
        
        /// <summary>
        /// _C_EgTagClearFIFOByIndex
        /// </summary>
        /// <param name="index"></param>
        /// <returns></returns>
        [DllImport("eghgapiWin.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "EgTagClearFIFOByIndex")]
        public static extern int _C_EgTagClearFIFOByIndex(ref EGOBJINDEX index);
        
        /// <summary>
        /// _C_EgTagClearFIFO
        /// </summary>
        /// <param name="stName"></param>
        /// <returns></returns>
        [DllImport("eghgapiWin.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "EgTagClearFIFO")]
        public static extern int _C_EgTagClearFIFO(string stName);


        //---------------------------------------------------------------------------------------
        //
        // Dataset
        //
        //---------------------------------------------------------------------------------------
        /// <summary>
        /// EgDatasetCreate
        /// </summary>
        /// <param name="stName"></param>
        /// <returns></returns>
        [DllImport("eghgapiWin.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "EgDatasetCreate")]
        public static extern int _C_EgDatasetCreate(string stName);
        
        /// <summary>
        /// EgDatasetAddTag
        /// </summary>
        /// <param name="stName"></param>
        /// <param name="stTagName"></param>
        /// <returns></returns>
        [DllImport("eghgapiWin.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "EgDatasetAddTag")]
        public static extern int _C_EgDatasetAddTag(string stName, string stTagName);
        
        /// <summary>
        /// EgDatasetGetSize
        /// </summary>
        /// <param name="stName"></param>
        /// <returns></returns>
        [DllImport("eghgapiWin.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "EgDatasetGetSize")]
        public static extern int _C_EgDatasetGetSize(string stName);

        /// <summary>
        /// EgDatasetSetBinary
        /// </summary>
        /// <param name="stName"></param>
        /// <param name="Source"></param>
        /// <param name="SourceSize"></param>
        /// <returns></returns>
        [DllImport("eghgapiWin.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "EgDatasetSetBinary")]
        public static extern int _C_EgDatasetSetBinary(string stName, IntPtr Source, UInt32 SourceSize);

        /// <summary>
        /// EgDatasetGetBinary
        /// </summary>
        /// <param name="stName"></param>
        /// <param name="pDest"></param>
        /// <param name="uisize"></param>
        /// <returns></returns>
        [DllImport("eghgapiWin.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "EgDatasetGetBinary")]
        public static extern int _C_EgDatasetGetBinary(string stName, IntPtr pDest, uint uisize);
        
        /// <summary>
        /// EgDatasetGetFirst
        /// </summary>
        /// <param name="stName"></param>
        /// <param name="pDlink"></param>
        /// <returns></returns>
        [DllImport("eghgapiWin.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "EgDatasetGetFirst")]
        public static extern int _C_EgDatasetGetFirst(string stName, IntPtr pDlink);
        
        /// <summary>
        /// EgDatasetGetNext
        /// </summary>
        /// <param name="pDlink"></param>
        /// <returns></returns>
        [DllImport("eghgapiWin.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "EgDatasetGetNext")]
        public static extern int _C_EgDatasetGetNext(IntPtr pDlink);

        /// <summary>
        /// _C_EgDatasetGetPropertyByIndex
        /// </summary>
        /// <param name="index"></param>
        /// <param name="propName"></param>
        /// <param name="pBuffer"></param>
        /// <param name="dwSize"></param>
        /// <returns></returns>
        [DllImport("eghgapiWin.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "EgDatasetGetPropertyByIndex")]
        public static extern int _C_EgDatasetGetPropertyByIndex(IntPtr index, string propName, IntPtr pBuffer, uint dwSize);

        /// <summary>
        /// _C_EgDatasetGetCount
        /// </summary>
        /// <param name="itemNum"></param>
        /// <returns></returns>
        [DllImport("eghgapiWin.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "EgDatasetGetCount")]
        public static extern int _C_EgDatasetGetCount(ref uint itemNum);

        /// <summary>
        /// _C_EgDatasetGetEnum
        /// </summary>
        /// <param name="pBuffer"></param>
        /// <param name="dwSize"></param>
        /// <param name="itemNum"></param>
        /// <returns></returns>
        [DllImport("eghgapiWin.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "EgDatasetGetEnum")]
        public static extern int _C_EgDatasetGetEnum(IntPtr pBuffer, uint dwSize, ref uint itemNum);

        //---------------------------------------------------------------------------------------
        //
        // Collector
        //
        //---------------------------------------------------------------------------------------
        /// <summary>
        /// EgCollectorCreate
        /// </summary>
        /// <param name="Name"></param>
        /// <param name="OwnerName"></param>
        /// <param name="Interval"></param>
        /// <param name="Priority"></param>
        /// <param name="ColSize"></param>
        /// <param name="RowSize"></param>
        /// <param name="DestName"></param>
        /// <returns></returns>
        [DllImport("eghgapiWin.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "EgCollectorCreate")]
        public static extern int _C_EgCollectorCreate(string Name, string OwnerName, uint Interval, byte Priority, uint ColSize, uint RowSize, string DestName);

        /// <summary>
        /// EgCollectorGetProperty
        /// </summary>
        /// <param name="stName"></param>
        /// <param name="propName"></param>
        /// <param name="pBuffer"></param>
        /// <param name="dwSize"></param>
        /// <returns></returns>
        [DllImport("eghgapiWin.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "EgCollectorGetProperty")]
        public static extern int _C_EgCollectorGetProperty(string stName, string propName, IntPtr pBuffer, uint dwSize);

        /// <summary>
        /// _C_EgCollectorGetPropertyByIndex
        /// </summary>
        /// <param name="index"></param>
        /// <param name="propName"></param>
        /// <param name="pBuffer"></param>
        /// <param name="dwSize"></param>
        /// <returns></returns>
        [DllImport("eghgapiWin.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "EgCollectorGetPropertyByIndex")]
        public static extern int _C_EgCollectorGetPropertyByIndex(IntPtr index, string propName, IntPtr pBuffer, uint dwSize);

        /// <summary>
        /// _C_EgCollectorGetCount
        /// </summary>
        /// <param name="itemNum"></param>
        /// <returns></returns>
        [DllImport("eghgapiWin.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "EgCollectorGetCount")]
        public static extern int _C_EgCollectorGetCount(ref uint itemNum);

        /// <summary>
        /// _C_EgCollectorGetEnum
        /// </summary>
        /// <param name="pBuffer"></param>
        /// <param name="dwSize"></param>
        /// <param name="itemNum"></param>
        /// <returns></returns>
        [DllImport("eghgapiWin.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "EgCollectorGetEnum")]
        public static extern int _C_EgCollectorGetEnum(IntPtr pBuffer, uint dwSize, ref uint itemNum);

        //---------------------------------------------------------------------------------------
        //
        // MailBox
        //
        //---------------------------------------------------------------------------------------
        /// <summary>
        /// EgMailboxCreateEx
        /// </summary>
        /// <param name="Name"></param>
        /// <param name="uiColSize"></param>
        /// <param name="uiRowSize"></param>
        /// <param name="pInfo"></param>
        /// <returns></returns>
        [DllImport("eghgapiWin.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "EgMailboxCreateEx")]
        public static extern int _C_EgMailboxCreateEx(string Name, uint uiColSize, uint uiRowSize, IntPtr pInfo);
        
        /// <summary>
        /// EgMailboxDelete
        /// </summary>
        /// <param name="pInfo"></param>
        /// <returns></returns>
        [DllImport("eghgapiWin.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "EgMailboxDelete")]
        public static extern int _C_EgMailboxDelete(IntPtr pInfo);
        
        /// <summary>
        /// EgMailboxOpen
        /// </summary>
        /// <param name="Name"></param>
        /// <param name="dwAccess"></param>
        /// <param name="pHandle"></param>
        /// <returns></returns>
        [DllImport("eghgapiWin.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "EgMailboxOpen")]
        public static extern int _C_EgMailboxOpen(string Name, uint dwAccess, IntPtr pHandle);
        
        /// <summary>
        /// EgMailboxClose
        /// </summary>
        /// <param name="pHandle"></param>
        /// <returns></returns>
        [DllImport("eghgapiWin.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "EgMailboxClose")]
        public static extern int _C_EgMailboxClose(IntPtr pHandle);
        
        /// <summary>
        /// EgMailboxSend
        /// </summary>
        /// <param name="pHandle"></param>
        /// <param name="SenderName"></param>
        /// <param name="MessageNo"></param>
        /// <param name="pData"></param>
        /// <param name="uiSize"></param>
        /// <returns></returns>
        [DllImport("eghgapiWin.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "EgMailboxSend")]
        public static extern int _C_EgMailboxSend(IntPtr pHandle, string SenderName, int MessageNo, IntPtr pData, uint uiSize);
        
        /// <summary>
        /// EgMailboxReceive
        /// </summary>
        /// <param name="pHandle"></param>
        /// <param name="SenderName"></param>
        /// <param name="MessageNo"></param>
        /// <param name="pData"></param>
        /// <param name="uiTimeOut"></param>
        /// <returns></returns>
        [DllImport("eghgapiWin.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "EgMailboxReceive")]
        public static extern int _C_EgMailboxReceive(IntPtr pHandle, StringBuilder SenderName, uint dwNameSize, ref int MessageNo, IntPtr pData, uint dwSize, uint uiTimeOut);
        
        /// <summary>
        /// EgMailboxGetInfo
        /// </summary>
        /// <param name="pHandle"></param>
        /// <param name="pInfo"></param>
        /// <returns></returns>
        [DllImport("eghgapiWin.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "EgMailboxGetInfo")]
        public static extern int _C_EgMailboxGetInfo(IntPtr pHandle, IntPtr pInfo);

        /// <summary>
        /// EgMailboxGetProperty
        /// </summary>
        /// <param name="stName"></param>
        /// <param name="propName"></param>
        /// <param name="pBuffer"></param>
        /// <param name="dwSize"></param>
        /// <returns></returns>
        [DllImport("eghgapiWin.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "EgMailboxGetProperty")]
        public static extern int _C_EgMailboxGetProperty(string stName, string propName, IntPtr pBuffer, uint dwSize);

        /// <summary>
        /// _C_EgMailboxGetPropertyByIndex
        /// </summary>
        /// <param name="index"></param>
        /// <param name="propName"></param>
        /// <param name="pBuffer"></param>
        /// <param name="dwSize"></param>
        /// <returns></returns>
        [DllImport("eghgapiWin.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "EgMailboxGetPropertyByIndex")]
        public static extern int _C_EgMailboxGetPropertyByIndex(IntPtr index, string propName, IntPtr pBuffer, uint dwSize);

        /// <summary>
        /// _C_EgMailboxGetCount
        /// </summary>
        /// <param name="itemNum"></param>
        /// <returns></returns>
        [DllImport("eghgapiWin.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "EgMailboxGetCount")]
        public static extern int _C_EgMailboxGetCount(ref uint itemNum);

        /// <summary>
        /// _C_EgMailboxGetEnum
        /// </summary>
        /// <param name="pBuffer"></param>
        /// <param name="dwSize"></param>
        /// <param name="itemNum"></param>
        /// <returns></returns>
        [DllImport("eghgapiWin.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "EgMailboxGetEnum")]
        public static extern int _C_EgMailboxGetEnum(IntPtr pBuffer, uint dwSize, ref uint itemNum);

        //---------------------------------------------------------------------------------------
        //
        // Service
        //
        //---------------------------------------------------------------------------------------
        /// <summary>
        /// EgServiceCreate
        /// </summary>
        /// <param name="Name"></param>
        /// <param name="path"></param>
        /// <param name="arg"></param>
        /// <param name="pid"></param>
        /// <returns></returns>
        [DllImport("eghgapiWin.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "EgServiceCreate")]
        public static extern int _C_EgServiceCreate(string Name, string path, string arg, IntPtr pid);

        /// <summary>
        /// EgServiceCreateEx
        /// </summary>
        /// <param name="Name"></param>
        /// <param name="path"></param>
        /// <param name="arg"></param>
        /// <param name="pid"></param>
        /// <param name="config"></param>
        /// <returns></returns>
        [DllImport("eghgapiWin.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "EgServiceCreateEx")]
        public static extern int _C_EgServiceCreateEx(string Name, string path, string arg, IntPtr pid, IntPtr config);

        /// <summary>
        /// EgServiceGetProperty
        /// </summary>
        /// <param name="stName"></param>
        /// <param name="propName"></param>
        /// <param name="pBuffer"></param>
        /// <param name="dwSize"></param>
        /// <returns></returns>
        [DllImport("eghgapiWin.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "EgServiceGetProperty")]
        public static extern int _C_EgServiceGetProperty(string stName, string propName, IntPtr pBuffer, uint dwSize);

        /// <summary>
        /// _C_EgServiceGetPropertyByIndex
        /// </summary>
        /// <param name="index"></param>
        /// <param name="propName"></param>
        /// <param name="pBuffer"></param>
        /// <param name="dwSize"></param>
        /// <returns></returns>
        [DllImport("eghgapiWin.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "EgServiceGetPropertyByIndex")]
        public static extern int _C_EgServiceGetPropertyByIndex(IntPtr index, string propName, IntPtr pBuffer, uint dwSize);

        /// <summary>
        /// _C_EgServiceGetCount
        /// </summary>
        /// <param name="itemNum"></param>
        /// <returns></returns>
        [DllImport("eghgapiWin.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "EgServiceGetCount")]
        public static extern int _C_EgServiceGetCount(ref uint itemNum);

        /// <summary>
        /// _C_EgServiceGetEnum
        /// </summary>
        /// <param name="pBuffer"></param>
        /// <param name="dwSize"></param>
        /// <param name="itemNum"></param>
        /// <returns></returns>
        [DllImport("eghgapiWin.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "EgServiceGetEnum")]
        public static extern int _C_EgServiceGetEnum(IntPtr pBuffer, uint dwSize, ref uint itemNum);

        //---------------------------------------------------------------------------------------
        //
        // etc
        //
        //---------------------------------------------------------------------------------------
        /// <summary>
        /// EgGetTime
        /// </summary>
        /// <param name="pTimeStmp"></param>
        /// <returns></returns>
        [DllImport("eghgapiWin.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "EgGetTime")]
        public static extern int _C_EgGetTime(IntPtr pTimeStmp);
        
        /// <summary>
        /// EgGetTimeVal
        /// </summary>
        /// <param name="pTimeVal"></param>
        /// <returns></returns>
        [DllImport("eghgapiWin.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "EgGetTimeVal")]
        public static extern int _C_EgGetTimeVal(IntPtr pTimeVal);
        
        /// <summary>
        /// EgDecode
        /// </summary>
        /// <param name="dst"></param>
        /// <param name="dstsize"></param>
        /// <param name="src"></param>
        /// <param name="srcsize"></param>
        /// <param name="addrformat"></param>
        /// <param name="actualsize"></param>
        /// <returns></returns>
        [DllImport("eghgapiWin.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "EgDecode")]
        public static extern int _C_EgDecode(IntPtr dst, uint dstsize, IntPtr src, uint srcsize, string addrformat, ref uint actualsize);
        
        /// <summary>
        /// EgEncode
        /// </summary>
        /// <param name="dst"></param>
        /// <param name="dstsize"></param>
        /// <param name="src"></param>
        /// <param name="srcsize"></param>
        /// <param name="addrformat"></param>
        /// <param name="actualsize"></param>
        /// <returns></returns>
        [DllImport("eghgapiWin.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "EgEncode")]
        public static extern int _C_EgEncode(IntPtr dst, uint dstsize, IntPtr src, uint srcsize, string addrformat, ref uint actualsize);
        
        /// <summary>
        /// EgMakeAddress
        /// </summary>
        /// <param name="offset"></param>
        /// <param name="size"></param>
        /// <param name="buff"></param>
        /// <param name="buffsize"></param>
        /// <returns></returns>
        [DllImport("eghgapiWin.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "EgMakeAddress")]
        public static extern IntPtr _C_EgMakeAddress(int offset, short size, StringBuilder buff, uint buffsize);
        
        /// <summary>
        /// EgParseAddress (const char* addrformat, int32_t *offset, size_t* size, int16_t *bit, uint8_t* typemode);
        /// </summary>
        /// <param name="addrformat"></param>
        /// <param name="offset"></param>
        /// <param name="size"></param>
        /// <param name="bit"></param>
        /// <param name="typemode"></param>
        /// <returns></returns>
        [DllImport("eghgapiWin.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "EgParseAddress")]
        public static extern int _C_EgParseAddress(string addrformat, ref int offset, ref uint size, ref short bit, ref byte typemode);
        
        /// <summary>
        /// EgFw_AddTagTrigger (const char* pTagName);
        /// </summary>
        /// <param name="tagName">トリガー対象のタグ名</param>
        /// <returns></returns>
        [DllImport("eghgapiWin.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "EgFW_AddTagTrigger")]
        public static extern int _C_EgFW_AddTagTrigger(string tagName);
    }
}
