////
/// Copyright(C) 2020 Micronet Co. All Rights Reserved.
//
/*****************************************************************************
* egosdep.c
*
*   RT-edge system OS depend call API header
*
\*****************************************************************************/
#pragma once
#include <stdint.h>
#include <stdbool.h>
#include "egAPI.h"
#include "edgeAPI_RTCD.h"

#ifdef __cplusplus
extern "C"
{
#endif

#ifdef  EGOSDEP_EXPORTS
#define egosdep_API
//#else
//#define egosdep_API __declspec(dllimport)
#endif

//------------------------------------------------------------------------------
// OS depend API`
//------------------------------------------------------------------------------

	//
	// Memory control
	//
	egosdep_API void* edge_createMemory(const char *Name, uint32_t dwSize, void **MemHandle);
	egosdep_API uint32_t edge_freeMemory(const char *Name, void *Memory, void *MemHandle);
	egosdep_API void* edge_map(void *MemHandle);
	egosdep_API uint32_t edge_unmap(void *MemPointer, void *MemHandle);
	egosdep_API uint32_t edge_closeHandle(void *Handle);

	egosdep_API void* edge_mallocMemory(size_t dwSize);
	egosdep_API uint32_t edge_freemallocMemory(void *Memory);
	
	//
	// Name control
	//
	egosdep_API uint32_t edge_catalog(void *MemHandle, const char *Name);
	egosdep_API uint32_t edge_uncatalog(void *Handle, const char *Name);
	egosdep_API void* edge_lookup(const char *Name);
	egosdep_API uint32_t edge_setNode(const char *Name);
	egosdep_API uint32_t edge_getNodeName(char *pNode, uint32_t dwSize);
	egosdep_API void* edge_lookup_memory(const char *NodeName, const char *ProcessName, const char *ObjectName, uint32_t *pSize);

	//
	// Name control
	//
	egosdep_API uint32_t edge_createService(const char *Name ,const char *path ,const char *arg, intptr_t *pid);
	egosdep_API uint32_t edge_createServiceEx(const char *Name, const char *path, const char *arg, intptr_t *pid, EDGESERVICECONFIG *config);

	//
	// RTCD mailbox (edge v2 feature) control
	//
	egosdep_API int32_t edge_creMbox(const char *pName, uint32_t dwLength, uint32_t dwNum, RTCDMEMINFO *pRtcdInfo);
	egosdep_API int32_t edge_delMbox(RTCDMEMINFO *pRtcdInfo);
	egosdep_API int32_t edge_openMbox(const char *pName, uint32_t dwAccess, RTCDHANDLE *pHandle);
	egosdep_API int32_t edge_closeMbox(RTCDHANDLE *pHandle);
	egosdep_API int32_t edge_sendMbox(RTCDHANDLE *pHandle, void *pBuffer, uint32_t dwSize);
	egosdep_API int32_t edge_recvMbox(RTCDHANDLE *pHandle, void *pBuffer, uint32_t dwBufferSize, uint32_t dwTimeout);
	egosdep_API int32_t edge_infoMbox(RTCDHANDLE *pHandle, EDGE_RTCD_HEADER *pInfo);
	egosdep_API int32_t edge_checkHandle(RTCDHANDLE *pHandle);
	egosdep_API int32_t edge_sendMboxEx(RTCDHANDLE *pHandle, void *pBuffer1, uint32_t dwSize1, void *pBuffer2, uint32_t dwSize2);
	egosdep_API int32_t edge_recvMboxEx(RTCDHANDLE *pHandle, void *pBuffer1, uint32_t dwBufferSize1, void *pBuffer2, uint32_t dwBufferSize2, uint32_t dwTimeout);

	//
	// Thread control
	//
	egosdep_API void* edge_createThread(uint16_t shPriority, uint32_t dwStackSize, void* func, void* arg, int32_t* errorNo);
	egosdep_API	uint8_t edge_deleteThread(void* hHandle);

	//
	// Common Function
	//
	egosdep_API void edge_sleep(uint32_t dwMillseconds);				//sleep
	egosdep_API void edge_getProcPath(char *pPath, uint32_t dwSize);	//s̃vZXpX擾
	egosdep_API int32_t edge_getTime(EDGE_TIME_STAMP *pTime);			//ݓ̃^CX^v擾
	egosdep_API int32_t edge_getTimeVal(EDGE_TIME_VAL *pTime);			//ݓ̃^CX^v擾
	egosdep_API int32_t edge_Localtime64(void* pTime, void* pLongTime);	//G|bNb猻ݎɕϊ
	egosdep_API int32_t edge_setPriority(void* hdl, uint8_t byPriority);		//vCIeB̐ݒ Windowsp
	egosdep_API int32_t edge_getCommmandLine(char *pPath, uint32_t dwSize);	//R}hC擾

	egosdep_API void edge_strncpy_s(char *dst, size_t dstmax, const char *src, size_t n);

	egosdep_API unsigned long long edge_GetPCounter();					//ptH[}XJE^[̎擾
	egosdep_API double edge_CalcTime(unsigned long long start_Count, unsigned long long finish_Count);//ptH[}XvZ nano

#ifdef __cplusplus
}
#endif