////
/// Copyright(C) 2020 Micronet Co. All Rights Reserved.
//
/*****************************************************************************
* egosdep.c
*
*   RT-edge system OS depend call API header
*
\*****************************************************************************/
#pragma once
#include <stdint.h>
#include <stdbool.h>
#include "edgeAPI_RTCD.h"

#ifdef __cplusplus
extern "C"
{
#endif

//#ifdef  EGOSDEP_EXPORTS
//#define egosdep_API
//#else
//#define egosdep_API __declspec(dllimport)
//#endif

//------------------------------------------------------------------------------
// OS depend API Wrapper`
//------------------------------------------------------------------------------

	//
	// Memory control
	//
	void* os_edge_createMemory(const char *Name, uint32_t dwSize, void **MemHandle);
	uint32_t os_edge_freeMemory(const char *Name, void *Memory, void *MemHandle);
	void* os_edge_map(void *MemHandle);
	uint32_t os_edge_unmap(void *MemPointer, void *MemHandle);
	uint32_t os_edge_closeHandle(void *Handle);

	void* os_edge_mallocMemory(size_t dwSize);
	uint32_t os_edge_freemallocMemory(void *Memory);

	//
	// Name control
	//
	uint32_t os_edge_catalog(void *MemHandle, const char *Name);
	uint32_t os_edge_uncatalog(void *Handle, const char *Name);
	void* os_edge_lookup(const char *Name);
	uint32_t os_edge_setNode(const char *Name);
	void* os_edge_lookup_memory(const char *NodeName, const char *ProcessName, const char *ObjectName, uint32_t *pSize);

	//
	// Name control
	//
	uint32_t os_edge_createService(const char *Name, const char *path, const char *arg, intptr_t *pid);
	uint32_t os_edge_createServiceEx(const char *Name, const char *path, const char *arg, intptr_t *pid, EDGESERVICECONFIG *config);

	//
	// RTCD mailbox (edge v2 feature) control
	//
	int32_t os_edge_creMbox(const char *pName, uint32_t dwLength, uint32_t dwNum, RTCDMEMINFO *pRtcdInfo);
	int32_t os_edge_delMbox(RTCDMEMINFO *pRtcdInfo);
	int32_t os_edge_openMbox(const char *pName, uint32_t dwAccess, RTCDHANDLE *pHandle);
	int32_t os_edge_closeMbox(RTCDHANDLE *pHandle);
	int32_t os_edge_sendMbox(RTCDHANDLE *pHandle, void *pBuffer, uint32_t dwSize);
	int32_t os_edge_recvMbox(RTCDHANDLE *pHandle, void *pBuffer, uint32_t dwBufferSize, uint32_t dwTimeout);
	int32_t os_edge_infoMbox(RTCDHANDLE *pHandle, EDGE_RTCD_HEADER *pInfo);
	int32_t os_edge_sendMboxEx(RTCDHANDLE *pHandle, void *pBuffer1, uint32_t dwSize1, void *pBuffer2, uint32_t dwSize2);
	int32_t os_edge_recvMboxEx(RTCDHANDLE *pHandle, void *pBuffer1, uint32_t dwBufferSize1, void *pBuffer2, uint32_t dwBufferSize2, uint32_t dwTimeout);

	//
	// Thread control
	//
	void* os_edge_createThread(uint16_t shPriority, uint32_t dwStackSize, void* func, void* arg, int32_t* errorNo);
	uint8_t os_edge_deleteThread(void* hHandle);

	//
	// Common Function
	//
	void os_edge_sleep(uint32_t dwMillseconds);				//sleep
	void os_edge_getProcPath(char *pPath, uint32_t dwSize);	//s̃vZXpX擾
	int32_t os_edge_getTime(EDGE_TIME_STAMP *pTime);			//ݓ̃^CX^v擾
	int32_t os_edge_getTimeVal(EDGE_TIME_VAL *pTime);			//ݓ̃^CX^v擾
	int32_t os_edge_Localtime64(void* pTime, void* pLongTime);	//G|bNb猻ݎɕϊ
	int32_t os_edge_setPriority(void* hdl, uint8_t byPriority);		//vCIeB̐ݒ Windowsp
	int32_t os_edge_getCommmandLine(char *pPath, uint32_t dwSize);	//R}hC擾

	void os_edge_strncpy_s(char *dst, size_t dstmax, const char *src, size_t n);
	int32_t os_edge_checkHandle(RTCDHANDLE *pHandle);				//nh`FbN֐

	int32_t os_edge_GetPCounter(unsigned long long *pCounter);						//ptH[}XJE^[̎擾
	int32_t os_edge_CalcTime(unsigned long long start_Count, unsigned long long finish_Count, double* pTime);//ptH[}XvZ nano

//------------------------------------------------------------------------------
// OS depend API INp`
//------------------------------------------------------------------------------

	//
	// Memory control
	//
	typedef void*		(__cdecl *DPedge_createMemory)(const char *Name, uint32_t dwSize, void **MemHandle);
	typedef uint32_t	(__cdecl *DPedge_freeMemory)(const char *Name, void *Memory, void *MemHandle);
	typedef void*		(__cdecl *DPedge_map)(void *MemHandle);
	typedef uint32_t	(__cdecl *DPedge_unmap)(void *MemPointer, void *MemHandle);
	typedef uint32_t	(__cdecl *DPedge_closeHandle)(void *Handle);

	typedef void*		(__cdecl *DPedge_mallocMemory)(size_t dwSize);
	typedef uint32_t	(__cdecl *DPedge_freemallocMemory)(void *Memory);

	////
	//// Name control
	////
	typedef uint32_t	(__cdecl *DPedge_catalog)(void *MemHandle, const char *Name);
	typedef uint32_t	(__cdecl *DPedge_uncatalog)(void *Handle, const char *Name);
	typedef void*		(__cdecl *DPedge_lookup)(const char *Name);
	typedef uint32_t	(__cdecl *DPedge_setNode)(const char *Name);
	typedef uint32_t	(__cdecl *DPedge_getNodeName)(char *pNode, uint32_t dwSize);
	typedef void*		(__cdecl *DPedge_lookup_memory)(const char *NodeName, const char *ProcessName, const char *ObjectName, uint32_t *pSize);

	////
	//// Service control
	////
	typedef uint32_t	(__cdecl *DPedge_createService)(const char *Name, const char *path, const char *arg, intptr_t *pid);
	typedef uint32_t	(__cdecl *DPedge_createServiceEx)(const char *Name, const char *path, const char *arg, intptr_t *pid, EDGESERVICECONFIG *config);

	////
	//// RTCD mailbox (edge v2 feature) control
	////
	typedef int32_t		(__cdecl *DPedge_creMbox)(const char *pName, uint32_t dwLength, uint32_t dwNum, RTCDMEMINFO *pRtcdInfo);
	typedef int32_t		(__cdecl *DPedge_delMbox)(RTCDMEMINFO *pRtcdInfo);
	typedef int32_t		(__cdecl *DPedge_openMbox)(const char *pName, uint32_t dwAccess, RTCDHANDLE *pHandle);
	typedef int32_t		(__cdecl *DPedge_closeMbox)(RTCDHANDLE *pHandle);
	typedef int32_t		(__cdecl *DPedge_sendMbox)(RTCDHANDLE *pHandle, void *pBuffer, uint32_t dwSize);
	typedef int32_t		(__cdecl *DPedge_recvMbox)(RTCDHANDLE *pHandle, void *pBuffer, uint32_t dwBufferSize, uint32_t dwTimeout);
	typedef int32_t		(__cdecl *DPedge_infoMbox)(RTCDHANDLE *pHandle, EDGE_RTCD_HEADER *pInfo);
	typedef int32_t		(__cdecl *DPedge_sendMboxEx)(RTCDHANDLE *pHandle, void *pBuffer1, uint32_t dwSize1, void *pBuffer2, uint32_t dwSize2);
	typedef int32_t		(__cdecl *DPedge_recvMboxEx)(RTCDHANDLE *pHandle, void *pBuffer1, uint32_t dwBufferSize1, void *pBuffer2, uint32_t dwBufferSize2, uint32_t dwTimeout);

	////
	//// Thread control
	////
	typedef void*		(__cdecl *DPedge_createThread)(uint16_t shPriority, uint32_t dwStackSize, void* func, void* arg, int32_t* errorNo);
	typedef uint8_t		(__cdecl *DPedge_deleteThread)(void* hHandle);

	////
	//// Common Function
	////
	typedef void		(__cdecl *DPedge_sleep)(uint32_t dwMillseconds);				//sleep
	typedef void		(__cdecl *DPedge_getProcPath)(char *pPath, uint32_t dwSize);	//s̃vZXpX擾
	typedef int32_t		(__cdecl *DPedge_getTime)(EDGE_TIME_STAMP *pTime);			//ݓ̃^CX^v擾
	typedef int32_t		(__cdecl *DPedge_getTimeVal)(EDGE_TIME_VAL *pTime);			//ݓ̃^CX^v擾
	typedef int32_t		(__cdecl *DPedge_Localtime64)(void* pTime, void* pLongTime);	//G|bNb猻ݎɕϊ
	typedef int32_t		(__cdecl *DPedge_setPriority)(void* hdl, uint8_t byPriority);		//vCIeB̐ݒ Windowsp
	typedef int32_t		(__cdecl *DPedge_getCommmandLine)(char *pPath, uint32_t dwSize);	//R}hC擾
	typedef void		(__cdecl *DPedge_strncpy_s)(char *dst, size_t dstmax, const char *src, size_t n);
	typedef int32_t		(__cdecl *DPedge_checkHandle)(RTCDHANDLE *pHandle);
	typedef unsigned long long (__cdecl *DPedge_GetPCounter)();						//ptH[}XJE^[̎擾
	typedef double		(__cdecl *DPedge_CalcTime)(unsigned long long start_Count, unsigned long long finish_Count);//ptH[}XvZ nano
	////
	//// Function pointer
	////
	extern edge_API DPedge_createMemory			edge_createMemory;
	extern edge_API DPedge_freeMemory			edge_freeMemory;
	extern edge_API DPedge_map					edge_map;
	extern edge_API DPedge_unmap				edge_unmap;
	extern edge_API DPedge_closeHandle			edge_closeHandle;
	extern edge_API DPedge_mallocMemory			edge_mallocMemory;
	extern edge_API DPedge_freemallocMemory		edge_freemallocMemory;
	extern edge_API DPedge_catalog				edge_catalog;
	extern edge_API DPedge_uncatalog			edge_uncatalog;
	extern edge_API DPedge_lookup				edge_lookup;
	extern edge_API DPedge_setNode				edge_setNode;
	extern edge_API DPedge_getNodeName			edge_getNodeName;
	extern edge_API DPedge_createService		edge_createService;
	extern edge_API DPedge_createServiceEx		edge_createServiceEx;
	extern edge_API DPedge_creMbox				edge_creMbox;
	extern edge_API DPedge_delMbox				edge_delMbox;
	extern edge_API DPedge_openMbox				edge_openMbox;
	extern edge_API DPedge_closeMbox			edge_closeMbox;
	extern edge_API DPedge_sendMbox				edge_sendMbox;
	extern edge_API DPedge_recvMbox				edge_recvMbox;
	extern edge_API DPedge_infoMbox				edge_infoMbox;
	extern edge_API DPedge_createThread			edge_createThread;
	extern edge_API DPedge_deleteThread			edge_deleteThread;
	extern edge_API DPedge_sleep				edge_sleep;
	extern edge_API DPedge_getProcPath			edge_getProcPath;
	extern edge_API DPedge_getTime				edge_getTime;
	extern edge_API DPedge_getTimeVal			edge_getTimeVal;
	extern edge_API DPedge_Localtime64			edge_Localtime64;
	extern edge_API DPedge_setPriority			edge_setPriority;
	extern edge_API DPedge_getCommmandLine		edge_getCommmandLine;
	extern edge_API DPedge_strncpy_s			edge_strncpy_s;
	extern edge_API DPedge_checkHandle			edge_checkHandle;
	extern edge_API DPedge_sendMboxEx			edge_sendMboxEx;
	extern edge_API DPedge_recvMboxEx			edge_recvMboxEx;
	extern edge_API DPedge_GetPCounter			edge_GetPCounter;
	extern edge_API DPedge_CalcTime				edge_CalcTime;
	extern edge_API DPedge_lookup_memory		edge_lookup_memory;

#ifdef __cplusplus
}
#endif