////
/// Copyright(C) 2020 Micronet Co. All Rights Reserved.
//
/*****************************************************************************
* egosdep.c
*
*   RT-edge system / OS depend module for NTX application
*
\*****************************************************************************/
#include <stdio.h>
#include <time.h>
#include <ntx.h>
#include "../eghgapiCommon/egosdep.h"
#include "../eghgapiCommon/egapi.h"
#include "../eghgapiCommon/egInternal.h"

static NTXLOCATION		hRtcdLoc = NTX_UNDEFINED_LOCATION;	// RTCDnodẽP[Vnh

/*****************************************************************************
* FUNCTION:		_edge_setNode
* DESCRIPTION:
*   ()RTCD쐬m[ho^
\*****************************************************************************/
uint32_t _edge_setNode(NTXLOCATION hLoc)
{
	NTXHANDLE hRoot;
	NTXHANDLE	hMem = NTX_BAD_NTXHANDLE;

	if (hRtcdLoc != NTX_UNDEFINED_LOCATION)
	{
		return EDGE_RTCD_EXIST_ERROR;	// Ɏ擾ς̏ꍇ͖
	}

	if (hLoc != NTX_UNDEFINED_LOCATION) {
		hRoot = ntxGetRootRtProcess(hLoc);
		if (hRoot != NTX_BAD_NTXHANDLE) {
			// m
			hMem = ntxAllocateRtMemory(hLoc, 1);
			if (hMem == NTX_BAD_NTXHANDLE)
				goto failed;

			// J^O
			ntxUncatalogNtxHandle(hRoot, hMem, (LPSTR)EGRTCDNODE);
			ntxCatalogNtxHandle(hRoot, hMem, (LPSTR)EGRTCDNODE);

			hRtcdLoc = hLoc;
			return EDGE_SUCCESS;
		}
	}
	return EDGE_LOCATION_ERROR;
failed:
	if (hMem != NTX_NULL_NTXHANDLE)
		ntxFreeRtMemory(hMem);
	return EDGE_LOCATION_ERROR;
}

/*****************************************************************************
* FUNCTION:		searchRTCD
* DESCRIPTION:
*   ()RTCD̃J^O
\*****************************************************************************/
NTXHANDLE searchRTCD(NTXLOCATION hLoc)
{
	NTXHANDLE	hRoot = NTX_BAD_NTXHANDLE;
	NTXHANDLE	hMem = NTX_BAD_NTXHANDLE;		// Lnh
	WORD		wRet;

	hRoot = ntxGetRootRtProcess(hLoc);	// w肳ꂽP[Ṽ[gvZX擾
	if (hRoot == NTX_BAD_NTXHANDLE) {
		return NTX_BAD_NTXHANDLE;
	}

	hMem = ntxLookupNtxHandle(hRoot, (LPSTR)EGRTCDNODE, NO_WAIT);	// RTCDbNAbv
	if (hMem == NTX_BAD_NTXHANDLE) {
		return NTX_BAD_NTXHANDLE;
	}

	wRet = ntxGetType(hMem);
	if (wRet == NTX_TYPE_RT_SHARED_MEMORY) {
		return hRoot;
	}

	return NTX_BAD_NTXHANDLE;
}


/*****************************************************************************
* FUNCTION:		GetLocationRTCD
* DESCRIPTION:
*   ()RTCD݂m[h̃P[Vnh擾
\*****************************************************************************/
NTXLOCATION GetLocationRTCD()
{
	NTXLOCATION hLoc;
	NTXHANDLE hRoot;
	NTXHANDLE	hMem = NTX_BAD_NTXHANDLE;		// Lnh

	if (hRtcdLoc != NTX_UNDEFINED_LOCATION)
	{
		return hRtcdLoc;							// Ɏ擾ςȂI
	}

	// find all nodes
	hLoc = ntxGetFirstLocation();			// ŏ̃P[V擾
	while (hLoc != NTX_UNDEFINED_LOCATION) {
		// m[hNĂ邩`FbN
		NTXSTATUS sts = ntxGetRtStatus(hLoc);
		if (sts == E_OK)
		{
			hRoot = searchRTCD(hLoc);					// RTCDJ^O
			if (hRoot != NTX_BAD_NTXHANDLE) {			// ꍇAnhێ
				hRtcdLoc = hLoc;
				break;
			}
		}
		hLoc = ntxGetNextLocation(hLoc);		// ̃P[V擾
	};

	if (hRtcdLoc == NTX_UNDEFINED_LOCATION) {
		// Ȃꍇ́A[Jm[h擾
		hLoc = NTX_LOCAL;
		NTXSTATUS sts = ntxGetRtStatus(hLoc);
		if (sts == E_OK)
		{
			_edge_setNode(hLoc);	// RTCDm[hݒ
		}
	}
	return hRtcdLoc;
}

/*****************************************************************************
* FUNCTION:		edge_setNode
* DESCRIPTION:
*   ()RTCD쐬m[ho^
\*****************************************************************************/
uint32_t edge_setNode(const char *Name)
{
	NTXLOCATION hLoc;
	NTXHANDLE	hMem = NTX_BAD_NTXHANDLE;

	if (hRtcdLoc != NTX_UNDEFINED_LOCATION) {
		return EDGE_RTCD_EXIST_ERROR;					// Ɏ擾ς̏ꍇ͖
	}

	hLoc = ntxGetLocationByName((LPSTR)Name);	// P[Vnh擾
	if (hLoc != NTX_UNDEFINED_LOCATION) {
		return _edge_setNode(hLoc);						// RTCDm[hݒ
	}
	return EDGE_LOCATION_ERROR;
}

/*****************************************************************************
* FUNCTION:		edge_getNodeName
* DESCRIPTION:
*   RTCDɰނɰޖԂ
\*****************************************************************************/
uint32_t edge_getNodeName(char *pNode, uint32_t dwSize)
{
	NTXLOCATION loc = GetLocationRTCD();
	NTXNODEINFO info;

	if (pNode == NULL || dwSize == 0) return EDGE_PARAM_INVALID;
	if (loc == NTX_UNDEFINED_LOCATION) return EDGE_LOCATION_ERROR;
	if (ntxGetRtNodeInfo(loc, &info) != E_OK) return EDGE_LOCATION_ERROR;

	strncpy_s(pNode, dwSize, info.nodeName, _TRUNCATE);
	pNode[dwSize - 1] = '\0';

	return EDGE_SUCCESS;
}

//------------------------------------------------------------
// lookup
// @return	قԂ
//------------------------------------------------------------
void* edge_lookup(const char *Name)
{
	WORD		wRet;
	NTXSTATUS	stat = 0;
	NTXLOCATION	hLoc = NTX_UNDEFINED_LOCATION;
	NTXHANDLE	hRoot = NTX_BAD_NTXHANDLE;
	NTXHANDLE	hMem = NTX_BAD_NTXHANDLE;

	//--------------------------------------------------------------------
	// lookupO`FbN
	//--------------------------------------------------------------------
	hLoc = GetLocationRTCD();										// P[Vnh擾
	if (hLoc == NTX_UNDEFINED_LOCATION) {
		return NULL;												// P[Vnh擾G[
	}

	stat = ntxGetRtStatus(hLoc);							// J[lNmF
	if (stat != E_OK) {
		return NULL;												// J[lNĂ܂
	}

	hRoot = ntxGetRootRtProcess(hLoc);						// [gvZXnh擾
	if (hRoot == NTX_BAD_NTXHANDLE) {
		return NULL;												// [gvZX擾G[
	}
	hMem = ntxLookupNtxHandle(hRoot, (LPSTR)Name, NO_WAIT);
	if (hMem == NTX_BAD_NTXHANDLE) {
		return NULL;												// bNAbvs
	}
	else {
		wRet = ntxGetType(hMem);
		if (wRet == NTX_ERROR) {
			ntxUncatalogNtxHandle(hRoot, hMem, (LPSTR)Name);		// J^OcĂԂ̂߃AJ^OăG[Ԃ
			return NULL;											// RTCD݂܂
		}
		else if (wRet == NTX_TYPE_RT_SHARED_MEMORY) {
			// ̂ߏp
		}
		else {
			return NULL;											// ɓOgĂ܂
		}
	}
	return hMem;
}

//------------------------------------------------------------
// lookup for INtime node
// @return	قԂ
//------------------------------------------------------------
void* edge_lookup_memory(const char *NodeName, const char *ProcessName, const char *ObjectName, uint32_t *pSize)
{
	WORD		wRet;
	NTXSTATUS	stat  = 0;
	NTXLOCATION	hLoc  = NTX_UNDEFINED_LOCATION;
	NTXHANDLE	hRoot = NTX_BAD_NTXHANDLE;
	NTXHANDLE	hProc = NTX_BAD_NTXHANDLE;
	NTXHANDLE	hOBJ  = NTX_BAD_NTXHANDLE;

	
	if ((NULL != NodeName) && ('\0' != *NodeName)) {		// NodeNamew肪ꍇ
		hLoc = ntxGetLocationByName(NodeName);				// Ώۃm[h̃P[Vnh擾
	} 
	else {													// ꍇ
		hLoc = GetLocationRTCD();							// RTCDm[h̃P[Vnh擾
	}

	if (hLoc == NTX_UNDEFINED_LOCATION) {
		return NULL;										// P[Vnh擾G[
	}

	stat = ntxGetRtStatus(hLoc);							// J[lNmF
	if (stat != E_OK) {
		return NULL;										// J[lNĂ܂
	}

	hRoot = ntxGetRootRtProcess(hLoc);						// [gvZXnh擾
	if (hRoot == NTX_BAD_NTXHANDLE) {
		return NULL;										// [gvZX擾G[
	}
	
	if ((NULL != ProcessName) && ('\0' != *ProcessName)) {	// ProcessNamew肪ꍇ
		hProc = ntxLookupNtxHandle(hRoot, (LPSTR)ProcessName, NO_WAIT);		// ΏۃvZX̌
	}
	else {													// ꍇ
		hProc = hRoot;										// 郋[gvZXZbg
	}

	if (hProc == NTX_BAD_NTXHANDLE) {
		return NULL;										// bNAbvs
	}
	
	hOBJ = ntxLookupNtxHandle(hProc, (LPSTR)ObjectName, NO_WAIT);	// ΏۃIuWFNǧ
	if (hOBJ == NTX_BAD_NTXHANDLE) {
		return NULL;												// bNAbvs
	}
		
	wRet = ntxGetType(hOBJ);										// IuWFNgĂ邩 ?
	if (wRet == NTX_ERROR) {
		ntxUncatalogNtxHandle(hProc, hOBJ, (LPSTR)ObjectName);		// J^OcĂԂ̂߃AJ^OăG[Ԃ
		return NULL;												// ΏۃIuWFNg݂͑܂
	}
	
	if (NULL != pSize) {
		*pSize = ntxGetRtSize(hOBJ);								// TCY̎擾
	}
	return hOBJ;
}

//------------------------------------------------------------
// createMemory
// @return	ϯߍςرڽԂ
//------------------------------------------------------------
void* edge_createMemory(const char *Name, uint32_t dwSize, void **MemHandle)
{
	NTXLOCATION	hLoc = NTX_UNDEFINED_LOCATION;
	NTXHANDLE   hMem = NTX_NULL_NTXHANDLE;

	hLoc = GetLocationRTCD();							// P[Vnh擾
	if (hLoc == NTX_UNDEFINED_LOCATION) {
		return NULL;									// P[Vnh擾G[
	}

	hMem = ntxAllocateRtMemory(hLoc, dwSize);
	if (hMem == NTX_BAD_NTXHANDLE) {
		return NULL;
	}

	if ((NULL != Name) && (0 != strlen(Name))) {			//Name`ꍇɌJ^O
		edge_catalog(hMem, Name);							// edgepcatalog֐Ăяo
	}

	if (MemHandle != NULL) *MemHandle = hMem;			//ُo͐悪ΕԂ(ꂪȂfreełȂ)
	return edge_map(hMem);
}

//------------------------------------------------------------
// freeMemory
//------------------------------------------------------------
uint32_t edge_freeMemory(const char *Name, void *Memory, void *MemHandle)
{
	WORD stat = 0;

	if ((NULL != Name) && (0 != strlen(Name))) {			// Name`ꍇɌAJ^O
		stat = edge_uncatalog(MemHandle, Name);				// J^O폜
		if(stat != EDGE_SUCCESS) {
			return stat;
		}
	}

	stat = ntxFreeRtMemory((NTXHANDLE)MemHandle);			//폜
	if (stat != E_OK) {
		return	EDGE_RTCD_DELETE_MEM_HANDLE_ERROR;			// nh폜G[
	}
	return EDGE_SUCCESS;
}
//------------------------------------------------------------
// map
//------------------------------------------------------------
void* edge_map(void *Memhandle)
{
	return ntxMapRtSharedMemoryEx(Memhandle, NTX_MAP_WRITE | NTX_MAP_UNALIGNED);
}
//------------------------------------------------------------
// unmap
//------------------------------------------------------------
uint32_t  edge_unmap(void *MemPointer, void *MemHandle)
{
	//---------------------------------------------------------------------------------
	// NTXł͂ŃA}bv(ntxUnmapRtSharedMemory)sȂdlƂB
	// [R]
	// NTX̏ꍇ͓L𕡐}bvĂSēAhXƂȂB(NTX̎dl)
	// Ⴆ΁A鋤L𕡐XbhɂOPENREAD/WRITEĂŒɁA
	// XbhACLOSEsƁALɃ}bsOꂽ邽߁A
	// XbhREAD/WRITEsƃANZXᔽOĂ܂B
	// ̂߁A}bsOꂽ̓vZXI܂ŕێĂƂɂB
	//---------------------------------------------------------------------------------

	return EDGE_SUCCESS;
}
//------------------------------------------------------------
// catalog
//------------------------------------------------------------
uint32_t edge_catalog(void *MemHandle, const char *Name)
{
	NTXSTATUS	stat = 0;
	NTXHANDLE	hRoot = NTX_BAD_NTXHANDLE;			// [gvZXnh
	NTXLOCATION	hLoc = NTX_UNDEFINED_LOCATION;

	//--------------------------------------------------------------------
	// O`FbN
	//--------------------------------------------------------------------
	hLoc = GetLocationRTCD();										// P[Vnh擾
	if (hLoc == NTX_UNDEFINED_LOCATION) {
		return EDGE_LOCATION_ERROR;									// P[Vnh擾G[
	}

	stat = ntxGetRtStatus(hLoc);							// J[lNmF
	if (stat != E_OK) {
		return EDGE_KERNEL_NOT_RUNNING;								// J[lNĂ܂
	}

	hRoot = ntxGetRootRtProcess(hLoc);						// [gvZXnh擾
	if (hRoot == NTX_BAD_NTXHANDLE) {
		return EDGE_ROOTPROCESS_ERROR;								// [gvZX擾G[
	}
	ntxUncatalogNtxHandle(hRoot, MemHandle, (LPSTR)Name);
	return ntxCatalogNtxHandle(hRoot, MemHandle, (LPSTR)Name);
}
//------------------------------------------------------------
// uncatalog
//------------------------------------------------------------
uint32_t edge_uncatalog(void *Handle, const char *Name)
{
	NTXSTATUS	stat = 0;
	NTXHANDLE	hRoot = NTX_BAD_NTXHANDLE;							// [gvZXnh
	NTXLOCATION	hLoc = NTX_UNDEFINED_LOCATION;

	if ((NULL == Name) || (0 == strlen(Name))) {					// Name`ꍇ͏I
		return EDGE_SUCCESS;
	}

	//--------------------------------------------------------------------
	// O`FbN
	//--------------------------------------------------------------------
	hLoc = GetLocationRTCD();										// P[Vnh擾
	if (hLoc == NTX_UNDEFINED_LOCATION) {
		return EDGE_LOCATION_ERROR;									// P[Vnh擾G[
	}

	stat = ntxGetRtStatus(hLoc);									// J[lNmF
	if (stat != E_OK) {
		return EDGE_KERNEL_NOT_RUNNING;								// J[lNĂ܂
	}

	hRoot = ntxGetRootRtProcess(hLoc);								// [gvZXnh擾
	if (hRoot == NTX_BAD_NTXHANDLE) {
		return EDGE_ROOTPROCESS_ERROR;								// [gvZX擾G[
	}

	stat = ntxUncatalogNtxHandle(hRoot, Handle, (LPSTR)Name);
	if (stat != E_OK) {
		return EDGE_RTCD_UNCATALOG_ERROR;							// AJ^OG[
	}

	return EDGE_SUCCESS;
}
//------------------------------------------------------------
// closeHandle(NTXł͕sv)
//------------------------------------------------------------
uint32_t edge_closeHandle(void *Handle)
{
	return EDGE_SUCCESS;
}
//------------------------------------------------------------
// mallocMemory(ʏ̃쐬)
//------------------------------------------------------------
void* edge_mallocMemory(size_t dwSize)
{
	return malloc(dwSize);
}
//------------------------------------------------------------
// freemallocMemory
//------------------------------------------------------------
uint32_t edge_freemallocMemory(void *Memory)
{
	free(Memory);
	return 1; //Ƃ肠(Windows͖߂l)
}
//------------------------------------------------------------
// createThread
//------------------------------------------------------------
void* edge_createThread(uint16_t shPriority, uint32_t dwStackSize, void* func, void* arg, int32_t* errorNo)
{
	HANDLE hdl = CreateThread(NULL, dwStackSize, (LPTHREAD_START_ROUTINE)func, arg, 0, NULL); //Xbh쐬
	if (hdl)																				  //ꍇ̓vCIeBݒ
	{
		*errorNo = edge_setPriority(hdl, (uint8_t)shPriority);	//priority̐ݒ
	}
	return hdl;
}
//------------------------------------------------------------
// deleteThread
//------------------------------------------------------------
uint8_t edge_deleteThread(void* hHandle)
{
	return CloseHandle((HANDLE)hHandle);
}
//------------------------------------------------------------
// sleep
//------------------------------------------------------------
void edge_sleep(uint32_t dwMillseconds)
{
	Sleep(dwMillseconds);
}
//------------------------------------------------------------
// getProcPath s̃vZXpX擾
//------------------------------------------------------------
void edge_getProcPath(char *pPath, uint32_t dwSize)
{
	uint32_t dwRet;
	//
	memset(pPath, 0x00, dwSize);

	//s̃vZX̃tpX擾
	dwRet = GetModuleFileName(NULL, pPath, dwSize);
	if (dwRet == 0)
	{
		//0ɐݒ
		memset(pPath, 0x00, dwSize);
	}
}
//------------------------------------------------------------
// Ԃ̎擾
//------------------------------------------------------------
int32_t edge_getTime(EDGE_TIME_STAMP *pTime)
{
	return edgeGetTimestamp(pTime);
}
//------------------------------------------------------------
// Ԃ̎擾
//------------------------------------------------------------
int32_t edge_getTimeVal(EDGE_TIME_VAL *pTime)
{
	return edgeGetTimeVal(pTime);
}
//------------------------------------------------------------
// Ԃ̕ϊ
//------------------------------------------------------------
int32_t edge_Localtime64(void* pTime, void* pLongTime)
{
	return _localtime64_s((struct tm*)pTime, (__time64_t const*)pLongTime);
}
//------------------------------------------------------------
// RTCDx[X[{bNX̐
//------------------------------------------------------------
int32_t edge_creMbox(const char *pName, uint32_t dwLength, uint32_t dwNum, RTCDMEMINFO *pRtcdInfo)
{
	return edgeCreateMailbox(pName, dwLength, dwNum, pRtcdInfo);
}
//------------------------------------------------------------
// RTCDx[X[{bNX̍폜
//------------------------------------------------------------
int32_t edge_delMbox(RTCDMEMINFO *pRtcdInfo)
{
	return edgeDeleteMailbox(pRtcdInfo);
}
//------------------------------------------------------------
// RTCDx[X[{bNX̃I[v
//------------------------------------------------------------
int32_t edge_openMbox(const char *pName, uint32_t dwAccess, RTCDHANDLE *pHandle)
{
	return edgeOpenMailbox(pName, dwAccess, pHandle);
}
//------------------------------------------------------------
// RTCDx[X[{bNX̃N[Y
//------------------------------------------------------------
int32_t edge_closeMbox(RTCDHANDLE *pHandle)
{
	return edgeCloseMailbox(pHandle);
}
//------------------------------------------------------------
// RTCDx[X[{bNXĂ̑M
//------------------------------------------------------------
int32_t edge_sendMbox(RTCDHANDLE *pHandle, void *pBuffer, uint32_t dwSize)
{
	return edgeSendData(pHandle, pBuffer, dwSize);
}
//------------------------------------------------------------
// RTCDx[X[{bNXĂ̑M
//------------------------------------------------------------
int32_t edge_sendMboxEx(RTCDHANDLE *pHandle, void *pBuffer1, uint32_t dwSize1, void *pBuffer2, uint32_t dwSize2)
{
	return edgeSendData4Mbox(pHandle, pBuffer1, dwSize1, pBuffer2, dwSize2);
}
//------------------------------------------------------------
// RTCDx[X[{bNX̎M
//------------------------------------------------------------
int32_t edge_recvMbox(RTCDHANDLE *pHandle, void *pBuffer, uint32_t dwBufferSize, uint32_t dwTimeout)
{
	return edgeReceiveData(pHandle, pBuffer, dwBufferSize, dwTimeout);
}
//------------------------------------------------------------
// RTCDx[X[{bNX̎M
//------------------------------------------------------------
int32_t edge_recvMboxEx(RTCDHANDLE *pHandle, void *pBuffer1, uint32_t dwBufferSize1, void *pBuffer2, uint32_t dwBufferSize2, uint32_t dwTimeout)
{
	return edgeReceiveData4Mbox(pHandle, pBuffer1, dwBufferSize1, pBuffer2, dwBufferSize2, dwTimeout);
}
//------------------------------------------------------------
// RTCDx[X[{bNX̍\擾
//------------------------------------------------------------
int32_t edge_infoMbox(RTCDHANDLE *pHandle, EDGE_RTCD_HEADER *pInfo)
{
	return edgeGetRTCDInfo(pHandle, pInfo);
}
//------------------------------------------------------------
// edgeT[rXN
//------------------------------------------------------------
uint32_t edge_createService(const char *Name, const char *path, const char *arg, intptr_t *pid)
{
	//SHELLEXECUTEINFO	sei = { 0 };
	char	*pFind;

	if (strlen(path) == 0) //`FbN
		return EDGE_PARAM_INVALID;

	pFind = strrchr(path, (int)'.');		// 납"."
	if (pFind != NULL)
	{
		if (_stricmp(pFind, ".rta") == 0)	//INtimepN
		{
			NTXPROCATTRIBS	str = { 0 };
			NTXHANDLE ret;
			str.dwObjDirSize = 3840;
			str.dwPoolMin = 0;
			str.dwPoolMax = 0xFFFFFFFF;
			str.dwVsegSize = 64 * 1024 * 1024;

			ret = ntxCreateRtProcess(NTX_LOCAL, (LPSTR)path, (LPSTR)arg, &str, 0);		// CreateRtProcessrtaN
			if (ret == NTX_BAD_NTXHANDLE)
			{
				uint32_t err = ntxGetLastRtError();	//G[Xe[^X擾
				*pid = (intptr_t)ret;
				return err;// (̂܂ܖ߂l)
			}
			*pid = (intptr_t)ret;
			return EDGE_SUCCESS;
		}
	}
	//else ".exe"

	char *newArgs = NULL;
	size_t size = strlen(path) + strlen(arg) + 10;
	newArgs = malloc(size);
	memset(newArgs, 0, size);
	// R}hC̐擪Ɏst@CpXt^
	sprintf_s(newArgs, size, "\"%s\" %s", path, arg);

	HRESULT hResult = E_FAIL;
	STARTUPINFO         tStartupInfo = { 0 };
	PROCESS_INFORMATION tProcessInfomation = { 0 };
	SECURITY_ATTRIBUTES thread = { 0 };
	SECURITY_ATTRIBUTES process = { 0 };
	thread.nLength = sizeof(SECURITY_ATTRIBUTES);
	process.nLength = sizeof(SECURITY_ATTRIBUTES);
	BOOL bResult = CreateProcess(
		path
		, (char*)newArgs
		, &process
		, &thread
		, FALSE
		, 0
		, NULL
		, NULL
		, &tStartupInfo
		, &tProcessInfomation
	);

	free(newArgs);

	if (0 == bResult) {
		return GetLastError();
	}
	*pid = (intptr_t)tProcessInfomation.dwProcessId;		// PID̎擾

	return EDGE_SUCCESS;
}

//------------------------------------------------------------
// edgeT[rXwm[hŋN
//------------------------------------------------------------
uint32_t edge_createServiceEx(const char *Name, const char *path, const char *arg, intptr_t *pid, EDGESERVICECONFIG *config)
{
	//SHELLEXECUTEINFO	sei = { 0 };
	char	*pFind;

	if (strlen(path) == 0) //`FbN
		return EDGE_PARAM_INVALID;

	pFind = strrchr(path, (int)'.');		// 납"."
	if (pFind != NULL)
	{
		if (_stricmp(pFind, ".rta") == 0)	//INtimepN
		{
			NTXPROCATTRIBS	str = { 0 };
			NTXHANDLE ret;
			NTXLOCATION loc;
			str.dwObjDirSize = 3840;
			str.dwPoolMin = 0;
			str.dwPoolMax = 0xFFFFFFFF;
			str.dwVsegSize = 64 * 1024 * 1024;

			if ((config != NULL) && (strlen(config->NodeName) != 0)) {	// m[hw肠̏ꍇ
				loc = ntxGetLocationByName((LPSTR)(config->NodeName));	// P[Vnh擾
				if (NTX_UNDEFINED_LOCATION == loc) {
					return EDGE_PARAM_INVALID;							// p[^s
				}
				// wm[hrtaN
				ret = ntxCreateRtProcess(loc, (LPSTR)path, (LPSTR)arg, &str, 0);
			}
			else {														// m[hwȂ̏ꍇ
				// [Jm[hrtaN
				ret = ntxCreateRtProcess(NTX_LOCAL, (LPSTR)path, (LPSTR)arg, &str, 0);
			}
			if (ret == NTX_BAD_NTXHANDLE)
			{
				uint32_t err = ntxGetLastRtError();	//G[Xe[^X擾
				*pid = (intptr_t)ret;
				return err;// (̂܂ܖ߂l)
			}
			*pid = (intptr_t)ret;
			return EDGE_SUCCESS;
		}
	}
	//else ".exe"

	char *newArgs = NULL;
	size_t size = strlen(path) + strlen(arg) + 10;
	newArgs = malloc(size);
	memset(newArgs, 0, size);
	// R}hC̐擪Ɏst@CpXt^
	sprintf_s(newArgs, size, "\"%s\" %s", path, arg);

	HRESULT hResult = E_FAIL;
	STARTUPINFO         tStartupInfo = { 0 };
	PROCESS_INFORMATION tProcessInfomation = { 0 };
	SECURITY_ATTRIBUTES thread = { 0 };
	SECURITY_ATTRIBUTES process = { 0 };
	thread.nLength = sizeof(SECURITY_ATTRIBUTES);
	process.nLength = sizeof(SECURITY_ATTRIBUTES);
	BOOL bResult = CreateProcess(
		path
		, (char*)newArgs
		, &process
		, &thread
		, FALSE
		, 0
		, NULL
		, NULL
		, &tStartupInfo
		, &tProcessInfomation
	);

	free(newArgs);

	if (0 == bResult) {
		return GetLastError();
	}
	*pid = (intptr_t)tProcessInfomation.dwProcessId;		// PID̎擾

	return EDGE_SUCCESS;
}

//------------------------------------------------------------
// Thread Priority̐ݒ Windowsp
//------------------------------------------------------------
int32_t edge_setPriority(void* hdl, uint8_t byPriority)
{
	uint32_t dwStatus = 0;
	uint32_t dwClPriority = NORMAL_PRIORITY_CLASS;
	int32_t inThPriority = THREAD_PRIORITY_NORMAL;

	if (byPriority < 8)		// 31 0`7 EG_PRIORITY_0_7
	{
		dwClPriority = REALTIME_PRIORITY_CLASS;
		inThPriority = THREAD_PRIORITY_TIME_CRITICAL;
	}
	else if (byPriority < 48) // 26 8`47 EG_PRIORITY_8_47
	{
		dwClPriority = REALTIME_PRIORITY_CLASS;
		inThPriority = THREAD_PRIORITY_HIGHEST;
	}
	else if (byPriority < 56) // 25 48`55 EG_PRIORITY_48_55
	{
		dwClPriority = REALTIME_PRIORITY_CLASS;
		inThPriority = THREAD_PRIORITY_ABOVE_NORMAL;
	}
	else if (byPriority < 64) // 24 56`63 EG_PRIORITY_56_63
	{
		dwClPriority = REALTIME_PRIORITY_CLASS;
		inThPriority = THREAD_PRIORITY_NORMAL;
	}
	else if (byPriority < 72) // 23 64`71 EG_PRIORITY_64_71
	{
		dwClPriority = REALTIME_PRIORITY_CLASS;
		inThPriority = THREAD_PRIORITY_BELOW_NORMAL;
	}
	else if (byPriority < 80) // 22 72`79 EG_PRIORITY_72_79
	{
		dwClPriority = REALTIME_PRIORITY_CLASS;
		inThPriority = THREAD_PRIORITY_LOWEST;
	}
	else if (byPriority < 128) // 16 80`127 EG_PRIORITY_80_127 
	{
		dwClPriority = REALTIME_PRIORITY_CLASS;
		inThPriority = THREAD_PRIORITY_IDLE;
	}
	else if (byPriority < 136) // 15 128`135 EG_PRIORITY_128_135
	{
		dwClPriority = HIGH_PRIORITY_CLASS;
		inThPriority = THREAD_PRIORITY_HIGHEST;
	}
	else if (byPriority < 144) // 14 136`143 EG_PRIORITY_136_143
	{
		dwClPriority = HIGH_PRIORITY_CLASS;
		inThPriority = THREAD_PRIORITY_ABOVE_NORMAL;
	}
	else if (byPriority < 152) // 13 144`151 EG_PRIORITY_144_151
	{
		dwClPriority = HIGH_PRIORITY_CLASS;
		inThPriority = THREAD_PRIORITY_NORMAL;
	}
	else if (byPriority < 160) // 12 152`159 EG_PRIORITY_152_159
	{
		dwClPriority = HIGH_PRIORITY_CLASS;
		inThPriority = THREAD_PRIORITY_BELOW_NORMAL;
	}
	else if (byPriority < 168) // 11 160`167 EG_PRIORITY_160_167
	{
		dwClPriority = HIGH_PRIORITY_CLASS;
		inThPriority = THREAD_PRIORITY_LOWEST;
	}
	else if (byPriority < 176) // 10 168`175 EG_PRIORITY_168_175
	{
		dwClPriority = NORMAL_PRIORITY_CLASS;
		inThPriority = THREAD_PRIORITY_NORMAL;
	}
	else if (byPriority < 184) // 9 176`183 EG_PRIORITY_176_183
	{
		dwClPriority = NORMAL_PRIORITY_CLASS;
		inThPriority = THREAD_PRIORITY_ABOVE_NORMAL;
	}
	else if (byPriority < 192) // 8 184`191 EG_PRIORITY_184_191
	{
		dwClPriority = NORMAL_PRIORITY_CLASS;
		inThPriority = THREAD_PRIORITY_NORMAL;
	}
	else if (byPriority < 200) // 7 192`199 EG_PRIORITY_192_199
	{
		dwClPriority = NORMAL_PRIORITY_CLASS;
		inThPriority = THREAD_PRIORITY_BELOW_NORMAL;
	}
	else if (byPriority < 208) // 6 200`207 EG_PRIORITY_200_207
	{
		dwClPriority = NORMAL_PRIORITY_CLASS;
		inThPriority = THREAD_PRIORITY_LOWEST;
	}
	else if (byPriority < 216) // 5 208`215 EG_PRIORITY_208_215
	{
		dwClPriority = IDLE_PRIORITY_CLASS;
		inThPriority = THREAD_PRIORITY_HIGHEST;
	}
	else if (byPriority < 224) // 4 216`223 EG_PRIORITY_216_223
	{
		dwClPriority = IDLE_PRIORITY_CLASS;
		inThPriority = THREAD_PRIORITY_ABOVE_NORMAL;
	}
	else if (byPriority < 232) // 3 224`231 EG_PRIORITY_224_231
	{
		dwClPriority = IDLE_PRIORITY_CLASS;
		inThPriority = THREAD_PRIORITY_NORMAL;
	}
	else if (byPriority < 240) // 2 232`239 EG_PRIORITY_232_239
	{
		dwClPriority = IDLE_PRIORITY_CLASS;
		inThPriority = THREAD_PRIORITY_BELOW_NORMAL;
	}
	else if (byPriority < 256) // 1 240`255 EG_PRIORITY_240_255
	{
		dwClPriority = IDLE_PRIORITY_CLASS;
		inThPriority = THREAD_PRIORITY_LOWEST;
	}

	if (TRUE == SetPriorityClass(GetCurrentProcess(), dwClPriority))	//sGetLastErrorԂ
	{
		if (FALSE == SetThreadPriority((HANDLE)hdl, inThPriority))
			dwStatus = GetLastError();
	}
	else
	{
		dwStatus = GetLastError();
	}
	return dwStatus;
}
//------------------------------------------------------------
// vO̎擾
//------------------------------------------------------------
int32_t edge_getCommmandLine(char *pArg, uint32_t dwSize)
{
	char* ptr = NULL;
	ptr = GetCommandLine();	//R}hC擾
	if (strlen(ptr) > 0 && strlen(ptr) < dwSize)	//A֐̃TCY菬ꍇRs[
		strcpy_s(pArg, dwSize, ptr); 
	else
		pArg[0] = '\0'; 
	
	return EDGE_SUCCESS;
}
/// <summary>
/// tRs[(t^\0nɊ܂܂Ȃ) _TRUNCATE(-1)dstmax-1
/// </summary>
/// <param name="dst"></param>
/// <param name="dstmax">dstobt@̃TCY</param>
/// <param name="src"></param>
/// <param name="n">Rs[TCY</param>
void edge_strncpy_s(char *dst, size_t dstmax, const char *src, size_t n)
{
	strncpy_s(dst, dstmax, src, n);
}
//------------------------------------------------------------
// RTCDx[X[{bNX̃nhL`FbN
//------------------------------------------------------------
int32_t edge_checkHandle(RTCDHANDLE *pHandle)
{
	return edgeHandleCheck(pHandle);
}
//------------------------------------------------------------
// @\  FptH[}XJE^̎g̎擾( Hz )
//   F
// ߂lFJE^g(Hz)
//------------------------------------------------------------
unsigned long long edge_GetPCounter(void)
{
	LARGE_INTEGER counter;
	QueryPerformanceCounter(&counter);		// _asm rdtscp Ƃ͈قȂ錋
	return counter.QuadPart;
}
/*==========================================
 * @\  FptH[}XJE^̎g̎擾( Hz )
 *   F
 * ߂lFJE^g(Hz)
/==========================================*/
unsigned long long GetPerformanceCFreq(void)
{
	LARGE_INTEGER freq;
	QueryPerformanceFrequency(&freq);
	return freq.QuadPart;
}
//------------------------------------------------------------
// @\  FԂ̎Zo( nano second )
//   Fstart_Count  Jn̶
//		   finish_Count I̶
// ߂lFx(nano second)
//------------------------------------------------------------
double edge_CalcTime(unsigned long long start_Count, unsigned long long finish_Count)
{
	double  dblSpeed;														// (1ۯɂ鎞)
	unsigned long long freq = GetPerformanceCFreq();							// JE^g̎擾 [Hz]
	double calcedTime = 0;
	if (freq != 0) {
		dblSpeed = (double)1000000000 / (double)GetPerformanceCFreq();			// (1ۯɂ鎞)̎擾 [nsec]
		calcedTime = ((double)(finish_Count - start_Count) * dblSpeed);		// w̸ۯɂ鎞Ԃ̎擾[nsec]
	}
	else {
		calcedTime = 0;
	}

	return calcedTime;
}