////
/// Copyright(C) 2020 Micronet Co. All Rights Reserved.
//
/*****************************************************************************
* egosdep.c
*
*   RT-edge system / OS depend module for pure Windows application
*
\*****************************************************************************/

#include <stdio.h>
#include <time.h>
#include <Windows.h>
#include "../eghgapiCommon/egosdep.h"
#include "../eghgapiCommon/egapi.h"


//------------------------------------------------------------
// setNode
//------------------------------------------------------------
uint32_t edge_setNode(const char *Name)
{
	return EDGE_SUCCESS;
}
//------------------------------------------------------------
// getNodeName
//------------------------------------------------------------
uint32_t edge_getNodeName(char *pNode, uint32_t dwSize)
{
	if (pNode == NULL || dwSize == 0) return EDGE_PARAM_INVALID;

	/* K(ǂgȂ) */
	strncpy_s(pNode, dwSize, ".", _TRUNCATE);
	pNode[dwSize - 1] = '\0';

	return EDGE_SUCCESS;
}
//------------------------------------------------------------
// lookup
// @return	قԂ
//------------------------------------------------------------
void* edge_lookup(const char *Name)
{
	return OpenFileMapping(FILE_MAP_ALL_ACCESS, FALSE, Name);
}
//------------------------------------------------------------
// lookup (Windows has no concept of nodes and process directories)
// @return	قԂ
//------------------------------------------------------------
void* edge_lookup_memory(const char *NodeName, const char *ProcessName, const char *ObjectName, uint32_t *pSize)
{
	void* ret = edge_lookup(ObjectName);
	/* @warning Win32 APIł̧ϯݸق̈滲ނ擾̎i݂Ȃ(Ȃ???????) */
	/*          ̂߁ApSizeɈˑ麰ނWindowsłł͓삵ȂB */
	return ret;
}
//------------------------------------------------------------
// createMemory
// @return	ϯߍςرڽԂ
//------------------------------------------------------------
void* edge_createMemory(const char *Name, uint32_t dwSize, void **MemHandle)
{
	HANDLE hMem = CreateFileMapping(INVALID_HANDLE_VALUE, NULL, PAGE_READWRITE, 0, dwSize, Name);
	if (hMem == NULL) return NULL;
	if (MemHandle != NULL) *MemHandle = hMem; /* قKvƂĂȂԂ */
	return edge_map(hMem);
}
//------------------------------------------------------------
// freeMemory
//------------------------------------------------------------
uint32_t edge_freeMemory(const char *Name, void *Memory, void *MemHandle)
{
	if (Memory != NULL)
	{
		if (UnmapViewOfFile(Memory) == FALSE)
			return EDGE_RTCD_UNMAPPING_ERROR;
	}
	if (edge_closeHandle(MemHandle) == FALSE)
		return EDGE_RTCD_DELETE_MEM_HANDLE_ERROR;

	return EDGE_SUCCESS;
}
//------------------------------------------------------------
// map
//------------------------------------------------------------
void* edge_map(void *Memhandle)
{
	return MapViewOfFile(Memhandle, FILE_MAP_ALL_ACCESS, 0, 0, 0);
}
//------------------------------------------------------------
// unmap
//------------------------------------------------------------
uint32_t  edge_unmap(void *MemPointer, void *MemHandle)
{
	if (UnmapViewOfFile((BYTE*)MemPointer) == FALSE)
		return EDGE_RTCD_UNMAPPING_ERROR;

	if (edge_closeHandle(MemHandle) == FALSE)
		return EDGE_RTCD_DELETE_MEM_HANDLE_ERROR;

	return EDGE_SUCCESS;
}
//------------------------------------------------------------
// catalog
//------------------------------------------------------------
uint32_t edge_catalog(void *MemHandle, const char *Name)
{
	return EDGE_SUCCESS; //WindowsVXeł̓IuWFNgɖ̂܂
}
//------------------------------------------------------------
// uncatalog
//------------------------------------------------------------
uint32_t edge_uncatalog(void *Handle, const char *Name)
{
	return EDGE_SUCCESS; //WindowsVXeł̓IuWFNgɖ̂܂
}
//------------------------------------------------------------
// closeHandle
//------------------------------------------------------------
uint32_t edge_closeHandle(void *Handle)
{
	if (CloseHandle((HANDLE)Handle) == FALSE)
		return EDGE_RTCD_DELETE_MEM_HANDLE_ERROR;
	return EDGE_SUCCESS;
}
//------------------------------------------------------------
// mallocMemory
//------------------------------------------------------------
void* edge_mallocMemory(size_t dwSize)
{
	return malloc(dwSize);
}
//------------------------------------------------------------
// freemallocMemory
//------------------------------------------------------------
uint32_t edge_freemallocMemory(void *Memory)
{
	free(Memory);
	return 1; //Ƃ肠(Windows͖߂l)
}
//------------------------------------------------------------
// createThread
//------------------------------------------------------------
void* edge_createThread(uint16_t shPriority, uint32_t dwStackSize, void* func, void* arg, int32_t* errorNo)
{
	HANDLE hdl;
	hdl = CreateThread(NULL, dwStackSize, (LPTHREAD_START_ROUTINE)func, arg, 0, NULL);
	if (hdl)																				  //ꍇ̓vCIeBݒ
	{
		*errorNo = edge_setPriority(hdl, (uint8_t)shPriority);	//priority̐ݒ
	}
	return hdl;
}
//------------------------------------------------------------
// deleteThread
//------------------------------------------------------------
uint8_t edge_deleteThread(void* hHandle)
{
	return CloseHandle((HANDLE)hHandle);
}
//------------------------------------------------------------
// sleep
//------------------------------------------------------------
void edge_sleep(uint32_t dwMillseconds)
{
	Sleep(dwMillseconds);
}
//------------------------------------------------------------
// getProcPath s̃vZXpX擾
//------------------------------------------------------------
void edge_getProcPath(char *pPath, uint32_t dwSize)
{
	uint32_t dwRet;
	//
	memset(pPath, 0x00, dwSize);

	//s̃vZX̃tpX擾
	dwRet = GetModuleFileName(NULL, pPath, dwSize);
	if (dwRet == 0)
	{
		//0ɐݒ
		memset(pPath, 0x00, dwSize);
	}
}
//------------------------------------------------------------
// Ԃ̎擾
//------------------------------------------------------------
int32_t edge_getTime(EDGE_TIME_STAMP *pTime)
{
	return edgeGetTimestamp(pTime);
}
//------------------------------------------------------------
// Ԃ̎擾
//------------------------------------------------------------
int32_t edge_getTimeVal(EDGE_TIME_VAL *pTime)
{
	return edgeGetTimeVal(pTime);
}
//------------------------------------------------------------
// Ԃ̕ϊ
//------------------------------------------------------------
int32_t edge_Localtime64(void* pTime, void* pLongTime)
{
	return _localtime64_s((struct tm*)pTime, (__time64_t const*)pLongTime);
}
//------------------------------------------------------------
// RTCDx[X[{bNX̐
//------------------------------------------------------------
int32_t edge_creMbox(const char *pName, uint32_t dwLength, uint32_t dwNum, RTCDMEMINFO *pRtcdInfo)
{
	return edgeCreateMailbox(pName, dwLength, dwNum, pRtcdInfo);
}
//------------------------------------------------------------
// RTCDx[X[{bNX̍폜
//------------------------------------------------------------
int32_t edge_delMbox(RTCDMEMINFO *pRtcdInfo)
{
	return edgeDeleteMailbox(pRtcdInfo);
}
//------------------------------------------------------------
// RTCDx[X[{bNX̃I[v
//------------------------------------------------------------
int32_t edge_openMbox(const char *pName, uint32_t dwAccess, RTCDHANDLE *pHandle)
{
	return edgeOpenMailbox(pName, dwAccess, pHandle);
}
//------------------------------------------------------------
// RTCDx[X[{bNX̃N[Y
//------------------------------------------------------------
int32_t edge_closeMbox(RTCDHANDLE *pHandle)
{
	return edgeCloseMailbox(pHandle);
}
//------------------------------------------------------------
// RTCDx[X[{bNXĂ̑M
//------------------------------------------------------------
int32_t edge_sendMbox(RTCDHANDLE *pHandle, void *pBuffer, uint32_t dwSize)
{
	return edgeSendData(pHandle, pBuffer, dwSize);
}
//------------------------------------------------------------
// RTCDx[X[{bNXĂ̑M
//------------------------------------------------------------
int32_t edge_sendMboxEx(RTCDHANDLE *pHandle, void *pBuffer1, uint32_t dwSize1, void *pBuffer2, uint32_t dwSize2)
{
	return edgeSendData4Mbox(pHandle, pBuffer1, dwSize1, pBuffer2, dwSize2);
}
//------------------------------------------------------------
// RTCDx[X[{bNX̎M
//------------------------------------------------------------
int32_t edge_recvMbox(RTCDHANDLE *pHandle, void *pBuffer, uint32_t dwBufferSize, uint32_t dwTimeout)
{
	return edgeReceiveData(pHandle, pBuffer, dwBufferSize, dwTimeout);
}
//------------------------------------------------------------
// RTCDx[X[{bNX̎M
//------------------------------------------------------------
int32_t edge_recvMboxEx(RTCDHANDLE *pHandle, void *pBuffer1, uint32_t dwBufferSize1, void *pBuffer2, uint32_t dwBufferSize2, uint32_t dwTimeout)
{
	return edgeReceiveData4Mbox(pHandle, pBuffer1, dwBufferSize1, pBuffer2, dwBufferSize2, dwTimeout);
}
//------------------------------------------------------------
// RTCDx[X[{bNX̍\擾
//------------------------------------------------------------
int32_t edge_infoMbox(RTCDHANDLE *pHandle, EDGE_RTCD_HEADER *pInfo)
{
	return edgeGetRTCDInfo(pHandle, pInfo);
}
//------------------------------------------------------------
// edgeT[rXN
//------------------------------------------------------------
uint32_t edge_createService(const char *Name, const char *path, const char *arg, intptr_t *pid)
{
	char *newArgs = NULL;
	size_t size = strlen(path) + strlen(arg) + 10;
	newArgs = malloc(size);
	memset(newArgs, 0, size);
	// R}hC̐擪Ɏst@CpXt^
	sprintf_s(newArgs, size, "\"%s\" %s", path, arg);

	HRESULT hResult = E_FAIL;
	STARTUPINFO         tStartupInfo = { 0 };
	PROCESS_INFORMATION tProcessInfomation = { 0 };
	SECURITY_ATTRIBUTES thread = { 0 };
	SECURITY_ATTRIBUTES process = { 0 };
	thread.nLength = sizeof(SECURITY_ATTRIBUTES);
	process.nLength = sizeof(SECURITY_ATTRIBUTES);
	BOOL bResult = CreateProcess(
		path
		, (char*)newArgs
		, &process
		, &thread
		, FALSE
		, 0
		, NULL
		, NULL
		, &tStartupInfo
		, &tProcessInfomation
	);

	free(newArgs);

	if (0 == bResult) {
		return GetLastError();
	}
	*pid = (intptr_t)tProcessInfomation.dwProcessId;		// PID̎擾
	return EDGE_SUCCESS;
}

// edgeT[rXN
//------------------------------------------------------------
uint32_t edge_createServiceEx(const char *Name, const char *path, const char *arg, intptr_t *pid, EDGESERVICECONFIG *config)
{
	return edge_createService(Name, path, arg, pid);
}
//------------------------------------------------------------
// Thread Priority̐ݒ Windowsp
//------------------------------------------------------------
int32_t edge_setPriority(void* hdl, uint8_t byPriority)
{
	uint32_t dwStatus = 0;
	uint32_t dwClPriority = NORMAL_PRIORITY_CLASS;	//ftHgWindows̃ftHg 10ݒ
	int32_t inThPriority = THREAD_PRIORITY_NORMAL;

	if (byPriority < 8)		// 31 0`7 EG_PRIORITY_0_7
	{
		dwClPriority = REALTIME_PRIORITY_CLASS;
		inThPriority = THREAD_PRIORITY_TIME_CRITICAL;
	}
	else if (byPriority < 48) // 26 8`47 EG_PRIORITY_8_47
	{
		dwClPriority = REALTIME_PRIORITY_CLASS;
		inThPriority = THREAD_PRIORITY_HIGHEST;
	}
	else if (byPriority < 56) // 25 48`55 EG_PRIORITY_48_55
	{
		dwClPriority = REALTIME_PRIORITY_CLASS;
		inThPriority = THREAD_PRIORITY_ABOVE_NORMAL;
	}
	else if (byPriority < 64) // 24 56`63 EG_PRIORITY_56_63
	{
		dwClPriority = REALTIME_PRIORITY_CLASS;
		inThPriority = THREAD_PRIORITY_NORMAL;
	}
	else if (byPriority < 72) // 23 64`71 EG_PRIORITY_64_71
	{
		dwClPriority = REALTIME_PRIORITY_CLASS;
		inThPriority = THREAD_PRIORITY_BELOW_NORMAL;
	}
	else if (byPriority < 80) // 22 72`79 EG_PRIORITY_72_79
	{
		dwClPriority = REALTIME_PRIORITY_CLASS;
		inThPriority = THREAD_PRIORITY_LOWEST;
	}
	else if (byPriority < 128) // 16 80`127 EG_PRIORITY_80_127 
	{
		dwClPriority = REALTIME_PRIORITY_CLASS;
		inThPriority = THREAD_PRIORITY_IDLE;
	}
	else if (byPriority < 136) // 15 128`135 EG_PRIORITY_128_135
	{
		dwClPriority = HIGH_PRIORITY_CLASS;
		inThPriority = THREAD_PRIORITY_HIGHEST;
	}
	else if (byPriority < 144) // 14 136`143 EG_PRIORITY_136_143
	{
		dwClPriority = HIGH_PRIORITY_CLASS;
		inThPriority = THREAD_PRIORITY_ABOVE_NORMAL;
	}
	else if (byPriority < 152) // 13 144`151 EG_PRIORITY_144_151
	{
		dwClPriority = HIGH_PRIORITY_CLASS;
		inThPriority = THREAD_PRIORITY_NORMAL;
	}
	else if (byPriority < 160) // 12 152`159 EG_PRIORITY_152_159
	{
		dwClPriority = HIGH_PRIORITY_CLASS;
		inThPriority = THREAD_PRIORITY_BELOW_NORMAL;
	}
	else if (byPriority < 168) // 11 160`167 EG_PRIORITY_160_167
	{
		dwClPriority = HIGH_PRIORITY_CLASS;
		inThPriority = THREAD_PRIORITY_LOWEST;
	}
	else if (byPriority < 176) // 10 168`175 EG_PRIORITY_168_175
	{
		dwClPriority = NORMAL_PRIORITY_CLASS;
		inThPriority = THREAD_PRIORITY_NORMAL;
	}
	else if (byPriority < 184) // 9 176`183 EG_PRIORITY_176_183
	{
		dwClPriority = NORMAL_PRIORITY_CLASS;
		inThPriority = THREAD_PRIORITY_ABOVE_NORMAL;
	}
	else if (byPriority < 192) // 8 184`191 EG_PRIORITY_184_191
	{
		dwClPriority = NORMAL_PRIORITY_CLASS;
		inThPriority = THREAD_PRIORITY_NORMAL;
	}
	else if (byPriority < 200) // 7 192`199 EG_PRIORITY_192_199
	{
		dwClPriority = NORMAL_PRIORITY_CLASS;
		inThPriority = THREAD_PRIORITY_BELOW_NORMAL;
	}
	else if (byPriority < 208) // 6 200`207 EG_PRIORITY_200_207
	{
		dwClPriority = NORMAL_PRIORITY_CLASS;
		inThPriority = THREAD_PRIORITY_LOWEST;
	}
	else if (byPriority < 216) // 5 208`215 EG_PRIORITY_208_215
	{
		dwClPriority = IDLE_PRIORITY_CLASS;
		inThPriority = THREAD_PRIORITY_HIGHEST;
	}
	else if (byPriority < 224) // 4 216`223 EG_PRIORITY_216_223
	{
		dwClPriority = IDLE_PRIORITY_CLASS;
		inThPriority = THREAD_PRIORITY_ABOVE_NORMAL;
	}
	else if (byPriority < 232) // 3 224`231 EG_PRIORITY_224_231
	{
		dwClPriority = IDLE_PRIORITY_CLASS;
		inThPriority = THREAD_PRIORITY_NORMAL;
	}
	else if (byPriority < 240) // 2 232`239 EG_PRIORITY_232_239
	{
		dwClPriority = IDLE_PRIORITY_CLASS;
		inThPriority = THREAD_PRIORITY_BELOW_NORMAL;
	}
	else if (byPriority < 256) // 1 240`255 EG_PRIORITY_240_255
	{
		dwClPriority = IDLE_PRIORITY_CLASS;
		inThPriority = THREAD_PRIORITY_LOWEST;
	}
	//
	// else = 10ݒ(ϐl)
	//
	if (TRUE == SetPriorityClass(GetCurrentProcess(), dwClPriority))	//sGetLastErrorԂ
	{
		if(FALSE == SetThreadPriority((HANDLE)hdl, inThPriority))
			dwStatus = GetLastError();
	}
	else
	{
		dwStatus = GetLastError();
	}
	return dwStatus;
}
//------------------------------------------------------------
// vO̎擾
//------------------------------------------------------------
int32_t edge_getCommmandLine(char *pArg, uint32_t dwSize)
{
	char* ptr = NULL;
	ptr = GetCommandLine();	//R}hC擾
	if (strlen(ptr) > 0 && strlen(ptr) < dwSize)	//A֐̃TCY菬ꍇRs[
		strcpy_s(pArg, dwSize, ptr);
	else
		pArg[0] = '\0';

	return EDGE_SUCCESS;
}
/// <summary>
/// tRs[(t^\0nɊ܂܂Ȃ) _TRUNCATE(-1)dstmax-1
/// </summary>
/// <param name="dst"></param>
/// <param name="dstmax">dstobt@̃TCY</param>
/// <param name="src"></param>
/// <param name="n">Rs[TCY</param>
void edge_strncpy_s(char *dst, size_t dstmax, const char *src, size_t n)
{
	strncpy_s(dst, dstmax, src, n);
}
//------------------------------------------------------------
// RTCDx[X[{bNX̃nhL`FbN
//------------------------------------------------------------
int32_t edge_checkHandle(RTCDHANDLE *pHandle)
{
	return edgeHandleCheck(pHandle);
}
//------------------------------------------------------------
// @\  FptH[}XJE^̎g̎擾( Hz )
//   F
// ߂lFJE^g(Hz)
//------------------------------------------------------------
unsigned long long edge_GetPCounter(void)
{
	LARGE_INTEGER counter;
	QueryPerformanceCounter(&counter);		// _asm rdtscp Ƃ͈قȂ錋
	return counter.QuadPart;
}
/*==========================================
 * @\  FptH[}XJE^̎g̎擾( Hz )
 *   F
 * ߂lFJE^g(Hz)
/==========================================*/
unsigned long long GetPerformanceCFreq(void)
{
	LARGE_INTEGER freq;
	QueryPerformanceFrequency(&freq);
	return freq.QuadPart;
}
//------------------------------------------------------------
// @\  FԂ̎Zo( nano second )
//   Fstart_Count  Jn̶
//		   finish_Count I̶
// ߂lFx(nano second)
//------------------------------------------------------------
double edge_CalcTime(unsigned long long start_Count, unsigned long long finish_Count)
{
	double  dblSpeed;														// (1ۯɂ鎞)
	unsigned long long freq = GetPerformanceCFreq();							// JE^g̎擾 [Hz]
	double calcedTime = 0;
	if (freq != 0) {
		dblSpeed = (double)1000000000 / (double)GetPerformanceCFreq();			// (1ۯɂ鎞)̎擾 [nsec]
		calcedTime = ((double)(finish_Count - start_Count) * dblSpeed);		// w̸ۯɂ鎞Ԃ̎擾[nsec]
	}
	else {
		calcedTime = 0;
	}

	return calcedTime;
}