﻿////
/// Copyright(C) 2020 Micronet Co. All Rights Reserved.
//
using RTedge;
using System;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Runtime.InteropServices;
using System.Windows.Data;
using System.Collections.Generic;

namespace EgBrow
{
    /// <summary>
    /// コンテナ(サービス)情報
    /// </summary>
    public class ServInfo
    {
        #region フィールド
        public string Name;         // コンテナ名
        public string Path;         // パス
        public string Arg;          // 起動引数
        public string NodeName;     // コンテナの起動ノード名
        public object Var_Status;   // ステータスインジケーター:Status
        public object Var_Error;    // ステータスインジケーター:Error
        public object Var_Run;      // ステータスインジケーター:Run
        public object Var_Live;     // ステータスインジケーター:Live
        public string MboxName;     // コンテナ通信用メールボックス名
        #endregion
    }

    /// <summary>
    /// ListViewに表示されるアイテム(１行)
    /// </summary>
    public class ServItem : INotifyPropertyChanged
    {
        #region フィールド
        /// <summary>
        /// コンテナ情報
        /// </summary>
        public ServInfo rec;
        /// <summary>
        /// リスト表示用No
        /// </summary>
        public int No;
        /// <summary>
        /// インデックス
        /// </summary>
        public edge_API.EGOBJINDEX index;
        #endregion

        #region プロパティ

        /// <summary>
        /// コンテナ名
        /// </summary>
        public string Name { get { return rec.Name; } }
        /// <summary>
        /// パス
        /// </summary>
        public string Path { get { return rec.Path; } }
        /// <summary>
        /// 起動引数
        /// </summary>
        public string Arg { get { return rec.Arg; } }
        /// <summary>
        /// コンテナを起動するノード名
        /// </summary>
        public string NodeName { get { return rec.NodeName; } }
        /// <summary>
        /// コンテナ通信用メールボックス名
        /// </summary>
        public string MboxName { get { return rec.MboxName; } }
        /// <summary>
        /// ステータスインジケーター:Status
        /// </summary>
        public string Status
        {
            get
            {
                try
                {
                    var val = (Byte)rec.Var_Status; return val == 1 ? "Working" : "-";
                }
                catch (Exception)
                {
                    return "NaN";
                }
            }
        }
        /// <summary>
        /// ステータスインジケーター:Error
        /// </summary>
        public string Error
        {
            get
            {
                try
                {
                    return rec.Var_Error.ToString();
                }
                catch (Exception)
                {
                    return "NaN";
                }
            }
        }
        /// <summary>
        /// ステータスインジケーター:Run
        /// </summary>
        public string Run
        {
            get
            {
                try
                {
                    return rec.Var_Run.ToString();
                }
                catch (Exception)
                {
                    return "NaN";
                }
            }
        }
        /// <summary>
        /// ステータスインジケーター:Live
        /// </summary>
        public string Live
        {
            get
            {
                try
                {
                    var val = (UInt32)rec.Var_Live; return val.ToString();
                }
                catch (Exception)
                {
                    return "NaN";
                }
            }
        }
        #endregion

        #region コンストラクタ
        public ServItem(int iNo, edge_API.EGOBJINDEX idx, ServInfo item)
        {
            No = iNo;       // リスト表示用No
            index = idx;    // インデックス情報
            rec = item;     // コンテナ情報
        }
        #endregion

        #region プライベート
        /// <summary>
        /// プロパティ変更通知
        /// </summary>
        public event PropertyChangedEventHandler PropertyChanged;
        private void Notifiy(String propertyName)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
        }
        #endregion

        #region パブリック
        /// <summary>
        /// データ更新指令
        /// </summary>
        public void Refresh()
        {
            Notifiy("Status");
            Notifiy("Error");
            Notifiy("Run");
            Notifiy("Live");
        }
        #endregion

    }

    /// <summary>
    /// ListViewに表示されるリストデータ構造
    /// </summary>
    public class ServList : ObservableCollection<ServItem>, INotifyPropertyChanged
    {
        public ServList()
        {
            // 複数スレッドからコレクション操作できるようにする
            BindingOperations.EnableCollectionSynchronization(this, new object());
        }
        // トータル数
        public int NumItems
        {
            get { return this.Count; }
        }
        public new event PropertyChangedEventHandler PropertyChanged;
        private void Notifiy(String propertyName)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
        }
        public void Refresh()
        {
            Notifiy("NumItems");
        }
        /// <summary>
        /// CSV出力用データ取得
        /// </summary>
        /// <returns></returns>
        public List<List<string>> GetCsvOutData()
        {
            // CSVに出力するためのデータを生成します。
            List<List<string>> data = new List<List<string>>();

            // ヘッダー部
            List<string> h = new List<string>();
            h.Add("Name");
            h.Add("Status");
            h.Add("Error");
            h.Add("Run");
            h.Add("Live");
            h.Add("Path");
            h.Add("Arg");
            h.Add("NodeName");
            h.Add("MboxName");
            data.Add(h);

            // データ部
            foreach (var item in this)
            {
                List<string> d = new List<string>();
                d.Add(item.Name);
                d.Add(item.Status);
                d.Add(item.Error);
                d.Add(item.Run);
                d.Add(item.Live);
                d.Add(item.Path);
                d.Add(item.Arg);
                d.Add(item.NodeName);
                d.Add(item.MboxName);
                data.Add(d);
            }
            return data;
        }

    }
}

