﻿////
/// Copyright(C) 2020 Micronet Co. All Rights Reserved.
//
using RTedge;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Runtime.InteropServices;
using System.Windows.Data;

namespace EgBrow
{
    /// <summary>
    /// コンテナ(サービス)情報
    /// </summary>
    public class DsetInfo
    {
        #region フィールド

        /// <summary>
        /// データセット名
        /// </summary>
        public string Name;
        /// <summary>
        /// タグへのリンク数
        /// </summary>
        public uint Count;
        /// <summary>
        /// タグリンク先のタグ情報
        /// </summary>
        public ObservableCollection<TagInfo> tagInfo;

        #endregion

        #region コンストラクタ
        /// <summary>
        /// コンストラクタ
        /// </summary>
        public DsetInfo()
        {
            Name = string.Empty;
            Count = 0;
            tagInfo = new ObservableCollection<TagInfo>();
        }
        #endregion
    }

    /// <summary>
    /// ListViewに表示されるアイテム(１行)
    /// </summary>
    public class DsetItem : INotifyPropertyChanged
    {
        #region フィールド

        /// <summary>
        /// データセット情報
        /// </summary>
        public DsetInfo rec;
        /// <summary>
        /// リスト表示用No
        /// </summary>
        public int No;
        /// <summary>
        /// インデックス
        /// </summary>
        public edge_API.EGOBJINDEX index;

        #endregion

        #region プロパティ

        /// <summary>
        /// データセット名
        /// </summary>
        public string Name { get { return rec.Name; } }
        /// <summary>
        /// タグへのリンク数
        /// </summary>
        public uint Count { get { return rec.Count; } }
        /// <summary>
        /// タグリンク情報
        /// </summary>
        public List<TagItem> TagItems { get; set; } = new List<TagItem>();

        #endregion

        #region コンストラクタ
        public DsetItem(int iNo, edge_API.EGOBJINDEX idx, DsetInfo item)
        {
            No = iNo;       // リスト表示用No
            index = idx;    // タグインデックス情報
            rec = item;     // タグ情報
            TagItems = new List<TagItem>();
            for (int i = 0; i < item.Count; i++)
            {
                TagItems.Add(new TagItem(item.tagInfo[i]));
            }
        }
        #endregion

        #region プライベート
        /// <summary>
        /// プロパティ変更通知
        /// </summary>
        public event PropertyChangedEventHandler PropertyChanged;
        private void Notifiy(String propertyName)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
        }
        #endregion

        #region パブリック
        /// <summary>
        /// データ更新指令
        /// </summary>
        public void Refresh()
        {
            Notifiy("Name");
        }
        #endregion

    }

    /// <summary>
    /// ListViewに表示されるリストデータ構造
    /// </summary>
    public class DsetList : ObservableCollection<DsetItem>, INotifyPropertyChanged
    {
        public DsetList()
        {
            // 複数スレッドからコレクション操作できるようにする
            BindingOperations.EnableCollectionSynchronization(this, new object());
        }
        // トータル数
        public int NumItems
        {
            get { return this.Count; }
        }
        public new event PropertyChangedEventHandler PropertyChanged;
        private void Notifiy(String propertyName)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
        }
        public void Refresh()
        {
            Notifiy("NumItems");
        }
        /// <summary>
        /// CSV出力用データ取得
        /// </summary>
        /// <returns></returns>
        public List<List<string>> GetCsvOutData()
        {
            // CSVに出力するためのデータを生成します。
            List<List<string>> data = new List<List<string>>();

            // ヘッダー部
            List<string> h = new List<string>();
            h.Add("Dataset");
            h.Add("TagRef Name");
            h.Add("Current Value");
            data.Add(h);

            // データ部
            foreach (var item in this)
            {
                // Datasetに定義されているタグ毎に出力します
                foreach (var tag in item.TagItems)
                {
                    List<string> d = new List<string>();
                    d.Add(item.Name);
                    d.Add(tag.Name);
                    d.Add(tag.CurrentValue);
                    data.Add(d);
                }
            }
            return data;
        }

    }
}

