﻿////
/// Copyright(C) 2020 Micronet Co. All Rights Reserved.
//
using RTedge;
using System;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Windows.Data;
using System.Collections.Generic;

namespace EgBrow
{
    /// <summary>
    /// メールボックス情報
    /// </summary>
    public class MboxInfo
    {
        #region フィールド
        public string Name;             // メールボックス名
        public uint ColSize;            // 1レコード長
        public uint RowSize;            // レコード数
        public uint WritePos;           // 書込位置
        public uint RcvDataNum;         // 受信データ件数(バッファーに溜まっている件数)
        #endregion  
    }

    /// <summary>
    /// ListViewに表示されるアイテム(１行)
    /// </summary>
    public class MboxItem : INotifyPropertyChanged
    {
        #region フィールド

        /// <summary>
        /// メールボックス情報
        /// </summary>
        public MboxInfo rec;
        /// <summary>
        /// リスト表示用No
        /// </summary>
        public int No;
        /// <summary>
        /// インデックス
        /// </summary>
        public edge_API.EGOBJINDEX index;
        /// <summary>
        /// 書込み位置の前回値
        /// </summary>
        private uint lastWritePosvalue;
        /// <summary>
        /// 受信データ件数の前回値
        /// </summary>
        private uint lastRcvDataNum;
        #endregion

        #region プロパティ
        public bool Update                                  // 書込み位置の変化有無
        {
            get
            {
                bool changed = (!rec.WritePos.Equals(lastWritePosvalue) || !rec.RcvDataNum.Equals(lastRcvDataNum));
                lastWritePosvalue = rec.WritePos;
                lastRcvDataNum = rec.RcvDataNum;
                return changed;     // true:変化あり false:変化なし
            }
        }
        /// <summary>
        /// メールボックス名
        /// </summary>
        public string Name { get { return rec.Name; } }
        /// <summary>
        /// 1レコード長
        /// </summary>
        public uint ColSize { get { return rec.ColSize; } }
        /// <summary>
        /// レコード数
        /// </summary>
        public uint RowSize { get { return rec.RowSize; } }
        /// <summary>
        /// 書込位置(書込みカウンタ)
        /// </summary>
        public uint WritePos { get { return rec.WritePos; } }
        /// <summary>
        /// 受信データ件数(バッファーに溜まっている件数)
        /// </summary>
        public uint RcvDataNum { get { return rec.RcvDataNum; } }

        #endregion

        #region コンストラクタ
        public MboxItem(int iNo, edge_API.EGOBJINDEX idx, MboxInfo item)
        {
            No = iNo;       // リスト表示用No
            index = idx;    // インデックス情報
            rec = item;     // アイテム情報
        }
        #endregion

        #region プライベート
        /// <summary>
        /// プロパティ変更通知
        /// </summary>
        public event PropertyChangedEventHandler PropertyChanged;
        private void Notify(String propertyName)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
        }
        #endregion

        #region パブリック
        /// <summary>
        /// データ更新指令
        /// </summary>
        public void Refresh()
        {
            Notify("Name");
            Notify("WritePos");
            Notify("RcvDataNum");
            Notify("Update");
        }
        #endregion

    }

    /// <summary>
    /// ListViewに表示されるリストデータ構造
    /// </summary>
    public class MboxList : ObservableCollection<MboxItem>, INotifyPropertyChanged
    {
        public MboxList()
        {
            // 複数スレッドからコレクション操作できるようにする
            BindingOperations.EnableCollectionSynchronization(this, new object());
        }
        // トータル数
        public int NumItems
        {
            get { return this.Count; }
        }
        public new event PropertyChangedEventHandler PropertyChanged;
        private void Notifiy(String propertyName)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
        }
        public void Refresh()
        {
            Notifiy("NumItems");
        }
        /// <summary>
        /// CSV出力用データ取得
        /// </summary>
        /// <returns></returns>
        public List<List<string>> GetCsvOutData()
        {
            // CSVに出力するためのデータを生成します。
            List<List<string>> data = new List<List<string>>();

            // ヘッダー部
            List<string> h = new List<string>();
            h.Add("Name");
            h.Add("ColSize");
            h.Add("RowSize");
            h.Add("WritePos");
            h.Add("Queue");
            data.Add(h);

            // データ部
            foreach (var item in this)
            {
                List<string> d = new List<string>();
                d.Add(item.Name);
                d.Add(item.ColSize.ToString());
                d.Add(item.RowSize.ToString());
                d.Add(item.WritePos.ToString());
                d.Add(item.RcvDataNum.ToString());
                data.Add(d);
            }
            return data;
        }

    }
}

