﻿////
/// Copyright(C) 2020 Micronet Co. All Rights Reserved.
//
using System;
using System.Linq;
using System.Timers;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Input;
using System.Runtime.InteropServices;
using System.Collections.Generic;
using RTedge;   // RT-edge system API
using EdgeCode; // RT-edge template code

namespace EgBrow
{
    /// <summary>
    /// MainWindow.xaml の相互作用ロジック
    /// </summary>
    public partial class MainWindow : Window
    {
        /// <summary>
        /// 
        /// </summary>
        edgeCode EgProc = null;                 //RTedge フレームワーク+APIを使用したコードクラス

        private Dictionary<TabIdx, Timer> dicTimer; // 表示更新タイマー
        public bool isUpdate = false;           // 一覧の更新中を示すフラグ
        public TabIdx selectedTab = 0;          // 表示中のタブインデックス
        public TagList taglist;                 // 表示用タグリスト
        public ServList servlist;               // 表示用コンテナリスト
        public ClctList clctlist;               // 表示用コレクタリスト
        public DsetList dsetlist;               // 表示用データセットリスト
        public MboxList mboxlist;               // 表示用メールボックスリスト

        public SettingIntv interval;

        /// <summary>
        /// コンストラクタ
        /// </summary>
        public MainWindow()
        {
            InitializeComponent();
            //AllocConsole();                     // [DEBUG用] コンソール出力用

            interval = new SettingIntv(1000);   // 更新周期初期化
            EgProc = new edgeCode();            // RTedge処理クラスの作成
            EgProc.mInstMainClose = Stop;       // フレームワーク内終了メッセージ処理時の関数指定
            EgProc.InitService();               // RTedge処理クラスの初期化

            isUpdate = true;                    // 一覧更新中フラグON
            InitDisplayData();                  // 画面表示データ初期化
            EgProc.GetTagList(ref taglist);     // タグ一覧取得
            EgProc.GetServList(ref servlist);   // コンテナ一覧取得
            EgProc.GetClctList(ref clctlist, ref servlist);   // コレクタ一覧取得
            EgProc.GetDsetList(ref dsetlist);   // データセット一覧取得
            EgProc.GetMboxList(ref mboxlist);   // メールボックス一覧取得
            isUpdate = false;                   // 一覧更新中フラグOFF

            SetupTimer();                       // 表示更新タイマー初期化
        }

        /// <summary>
        /// サービス終了処理
        /// </summary>
        public void Stop()
        {
            try
            {
                Dispatcher.Invoke((Action)(() =>
                {
                    Close();                        //Window Close関数呼び出し
                }));
            }
            catch (Exception ex)
            {
                EgProc.LOG(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
            }
            ;
        }

        /// <summary>
        /// Windowクローズ処理
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Window_Closing(object sender, System.ComponentModel.CancelEventArgs e)
        {
            try
            {
                EgProc.KillService();           //サービス削除関数呼び出し<必須>
                //EgProc.EGAPI.EgFinalize();      //APIのFinalize処理追加 <必須>
                EgProc.Release();               //後処理
                //[TODO] : 必要に応じて終了処理追加
            }
            catch (Exception ex)
            {
                EgProc.LOG(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
            }
        }
        /// <summary>
        /// [DEBUG用] コンソール出力するための定義
        /// </summary>
        /// <returns></returns>
        [DllImport("kernel32.dll", SetLastError = true)]
        [return: MarshalAs(UnmanagedType.Bool)]
        static extern bool AllocConsole();


        #region Private

        /// <summary>
        /// 表示用データ初期化
        /// </summary>
        private void InitDisplayData()
        {
            // data instance
            taglist     = new TagList();
            servlist    = new ServList();
            clctlist    = new ClctList();
            dsetlist    = new DsetList();
            mboxlist    = new MboxList();

            // Binding
            DataContext = this;
            // data binding for tag
            lstData.ItemsSource = taglist;
            ItemCount.DataContext = taglist;
            // data binding for container
            servData.ItemsSource = servlist;
            servCount.DataContext = servlist;
            // data binding for collector
            clctData.ItemsSource = clctlist;
            clctCount.DataContext = clctlist;
            // data binding for dataset
            dsetData.DataContext = dsetlist;
            dsetCount.DataContext = dsetlist;
            // data binding for mailbox
            mboxData.ItemsSource = mboxlist;
            mboxCount.DataContext = mboxlist;
        }

        /// <summary>
        /// タグの現在値更新
        /// </summary>
        private void UpdateCurrentData(TabIdx idx)
        {
            // 一覧更新中の場合は処理しない
            if (isUpdate) { return; } 

            // 該当タブが表示中の場合のみ、タグ値の更新処理を行う
            if (selectedTab != idx) { return; }

            switch (idx)
            {
                case TabIdx.TAG:            // タグ一覧表示中
                    EgProc.UpdateTagsVal(ref taglist);          // タグ値更新
                    break;
                case TabIdx.CONTAINER:      // コンテナ一覧表示中
                    EgProc.UpdateContainersVal(ref servlist);   // タグ値更新(コンテナのステータスインジケーター)
                    break;
                case TabIdx.DATASET:        // データセット一覧表示中
                    EgProc.UpdateDsetTagsVal(ref dsetlist);     // タグ値更新(データセット内のタグ)
                    break;
                case TabIdx.MAILBOX:        // メールボックス一覧表示中
                    EgProc.UpdateMboxVal(ref mboxlist);         // メールボックス カウンタ更新
                    break;
                case TabIdx.COLLECTOR:
                    EgProc.UpdateCollectorVal(ref clctlist);
                    break;
                default:
                    break;
            }
        }

        /// <summary>
        /// 一覧更新処理
        /// </summary>
        /// <param name="tabIndex"></param>
        /// <param name="boForced"></param>
        private void UpdateListItem(TabIdx tabIndex, bool boForced = false)
        {
            // -------------------------------------------------
            // NOTE: 表示中のタブでオブジェクト数に変化がある
            //       場合のみ更新処理を行う
            //       ※強制更新フラグON時は毎回更新
            // -------------------------------------------------
            if (isUpdate) { return; }

            isUpdate = true;                // 一覧更新中フラグON
            switch (tabIndex)
            {
                case TabIdx.TAG:            // タグ一覧
                    if ((taglist.Count != EgProc.GetTagListCount())     // 総数が変化した場合
                        || (boForced))                                  // または、強制更新
                    {
                        taglist.Clear();                                // 一覧クリア
                        EgProc.GetTagList(ref taglist);                 // 一覧取得
                    }
                    break;
                case TabIdx.CONTAINER:      // コンテナ一覧
                    if ((servlist.Count != EgProc.GetServListCount())   // 総数が変化した場合
                        || (boForced))                                  // または、強制更新
                    {
                        servlist.Clear();                               // 一覧クリア
                        EgProc.GetServList(ref servlist);               // 一覧取得
                    }
                    break;
                case TabIdx.COLLECTOR:      // コレクタ一覧
                    if ((clctlist.Count != EgProc.GetClctListCount())   // 総数が変化した場合
                        || (boForced))                                  // または、強制更新
                    {
                        clctlist.Clear();                               // 一覧クリア
                        EgProc.GetClctList(ref clctlist, ref servlist); // 一覧取得
                    }
                    break;
                case TabIdx.DATASET:        // データセット一覧
                    if ((dsetlist.Count != EgProc.GetDsetListCount())   // 総数が変化した場合
                        || (boForced))                                  // または、強制更新
                    {
                        dsetlist.Clear();                               // 一覧クリア
                        EgProc.GetDsetList(ref dsetlist);               // 一覧取得
                    }
                    break;
                case TabIdx.MAILBOX:        // メールボックス一覧
                    if ((mboxlist.Count != EgProc.GetMboxListCount())   // 総数が変化した場合
                        || (boForced))                                  // または、強制更新
                    {
                        mboxlist.Clear();                               // 一覧クリア
                        EgProc.GetMboxList(ref mboxlist);               // 一覧取得
                    }
                    break;
                default:
                    break;
            }
            isUpdate = false;               // 一覧更新中フラグOFF
        }
        #endregion

        #region Event
        /// <summary>
        /// ListViewのヘッダをクリックしてソートする
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void listHeader_Click(object sender, RoutedEventArgs e)
        {
            var header = e.OriginalSource as GridViewColumnHeader;
            if (header == null) // ヘッダーのクリック以外のイベントを除外
                return;

            // 列の外部分は無視
            if (header.Column == null)
                return;

            // 列を取得
            var binding = (Binding)header.Column.DisplayMemberBinding;
            var path = binding.Path.Path;

            var listview = (ListView)sender;
            var pre_sort_items = listview.ItemsSource.Cast<object>();
            // 昇順にソート
            var sorted_items = pre_sort_items.OrderBy(row =>
            {
                return row.GetType().GetProperty(path).GetValue(row);
            }).ToList();

            // すでに昇順なら降順にする
            if (sorted_items.SequenceEqual(pre_sort_items))
            {
                sorted_items.Reverse();
            }

            listview.ItemsSource = sorted_items;
        }

        /// <summary>
        /// 書き込み値の生成
        /// </summary>
        /// <param name="itemx"></param>
        /// <returns></returns>
        private string fnCreateWriteVar(TagItem itemx)
        {
            string WriteVar = "";

            // タグタイプによって書き込み値の書式を変える
            switch (itemx.rec.Type)
            {
                case (ushort)EGDEFINE.egTagDataType.FIFO:
                    if (itemx.rec.FIFO_Remain != 0) goto case (ushort)EGDEFINE.egTagDataType.Segment;
                    WriteVar = itemx.fnCreateSegmentValue(new byte[(itemx.rec.Var as byte[])?.Length ?? 0]);    /* FIFOが空の場合は0ﾌｨﾙﾊﾞｲﾄ列を提供 */
                    break;
                case (ushort)EGDEFINE.egTagDataType.Segment:
                    WriteVar = itemx.fnCreateSegmentValue(itemx.rec.Var);   // 書き込み値を現在値で初期化
                    break;
                default:
                    WriteVar = itemx.rec.Var.ToString();                    // 書き込み値を現在値で初期化
                    break;
            }

            return WriteVar;
        }

        /// <summary>
        /// ListView Itemのダブルクリックイベント、詳細ダイアログ表示
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void LstData_MouseDoubleClick(object sender, MouseButtonEventArgs e)
        {
            // 選択無し
            if (lstData.SelectedItems.Count == 0)
                return;

            // アイテム決定
            TagItem itemx = (TagItem)lstData.SelectedItems[0];
            itemx.rec.WriteVar = fnCreateWriteVar(itemx);      // 書き込み値を現在値で初期化

            // 詳細ウィンドウ開く
            var dw = new TagDetailWindow(EgProc);
            dw.Owner = this;
            dw.DataContext = itemx;
            dw.ShowDialog();
        }

        /// <summary>
        /// データセット内のタグのダブルクリックイベント、詳細ダイアログ表示
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void dsetTagRef_MouseDoubleClick(object sender, MouseButtonEventArgs e)
        {
            // 選択無し
            if (dsTagRefs.SelectedItems.Count == 0)
                return;

            // アイテム決定
            TagItem itemx = (TagItem)dsTagRefs.SelectedItems[0];
            itemx.rec.WriteVar = fnCreateWriteVar(itemx);      // 書き込み値を現在値で初期化

            // 詳細ウィンドウ開く
            var dw = new TagDetailWindow(EgProc);
            dw.Owner = this;
            dw.DataContext = itemx;
            dw.ShowDialog();
        }

        /// <summary>
        /// タグ一覧更新ボタンクリックイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void TagUpdateButton_Click(object sender, RoutedEventArgs e)
        {
            // TODO: 未テストです。機能有効にする場合は、テスト必須。
            //isUpdate = true;                // 更新中フラグON
            //System.Threading.Thread.Sleep(250);
            //taglist.Clear();                // 一覧クリア
            //EgProc.GetTagList(ref taglist); // タグ一覧更新
            //isUpdate = false;               // 更新中フラグOFF
        }

        /// <summary>
        /// タブ切替イベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void TabSelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            TabIdx tabIndex = (TabIdx)tabControl.SelectedIndex; // 表示中タブ取得
            if (tabIndex == selectedTab) { return; }    // 変化なし時は無処理で抜ける

            // -------------------------------------------------
            // NOTE: タブ切替のタイミングで一覧を再取得します
            // -------------------------------------------------
            StopTimer();                            // 更新タイマー停止
            UpdateListItem(tabIndex);               // 一覧更新
            selectedTab = tabIndex;                 // 表示中タブ更新
            UpdateCurrentData((TabIdx)tabIndex);    // 現在値更新
            StartTimer();                           // 更新タイマー開始
        }

        /// <summary>
        /// 更新周期設定メニュークリックイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuItemSetting_Click(object sender, RoutedEventArgs e)
        {
            // 設定画面表示
            var dw = new SettingWindow(interval);
            dw.Owner = this;
            dw.DataContext = interval;
            dw.ShowDialog();

            // 設定更新
            StopTimer();            // 更新タイマー停止
            SetIntervalTimer();     // 更新周期設定
            StartTimer();           // 更新タイマー開始
        }

        /// <summary>
        /// 一覧更新ボタンクリックイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ListUpdateButton_Click(object sender, RoutedEventArgs e)
        {
            TabIdx idx = (TabIdx)tabControl.SelectedIndex;  // 表示中タブ取得
            StopTimer();                                    // 更新タイマー停止
            UpdateListItem(idx, true);                      // 一覧更新（強制更新）
            UpdateCurrentData(idx);                         // 現在値更新
            StartTimer();                                   // 更新タイマー開始
        }

        /// <summary>
        /// CSV出力ボタンクリックイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void CsvOutputButton_Click(object sender, RoutedEventArgs e)
        {
            // 更新タイマー停止
            StopTimer();

            // 表示中タブ情報をCSVファイルに出力します
            switch(selectedTab)
            {
                case TabIdx.TAG:        CsvUtil.WriteCsv("Dump_Tags_"        , taglist.GetCsvOutData());  break;
                case TabIdx.CONTAINER:  CsvUtil.WriteCsv("Dump_Containers_"  , servlist.GetCsvOutData()); break;
                case TabIdx.COLLECTOR:  CsvUtil.WriteCsv("Dump_Collectors_"  , clctlist.GetCsvOutData()); break;
                case TabIdx.DATASET:    CsvUtil.WriteCsv("Dump_Datasets_"    , dsetlist.GetCsvOutData()); break;
                case TabIdx.MAILBOX:    CsvUtil.WriteCsv("Dump_Mailboxes_"   , mboxlist.GetCsvOutData()); break;
                default: break;
            }

            // 更新タイマー開始
            StartTimer();
        }

        /// <summary>
        /// 終了メニュークリックイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuItemClose_Click(object sender, RoutedEventArgs e)
        {
            Close();
        }

        #endregion
    }
}
