﻿////
/// Copyright(C) 2020 Micronet Co. All Rights Reserved.
//
using RTedge;
using System;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Windows.Data;
using System.Collections.Generic;

namespace EgBrow
{
    /// <summary>
    /// タグ情報
    /// </summary>
    public class TagInfo
    {
        #region フィールド
        public UInt16 Type;         // データ型
        public UInt16 Size;         // データサイズ
        public object Var;          // 値
        public string Name;         // タグ名
        public string Comment;      // コメント
        public string Source;       // ソース(アドレス)
        public string WriteVar;     // 値(書き込み用)
        /* ﾃﾞｰﾀ型固有ﾌﾟﾛﾊﾟﾃｨｴﾘｱ */
        public UInt16 FIFO_RecordCount; // ﾚｺｰﾄﾞ総数
        public UInt16 FIFO_Remain;      // ﾚｺｰﾄﾞ残数
        #endregion
    }

    /// <summary>
    /// ListViewに表示されるアイテム(１行)
    /// </summary>
    public class TagItem : INotifyPropertyChanged
    {
        #region フィールド
        public TagInfo rec;                                 // タグ情報
        public int No;                                      // リスト表示用No
        public edge_API.EGOBJINDEX index;                   // タグインデックス
        private object lastvalue;                            // タグ値の前回値
        #endregion

        #region プロパティ
        public bool Update                                  // タグ値の変化有無
        {
            get { bool changed = (!rec.Var.Equals(lastvalue)); lastvalue = rec.Var; return changed; }          // true:変化あり false:変化なし
        }
        public string Name
        {
            get { return rec.Name; }                        // タグ名
        }
        public UInt16 Type
        {
            get { return rec.Type; }                        // タグの型
        }
        public string Size
        {
            get { return rec.Size.ToString(); }             // タグのサイズ
        }
        public string Source
        {
            get { return rec.Source; }                      // タグのソース
        }
        public string Comment
        {
            get { return rec.Comment; }                     // タグのコメント
        }
        protected static Dictionary<ushort, string> dic_type_name;
        public string TypeString                            // タグのデータ型(表示用文字列)
        {
            get
            {
                if (dic_type_name == null)
                {
                    /* 初回ｱｸｾｽ時に文字列変換ﾃｰﾌﾞﾙを作成しておく */
                    dic_type_name = new Dictionary<ushort, string>();
                    foreach (var m in Enum.GetNames(typeof(EGDEFINE.egTagDataType)))
                    {
                        dic_type_name.Add((ushort)(EGDEFINE.egTagDataType)Enum.Parse(typeof(EGDEFINE.egTagDataType), m), m);
                    }
                }
                return (dic_type_name.ContainsKey(rec.Type))
                    ? dic_type_name[rec.Type]
                    : rec.Size.ToString();
            }
        }
        public string CurrentValue             // タグの現在値
        {
            get
            {
                try
                {
                    switch (rec.Type)
                    {
                        case (ushort)EGDEFINE.egTagDataType.Boolean:    return rec.Var.ToString() + " (" + (Convert.ToBoolean(rec.Var) ? "1" : "0") + ")";
                        case (ushort)EGDEFINE.egTagDataType.SByte:      var val1 = (SByte)rec.Var; return val1.ToString("x2") + " (" + val1.ToString() + ")";
                        case (ushort)EGDEFINE.egTagDataType.Byte:       var val2 = (Byte)rec.Var; return val2.ToString("x2") + " (" + val2.ToString() + ")";
                        case (ushort)EGDEFINE.egTagDataType.Int16:      var val3 = (Int16)rec.Var; return val3.ToString("x4") + " (" + val3.ToString() + ")";
                        case (ushort)EGDEFINE.egTagDataType.UInt16:     var val4 = (UInt16)rec.Var; return val4.ToString("x4") + " (" + val4.ToString() + ")";
                        case (ushort)EGDEFINE.egTagDataType.Int32:      var val5 = (Int32)rec.Var; return val5.ToString("x8") + " (" + val5.ToString() + ")";
                        case (ushort)EGDEFINE.egTagDataType.UInt32:     var val6 = (UInt32)rec.Var; return val6.ToString("x8") + " (" + val6.ToString() + ")";
                        case (ushort)EGDEFINE.egTagDataType.Int64:      var val7 = (Int64)rec.Var; return val7.ToString("x16") + " (" + val7.ToString() + ")";
                        case (ushort)EGDEFINE.egTagDataType.UInt64:     var val8 = (UInt64)rec.Var; return val8.ToString("x16") + " (" + val8.ToString() + ")";
                        case (ushort)EGDEFINE.egTagDataType.Float:      var val9 = (float)rec.Var; return val9.ToString("F7");
                        case (ushort)EGDEFINE.egTagDataType.Double:     var val10 = (double)rec.Var; return val10.ToString("F15");
                        case (ushort)EGDEFINE.egTagDataType.String:     var val11 = (string)rec.Var; return val11;
                        case (ushort)EGDEFINE.egTagDataType.Segment:    var val12 = fnCreateSegmentValue(rec.Var); return val12;
                        case (ushort)EGDEFINE.egTagDataType.FIFO:       var val13 = $"{((rec.FIFO_Remain != 0) ? fnCreateSegmentValue(rec.Var) : "-")} [{rec.FIFO_Remain} / {rec.FIFO_RecordCount}]"; return val13;
                        default: return rec.Var.ToString();
                    }
                }
                catch (Exception)
                {
                    return "NaN";
                }
            }
        }
        public string WriteValue
        {
            get { return rec.WriteVar; }
            set
            {
                if (rec.WriteVar != value)
                {
                    rec.WriteVar = value;
                    Notifiy("WriteValue");
                }
            }
        }
        #endregion

        #region "コンストラクタ"
        public TagItem(int iNo, edge_API.EGOBJINDEX idx, TagInfo item)
        {
            No = iNo;       // リスト表示用No
            index = idx;    // タグインデックス情報
            rec = item;     // タグ情報
        }

        public TagItem(TagInfo item)
        {
            rec = item;     // タグ情報
            No = 0;         // リスト表示用No
            index = new edge_API.EGOBJINDEX();    // タグインデックス情報
        }
        #endregion

        #region プライベート
        /// <summary>
        /// プロパティ変更通知
        /// </summary>
        public event PropertyChangedEventHandler PropertyChanged;
        private void Notifiy(String propertyName)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
        }
        #endregion

        #region パブリック
        /// <summary>
        /// データ更新指令
        /// </summary>
        public void Refresh()
        {
            Notifiy("CurrentValue");
            Notifiy("Update");
        }

        public string fnCreateSegmentValue(object Var)
        {
            string srRet = "";

            if (Var is byte[])
            {
                // バイト配列の場合、ダンプ出力（例：00|00| .. |00）
                byte[] bySegment = (byte[])Var;
                for (var i = 0; i < bySegment.Length; i++)
                {
                    srRet += bySegment[i].ToString("X2");   // 16進数に変更
                    if ((i + 1) < bySegment.Length)
                    {
                        srRet += "|";                       // 最終配列でない場合は"|"を付加
                    }
                }
            }

            return srRet;
        }
        #endregion
    }

    /// <summary>
    /// ListViewに表示されるリストデータ構造
    /// </summary>
    public class TagList : ObservableCollection<TagItem>, INotifyPropertyChanged
    {
        public TagList()
        {
            // 複数スレッドからコレクション操作できるようにする
            BindingOperations.EnableCollectionSynchronization(this, new object());
        }
        // タグのトータル数
        public int NumItems
        {
            get { return this.Count; }
        }
        public new event PropertyChangedEventHandler PropertyChanged;
        private void Notifiy(String propertyName)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
        }
        public void Refresh()
        {
            Notifiy("NumItems");
        }
        /// <summary>
        /// CSV出力用データ取得
        /// </summary>
        /// <returns></returns>
        public List<List<string>> GetCsvOutData()
        {
            // CSVに出力するためのデータを生成します。
            List<List<string>> data = new List<List<string>>();

            // ヘッダー部
            List<string> h = new List<string>();
            h.Add("Name");
            h.Add("Current Value");
            h.Add("Type");
            h.Add("Source");
            h.Add("Comment");
            data.Add(h);

            // データ部
            foreach (var item in this)
            {
                List<string> d = new List<string>();
                d.Add(item.Name);
                d.Add(item.CurrentValue);
                d.Add(item.TypeString);
                d.Add(item.Source);
                d.Add(item.Comment);
                data.Add(d);
            }
            return data;
        }
    }
}

