﻿////
/// Copyright(C) 2020 Micronet Co. All Rights Reserved.
//
using System;
using System.Windows;
using System.Windows.Data;
using EdgeCode; // RT-edge template code

namespace EgBrow
{
    /// <summary>
    /// TagDetailWindow.xaml の相互作用ロジック
    /// </summary>
    public partial class TagDetailWindow : Window
    {
        edgeCode EgProc = null;

        public TagDetailWindow(edgeCode egProc)
        {
            InitializeComponent();
            EgProc = egProc;
        }

        /// <summary>
        /// タグ書き込みボタンクリックイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void TagWriteButton_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                int ret;
                TagItem item = (TagItem)this.DataContext;           // タグ情報取得
                object objWriteVal = EgProc.ConvertData(item);      // タグ書込み用に変換
                ret = EgProc.WriteTagValue(item.Name, objWriteVal); // タグ書き込み
                if (ret != 0)
                {
                    // エラー
                    MessageBox.Show("EgTagWrite Error. [" + ret.ToString() + "]", "Error", MessageBoxButton.OK, MessageBoxImage.Error);
                }
            }
            catch(Exception ex)
            {
                MessageBox.Show("EgTagWrite Error. [" + ex.Message + "]", "Error", MessageBoxButton.OK, MessageBoxImage.Error);
            }
        }
    }

    [ValueConversion(typeof(bool), typeof(bool))]
    public class InverseBooleanConverter : IValueConverter
    {
        #region IValueConverter Members

        public object Convert(object value, Type targetType, object parameter,
            System.Globalization.CultureInfo culture)
        {
            if (targetType != typeof(bool))
                throw new InvalidOperationException("The target must be a boolean");

            return !(bool)value;
        }

        public object ConvertBack(object value, Type targetType, object parameter,
            System.Globalization.CultureInfo culture)
        {
            throw new NotSupportedException();
        }

        #endregion
    }
}
