﻿////
/// Copyright(C) 2020 Micronet Co. All Rights Reserved.
//
using System;
using System.Collections.Generic;
using System.Timers;
using System.Windows;

namespace EgBrow
{
    public partial class MainWindow : Window
    {

        /// <summary>
        /// 画面表示更新用タイマー初期化
        /// </summary>
        private void SetupTimer()
        {
            dicTimer = new Dictionary<TabIdx, Timer>();

            // タブ毎の更新周期設定
            dicTimer.Add(TabIdx.TAG,        new Timer(interval.IntvTag));
            dicTimer.Add(TabIdx.CONTAINER,  new Timer(interval.IntvTag));
            dicTimer.Add(TabIdx.COLLECTOR,  new Timer(interval.IntvCollector));
            dicTimer.Add(TabIdx.DATASET,    new Timer(interval.IntvTag));
            dicTimer.Add(TabIdx.MAILBOX,    new Timer(interval.IntvMailbox));

            // タブ毎のタイマーイベント設定
            dicTimer[TabIdx.TAG].Elapsed        += (ss, ee) => { UpdateCurrentData(TabIdx.TAG); };
            dicTimer[TabIdx.CONTAINER].Elapsed  += (ss, ee) => { UpdateCurrentData(TabIdx.CONTAINER); };
            dicTimer[TabIdx.COLLECTOR].Elapsed  += (ss, ee) => { UpdateCurrentData(TabIdx.COLLECTOR); };
            dicTimer[TabIdx.DATASET].Elapsed    += (ss, ee) => { UpdateCurrentData(TabIdx.DATASET); };
            dicTimer[TabIdx.MAILBOX].Elapsed    += (ss, ee) => { UpdateCurrentData(TabIdx.MAILBOX); };

            // タイマー開始 (表示中タブの更新用タイマーを開始します)
            StartTimer();

            // 画面閉じる時に全てのタイマー破棄
            this.Closing += (s, e) =>
            {
                foreach (var tim in dicTimer)
                {
                    tim.Value.Stop();
                    tim.Value.Dispose();
                }
            };
        }

        /// <summary>
        /// タイマー周期設定
        /// </summary>
        private void SetIntervalTimer()
        {
            try
            {
                if (interval.IntvTag > 0)
                    dicTimer[TabIdx.TAG].Interval = interval.IntvTag;
                if (interval.IntvContainer > 0)
                    dicTimer[TabIdx.CONTAINER].Interval = interval.IntvContainer;
                if (interval.IntvCollector > 0)
                    dicTimer[TabIdx.COLLECTOR].Interval = interval.IntvCollector;
                if (interval.IntvDataset > 0)
                    dicTimer[TabIdx.DATASET].Interval = interval.IntvDataset;
                if (interval.IntvMailbox > 0)
                    dicTimer[TabIdx.MAILBOX].Interval = interval.IntvMailbox;
            }
            catch (Exception ex)
            {
                EgProc.LOG(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
            }
        }

        /// <summary>
        /// 更新用タイマー開始
        /// </summary>
        private void StartTimer()
        {
            // 表示中タブの更新用タイマーをStart
            dicTimer[selectedTab].Start();
        }

        /// <summary>
        /// 更新用タイマー停止
        /// </summary>
        /// <param name="isAllStop"></param>
        private void StopTimer(bool isAllStop = false)
        {
            if (isAllStop)
            {
                // 全ての更新用タイマーをStop
                foreach (var time in dicTimer)
                {
                    time.Value.Stop();
                }
            }
            else
            {
                // 表示中タブの更新用タイマーをStop
                dicTimer[selectedTab].Start();
            }
        }
    }
}
