﻿////
/// Copyright(C) 2020 Micronet Co. All Rights Reserved.
//
using System;
using System.Threading.Tasks;
using RTedge;   // RT-edge system API
using System.Runtime.CompilerServices;
using System.Threading;
using EgBrow;

namespace EdgeCode
{
    /// <summary>
    /// RT-edge API/フレームワーク 各種値定義クラス
    /// </summary>
    public static class EDGECONST
    {
        /// <summary>
        /// 各種定義追加
        /// </summary>

        ////
        /// サービス名
        //
        public const string SERVICENAME = "EgBrow";                                           // [TODO](*******必須*******) : exe拡張子を除くプログラムファイル名に変更
        ////
        /// サービスインジケータ名
        //        
        public const string IND_MYSERV_STATUS = "SERVICE." + SERVICENAME + ".Status";           // サービス起動
        public const bool SERVICE_UP = true;                                                    // 起動中
        public const bool SERVICE_DOWN = false;                                                 // 停止中
        public const string IND_MYSERV_ERROR = "SERVICE." + SERVICENAME + ".Error";             // サービスエラー発生中
        public const string IND_MYSERV_RUN = "SERVICE." + SERVICENAME + ".Run";		            // サービス実行中
        public const string IND_MYSERV_LIVE = "SERVICE." + SERVICENAME + ".Live";               // サービス実行カウンタ
        ////
        /// サービスプロパティタグ名
        //        
        public const string PROP_MYSERV_MODE = "SERVICE." + SERVICENAME + ".Mode";              // サービス動作モード
        public const int MODE_AUTO = 1;                                                         // 入出自動サイクル
        public const int MODE_SEMIAUTO = 2;                                                     // 入力サイクル＋出力デマンド
        public const int MODE_MANUAL = 3;                                                       // 入出力デマンド
        public const int DEFAULT_MODE = MODE_SEMIAUTO;                                          // デフォルト値
        public const string PROP_MYSERV_CYCLE = "SERVICE." + SERVICENAME + ".Cycle";            // サービス動作サイクル(ms)
        public const int DEFAULT_CYCLE = 1;
        public const string PROP_MYSERV_INPRIO = "SERVICE." + SERVICENAME + ".InPriority";      // Edgeシステムへの入力サービススレッドプライオリティ
        public const int DEFAULT_INPRIO = 148;
        public const string PROP_MYSERV_OUTPRIO = "SERVICE." + SERVICENAME + ".OutPriority";    // Edgeシステムからの出力サービススレッドプライオリティ	
        public const int DEFAULT_OUTPRIO = 149;
        ////
        /// サービス提供タグ名
        //  [TODO] : 任意のタグ追加 (XMLで定義することを推奨)
        // public static string TAG_MYSERV_XXX = "XXX";
        ////
        /// サービス固有メッセージ
        //  [TODO] : 任意のメッセージID追加(ユーザメッセージ 20000～29999番指定可)
        // public static int EM_SERVICE_XXX = 20000;    //任意の番号
    }
    /// <summary>
    /// RT-edge API/フレームワーク実装クラス
    /// </summary>
    public class edgeCode
    {
        ////
        /// サービス内部状態
        //
        public class MYSTATE
        {
            public bool State;                  // インジケータ.エッジシステム接続状態
            public bool Run;                    // インジケータ.サービス実行状態
            public bool Error;                  // インジケータ.サービス異常状態
            public UInt32 Live;                 // インジケータ.サービス実行カウンタ
            public UInt64 StartTime;            // インジケータ.サービス開始時のタイムスタンプ
            public byte Mode;                   // プロパティ.サービス実行モード
            public UInt32 Cycle;                // プロパティ.サービス実行サイクル(ms)	[TODO] : 必要に応じて実装
            public byte InPriority;             // プロパティ.サービス(入力)プライオリティ	[TODO] : 必要に応じて実装
            public byte OutPriority;            // プロパティ.サービス(出力)プライオリティ	[TODO] : 必要に応じて実装
            public SemaphoreSlim hInTrigSem;    // 入力サービストリガセマフォハンドル	[TODO] : 必要に応じて実装
            public SemaphoreSlim hOutTrigSem;   // 出力サービストリガセマフォハンドル	[TODO] : 必要に応じて実装
            //[TODO] : 任意のステータス追加
            public MYSTATE()
            {
                State = false;
                Run = false;
                Error = false;
                Live = 0;
                StartTime = 0;
                Mode = 0;
                Cycle = 1000;
                InPriority = 149;
                OutPriority = 148;
                hInTrigSem = null;
                hOutTrigSem = null;
            }
        }

        ////
        /// edge_APIクラス インスタンス化 
        //
        public edge_API EGAPI = null;                               //edge APIのインスタンス
        public MYSTATE STATE = null;                                //サービス状態クラスのインスタンス
        ////
        /// スレッド変数 
        //
        private CancellationTokenSource mThreadAutoTS = null;       //自動更新用スレッド       
        private CancellationTokenSource mThreadInServiceTS = null;  //入力スレッドキャンセル処理用変数
        private CancellationTokenSource mThreadOutServiceTS = null; //出力スレッドキャンセル処理用変数
        ////
        /// メイン処理終了用コールバック関数
        //
        public delegate void CallbackMainClose();
        public CallbackMainClose mInstMainClose = null;
        /// <summary>
        /// コンストラクタ
        /// </summary>
        /// <param name="myMsgHndlers">メッセージハンドラー関数を指定</param>
        public edgeCode() 
        {
            EGAPI = new edge_API();
            STATE = new MYSTATE();
        }
        /// <summary>
        /// 後処理処理(デストラクタ)
        /// [TODO] : 必要に応じて変更
        /// </summary>
        public void Release()
        {
            try
            {
                if (mThreadInServiceTS != null)     //入力スレッド終了処理
                {
                    mThreadInServiceTS.Cancel();    //キャンセル状態をセットし、スレッド処理で抜ける
                    mThreadInServiceTS = null;
                }

                if (mThreadOutServiceTS != null)    //出力スレッド終了処理
                {
                    mThreadOutServiceTS.Cancel();   //キャンセル状態をセットし、スレッド処理で抜ける
                    mThreadOutServiceTS = null;
                }

                if (mThreadAutoTS != null)          //自動更新スレッド終了処理
                {
                    mThreadAutoTS.Cancel();         //キャンセル状態をセットし、スレッド処理で抜ける
                    mThreadAutoTS = null;
                }
                
                if (STATE.hInTrigSem != null)
                    STATE.hInTrigSem.Dispose();
                if (STATE.hOutTrigSem != null)
                    STATE.hOutTrigSem.Dispose();
            }
            catch (Exception ex)
            {
                LOG(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
            }
        }
        
        /// <summary>
        /// Edgeメッセージハンドラ
        /// [TODO] : 必要に応じてメッセージ・処理を追加
        /// </summary>
        /// <param name="senderName">メッセージ送信元</param>
        /// <param name="messNo">メッセージ番号</param>
        /// <param name="paramArray">メッセージパラメータ</param>
        /// <returns></returns>
        object WMLOCK = new object();
        public Int32 MsgHandler(string senderName, int messNo, byte[] paramArray)
        {
            try
            {
                int result = 0;
                lock (WMLOCK) //排他制御 頻繁にやり取りする場合用 不要の場合は削除
                {
                    switch (messNo) //メッセージ番号毎に処理
                    {
                        case (int)EGDEFINE.EGMSG.EM_SERVICE_RUN:    //サービス開始指令
                            result = StartService();
                            break;
                        case (int)EGDEFINE.EGMSG.EM_SERVICE_STOP:   // サービス終了指令
                            {
                                if (mInstMainClose != null)
                                    mInstMainClose();               // コールバック関数からは戻りません
                                else
                                    result = KillService();
                            }
                            break;
                        case (int)EGDEFINE.EGMSG.EM_SERVICE_PAUSE:  // サービス一時停止指令
                            result = PauseService();
                            break;
                        case (int)EGDEFINE.EGMSG.EM_SERVICE_UPDATE: // サービスアップデート指令
                            switch ((int)STATE.Mode)
                            {
                                case EDGECONST.MODE_AUTO:
                                    break;
                                case EDGECONST.MODE_SEMIAUTO:
                                    KickOutput();
                                    break;
                                case EDGECONST.MODE_MANUAL:
                                    KickInput();
                                    KickOutput();
                                    break;
                                default:
                                    break;
                            }
                            break;
                        //
                        // [TODO] : 任意のメッセージ処理を追加
                        //
                        default:
                            break;
                    }
                }
                return result;
            }
            catch (Exception ex)
            {
                LOG(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return -1;
            }
        }
        /// <summary>
        /// Edgeシステム入出力自動サイクルスレッド
        /// 必要が無ければ変更不要
        /// </summary>
        public async void ThreadAutoCycle(CancellationToken cancelToken)
        {
            try
            {
				await Task.Run(() =>
                {
                    try
                    {

                        while (true)
                        {
                            if (cancelToken.IsCancellationRequested)        // スレッドを抜ける処理
                                break;

                            ++STATE.Live;                                   // サービス実行カウンタ増加
                            GetProperty();                                  // サービスプロパティのリロード

                            if (STATE.Run)                                  // サービス実行状態なら処理する
                            {
                                switch (STATE.Mode)                         // サービス動作モードに応じて分岐
                                {
                                    case EDGECONST.MODE_AUTO:               // 自動サイクルモード時（入出力共に自動更新する）
                                        KickInput();                      	// Input処理実行
                                        KickOutput();						// Output処理実行
                                        break;
                                    case EDGECONST.MODE_SEMIAUTO:           // 半自動サイクル時（入力のみ自動更新、出力はEM_SERVICE_UPDATEで指令）
                                        KickInput();                      	// Input処理実行
                                        break;
                                    case EDGECONST.MODE_MANUAL:             // 手動サイクル時（入出力はEM_SERVICE_UPDATEで指令するので無処理）
                                        break;
                                }
                            }

                            UpdateIndicator();                              // サービスインジケータ更新
                            System.Threading.Thread.Sleep((int)STATE.Cycle);// サイクル待ち
                        }
                    }
                    catch (Exception ex)
                    {
                        LOG(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                    }
                });
            }
            catch (Exception ex)
            {
                LOG(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
            }
        }
        /// <summary>
        /// Edgeシステムへのデータ入力
        /// [TODO] : 必要に応じて値を書き込むタグと値を指定
        /// </summary>
        public async void ThreadInService(CancellationToken cancelToken)
        {
            try
            {
                int result = 0;
                while (true)
                { 
                    await STATE.hInTrigSem.WaitAsync();
                    // キャンセルトークンが来たら終了
                    if (cancelToken.IsCancellationRequested)
                        break;
                    //タスク処理を開始する
                    try
                    {
                        //
                        //[TODO] : 必要に応じて任意のEgTagから値読込を実装
                        //例) 時間を取得
                        // 以下例で示す ts はEDGE_TIME_STAMPのインスタンス 
                        //result |= EgTagRead(TAG_MYSERV_XXXX1, &ts.time, sizeof(ts.time));			// 任意の値
                        //result |= EgTagRead(TAG_MYSERV_XXXX2, &ts.millisecond, , sizeof(ts.millisecond));	// システム時刻格納(ms)

                        STATE.Error = (result != 0);    // エラーインジケータを更新

                    }
                    finally
                    {
                        //Releaseはトリガ処理で行うためここでは呼ばない
                    }
                }
            }
            catch (Exception ex)
            {
                LOG(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
            }
        }
        /// <summary>
        /// Edgeシステムからのデータ出力
        /// [TODO] : 必要に応じて値を読み込むタグを指定
        /// </summary>
        public async void ThreadOutService(CancellationToken cancelToken)
        {
            try
            {
                int result = 0;
                while (true)
                {
                    await STATE.hOutTrigSem.WaitAsync();
                    // キャンセルトークンが来たら終了
                    if (cancelToken.IsCancellationRequested)
                        break;
                    //タスク処理を開始する
                    try
                    {
                        //
                        //[TODO] : 必要に応じて任意のEgTagから値読込を実装
                        //例) 時間を取得
                        // 以下例で示す ts はEDGE_TIME_STAMPのインスタンス 
                        //result |= EgTagRead(TAG_MYSERV_XXXX1, &ts.time, sizeof(ts.time));			// 任意の値
                        //result |= EgTagRead(TAG_MYSERV_XXXX2, &ts.millisecond, , sizeof(ts.millisecond));	// システム時刻格納(ms)

                        STATE.Error = (result != 0);	// エラーインジケータを更新
                    }
                    finally
                    {
                        //Releaseはトリガ処理で行うためここでは呼ばない
                    }
                }
            }
            catch (Exception ex)
            {
                LOG(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
            }
        }
        /// <summary>
        /// 入力サービスを1回キックする
        /// 必要が無ければ変更不要
        /// </summary>
        public void KickInput()
        {
            try
            {
                STATE.hInTrigSem.Release();
            }
            catch (Exception ex)
            {
                LOG(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
            }
        }
        /// <summary>
        /// 出力サービスを1回キックする
        /// 必要が無ければ変更不要
        /// </summary>
        public void KickOutput()
        {
            try
            {
                STATE.hOutTrigSem.Release();
            }
            catch (Exception ex)
            {
                LOG(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
            }
        }
        /// <summary>
        /// サービスタグ生成 ※すでに生成されている場合は無視して継続します
        /// [TODO] : 必要に応じて作成するタグを指定(タグの生成はXMLに定義し自動生成することを推奨)
        /// </summary>
        /// <returns></returns>
        public int CreateServiceTags()
        {
            try
            {
                int result = 0;
                // サービスタグ類の生成
                // [TODO] : 必要に応じて以下コードを修正
                //result |= EGAPI.EgTagCreate(EDGECONST.TAG_MYSERV_XXX, (ushort)EGDEFINE.egTagDataType.UInt64, "", "sample tag");
                return result;
            }
            catch (Exception ex)
            {
                LOG(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return -1;
            }
        }
        /// <summary>
        /// サービスインジケータの更新 ※更新に失敗しても無視して継続します
        /// 必要が無ければ変更不要
        /// </summary>
        /// <returns>正常時０</returns>
        public int UpdateIndicator()
        {
            try
            {
                // インジケータタグはフレームワークによって自動生成済
                int result = 0;

                // NOTE: ★★★EdgeMonツールでは非対応とする。コメントアウト。
                //result |= EGAPI.EgTagWrite(EDGECONST.IND_MYSERV_STATUS, STATE.State);   // サービス起動状態
                //result |= EGAPI.EgTagWrite(EDGECONST.IND_MYSERV_RUN, STATE.Run);        // サービス実行状態
                //result |= EGAPI.EgTagWrite(EDGECONST.IND_MYSERV_LIVE, STATE.Live);      // サービス実行カウンタ
                //result |= EGAPI.EgTagWrite(EDGECONST.IND_MYSERV_ERROR, STATE.Error);    // サービス異常状態
                return result;
            }
            catch (Exception ex)
            {
                LOG(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return -1;
            }
        }
        /// <summary>
        /// サービスプロパティの取得 ※取得できない時はデフォルト設定値を採用します
        /// [TODO] : 必要に応じて設定項目の増減指定
        /// </summary>
        /// <returns></returns>
        public int GetProperty()
        {
            try
            {
                // NOTE: ★★★EdgeMonツールでは非対応とする。コメントアウト。
                //// プロパティタグはXML定義可能　無いときはデフォルト設定値を採用します
                //int result = 0;
                //object value = new object();
                //result = EGAPI.EgTagRead(EDGECONST.PROP_MYSERV_MODE, ref value);    // サービス動作モードの取得
                //if (result == 0)
                //    STATE.Mode = (byte)value;
                //else
                //    STATE.Mode = EDGECONST.DEFAULT_MODE;				            // デフォルト

                //result = EGAPI.EgTagRead(EDGECONST.PROP_MYSERV_CYCLE, ref value);   // サービス処理サイクルの取得
                //if (result == 0)
                //    STATE.Cycle = (uint)value;
                //else
                //    STATE.Cycle = EDGECONST.DEFAULT_CYCLE;				            // デフォルト

                //result = EGAPI.EgTagRead(EDGECONST.PROP_MYSERV_INPRIO, ref value);  // 入力サービススレッドのプライオリティ指定
                //if (result == 0)
                //    STATE.InPriority = (byte)value;
                //else
                //    STATE.InPriority = EDGECONST.DEFAULT_INPRIO;                    // デフォルト

                //result = EGAPI.EgTagRead(EDGECONST.PROP_MYSERV_INPRIO, ref value);  // 出力サービススレッドのプライオリティ指定
                //if (result == 0)
                //    STATE.OutPriority = (byte)value;
                //else
                //    STATE.OutPriority = EDGECONST.DEFAULT_OUTPRIO;                  // デフォルト

                //
                // [TODO] : 任意のサービスプロパティを指定
                //
                return 0;
            }
            catch (Exception ex)
            {
                LOG(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return -1;
            }
        }
        /// <summary>
        /// サービス開始 ※入出力可能になります
        /// 必要が無ければ変更不要
        /// </summary>
        /// <returns>正常時０</returns>
        public int StartService()
        {
            try
            {
                STATE.Run = true;       // サービス実行
                UpdateIndicator();      // サービスインジケータ更新
                return 0;
            }
            catch (Exception ex)
            {
                LOG(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return -1;
            }
        }
        /// <summary>
        /// サービス一時停止 ※入出力されなくなります
        /// 必要が無ければ変更不要
        /// </summary>
        /// <returns></returns>
        public int PauseService()
        {
            try
            {
                STATE.Run = false;      // サービス一時停止
                UpdateIndicator();      // サービスインジケータ更新
                return 0;
            }
            catch (Exception ex)
            {
                LOG(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return -1;
            }
        }
        /// <summary>
        /// サービスの初期化
        /// </summary>
        /// <returns>正常時０</returns>
        public Int32 InitService()
        {
            try
            {
                Int32 result = 0;
                LOG("[LOG] Start " + EDGECONST.SERVICENAME + " Service");
                edge_API.EDGECONFIG config = new edge_API.EDGECONFIG(); //設定情報の初期化
                config = EGAPI.EDGE_CONFIG_DEFAULT;                     //初期値設定

                config.mInstMessFunc = MsgHandler;                      //Edgeメッセージユーザハンドラの指定

                // NOTE: ★★★EdgeMonツールでは非対応とする。コメントアウト。
                //result = EGAPI.EgInit(config);                          // Edgeフレームワークの初期化
                //if (result != 0)                                        // 正常終了以外の場合はコンソールにメッセージ出力
                //    WARN(result);

                result = GetProperty();                                 // サービスプロパティのロード処理
                if (result != 0)                                        // 正常終了以外の場合はコンソールにメッセージ出力
                    WARN(result);

                result = CreateServiceTags();                           // サービスインジケータの生成
                if (result != 0)                                        // 正常終了以外の場合はコンソールにメッセージ出力
                    WARN(result);

                STATE.hInTrigSem = new SemaphoreSlim(1, 1);             // 入力サービススレッド用トリガセマフォ
                STATE.hOutTrigSem = new SemaphoreSlim(1, 1);            // 出力サービススレッド用トリガセマフォ

                mThreadInServiceTS = new CancellationTokenSource();     //スレッド終了用token指定
                CancellationToken cInToken = mThreadInServiceTS.Token;    
                ThreadInService(cInToken);                              //入力スレッド処理作成始

                mThreadOutServiceTS = new CancellationTokenSource();    //スレッド終了用token指定
                CancellationToken cOutToken = mThreadOutServiceTS.Token;
                ThreadOutService(cOutToken);                            //出力スレッド処理作成始

                mThreadAutoTS = new CancellationTokenSource();          //スレッド終了用token指定
                CancellationToken cAutoToken = mThreadAutoTS.Token;
                ThreadAutoCycle(cAutoToken);                            //自動更新スレッド処理作成始

                STATE.State = EDGECONST.SERVICE_UP;                     // サービスアップ

                return result;
            }
            catch (Exception ex)
            {
                LOG(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return -1;
            }
        }
        /// <summary>
        /// サービス削除（プロセス停止）
        /// </summary>
        /// <returns>ん</returns>
        public int KillService()
        {
            try
            {
                PauseService();                         // サービス一時停止

                STATE.State = EDGECONST.SERVICE_DOWN;   //
                UpdateIndicator();				        // サービスインジケータ更新
                LOG("[LOG] End " + EDGECONST.SERVICENAME + " Service");
                return 0;
            }
            catch (Exception ex)
            {
                LOG(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return -1;
            }
        }
        /// <summary>
        /// [DEBUG用] メッセージ出力
        /// </summary>
        /// <param name="message"></param>
        /// <param name="filePath"></param>
        /// <param name="lineNumber"></param>
        public void WARN(int reason,
                         [CallerFilePath] string filePath = "",
                         [CallerLineNumber] int lineNumber = 0)
        {
            try
            {
                Console.WriteLine(EDGECONST.SERVICENAME + " service warning at file(" + filePath + ") ,line(" + lineNumber.ToString() +
                                   ")  ,reason 0x" + reason.ToString("x8"));
            }
            catch
            { }
        }
        /// <summary>
        /// [DEBUG用] メッセージ出力
        /// </summary>
        /// <param name="message"></param>
        public void LOG(string message)
        {
            try
            {
                Console.WriteLine(EDGECONST.SERVICENAME + " : " + message);
            }
            catch
            { }
        }

        #region Tag

        /// <summary>
        /// Tag数取得
        /// </summary>
        /// <returns></returns>
        public uint GetTagListCount()
        {
            uint itemNum = 0;
            int rtn = EGAPI.EgTagGetCount(ref itemNum);         // タグ総数取得
            if (rtn == 0)
                return itemNum;
            else
                return 0;
        }

        /// <summary>
        /// Tag一覧取得
        /// </summary>
        public void GetTagList(ref TagList _taglist)
        {
            try
            {
                uint itemNum = 0;
                int rtn = EGAPI.EgTagGetCount(ref itemNum);         // タグ総数取得
                if (rtn != 0) { return; }                           // エラー時は処理を抜ける

                edge_API.EGOBJINDEX[] idxArray = new edge_API.EGOBJINDEX[itemNum];
                rtn = EGAPI.EgTagGetEnum(ref idxArray, ref itemNum);// タグ一覧取得
                if (rtn != 0) { return; }                           // エラー時は処理を抜ける

                for (int i = 0; i < itemNum; i++)                   // タグ数分処理ループ
                {
                    TagInfo tag = GetTagProperty(idxArray[i]);      // タグ情報取得
                    if (tag == null) { continue; }                  // エラー時は次のタグへ

                    _taglist.Add(new TagItem(i, idxArray[i], tag)); // リストへ追加
                }
                _taglist.Refresh();
            }
            catch (Exception ex)
            {
                LOG(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
            }
        }

        /// <summary>
        /// タグ情報取得
        /// </summary>
        /// <param name="idx"></param>
        /// <returns></returns>
        public TagInfo GetTagProperty(edge_API.EGOBJINDEX idx)
        {
            try
            {
                int rtn = 0;
                object objName = new object();
                object objType = new object();
                object objSize = new object();
                object objSource = new object();
                object objComment = new object();
                object objVal = new object();
                object objFIFORecordCount = null;
                ushort wFIFORemain = 0;
                rtn |= EGAPI.EgTagGetPropertyByIndex(idx, "Name", ref objName);         // タグ名
                rtn |= EGAPI.EgTagGetPropertyByIndex(idx, "Type", ref objType);         // データ型
                rtn |= EGAPI.EgTagGetPropertyByIndex(idx, "Size", ref objSize);         // データサイズ
                rtn |= EGAPI.EgTagGetPropertyByIndex(idx, "Source", ref objSource);     // ソース
                rtn |= EGAPI.EgTagGetPropertyByIndex(idx, "Comment", ref objComment);   // コメント
                rtn |= EGAPI.EgTagReadByIndex(idx, ref objVal);                         // 値
                if (objType is ushort t && t == (ushort)EGDEFINE.egTagDataType.FIFO)
                {
                    /* FIFOﾀｸﾞ固有の追加情報を取得 */
                    objFIFORecordCount = new object();
                    rtn |= EGAPI.EgTagGetPropertyByIndex(idx, "RecordCount", ref objFIFORecordCount);
                    rtn |= EGAPI.EgTagGetRemainFIFOByIndex(idx, ref wFIFORemain);
                }
                if (rtn == 0)       // エラーなし
                {
                    // データ格納
                    TagInfo tag = new TagInfo();
                    tag.Name = objName.ToString();
                    tag.Type = (ushort)objType;
                    tag.Size = (ushort)objSize;
                    tag.Source = objSource.ToString();
                    tag.Comment = objComment.ToString();
                    tag.Var = objVal;
                    if (objFIFORecordCount != null) tag.FIFO_RecordCount = (ushort)objFIFORecordCount;
                    tag.FIFO_Remain = wFIFORemain;
                    return tag;
                }
                else                // エラーあり
                {
                    LOG("GetTagProperty Error: " + rtn.ToString());
                    return null;
                }
            }
            catch (Exception ex)
            {
                LOG(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return null;
            }
        }
        /// <summary>
        /// タグ情報取得
        /// </summary>
        /// <param name="idx"></param>
        /// <returns></returns>
        public TagInfo GetTagProperty(string name)
        {
            try
            {
                int rtn = 0;
                object objName = new object();
                object objType = new object();
                object objSize = new object();
                object objSource = new object();
                object objComment = new object();
                object objVal = new object();
                object objFIFORecordCount = null;
                ushort wFIFORemain = 0;
                rtn |= EGAPI.EgTagGetProperty(name, "Name", ref objName);         // タグ名
                rtn |= EGAPI.EgTagGetProperty(name, "Type", ref objType);         // データ型
                rtn |= EGAPI.EgTagGetProperty(name, "Size", ref objSize);         // データサイズ
                rtn |= EGAPI.EgTagGetProperty(name, "Source", ref objSource);     // ソース
                rtn |= EGAPI.EgTagGetProperty(name, "Comment", ref objComment);   // コメント
                rtn |= EGAPI.EgTagRead(name, ref objVal);                         // 値
                if (objType is ushort t && t == (ushort)EGDEFINE.egTagDataType.FIFO)
                {
                    /* FIFOﾀｸﾞ固有の追加情報を取得 */
                    objFIFORecordCount = new object();
                    rtn |= EGAPI.EgTagGetProperty(name, "RecordCount", ref objFIFORecordCount);
                    rtn |= EGAPI.EgTagGetRemainFIFO(name, ref wFIFORemain);
                }
                if (rtn == 0)       // エラーなし
                {
                    // データ格納
                    TagInfo tag = new TagInfo();
                    tag.Name = objName.ToString();
                    tag.Type = (ushort)objType;
                    tag.Size = (ushort)objSize;
                    tag.Source = objSource.ToString();
                    tag.Comment = objComment.ToString();
                    tag.Var = objVal;
                    if (objFIFORecordCount != null) tag.FIFO_RecordCount = (ushort)objFIFORecordCount;
                    tag.FIFO_Remain = wFIFORemain;
                    return tag;
                }
                else                // エラーあり
                {
                    LOG("GetTagProperty Error: " + rtn.ToString());
                    return null;
                }
            }
            catch (Exception ex)
            {
                LOG(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return null;
            }
        }

        /// <summary>
        /// タグ値更新
        /// </summary>
        public void UpdateTagsVal(ref TagList _taglist)
        {
            try
            {
                object val = new object();
                foreach (var item in _taglist)
                {
                    EGAPI.EgTagReadByIndex(item.index, ref val);
                    item.rec.Var = val;

                    /* FIFOﾀｸﾞ固有処理 */
                    if (item.rec.Type == (ushort)EGDEFINE.egTagDataType.FIFO) EGAPI.EgTagGetRemainFIFOByIndex(item.index, ref item.rec.FIFO_Remain);

                    item.Refresh();
                }
            }
            catch (Exception ex)
            {
                LOG(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
            }
        }

        /// <summary>
        /// タグ値更新(データセット内のタグ)
        /// </summary>
        public void UpdateDsetTagsVal(ref DsetList _list)
        {
            try
            {
                object val = new object();
                foreach (var dsItem in _list)
                {
                    foreach (var tag in dsItem.TagItems)
                    {
                        EGAPI.EgTagRead(tag.Name, ref val);
                        tag.rec.Var = val;
                        tag.Refresh();
                    }
                }
            }
            catch (Exception ex)
            {
                LOG(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
            }
        }

        /// <summary>
        /// EgMailbox情報を取得し、受信データ件数を算出する
        /// </summary>
        /// <param name="mbName"></param>
        /// <param name="writePos"></param>
        /// <param name="rcvNum"></param>
        /// <returns></returns>
        public int GetMailboxInfo(string mbName, out uint writePos, out uint rcvNum)
        {
            edge_API.RTCDHEADER stHeader = new edge_API.RTCDHEADER();
            object val = new object();
            uint wp, rp, total;         // 未受信データ件数算出用
            uint rcvDataNum;            // 未受信データ件数(リングバッファに溜まっている件数)

            // 値初期化
            writePos = 0;
            rcvNum = 0;

            // EgMailbox情報取得
            int rtn = EGAPI._EgMailboxGetInfo(mbName, ref stHeader);
            if (rtn == 0) //エラー無し
            {
                wp = stHeader.write_pos;
                rp = stHeader.read_pos;
                total = stHeader.record_num;
                rcvDataNum = 0;

                // 未受信データ件数の算出
                if (wp == rp)               // 書込位置と読込位置が同じ
                {
                    rcvDataNum = 0;         // 未受信データなし
                }
                else if (wp < rp)           // 読込位置のほうが大きい(リングバッファ終端を跨ぐ)
                {
                    rcvDataNum = total - rp + wp;
                }
                else                        // 上記以外
                {
                    rcvDataNum = wp - rp;
                }

                // 取得値を返却
                writePos = wp;
                rcvNum = rcvDataNum;
            }
            return rtn;
        }

        /// <summary>
        /// タグのデータ型にあわせて書き込み値を変換します
        /// </summary>
        /// <param name="item"></param>
        /// <returns></returns>
        public object ConvertData(TagItem item)
        {
            object ret = null;
            try
            {
                bool boConvert;     // true:変換成功 / false:変換失敗
                int i;
                byte byVal;
                switch (item.Type)
                {
                    case (ushort)EGDEFINE.egTagDataType.Boolean:ret = bool.Parse(item.WriteValue); break;
                    case (ushort)EGDEFINE.egTagDataType.SByte:  ret = sbyte.Parse(item.WriteValue); break;
                    case (ushort)EGDEFINE.egTagDataType.Byte:   ret = byte.Parse(item.WriteValue); break;
                    case (ushort)EGDEFINE.egTagDataType.Int16:  ret = short.Parse(item.WriteValue); break;
                    case (ushort)EGDEFINE.egTagDataType.UInt16: ret = ushort.Parse(item.WriteValue); break;
                    case (ushort)EGDEFINE.egTagDataType.Int32:  ret = int.Parse(item.WriteValue); break;
                    case (ushort)EGDEFINE.egTagDataType.UInt32: ret = uint.Parse(item.WriteValue); break;
                    case (ushort)EGDEFINE.egTagDataType.Int64:  ret = long.Parse(item.WriteValue); break;
                    case (ushort)EGDEFINE.egTagDataType.UInt64: ret = ulong.Parse(item.WriteValue); break;
                    case (ushort)EGDEFINE.egTagDataType.Float:  ret = float.Parse(item.WriteValue); break;
                    case (ushort)EGDEFINE.egTagDataType.Double: ret = double.Parse(item.WriteValue); break;
                    case (ushort)EGDEFINE.egTagDataType.String: ret = item.WriteValue; break;
                    case (ushort)EGDEFINE.egTagDataType.Segment:
                    case (ushort)EGDEFINE.egTagDataType.FIFO:
                        // "00 00 00 00 ..." -> バイト配列に変換する
                        boConvert = true;                                   // デフォルトは変換成功
                        string[] srSegs = item.WriteValue.Split('|');       // '|'で分解
                        byte[] bySegs = new byte[srSegs.Length];            // バイト配列の確保
                        for (i = 0; i < bySegs.Length; i++)
                        {
                            // 空文字はエラー
                            if (srSegs[i] == "")
                            {
                                boConvert = true;           // 変換失敗
                                break;                      // 変換処理を抜ける
                            }

                            // 入力値を変換
                            if (byte.TryParse(srSegs[i], System.Globalization.NumberStyles.HexNumber, null, out byVal))
                            {
                                bySegs[i] = byVal;          // 変換成功
                            }
                            else
                            {
                                boConvert = true;           // 変換失敗
                                break;                      // 変換処理を抜ける
                            }
                        }
                        if (boConvert) { ret = bySegs; }    // 変換成功した場合、戻り値にセット
                        break;
                    default: break;
                }
                return ret;
            }
            catch (Exception ex)
            {
                LOG(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                throw;
            }
        }

        /// <summary>
        /// タグ書き込み
        /// </summary>
        /// <param name="tagName"></param>
        /// <param name="val"></param>
        /// <returns></returns>
        public int WriteTagValue(string tagName, object val)
        {
            try
            {
                return EGAPI.EgTagWrite(tagName, val);
            }
            catch (Exception ex)
            {
                LOG(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                throw;
            }
        }

        #endregion

        #region Container

        /// <summary>
        /// コンテナ数取得
        /// </summary>
        /// <returns></returns>
        public uint GetServListCount()
        {
            uint itemNum = 0;
            int rtn = EGAPI.EgServiceGetCount(ref itemNum);         // コンテナ総数取得
            if (rtn == 0)
                return itemNum;
            else
                return 0;
        }

        /// <summary>
        /// コンテナ一覧取得
        /// </summary>
        public void GetServList(ref ServList _list)
        {
            try
            {
                uint itemNum = 0;
                int rtn = EGAPI.EgServiceGetCount(ref itemNum);     // 総数取得
                if (rtn != 0) { return; }                           // エラー時は処理を抜ける

                edge_API.EGOBJINDEX[] idxArray = new edge_API.EGOBJINDEX[itemNum];
                rtn = EGAPI.EgServiceGetEnum(ref idxArray, ref itemNum);    // 一覧取得
                if (rtn != 0) { return; }                           // エラー時は処理を抜ける

                for (int i = 0; i < itemNum; i++)                   // コンテナ数分処理ループ
                {
                    ServInfo info = GetServProperty(idxArray[i]);   // コンテナ情報取得
                    if (info == null) { continue; }                 // エラー時は次のタグへ

                    _list.Add(new ServItem(i, idxArray[i], info));  // リストへ追加
                }
                _list.Refresh();
            }
            catch (Exception ex)
            {
                LOG(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
            }
        }

        /// <summary>
        /// コンテナ情報(プロパティ)取得
        /// </summary>
        /// <param name="idx"></param>
        /// <returns></returns>
        public ServInfo GetServProperty(edge_API.EGOBJINDEX idx)
        {
            try
            {
                int rtn = 0;
                object objName = new object();
                object objPath = new object();
                object objArg = new object();
                object objNodeName = new object();
                object objVarStatus = new object();
                object objVarError = new object();
                object objVarRun = new object();
                object objVarLive = new object();
                object objMboxName = new object();
                // プロパティ情報取得
                rtn |= EGAPI.EgServiceGetPropertyByIndex(idx, "Name", ref objName);
                rtn |= EGAPI.EgServiceGetPropertyByIndex(idx, "Path", ref objPath);
                rtn |= EGAPI.EgServiceGetPropertyByIndex(idx, "Arg", ref objArg);
                rtn |= EGAPI.EgServiceGetPropertyByIndex(idx, "NodeName", ref objNodeName);
                rtn |= EGAPI.EgServiceGetPropertyByIndex(idx, "MailboxName", ref objMboxName);
                // ステータスインジケーター取得
                rtn |= EGAPI.EgTagRead(string.Format("SERVICE.{0}.Status", objName.ToString()), ref objVarStatus);
                rtn |= EGAPI.EgTagRead(string.Format("SERVICE.{0}.Error", objName.ToString()), ref objVarError);
                rtn |= EGAPI.EgTagRead(string.Format("SERVICE.{0}.Run", objName.ToString()), ref objVarRun);
                rtn |= EGAPI.EgTagRead(string.Format("SERVICE.{0}.Live", objName.ToString()), ref objVarLive);
                if (rtn == 0)       // エラーなし
                {
                    // データ格納
                    ServInfo info = new ServInfo();
                    info.Name = objName.ToString();
                    info.Path = objPath.ToString();
                    info.Arg = objArg.ToString();
                    info.NodeName = objNodeName.ToString();
                    info.Var_Status = objVarStatus;
                    info.Var_Error = objVarError;
                    info.Var_Run = objVarRun;
                    info.Var_Live = objVarLive;
                    info.MboxName = objMboxName.ToString();
                    return info;
                }
                else                // エラーあり
                {
                    LOG("GetServProperty Error: " + rtn.ToString());
                    return null;
                }
            }
            catch (Exception ex)
            {
                LOG(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return null;
            }
        }

        /// <summary>
        /// コンテナ情報更新
        /// </summary>
        public void UpdateContainersVal(ref ServList _list)
        {
            try
            {
                object val = new object();
                foreach (var item in _list)
                {
                    EGAPI.EgTagRead(string.Format("SERVICE.{0}.Status", item.Name), ref val);   // Status
                    item.rec.Var_Status = val;
                    EGAPI.EgTagRead(string.Format("SERVICE.{0}.Error", item.Name), ref val);    // Error
                    item.rec.Var_Error = val;
                    EGAPI.EgTagRead(string.Format("SERVICE.{0}.Run", item.Name), ref val);      // Run
                    item.rec.Var_Run = val;
                    EGAPI.EgTagRead(string.Format("SERVICE.{0}.Live", item.Name), ref val);     // Live
                    item.rec.Var_Live = val;

                    item.Refresh();
                }
            }
            catch (Exception ex)
            {
                LOG(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
            }
        }

        #endregion

        #region Collector

        /// <summary>
        /// コレクタ数取得
        /// </summary>
        /// <returns></returns>
        public uint GetClctListCount()
        {
            uint itemNum = 0;
            int rtn = EGAPI.EgCollectorGetCount(ref itemNum);   // コレクタ総数取得
            if (rtn == 0)
                return itemNum;
            else
                return 0;
        }

        /// <summary>
        /// コレクタ一覧取得
        /// </summary>
        public void GetClctList(ref ClctList _list, ref ServList _servList)
        {
            try
            {
                uint itemNum = 0;
                int rtn = EGAPI.EgCollectorGetCount(ref itemNum);   // 総数取得
                if (rtn != 0) { return; }                           // エラー時は処理を抜ける

                edge_API.EGOBJINDEX[] idxArray = new edge_API.EGOBJINDEX[itemNum];
                rtn = EGAPI.EgCollectorGetEnum(ref idxArray, ref itemNum);  // 一覧取得
                if (rtn != 0) { return; }                           // エラー時は処理を抜ける

                for (int i = 0; i < itemNum; i++)                   // コレクタ数分処理ループ
                {
                    ClctInfo info = GetClctProperty(idxArray[i], ref _servList);   // コレクタ情報取得
                    if (info == null) { continue; }                 // エラー時は次のタグへ

                    _list.Add(new ClctItem(i, idxArray[i], info));  // リストへ追加
                }
                _list.Refresh();
            }
            catch (Exception ex)
            {
                LOG(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
            }
        }

        /// <summary>
        /// コレクタ情報(プロパティ)取得
        /// </summary>
        /// <param name="idx"></param>
        /// <returns></returns>
        public ClctInfo GetClctProperty(edge_API.EGOBJINDEX idx, ref ServList _servList)
        {
            try
            {
                int rtn = 0;
                object objName = new object();
                object objOwnerName = new object();
                object objInterval = new object();
                object objPriority = new object();
                object objColSize = new object();
                object objRowSize = new object();
                object objDsetName = new object();
                // プロパティ情報取得
                rtn |= EGAPI.EgCollectorGetPropertyByIndex(idx, "Name", ref objName);
                rtn |= EGAPI.EgCollectorGetPropertyByIndex(idx, "OwnerName", ref objOwnerName);
                rtn |= EGAPI.EgCollectorGetPropertyByIndex(idx, "Interval", ref objInterval);
                rtn |= EGAPI.EgCollectorGetPropertyByIndex(idx, "Priority", ref objPriority);
                rtn |= EGAPI.EgCollectorGetPropertyByIndex(idx, "ColSize", ref objColSize);
                rtn |= EGAPI.EgCollectorGetPropertyByIndex(idx, "RowSize", ref objRowSize);
                rtn |= EGAPI.EgCollectorGetPropertyByIndex(idx, "DatasetName", ref objDsetName);
                if (rtn == 0)       // エラーなし
                {
                    // データ格納
                    ClctInfo info   = new ClctInfo();
                    info.Name       = objName.ToString();
                    info.OwnerName  = objOwnerName.ToString();
                    info.Interval   = (uint)objInterval;
                    info.Priority   = (byte)objPriority;
                    info.ColSize    = (uint)objColSize;
                    info.RowSize    = (uint)objRowSize;
                    info.DsetName   = objDsetName.ToString();

                    // コレクタタイプ判定
                    info.Type = false;
                    foreach (var ctn in _servList)
                    {
                        if (ctn.Name.Equals(info.OwnerName))    // コンテナ一覧からオーナー名を検索
                        {
                            if (System.IO.Path.GetExtension(ctn.Path).Equals(".rta"))   // ファイル拡張子をチェック
                            {
                                info.Type = true;               // INtimeと判断
                            }
                            break;
                        }
                    }

                    return info;
                }
                else                // エラーあり
                {
                    LOG("GetClctProperty Error: " + rtn.ToString());
                    return null;
                }
            }
            catch (Exception ex)
            {
                LOG(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return null;
            }
        }

        /// <summary>
        /// コレクター情報更新
        /// </summary>
        public void UpdateCollectorVal(ref ClctList _list)
        {
            try
            {
                int rtn;
                foreach (var clctItem in _list)
                {
                    rtn = GetMailboxInfo(clctItem.Name, out uint writePos, out uint rcvDataNum);
                    if (rtn == 0)
                    {
                        clctItem.rec.WritePos = writePos;
                        clctItem.rec.RcvDataNum = rcvDataNum;
                        clctItem.Refresh();
                    }
                }
            }
            catch (Exception ex)
            {
                LOG(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
            }
        }
        #endregion

        #region Dataset

        /// <summary>
        /// データセット数取得
        /// </summary>
        /// <returns></returns>
        public uint GetDsetListCount()
        {
            uint itemNum = 0;
            int rtn = EGAPI.EgDatasetGetCount(ref itemNum);     // データセット総数取得
            if (rtn == 0)
                return itemNum;
            else
                return 0;
        }

        /// <summary>
        /// データセット一覧取得
        /// </summary>
        public void GetDsetList(ref DsetList _list)
        {
            try
            {
                uint itemNum = 0;
                int rtn = EGAPI.EgDatasetGetCount(ref itemNum);     // 総数取得
                if (rtn != 0) { return; }                           // エラー時は処理を抜ける

                edge_API.EGOBJINDEX[] idxArray = new edge_API.EGOBJINDEX[itemNum];
                rtn = EGAPI.EgDatasetGetEnum(ref idxArray, ref itemNum);    // 一覧取得
                if (rtn != 0) { return; }                           // エラー時は処理を抜ける

                for (int i = 0; i < itemNum; i++)                   // データセット数分処理ループ
                {
                    DsetInfo info = GetDsetProperty(idxArray[i]);   // データセット情報取得
                    if (info == null) { continue; }                 // エラー時は次のタグへ

                    _list.Add(new DsetItem(i, idxArray[i], info));  // リストへ追加
                }
                _list.Refresh();
            }
            catch (Exception ex)
            {
                LOG(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
            }
        }

        /// <summary>
        /// データセット情報(プロパティ)取得
        /// </summary>
        /// <param name="idx"></param>
        /// <returns></returns>
        public DsetInfo GetDsetProperty(edge_API.EGOBJINDEX idx)
        {
            try
            {
                int rtn = 0;
                object objName = new object();
                object objCount = new object();
                edge_API.TDSLINK dsl = new edge_API.TDSLINK();
                // プロパティ情報取得
                rtn |= EGAPI.EgDatasetGetPropertyByIndex(idx, "Name", ref objName);
                rtn |= EGAPI.EgDatasetGetPropertyByIndex(idx, "ItemNum", ref objCount);

                if (rtn == 0)       // エラーなし
                {
                    // データ格納
                    DsetInfo info = new DsetInfo();
                    info.Name = objName.ToString();
                    info.Count = (uint)objCount;
                    // データセット内のタグ情報取得
                    for (uint dsLinkCnt = 0; dsLinkCnt < info.Count; dsLinkCnt++)
                    {
                        if (dsLinkCnt == 0) { rtn = EGAPI.EgDatasetGetFirst(info.Name, ref dsl); }  // 先頭タグ取得
                        else { rtn = EGAPI.EgDatasetGetNext(ref dsl); }                             // 次のタグ取得

                        if (rtn == 0)
                        {
                            info.tagInfo.Add(GetTagProperty(dsl.pTagName)); // タグ名から詳細情報を取得してリストに追加
                        }        
                        else if ((uint)rtn == 0x80000503) { break; }        // 次のタグがない場合は終了
                        else { LOG(string.Format("# [ERROR] : EgDatasetGetFirst/Next Status:0x{0}\n", rtn.ToString("X"))); }
                    }
                    return info;
                }
                else                // エラーあり
                {
                    LOG("GetClctProperty Error: " + rtn.ToString());
                    return null;
                }
            }
            catch (Exception ex)
            {
                LOG(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return null;
            }
        }

        #endregion

        #region Mailbox

        /// <summary>
        /// メールボックス数取得
        /// </summary>
        /// <returns></returns>
        public uint GetMboxListCount()
        {
            uint itemNum = 0;
            int rtn = EGAPI.EgMailboxGetCount(ref itemNum);         // メールボックス総数取得
            if (rtn == 0)
                return itemNum;
            else
                return 0;
        }

        /// <summary>
        /// メールボックス一覧取得
        /// </summary>
        public void GetMboxList(ref MboxList _list)
        {
            try
            {
                uint itemNum = 0;
                int rtn = EGAPI.EgMailboxGetCount(ref itemNum);     // 総数取得
                if (rtn != 0) { return; }                           // エラー時は処理を抜ける

                edge_API.EGOBJINDEX[] idxArray = new edge_API.EGOBJINDEX[itemNum];
                rtn = EGAPI.EgMailboxGetEnum(ref idxArray, ref itemNum);// 一覧取得
                if (rtn != 0) { return; }                           // エラー時は処理を抜ける

                for (int i = 0; i < itemNum; i++)                   // メールボックス数分処理ループ
                {
                    MboxInfo info = GetMboxProperty(idxArray[i]);   // メールボックス情報取得
                    if (info == null) { continue; }                 // エラー時は次のタグへ

                    _list.Add(new MboxItem(i, idxArray[i], info));  // リストへ追加
                }
                _list.Refresh();
            }
            catch (Exception ex)
            {
                LOG(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
            }
        }

        /// <summary>
        /// メールボックス情報(プロパティ)取得
        /// </summary>
        /// <param name="idx"></param>
        /// <returns></returns>
        public MboxInfo GetMboxProperty(edge_API.EGOBJINDEX idx)
        {
            try
            {
                int rtn = 0;
                object objName = new object();
                object objColSize = new object();
                object objRowSize = new object();
                // プロパティ情報取得
                rtn |= EGAPI.EgMailboxGetPropertyByIndex(idx, "Name", ref objName);
                rtn |= EGAPI.EgMailboxGetPropertyByIndex(idx, "ColSize", ref objColSize);
                rtn |= EGAPI.EgMailboxGetPropertyByIndex(idx, "RowSize", ref objRowSize);

                if (rtn == 0)       // エラーなし
                {
                    // データ格納
                    MboxInfo info = new MboxInfo();
                    info.Name = objName.ToString();
                    info.ColSize = (uint)objColSize;
                    info.RowSize = (uint)objRowSize;
                    //if(info.Name.Length > 0)
                    //{
                    //    edge_API.RTCDHEADER stHeader = new edge_API.RTCDHEADER();
                    //    rtn |= EGAPI._EgMailboxGetInfo(info.Name, ref stHeader);
                    //    if(rtn == 0) //エラー無し
                    //    {
                    //        info.WritePos = stHeader.write_pos;
                    //    }
                    //}
                    return info;
                }
                else                // エラーあり
                {
                    LOG("GetMboxProperty Error: " + rtn.ToString());
                    return null;
                }
            }
            catch (Exception ex)
            {
                LOG(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return null;
            }
        }

        /// <summary>
        /// メールボックス値更新
        /// </summary>
        public void UpdateMboxVal(ref MboxList _list)
        {
            try
            {
                int rtn;
                foreach (var dsItem in _list)
                {
                    rtn = GetMailboxInfo(dsItem.Name, out uint writePos, out uint rcvDataNum);
                    if (rtn == 0)
                    {
                        dsItem.rec.WritePos = writePos;
                        dsItem.rec.RcvDataNum = rcvDataNum;
                        dsItem.Refresh();
                    }
                }
            }
            catch (Exception ex)
            {
                LOG(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
            }
        }

        #endregion

    }
}
