﻿////
/// Copyright(C) 2020 Micronet Co. All Rights Reserved.
//
//
//   Edgeサービス停止要求のサンプル
//
using RTedge;
using System;
using EgBrow;

namespace EgShDown
{
    /// <summary>
    /// RT-edge API/フレームワーク 各種値定義クラス
    /// </summary>
    public static class EDGECONST
    {
        /// <summary>
        /// 各種定義追加
        /// </summary>

        ////
        /// サービス名
        //
        public const string SERVICENAME = "EgShDown";                                           // [TODO](*******必須*******) : exe拡張子を除くプログラムファイル名に変更
        ////
        /// サービスインジケータ名
        //        
        public const string IND_MYSERV_STATUS = "SERVICE." + SERVICENAME + ".Status";           // サービス起動
        public const bool SERVICE_UP = true;                                                    // 起動中
        public const bool SERVICE_DOWN = false;                                                 // 停止中
        public const string IND_MYSERV_ERROR = "SERVICE." + SERVICENAME + ".Error";             // サービスエラー発生中
        public const string IND_MYSERV_RUN = "SERVICE." + SERVICENAME + ".Run";		            // サービス実行中
        public const string IND_MYSERV_LIVE = "SERVICE." + SERVICENAME + ".Live";               // サービス実行カウンタ
        ////
        /// サービスプロパティタグ名
        //        
        public const string PROP_MYSERV_MODE = "SERVICE." + SERVICENAME + ".Mode";              // サービス動作モード
        public const int MODE_AUTO = 1;                                                         // 入出自動サイクル
        public const int MODE_SEMIAUTO = 2;                                                     // 入力サイクル＋出力デマンド
        public const int MODE_MANUAL = 3;                                                       // 入出力デマンド
        public const int DEFAULT_MODE = MODE_SEMIAUTO;                                          // デフォルト値
        public const string PROP_MYSERV_CYCLE = "SERVICE." + SERVICENAME + ".Cycle";            // サービス動作サイクル(ms)
        public const int DEFAULT_CYCLE = 1;
        public const string PROP_MYSERV_INPRIO = "SERVICE." + SERVICENAME + ".InPriority";      // Edgeシステムへの入力サービススレッドプライオリティ
        public const int DEFAULT_INPRIO = 148;
        public const string PROP_MYSERV_OUTPRIO = "SERVICE." + SERVICENAME + ".OutPriority";    // Edgeシステムからの出力サービススレッドプライオリティ	
        public const int DEFAULT_OUTPRIO = 149;
        ////
        /// サービス提供タグ名
        //  [TODO] : 任意のタグ追加 (XMLで定義することを推奨)
        // public static string TAG_MYSERV_XXX = "XXX";
        ////
        /// サービス固有メッセージ
        //  [TODO] : 任意のメッセージID追加(ユーザメッセージ 20000～29999番指定可)
        // public static int EM_SERVICE_XXX = 20000;    //任意の番号
    }

    class Program
    {
        /// <summary>
        /// 指定されたEdgeサービスに停止指令をかけます
        /// Edgeフレームワークを持っているサービス(EgInit済)に作用します
        /// </summary>
        /// <param name="targetservicename">対象Edgeサービス名</param>
        /// <returns>正常終了時true</returns>
        static bool EgServiceShutdown(string targetservicename )
        {
            try
            {
                edge_API EGAPI = new edge_API();
                var ret = EGAPI.EgMailboxSendSC(targetservicename, "", (int)EGDEFINE.EGMSG.EM_SERVICE_STOP);     // サービス停止指令を発行
                return true;
            }
            catch (Exception)
            {
            }
            return false;
        }

        /// <summary>
        /// プログラムメイン
        /// </summary>
        /// <param name="args"></param>
        static void Main(string[] args)
        {
            try
            {
                Container   EgCon = new Container();
                ServList    ConList = new ServList();

                EgCon.GetServList(ref ConList);                 // コンテナリストを取得

                for( int i = ConList.Count; 0 < i ; i-- )
                {
                    EgServiceShutdown(ConList[(i - 1)].Name);    // サービス停止指令
                }
            }
            catch(Exception)
            {
            }
        }
    
    }

    #region Container
    public class Container
    {
        public edge_API EGAPI;

        /// <summary>
        /// コンストラクタ
        /// </summary>
        public Container()
        {
            EGAPI = new edge_API();
        }

        /// <summary>
        /// [DEBUG用] メッセージ出力
        /// </summary>
        /// <param name="message"></param>
        public void LOG(string message)
        {
            try
            {
                Console.WriteLine(EDGECONST.SERVICENAME + " : " + message);
            }
            catch
            { }
        }

        /// <summary>
        /// コンテナ数取得
        /// </summary>
        /// <returns></returns>
        public uint GetServListCount()
        {
            uint itemNum = 0;
            int rtn = EGAPI.EgServiceGetCount(ref itemNum);         // コンテナ総数取得
            if (rtn == 0)
                return itemNum;
            else
                return 0;
        }

        /// <summary>
        /// コンテナ一覧取得
        /// </summary>
        public void GetServList(ref ServList _list)
        {
            try
            {
                uint itemNum = 0;
                int rtn = EGAPI.EgServiceGetCount(ref itemNum);     // 総数取得
                if (rtn != 0) { return; }                           // エラー時は処理を抜ける

                edge_API.EGOBJINDEX[] idxArray = new edge_API.EGOBJINDEX[itemNum];
                rtn = EGAPI.EgServiceGetEnum(ref idxArray, ref itemNum);    // 一覧取得
                if (rtn != 0) { return; }                           // エラー時は処理を抜ける

                for (int i = 0; i < itemNum; i++)                   // コンテナ数分処理ループ
                {
                    ServInfo info = GetServProperty(idxArray[i]);   // コンテナ情報取得
                    if (info == null) { continue; }                 // エラー時は次のタグへ

                    _list.Add(new ServItem(i, idxArray[i], info));  // リストへ追加
                }
                _list.Refresh();
            }
            catch (Exception ex)
            {
                LOG(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
            }
        }

        /// <summary>
        /// コンテナ情報(プロパティ)取得
        /// </summary>
        /// <param name="idx"></param>
        /// <returns></returns>
        public ServInfo GetServProperty(edge_API.EGOBJINDEX idx)
        {
            try
            {
                int rtn = 0;
                object objName = new object();
                object objPath = new object();
                object objArg = new object();
                object objNodeName = new object();
                object objVarStatus = new object();
                object objVarError = new object();
                object objVarRun = new object();
                object objVarLive = new object();
                object objMboxName = new object();
                // プロパティ情報取得
                rtn |= EGAPI.EgServiceGetPropertyByIndex(idx, "Name", ref objName);
                rtn |= EGAPI.EgServiceGetPropertyByIndex(idx, "Path", ref objPath);
                rtn |= EGAPI.EgServiceGetPropertyByIndex(idx, "Arg", ref objArg);
                rtn |= EGAPI.EgServiceGetPropertyByIndex(idx, "NodeName", ref objNodeName);
                rtn |= EGAPI.EgServiceGetPropertyByIndex(idx, "MailboxName", ref objMboxName);
                // ステータスインジケーター取得
                rtn |= EGAPI.EgTagRead(string.Format("SERVICE.{0}.Status", objName.ToString()), ref objVarStatus);
                rtn |= EGAPI.EgTagRead(string.Format("SERVICE.{0}.Error", objName.ToString()), ref objVarError);
                rtn |= EGAPI.EgTagRead(string.Format("SERVICE.{0}.Run", objName.ToString()), ref objVarRun);
                rtn |= EGAPI.EgTagRead(string.Format("SERVICE.{0}.Live", objName.ToString()), ref objVarLive);
                if (rtn == 0)       // エラーなし
                {
                    // データ格納
                    ServInfo info = new ServInfo();
                    info.Name = objName.ToString();
                    info.Path = objPath.ToString();
                    info.Arg = objArg.ToString();
                    info.NodeName = objNodeName.ToString();
                    info.Var_Status = objVarStatus;
                    info.Var_Error = objVarError;
                    info.Var_Run = objVarRun;
                    info.Var_Live = objVarLive;
                    info.MboxName = objMboxName.ToString();
                    return info;
                }
                else                // エラーあり
                {
                    LOG("GetServProperty Error: " + rtn.ToString());
                    return null;
                }
            }
            catch (Exception ex)
            {
                LOG(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return null;
            }
        }

        /// <summary>
        /// コンテナ情報更新
        /// </summary>
        public void UpdateContainersVal(ref ServList _list)
        {
            try
            {
                object val = new object();
                foreach (var item in _list)
                {
                    EGAPI.EgTagRead(string.Format("SERVICE.{0}.Status", item.Name), ref val);   // Status
                    item.rec.Var_Status = val;
                    EGAPI.EgTagRead(string.Format("SERVICE.{0}.Error", item.Name), ref val);    // Error
                    item.rec.Var_Error = val;
                    EGAPI.EgTagRead(string.Format("SERVICE.{0}.Run", item.Name), ref val);      // Run
                    item.rec.Var_Run = val;
                    EGAPI.EgTagRead(string.Format("SERVICE.{0}.Live", item.Name), ref val);     // Live
                    item.rec.Var_Live = val;

                    item.Refresh();
                }
            }
            catch (Exception ex)
            {
                LOG(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
            }
        }

        #endregion
    }
}
