﻿////////////////////////////////////////////////////////////////////////////////////////
/// 機能       :   RT-edge XML編集ツール　編集機能 「追加」画面コードビハインド
/// 作成日     :   2022/02/21
/// 作成者     :   mizuno
/// 
/// RT-edge XML編集ツール 機能仕様書
/// file://192.168.1.240/Jobs/KA1601(RT-edge開発）/trunk/0051_Documnet(社内)/0200_egCoreService/20211021_RT-edge_XML編集ツール_機能仕様書.pptx
/// P14. 機能詳細 編集機能 「追加」 TagRef(IN/OUT)/CollectorRef
///
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;
using RTedgeXmlObj;

namespace EgConfig
{
    /// <summary>
    /// AddElemRef.xaml の相互作用ロジック
    /// </summary>
    public partial class AddElemRef : Window
    {
        private Define.Edit.AddElemType type;   // 追加項目タイプ
        private MainWindow mainWindow;          // メインウインドウの参照

        private Dataset dataset;

        /// <summary>
        /// プロパティ
        /// </summary>
        private string _elemType;
        public string ElemType
        {
            get { return _elemType; }
            set
            {
                _elemType = value;
            }
        }

        /// <summary>
        /// コンストラクタ
        /// </summary>
        /// <param name="type"></param>
        /// <param name="mainWindow"></param>
        /// <param name="dataset"></param>
        public AddElemRef(Define.Edit.AddElemType type, MainWindow mainWindow, Dataset dataset)
        {
            InitializeComponent();

            this.type = type;
            this.mainWindow = mainWindow;

            this.dataset = dataset;
            this.dataset.TagRefs = new List<TagRef>();

            DataContext = this;                                 // DataContextを登録
            ElemType = this.type.ToString();
            ElemRefLBX.ItemsSource = MainWindow.tagRefNames;    // リストボックスにデータを設定
        }
        /// <summary>
        /// コンストラクタ
        /// </summary>
        /// <param name="type"></param>
        /// <param name="mainWindow"></param>
        public AddElemRef(Define.Edit.AddElemType type, MainWindow mainWindow)
        {
            InitializeComponent();

            this.type = type;
            this.mainWindow = mainWindow;
            
            DataContext = this;                                 // DataContextを登録
            ElemType = this.type.ToString();

            ///////////////////////////////////////
            // リストボックスにデータを設定
            ///////////////////////////////////////
            if (this.type == Define.Edit.AddElemType.CollectorRef)
            {
                ElemRefLBX.ItemsSource = MainWindow.collectorNames;
            }
            else
            {
                ElemRefLBX.ItemsSource = MainWindow.tagRefNames;
            }
        }

        /// <summary>
        /// OKボタン押下時の処理
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void OkBtn_Click(object sender, RoutedEventArgs e)
        {
            switch (type)
            {
                case Define.Edit.AddElemType.Dataset:
                    AddDataset_TagRef();
                    break;
                case Define.Edit.AddElemType.DatasetTagRef:
                    AddDatasetTagRef();
                    break;
                case Define.Edit.AddElemType.TagRef:
                    AddTagRef();
                    break;
                case Define.Edit.AddElemType.TagRefIN:
                    AddTagRefIN();
                    break;
                case Define.Edit.AddElemType.TagRefOUT:
                    AddTagRefOUT();
                    break;
                case Define.Edit.AddElemType.CollectorRef:
                    AddCollectorRef();
                    break;
                default:
                    break;
            }

            
        }
        /// <summary>
        /// キャンセルボタン押下時の処理
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void CancelBtn_Click(object sender, RoutedEventArgs e)
        {
            switch (type)
            {
                case Define.Edit.AddElemType.Dataset:
                    AddDataset();
                    this.DialogResult = true;
                    break;
                default:
                    this.DialogResult = false;
                    break;
            }

            this.Close();                                       // 画面を閉じる
        }

        /// <summary>
        /// Datasetの追加処理
        /// </summary>
        private bool AddDataset()
        {
            bool isDone = false;

            // TreeItemで選択している項目を取得
            if (mainWindow.XmlTreeView.SelectedItem is TreeItem selectedItem)
            {
                int index = MainWindow.projMgmt.SearchEciIndex(selectedItem.Filename);  // ECIファイル名から管理しているECIファイルリストのIndexを取得

                // ECIファイル管理データにDatasetを追加
                MainWindow.projMgmt.AddDatasetElem(selectedItem, this.dataset);

                ////////////////////////////////////////////////////////
                // ECIファイル管理データから、追加したDatasetを取得
                ////////////////////////////////////////////////////////
                int datasetIndex = 0;
                // Serviceを取得
                TreeItem Service = selectedItem.SearchService();
                if (Service == null) //トップのDatasetの追加処理
                {
                    datasetIndex = MainWindow.projMgmt.eciList[index].eciData.Datasets.Count - 1;
                    this.dataset = MainWindow.projMgmt.eciList[index].eciData.Datasets[datasetIndex];

                }
                else
                { 
                    datasetIndex = MainWindow.projMgmt.eciList[index].eciData.Services[Service.Index].Datasets.Count - 1;
                    this.dataset = MainWindow.projMgmt.eciList[index].eciData.Services[Service.Index].Datasets[datasetIndex];
                }
                ////////////////////////////////////////////////////////
                // TreeItemにDatasetを追加
                ////////////////////////////////////////////////////////
                selectedItem.AddDataset(this.dataset);
            }
            
            return isDone;
        }

        /// <summary>
        /// Dataset/TagRefの追加処理
        /// </summary>
        private void AddDataset_TagRef()
        {
            // TreeItemで選択している項目を取得
            if (mainWindow.XmlTreeView.SelectedItem is TreeItem selectedItem)
            {
                int index = MainWindow.projMgmt.SearchEciIndex(selectedItem.Filename);
                Dataset dataset;

                ////////////////////////////////////////////////////////
                // 選択項目からTagRef取得し、Datasetに設定
                ////////////////////////////////////////////////////////
                foreach (TagRef tagRef in ElemRefLBX.SelectedItems)
                {
                    this.dataset.TagRefs.Add(tagRef);
                }

                // ECIファイル管理データにDatasetを追加
                MainWindow.projMgmt.AddDatasetElem(selectedItem, this.dataset);

                ////////////////////////////////////////////////////////
                // ECIファイル管理データから、追加したDatasetを取得
                ////////////////////////////////////////////////////////
                int datasetIndex = 0;
                // Serviceを取得
                TreeItem Service = selectedItem.SearchService();
                if (Service == null) //トップのDatasetの追加処理
                {
                    datasetIndex = MainWindow.projMgmt.eciList[index].eciData.Datasets.Count - 1;
                    dataset = MainWindow.projMgmt.eciList[index].eciData.Datasets[datasetIndex];

                }
                else
                {
                    datasetIndex = MainWindow.projMgmt.eciList[index].eciData.Services[Service.Index].Datasets.Count - 1;
                    dataset = MainWindow.projMgmt.eciList[index].eciData.Services[Service.Index].Datasets[datasetIndex];
                }

                ////////////////////////////////////////////////////////
                // TreeItemにDatasetを追加
                ////////////////////////////////////////////////////////
                selectedItem.AddDataset(dataset);

                this.DialogResult = true;
            }
            
            this.Close();
        }
        /// <summary>
        /// DatasetTagRef追加処理
        /// </summary>
        private void AddDatasetTagRef()
        {
            // TreeItemで選択している項目を取得
            if (mainWindow.XmlTreeView.SelectedItem is TreeItem selectedItem)
            {
                int index = MainWindow.projMgmt.SearchEciIndex(selectedItem.Filename);
                List<TagRef> tagRefs = new List<TagRef>();
                Dataset dataset = new Dataset();

                ////////////////////////////////////////////////////////
                // 選択項目からTagRef取得する
                ////////////////////////////////////////////////////////
                foreach (TagRef tagRef in ElemRefLBX.SelectedItems)
                {
                    tagRefs.Add(tagRef);
                }

                // ECIファイル管理データにDatasetTagRefを追加
                dataset = MainWindow.projMgmt.GetDataset(selectedItem);
                dataset.TagRefs.AddRange(tagRefs);

                ////////////////////////////////////////////////////////
                // Serviceのindex取得
                ////////////////////////////////////////////////////////
                int datasetIndex = selectedItem.Index;

                switch (selectedItem.ItemType)
                {
                    case (int)ItemType.DatasetTagRefs:
                        datasetIndex = selectedItem.Parent.Index;
                        break;
                    case (int)ItemType.DatasetTagRef:
                        datasetIndex = selectedItem.Parent.Parent.Index;
                        break;
                    default:
                        break;
                }

                selectedItem.AddDatasetTagRef(datasetIndex, tagRefs);    // TreeItemにDatasetを追加

                this.DialogResult = true;
            }
            
            this.Close();
        }
        /// <summary>
        /// TagRef追加処理
        /// </summary>
        private void AddTagRef()
        {
            // TreeItemで選択している項目を取得
            if (mainWindow.XmlTreeView.SelectedItem is TreeItem selectedItem)
            {
                int index = MainWindow.projMgmt.SearchEciIndex(selectedItem.Filename);
                List<TagRef> tagRefs = new List<TagRef>();
                Service service;

                ////////////////////////////////////////////////////////
                // 選択項目からTagRef取得する
                ////////////////////////////////////////////////////////
                foreach (TagRef tagRef in ElemRefLBX.SelectedItems)
                {
                    tagRefs.Add(tagRef);
                }

                ////////////////////////////////////////////////////////
                // 選択したTagRefを追加する
                ////////////////////////////////////////////////////////
                service = MainWindow.projMgmt.GetService(selectedItem); // TreeViewで選択しているサービスを取得
                if(service.TagRefs == null)
                {
                    service.TagRefs = new List<TagRef>();
                }
                service.TagRefs.AddRange(tagRefs);

                ////////////////////////////////////////////////////////
                // Serviceのindex取得
                ////////////////////////////////////////////////////////
                int serviceIndex = selectedItem.Index;
                switch (selectedItem.ItemType)
                {
                    case (int)ItemType.TagRefs:
                        serviceIndex = selectedItem.Parent.Index;
                        break;
                    case (int)ItemType.TagRef:
                        serviceIndex = selectedItem.Parent.Parent.Index;
                        break;
                    default:
                        break;
                }

                selectedItem.AddTagRef(serviceIndex, tagRefs);   // TreeItemにTagRefを追加する

                this.DialogResult = true;
            }
            
            this.Close();
            
        }
        /// <summary>
        /// TagRefIN追加処理
        /// </summary>
        private void AddTagRefIN()
        {
            // TreeItemで選択している項目を取得
            if (mainWindow.XmlTreeView.SelectedItem is TreeItem selectedItem)
            {
                int index = MainWindow.projMgmt.SearchEciIndex(selectedItem.Filename);
                List<TagRef> tagRefs = new List<TagRef>();
                Service service = new Service();

                ////////////////////////////////////////////////////////
                // 選択項目からTagRef取得する
                ////////////////////////////////////////////////////////
                foreach (TagRef tagRef in ElemRefLBX.SelectedItems)
                {
                    tagRefs.Add(tagRef);
                }

                ////////////////////////////////////////////////////////
                // ECIファイル管理データのTagRefINに追加する
                ////////////////////////////////////////////////////////
                service = MainWindow.projMgmt.GetService(selectedItem);     // TreeViewで選択しているサービスを取得
                if (service.TagRefs_IN == null)
                {
                    service.TagRefs_IN = new List<TagRef>();
                }
                service.TagRefs_IN.AddRange(tagRefs);                       // 選択したTagRefを追加する

                ////////////////////////////////////////////////////////
                // Serviceのindex取得
                ////////////////////////////////////////////////////////
                int serviceIndex = selectedItem.Index;
                switch (selectedItem.ItemType)
                {
                    case (int)ItemType.TagRefsIN:
                        serviceIndex = selectedItem.Parent.Index;
                        break;
                    case (int)ItemType.TagRefIN:
                        serviceIndex = selectedItem.Parent.Parent.Index;
                        break;
                    default:
                        break;
                }

                selectedItem.AddTagRefIN(serviceIndex, tagRefs); // TreeItemにTagRefINを追加

                this.DialogResult = true;
            }
            
            this.Close();
        }
        /// <summary>
        /// TagRefOUT追加処理
        /// </summary>
        private void AddTagRefOUT()
        {
            // TreeItemで選択している項目を取得
            if (mainWindow.XmlTreeView.SelectedItem is TreeItem selectedItem)
            {
                int index = MainWindow.projMgmt.SearchEciIndex(selectedItem.Filename);
                List<TagRef> tagRefs = new List<TagRef>();
                Service service = new Service();

                ////////////////////////////////////////////////////////
                // 選択項目からTagRef取得する
                ////////////////////////////////////////////////////////
                foreach (TagRef tagRef in ElemRefLBX.SelectedItems)
                {
                    tagRefs.Add(tagRef);
                }

                ////////////////////////////////////////////////////////
                // ECIファイル管理データのTagRefOUTに追加する
                ////////////////////////////////////////////////////////
                service = MainWindow.projMgmt.GetService(selectedItem);     // TreeViewで選択しているサービスを取得
                if (service.TagRefs_OUT == null)
                {
                    service.TagRefs_OUT = new List<TagRef>();
                }
                service.TagRefs_OUT.AddRange(tagRefs);                      // 選択したTagRefを追加する

                ////////////////////////////////////////////////////////
                // Serviceのindex取得
                ////////////////////////////////////////////////////////
                int serviceIndex = selectedItem.Index;
                switch (selectedItem.ItemType)
                {
                    case (int)ItemType.TagRefsOUT:
                        serviceIndex = selectedItem.Parent.Index;
                        break;
                    case (int)ItemType.TagRefOUT:
                        serviceIndex = selectedItem.Parent.Parent.Index;
                        break;
                    default:
                        break;
                }

                selectedItem.AddTagRefOUT(serviceIndex, tagRefs);    // TreeItemにTagRefOUTを追加

                this.DialogResult = true;
            }
            
            this.Close();
        }
        /// <summary>
        /// CollectorRef追加処理
        /// </summary>
        private void AddCollectorRef()
        {
            // TreeItemで選択している項目を取得
            if (mainWindow.XmlTreeView.SelectedItem is TreeItem selectedItem)
            {
                int index = MainWindow.projMgmt.SearchEciIndex(selectedItem.Filename);
                List<CollectorRef> collectorRefs = new List<CollectorRef>();
                Service service = new Service();

                ////////////////////////////////////////////////////////
                // 選択項目からCollectorRefを取得する
                ////////////////////////////////////////////////////////
                foreach (Collector collector in ElemRefLBX.SelectedItems)
                {
                    collectorRefs.Add(new CollectorRef(collector.NAME));
                }

                ////////////////////////////////////////////////////////
                // ECIファイル管理データのCollectorRefsに追加する
                ////////////////////////////////////////////////////////
                service = MainWindow.projMgmt.GetService(selectedItem);     // TreeViewで選択しているサービスを取得
                if (service.CollectorRefs == null)
                {
                    service.CollectorRefs = new List<CollectorRef>();
                }
                service.CollectorRefs.AddRange(collectorRefs);              // 選択したCollectorRefを追加する

                ////////////////////////////////////////////////////////
                // Serviceのindex取得
                ////////////////////////////////////////////////////////
                int serviceIndex = selectedItem.Index;
                switch (selectedItem.ItemType)
                {
                    case (int)ItemType.CollectorRefs:
                        serviceIndex = selectedItem.Parent.Index;
                        break;
                    case (int)ItemType.CollectorRef:
                        serviceIndex = selectedItem.Parent.Parent.Index;
                        break;
                    default:
                        break;
                }

                selectedItem.AddCollectorRef(serviceIndex, collectorRefs);    // TreeItemにCollectorRefを追加

                this.DialogResult = true;
            }
            
            this.Close();
            
        }
    }
}
