﻿////////////////////////////////////////////////////////////////////////////////////////
/// 機能       :   RT-edge XML編集ツール　編集機能 「追加」画面コードビハインド
/// 作成日     :   2022/02/21
/// 作成者     :   mizuno
/// 
/// RT-edge XML編集ツール 機能仕様書
/// file://192.168.1.240/Jobs/KA1601(RT-edge開発）/trunk/0051_Documnet(社内)/0200_egCoreService/20211021_RT-edge_XML編集ツール_機能仕様書.pptx
/// P14. 機能詳細 編集機能 「追加」 Tag /Dataset/Collector/Mailbox/Service
/// 
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;
using RTedgeXmlObj;
using System.IO;
using System.Windows.Forms;

namespace EgConfig
{
    /// <summary>
    /// AddElement.xaml の相互作用ロジック
    /// </summary>
    public partial class AddElement : Window
    {
        private Define.Edit.AddElemType type;   // 追加項目タイプ
        private MainWindow mainWindow;          // メインウインドウの参照

        private string _elemType;
        public string ElemType
        {
            get { return _elemType; }
            set
            {
                _elemType = value;
            }
        }

        /// <summary>
        /// コンストラクタ
        /// </summary>
        /// <param name="type"></param>
        /// <param name="mainWindow"></param>
        public AddElement(Define.Edit.AddElemType type, MainWindow mainWindow)
        {
            InitializeComponent();

            this.type = type;
            this.mainWindow = mainWindow;

            DataContext = this;                 // DataContextを登録
            ElemType = this.type.ToString();

            switch (this.type)
            {
                case Define.Edit.AddElemType.Tag:
                case Define.Edit.AddElemType.Dataset:
                case Define.Edit.AddElemType.Collector:
                case Define.Edit.AddElemType.Mailbox:
                    AddElementLbl.ContentStringFormat = "{0}を追加します。名前を入力してください。";
                    AddElementLbl.Content = ElemType;
                    break;
                case Define.Edit.AddElemType.Service:
                    AddElementLbl.Content = "サービスコンテナを追加します。サービスコンテナ名を入力してください。";
                    break;
                default:
                    break;
            }
        }


        /// <summary>
        /// OKボタンクリック
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void OkBtn_Click(object sender, RoutedEventArgs e)
        {
            bool isDone = false;

            /////////////////////////////////////
            // 追加項目ごとに処理を行う
            /////////////////////////////////////
            switch (type)
            {
                case Define.Edit.AddElemType.Tag:
                    isDone = AddTag();
                    break;
                case Define.Edit.AddElemType.Dataset:
                    isDone = AddDataset();
                    break;
                case Define.Edit.AddElemType.Collector:
                    isDone = AddCollector();
                    break;
                case Define.Edit.AddElemType.Mailbox:
                    isDone = AddMailbox();
                    break;
                case Define.Edit.AddElemType.Service:
                    isDone = AddService();
                    break;
                default:
                    break;
            }
            
            if(isDone == true)
            {
                this.DialogResult = true;
                this.Close();
            }
        }

        private void CancelBtn_Click(object sender, RoutedEventArgs e)
        {
            this.Close();   // 画面を閉じる
        }

        /// <summary>
        /// Tagの追加
        /// </summary>
        private bool AddTag()
        {
            bool isDone = false;
            int errorCode;

            // TreeItemで選択している項目を取得
            if (mainWindow.XmlTreeView.SelectedItem is TreeItem selectedItem)
            {
                Tag tag = new Tag();

                // Tag要素Name属性の入力チェック
                errorCode = CheckConsistencyOfElem.TagName(ElementNameTBX.Text);
                if (errorCode == Define.ErrorCode.NULL)
                {
                    System.Windows.Forms.MessageBox.Show(EgConfig.Properties.Resources.MSGBOX_ERROR_000_MSGTEXT, EgConfig.Properties.Resources.MSGBOX_ERROR_000_CAPTION, MessageBoxButtons.OK, MessageBoxIcon.Hand);
                    return isDone;
                }
                else if (errorCode == Define.ErrorCode.EXCEPT_HWAC)
                {
                    System.Windows.Forms.MessageBox.Show(EgConfig.Properties.Resources.MSGBOX_ERROR_001_MSGTEXT, EgConfig.Properties.Resources.MSGBOX_ERROR_001_CAPTION, MessageBoxButtons.OK, MessageBoxIcon.Hand);
                    return isDone;
                }
                else if (errorCode == Define.ErrorCode.OVER_BYTE)
                {
                    System.Windows.Forms.MessageBox.Show(String.Format(EgConfig.Properties.Resources.MSGBOX_ERROR_002_MSGTEXT, "48"), EgConfig.Properties.Resources.MSGBOX_ERROR_002_CAPTION, MessageBoxButtons.OK, MessageBoxIcon.Hand);
                    return isDone;
                }


                tag.NAME = ElementNameTBX.Text;
                tag.VALUE = "0";
                MainWindow.projMgmt.AddTag(selectedItem, tag); // 管理データにTag追加
                selectedItem.AddTag2Tags(tag);

                isDone = true;
            }
            
            return isDone;
        }

        /// <summary>
        /// データセットの追加
        /// </summary>
        private bool AddDataset()
        {
            bool isDone = false;
            int errorCode;
            Dataset dataset = new Dataset();

            // Dataset要素Name属性の入力チェック
            errorCode = CheckConsistencyOfElem.DatasetName(ElementNameTBX.Text);
            if (errorCode == Define.ErrorCode.NULL)
            {
                System.Windows.Forms.MessageBox.Show(EgConfig.Properties.Resources.MSGBOX_ERROR_000_MSGTEXT, EgConfig.Properties.Resources.MSGBOX_ERROR_000_CAPTION, MessageBoxButtons.OK, MessageBoxIcon.Hand);
                return isDone;
            }
            else if (errorCode == Define.ErrorCode.EXCEPT_HWAC)
            {
                System.Windows.Forms.MessageBox.Show(EgConfig.Properties.Resources.MSGBOX_ERROR_001_MSGTEXT, EgConfig.Properties.Resources.MSGBOX_ERROR_001_CAPTION, MessageBoxButtons.OK, MessageBoxIcon.Hand);
                return isDone;
            }
            else if (errorCode == Define.ErrorCode.OVER_BYTE)
            {
                System.Windows.Forms.MessageBox.Show(String.Format(EgConfig.Properties.Resources.MSGBOX_ERROR_002_MSGTEXT, "32"), EgConfig.Properties.Resources.MSGBOX_ERROR_002_CAPTION, MessageBoxButtons.OK, MessageBoxIcon.Hand);
                return isDone;
            }

            dataset.NAME = ElementNameTBX.Text;

            AddElemRef addElemRef = new AddElemRef(Define.Edit.AddElemType.Dataset, mainWindow, dataset);
            addElemRef.ShowDialog();
            if(addElemRef.DialogResult == true)
            {
                
            }

            isDone = true;
            return isDone;
        }
        /// <summary>
        /// Collectorの追加
        /// </summary>
        private bool AddCollector()
        {
            bool isDone = false;
            int errorCode;

            TreeItem selectedItem = (TreeItem)mainWindow.XmlTreeView.SelectedItem;  // TreeItemで選択している項目を取得
            int index = MainWindow.projMgmt.SearchEciIndex(selectedItem.Filename);  // ECIファイル名から管理しているECIファイルリストのIndexを取得
            Service service;
            
            if(selectedItem == null)
            {
                return isDone;
            }

            // Collector要素Name属性の入力チェック
            errorCode = CheckConsistencyOfElem.CollectorName(ElementNameTBX.Text);
            if (errorCode == Define.ErrorCode.NULL)
            {
                System.Windows.Forms.MessageBox.Show(EgConfig.Properties.Resources.MSGBOX_ERROR_000_MSGTEXT, EgConfig.Properties.Resources.MSGBOX_ERROR_000_CAPTION, MessageBoxButtons.OK, MessageBoxIcon.Hand);
                return isDone;
            }
            else if (errorCode == Define.ErrorCode.EXCEPT_HWAC)
            {
                System.Windows.Forms.MessageBox.Show(EgConfig.Properties.Resources.MSGBOX_ERROR_001_MSGTEXT, EgConfig.Properties.Resources.MSGBOX_ERROR_001_CAPTION, MessageBoxButtons.OK, MessageBoxIcon.Hand);
                return isDone;
            }
            else if (errorCode == Define.ErrorCode.OVER_BYTE)
            {
                System.Windows.Forms.MessageBox.Show(String.Format(EgConfig.Properties.Resources.MSGBOX_ERROR_002_MSGTEXT, "8"), EgConfig.Properties.Resources.MSGBOX_ERROR_002_CAPTION, MessageBoxButtons.OK, MessageBoxIcon.Hand);
                return isDone;
            }

            //////////////////////////////////////
            // 内部データに追加
            //////////////////////////////////////
            Collector collector = new Collector();  // Collector作成
            collector.NAME = ElementNameTBX.Text;

            // Serviceの取得
            service = MainWindow.projMgmt.GetService(selectedItem);     // TreeViewで選択しているサービスを取得
            if(service.Collectors == null)
            {
                service.Collectors = new List<Collector>();
            }
            service.Collectors.Add(collector);                          // 選択したサービスにCollectorを追加する

            //////////////////////////////////////
            // TreeItemに追加
            //////////////////////////////////////
            int serviceIndex = selectedItem.Index;

            switch (selectedItem.ItemType)
            {
                case (int)ItemType.Collectors:
                    serviceIndex = selectedItem.Parent.Index;
                    break;
                case (int)ItemType.Collector:
                    serviceIndex = selectedItem.Parent.Parent.Index;
                    break;
                default:
                    break;
            }

            selectedItem.AddCollector(serviceIndex, collector);

            isDone = true;
            return isDone;
        }
        /// <summary>
        /// Mailboxの追加
        /// </summary>
        private bool AddMailbox()
        {
            bool isDone = false;
            int errorCode;

            // TreeItemで選択している項目を取得
            if (mainWindow.XmlTreeView.SelectedItem is TreeItem selectedItem)
            {
                MailBox mailbox = new MailBox();                            // Mailbox作成

                // MailBox要素Name属性の入力チェック
                errorCode = CheckConsistencyOfElem.MailboxName(ElementNameTBX.Text);
                if (errorCode == Define.ErrorCode.NULL)
                {
                    System.Windows.Forms.MessageBox.Show(EgConfig.Properties.Resources.MSGBOX_ERROR_000_MSGTEXT, EgConfig.Properties.Resources.MSGBOX_ERROR_000_CAPTION, MessageBoxButtons.OK, MessageBoxIcon.Hand);
                    return isDone;
                }
                else if (errorCode == Define.ErrorCode.EXCEPT_HWAC)
                {
                    System.Windows.Forms.MessageBox.Show(EgConfig.Properties.Resources.MSGBOX_ERROR_001_MSGTEXT, EgConfig.Properties.Resources.MSGBOX_ERROR_001_CAPTION, MessageBoxButtons.OK, MessageBoxIcon.Hand);
                    return isDone;
                }
                else if (errorCode == Define.ErrorCode.OVER_BYTE)
                {
                    System.Windows.Forms.MessageBox.Show(String.Format(EgConfig.Properties.Resources.MSGBOX_ERROR_002_MSGTEXT, "8"), EgConfig.Properties.Resources.MSGBOX_ERROR_002_CAPTION, MessageBoxButtons.OK, MessageBoxIcon.Hand);
                    return isDone;
                }

                mailbox.NAME = ElementNameTBX.Text;
                MainWindow.projMgmt.AddMailbox(selectedItem, mailbox);
                selectedItem.AddMailbox2Mailboxes(mailbox);

                isDone = true;
            }

            return isDone;
        }
        /// <summary>
        /// Serviceの追加
        /// </summary>
        private bool AddService()
        {
            bool isDone = false, isManaged;
            string stServiceName = "";
            int errorCode;
            TreeItem selectedItem = (TreeItem)mainWindow.XmlTreeView.SelectedItem;  // TreeItemで選択している項目を取得

            string filename, eciFilename;
            AddService addService;                      // ダイアログ
            eciFilename = ElementNameTBX.Text + ".xml"; // 入力したECIファイル名

            if (selectedItem == null)                                                // TreeViewのアイテム未選択時は追加処理を行わない
            {
                return isDone;
            }

            // Service要素Name属性の入力チェック
            if (ElementNameTBX.Text.Length > 0 && ElementNameTBX.Text != "*") //名前がある場合は従来通のチェック 無い場合は共通設定とする(名前が*の場合も同様)
            {
                errorCode = CheckConsistencyOfElem.ServiceName(ElementNameTBX.Text);
                if (errorCode == Define.ErrorCode.NULL)
                {
                    System.Windows.Forms.MessageBox.Show(EgConfig.Properties.Resources.MSGBOX_ERROR_000_MSGTEXT, EgConfig.Properties.Resources.MSGBOX_ERROR_000_CAPTION, MessageBoxButtons.OK, MessageBoxIcon.Hand);
                    return false;
                }
                else if (errorCode == Define.ErrorCode.EXCEPT_HWAC)
                {
                    System.Windows.Forms.MessageBox.Show(EgConfig.Properties.Resources.MSGBOX_ERROR_001_MSGTEXT, EgConfig.Properties.Resources.MSGBOX_ERROR_001_CAPTION, MessageBoxButtons.OK, MessageBoxIcon.Hand);
                    return false;
                }
                else if (errorCode == Define.ErrorCode.OVER_BYTE)
                {
                    System.Windows.Forms.MessageBox.Show(String.Format(EgConfig.Properties.Resources.MSGBOX_ERROR_002_MSGTEXT, Define.DEF_EG_MAX_SERVICENAME), EgConfig.Properties.Resources.MSGBOX_ERROR_002_CAPTION, MessageBoxButtons.OK, MessageBoxIcon.Hand);
                    return false;
                }


                // ファイルに使用できない文字が含まれていないか確認する
                if (0 <= ElementNameTBX.Text.IndexOfAny(System.IO.Path.GetInvalidFileNameChars()))
                {
                    System.Windows.Forms.MessageBox.Show(Properties.Resources.MSGBOX_ERROR_009_MSGTEXT, Properties.Resources.MSGBOX_ERROR_009_CAPTION, MessageBoxButtons.OK, MessageBoxIcon.Hand);
                    return isDone;
                }

                ///////////////////////////////////////////////////////////////
                // プロジェクトを開いている場合、管理するECIファイルを追加
                ///////////////////////////////////////////////////////////////
                if ((mainWindow.State & Define.MASK_STATE) == Define.STATE_PROJ)
                {
                    // ファイルパス作成
                    try
                    {
                        filename = System.IO.Path.GetDirectoryName(MainWindow.projMgmt.eciProjName);
                        filename = System.IO.Path.Combine(filename, eciFilename);
                    }
                    catch (Exception)
                    {
                        return false;
                    }

                    // Serviceと同名のECIファイルをプロジェクトで管理していない場合、読み込みを行う
                    isManaged = MainWindow.projMgmt.SearchListForSameName(eciFilename);
                    if (isManaged == false)
                    {
                        // ファイル存在有無確認、有ならそのECIファイルを読み込み、無いならECIファイル作成し読み込み
                        if (File.Exists(filename) == true)
                        {
                            addService = new AddService(EgConfig.AddService.Mode.AddXML, filename, mainWindow);     // 追加しますかダイアログ表示
                            addService.WindowStartupLocation = WindowStartupLocation.CenterScreen;
                            addService.ShowDialog();
                        }
                        else
                        {
                            addService = new AddService(EgConfig.AddService.Mode.CreateXML, filename, mainWindow);  // 新規作成するかダイアログ表示
                            addService.WindowStartupLocation = WindowStartupLocation.CenterScreen;
                            addService.ShowDialog();

                            if (addService.DialogResult == true)
                            {
                                addService = new AddService(EgConfig.AddService.Mode.AddedXML, filename, mainWindow);// 新規作成し、プロジェクトに追加したダイアログ表示
                                addService.WindowStartupLocation = WindowStartupLocation.CenterScreen;
                                addService.ShowDialog();
                            }

                        }
                    }
                }
                stServiceName = ElementNameTBX.Text;    //テキストボックスに記載の名前追加
            }
            else//共通名設定
            {
                //作成するか確認
                DialogResult dialogResult = System.Windows.Forms.MessageBox.Show(Properties.Resources.MSGBOX_WARN_010_MSGTEXT, Properties.Resources.MSGBOX_WARN_003_CAPTION, MessageBoxButtons.OKCancel, MessageBoxIcon.Exclamation);
                if (dialogResult != System.Windows.Forms.DialogResult.OK)
                {
                    return false;
                }
                
                //作成する場合は以下処理
                stServiceName = Define.COMMON_SERVICE_NAME;
                //共通設定が既にある場合はエラーとする
                TreeItem Services;
                // Servicesを検索する
                Services = selectedItem.SearchServices();
                for(int i=0; i<Services.Children.Count; i++)
                {
                    if(Services.Children[i].ItemType == (int)EgConfig.ItemType.Service)
                    {
                        if (Services.Children[i].Name == Define.COMMON_SERVICE_NAME)
                        {
                            System.Windows.Forms.MessageBox.Show(String.Format(EgConfig.Properties.Resources.MSGBOX_ERROR_022_MSGTEXT, Define.DEF_EG_MAX_SERVICENAME), EgConfig.Properties.Resources.MSGBOX_ERROR_002_CAPTION, MessageBoxButtons.OK, MessageBoxIcon.Hand);
                            return false;
                        }
                    }

                }
            }

            ///////////////////////////////////////////////////////////////
            // サービスを追加する
            ///////////////////////////////////////////////////////////////
            Service service = new Service()
            {
                Tags            = new List<Tag>(),
                Datasets        = new List<Dataset>(),
                TagRefs         = new List<TagRef>(),
                TagRefs_IN      =new List<TagRef>(),
                TagRefs_OUT     =new List<TagRef>(),
                Collectors      =new List<Collector>(),
                CollectorRefs   =new List<CollectorRef>(),
                MailBoxes       =new List<MailBox>()
            };
            service.NAME = stServiceName;


            MainWindow.projMgmt.AddService(selectedItem.Filename, service); // サービス管理データに追加

            // TreeItem更新
            selectedItem.AddService2Services(service);

            isDone = true;
            return isDone;
        }
    }
}
