﻿////////////////////////////////////////////////////////////////////////////////////////
/// 機能       :   RT-edge XML編集ツール　編集機能 「追加」画面コードビハインド
/// 作成日     :   2022/02/21
/// 作成者     :   mizuno
/// 
/// RT-edge XML編集ツール 機能仕様書
/// file://192.168.1.240/Jobs/KA1601(RT-edge開発）/trunk/0051_Documnet(社内)/0200_egCoreService/20211021_RT-edge_XML編集ツール_機能仕様書.pptx
/// P14. 機能詳細 編集機能 「追加」 Service
///
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;
using RTedgeXmlObj;
using EgBoot;

namespace EgConfig
{
    /// <summary>
    /// AddService.xaml の相互作用ロジック
    /// </summary>
    public partial class AddService : Window
    {
        private Mode mode;
        private string xmlFilename;
        private MainWindow mainWindow;          // メインウインドウの参照

        public enum Mode
        {
            AddXML = 0,
            CreateXML = 1,
            AddedXML = 2
        }

        public AddService(Mode mode, string xmlFilename, MainWindow mainWindow)
        {
            InitializeComponent();

            this.mode = mode;
            this.xmlFilename = xmlFilename;
            this.mainWindow = mainWindow;
            //////////////////////////////////////
            // modeにより追加する画面を変更
            //////////////////////////////////////
            switch (this.mode)
            {
                case Mode.AddXML:
                    AddXML();
                    break;
                case Mode.CreateXML:
                    CreateXML();
                    break;
                case Mode.AddedXML:
                    AddedXML();
                    break;
                default:
                    break;
            }
        }

        /// <summary>
        /// Serviceの既存追加画面
        /// </summary>
        private void AddXML()
        {
            string xmlFile = System.IO.Path.GetFileName(xmlFilename);

            //////////////////////////////////
            // 画面のレイアウト設定
            //////////////////////////////////
            ExplanationLbl.Content = xmlFile + "が既に存在しています。\nプロジェクトに追加しますか？";
            Yes_Btn.Visibility = Visibility.Visible;
            No_Btn.Visibility = Visibility.Visible;

            Yes_Btn.Click += new RoutedEventHandler(AddXML_YES__Click);
        }

        /// <summary>
        /// Serviceの新規追加画面
        /// </summary>
        private void CreateXML()
        {
            string xmlFile = System.IO.Path.GetFileName(xmlFilename);

            //////////////////////////////////
            // 画面のレイアウト設定
            //////////////////////////////////
            ExplanationLbl.Content = xmlFile + "を新規作成しますか？";
            Yes_Btn.Visibility = Visibility.Visible;
            No_Btn.Visibility = Visibility.Visible;

            Yes_Btn.Click += new RoutedEventHandler(CreateXML_YES_Click);
        }

        /// <summary>
        /// Serviceの画面追加完了表示
        /// </summary>
        private void AddedXML()
        {
            string xmlFile = System.IO.Path.GetFileName(xmlFilename);

            //////////////////////////////////
            // 画面のレイアウト設定
            //////////////////////////////////
            ExplanationLbl.Content = xmlFile + "を新規作成し、\nプロジェクトに追加しました。";
            Yes_Btn.Visibility = Visibility.Collapsed;
            No_Btn.Visibility = Visibility.Collapsed;
            OK_Btn.Visibility = Visibility.Visible;

            OK_Btn.Click += new RoutedEventHandler(OK_Btn_Click);
        }

        /// <summary>
        /// OKボタンクリック時
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void OK_Btn_Click(object sender, RoutedEventArgs e)
        {
            this.Close();       // 画面を閉じる
        }

        /// <summary>
        /// はいボタンクリック時、既存のxmlをプロジェクトに追加
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void AddXML_YES__Click(object sender, RoutedEventArgs e)
        {
            TreeItem treeItem;

            ////////////////////////////////////
            // 管理データに読み込み
            ////////////////////////////////////
            MainWindow.projMgmt.AddEciFile(xmlFilename);

            ////////////////////////////////////
            // TreeItem更新
            ////////////////////////////////////
            treeItem = TreeItem.EciFileMgmt2TreeItem(MainWindow.projMgmt.eciList[MainWindow.projMgmt.eciList.Count - 1]);       // 読み込んだECIファイルをTreeItemにする
            MainWindow.treeItems.Add(treeItem);                                                                                 // TreeItemに追加

            MainWindow.isFileChanged.Add(false);                                                                                // TreeItemの変更状態を追加


            this.Close();                                                                                                       // 画面を閉じる
        }

        /// <summary>
        /// xmlファイルを新規追加する
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void CreateXML_YES_Click(object sender, RoutedEventArgs e)
        {
            string xmlFile = System.IO.Path.GetFileName(xmlFilename);
            TreeItem treeItem;

            /////////////////////////////////
            // RTedgeデータ構造初期化
            /////////////////////////////////
            RTedge rtedge = new RTedge();
            rtedge.Tags = new List<Tag>();                      // Tags要素(リスト)の初期化
            rtedge.Datasets = new List<Dataset>();              // DataSets要素(リスト)の初期化
            rtedge.Services = new List<Service>();              // Services要素(リスト)の初期化

            Service service = new Service();                    // サービス(コンテナ)要素の初期化
            service.TagRefs = new List<TagRef>();
            service.TagRefs_IN = new List<TagRef>();
            service.TagRefs_OUT = new List<TagRef>();
            service.Collectors = new List<Collector>();
            service.CollectorRefs = new List<CollectorRef>();
            service.MailBoxes = new List<MailBox>();
            service.Name = xmlFile.Replace(".xml", "");         // サービス名は、入力したコンテナ名を使用する
            rtedge.Services.Add(service);                       // Services要素にServiceを追加する

            /////////////////////////////////
            // XMLファイル作成
            /////////////////////////////////
            MnXmlUtility.SaveToFile(xmlFilename, rtedge, typeof(RTedge));                                        // XMLシリアライズ

            // 管理データに読み込み
            MainWindow.projMgmt.AddEciFile(xmlFilename);

            // TreeView更新
            treeItem = TreeItem.EciFileMgmt2TreeItem(MainWindow.projMgmt.eciList[MainWindow.projMgmt.eciList.Count - 1]);       // 読み込んだECIファイルをTreeItemにする
            MainWindow.treeItems.Add(treeItem);                                                                                 // TreeItemに追加
            MainWindow.isFileChanged.Add(false);                                                                                // TreeItemの変更状態を追加

            this.DialogResult = true;
            this.Close();                                                                                                       // 画面を閉じる
        }

        /// <summary>
        /// いいえボタンクリック時、画面を閉じる
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void No_Btn_Click(object sender, RoutedEventArgs e)
        {
            this.Close();
        }
    }
}
