﻿////////////////////////////////////////////////////////////////////////////////////////
/// 機能       :   RT-edge XML編集ツール 起動順序設定画面のコードビハインド
/// 作成日     :   2022/02/21
/// 作成者     :   mizuno
/// 
/// RT-edge XML編集ツール 機能仕様書
/// file://192.168.1.240/Jobs/KA1601(RT-edge開発）/trunk/0051_Documnet(社内)/0200_egCoreService/20211021_RT-edge_XML編集ツール_機能仕様書.pptx
/// P10. 基本機能 起動順序設定
/// P21. 機能詳細 起動順序設定
/// 
using RTedgeXmlObj;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;

namespace EgConfig
{
    /// <summary>
    /// BootOrderSetting.xaml の相互作用ロジック
    /// </summary>
    public partial class BootOrderSetting : Window
    {
        private MainWindow mainWindow;
        private ObservableCollection<ListItem> listItems = new ObservableCollection<ListItem>();
        private TreeItem services;
        private EciFileMgmt eciFileMgmt;

        public BootOrderSetting(MainWindow mainWindow)
        {
            InitializeComponent();

            this.mainWindow = mainWindow;
            
            if(this.mainWindow.XmlTreeView.SelectedItem is TreeItem selectedItem)
            {
                // ボタンクリックイベント登録
                OK_btn.Click += OK_btn_Click;
                CANCEL_btn.Click += CANCEL_btn_Click;
                UpRow_btn.Click += UpRow_btn_Click;
                DownRow_btn.Click += DownRow_btn_Click;

                // Serviceデータ取得
                services = selectedItem.SearchForItemFromRoot(ItemType.Services);
                if(services is TreeItem)
                {
                    // listItemsにする
                    foreach(TreeItem ch in services.Children)
                    {
                        listItems.Add(new ListItem { Name = ch.Name, Data = ch });
                    }
                    // ListBoxにTreeItem設定
                    listBox.ItemsSource = listItems;
                    listBox.DisplayMemberPath = nameof(ListItem.Name);
                    listBox.SelectedValuePath = nameof(ListItem.Data);

                    // TreeViewで選択しているECIファイルの管理クラスを取得
                    this.eciFileMgmt = MainWindow.projMgmt.SearchEciFileMgmt(services.Filename);
                }
            }
        }

        /// <summary>
        /// リストボックスに表示する用
        /// </summary>
        class ListItem
        {
            public string Name { get; set; }
            public TreeItem Data { get; set; }
        }

        /// <summary>
        /// OKボタンクリック
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void OK_btn_Click(object sender, RoutedEventArgs e)
        {
            int i, j;

            //////////////////////////////////////////////////////////////////////////////////
            // リストボックスのアイテム順になるようにTreeItemのServiceの順番を入れ替える
            //////////////////////////////////////////////////////////////////////////////////
            for (i = 0; i < listItems.Count; i++)                   // リストボックスの参照用インデックス
            {
                for(j = i; j<services.Children.Count; j++)          // リストボックスに対応するサービスの検索用インデックス
                {
                    // リストボックスと同じインデックスにサービスを移動する
                    if (listItems[i].Data == services.Children[j])
                    {
                        if(i != j)  // 既に同じインデックスの場合移動は行わない
                        {
                            services.Children.Move(j, i);   // TreeItem移動

                            // Data移動
                            switch (MainWindow.projMgmt.GetItemData(services))
                            {
                                case List<Service> services:
                                    Service service = services[j];
                                    services.RemoveAt(j);
                                    services.Insert(i, service);
                                    break;
                                default:
                                    break;
                            }

                            // ファイル変更フラグをONにする
                            if (this.eciFileMgmt.isChanged == false)
                            {
                                // TreeItemの変更フラグをONにする
                                foreach (TreeItem treeItem in MainWindow.treeItems)
                                {
                                    if (treeItem.Filename == this.services.Filename)
                                    {
                                        treeItem.IsChanged = true;
                                    }
                                }

                                this.eciFileMgmt.isChanged = true;  
                            }
                        }
                        break;
                    }
                }
            }

            services.UpdateIndexOfChildren();   // 子要素のインデックスを更新する
            this.Close();                       // 画面を閉じる
        }

        /// <summary>
        /// CANCELボタンクリック
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void CANCEL_btn_Click(object sender, RoutedEventArgs e)
        {
            this.Close();
        }

        /// <summary>
        /// 項目を上に移動する
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void UpRow_btn_Click(object sender, RoutedEventArgs e)
        {
            int index;

            // リストボックスで選択しているインデックスを取得
            index = listBox.SelectedIndex;
            if (index <= 0) // 未選択または最初の項目選択時は以下の処理は行わない
            {
                return;
            }

            // 選択項目を上に移動する
            if(listBox.SelectedItem is ListItem selectedItem)
            {
                listItems.Move(index, index - 1);       // 項目を入れ替える
            }
            
        }

        /// <summary>
        /// 項目を下に移動する
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void DownRow_btn_Click(object sender, RoutedEventArgs e)
        {
            int index;

            index = listBox.SelectedIndex;
            if(0 <= index && index < listBox.Items.Count - 1 )
            {
                listItems.Move(index, index + 1);       // 項目を入れ替える
            }
        }
    }
}
