﻿////////////////////////////////////////////////////////////////////////////////////////
/// 機能       :   RT-edge XML編集ツール　新規ファイル作成画面コードビハインド
/// 作成日     :   2022/02/21
/// 作成者     :   mizuno
/// 
/// RT-edge XML編集ツール 機能仕様書
/// file://192.168.1.240/Jobs/KA1601(RT-edge開発）/trunk/0051_Documnet(社内)/0200_egCoreService/20211021_RT-edge_XML編集ツール_機能仕様書.pptx
/// P7. 基本機能 新規ファイル作成画面
/// 
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;
using System.Windows.Forms;                 // FolderBrowserDialog()使用
using System.IO;                            // StreamWriterクラス使用
using System.Xml.Serialization;             // XmlSerializerクラス使用
using RTedgeXmlObj;                         // ECIファイルのデータ構造定義使用
using EgBoot;                               // MnXmlUtilityクラス使用
using System.Collections.ObjectModel;

namespace EgConfig
{
    /// <summary>
    /// CreateNewFile.xaml の相互作用ロジック
    /// </summary>
    public partial class CreateNewFile : Window
    {
        public string Path { get; set; } = "";  // 保存先のファイルパス

        public CreateNewFile()
        {
            InitializeComponent();
            SaveFolderTextBox.Text = AppDomain.CurrentDomain.BaseDirectory; // 実行ファイルパスを設定する
        }


        public CreateNewFile(string dir)
        {
            InitializeComponent();

            SaveFolderTextBox.Text = dir;
            
            FolderLbl.Visibility = Visibility.Hidden;
            SaveFolderTextBox.Visibility = Visibility.Hidden;
            RefBtn.Visibility = Visibility.Hidden;
            
        }

        /// <summary>
        /// 参照ボタンクリックイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ReferenceBtn_Click(object sender, RoutedEventArgs e)
        {
            // フォルダ選択ダイアログを表示する
            using (FolderBrowserDialog fbd = new FolderBrowserDialog())
            {
                fbd.Description = "フォルダを選択してください";
                fbd.SelectedPath = SaveFolderTextBox.Text;
                fbd.ShowNewFolderButton = false;

                if (fbd.ShowDialog() != System.Windows.Forms.DialogResult.OK)
                {
                    return;
                }
                SaveFolderTextBox.Text = fbd.SelectedPath;

            }
        }

        /// <summary>
        /// OKボタンクリックイベント
        /// ----------------------------
        /// ECIファイルを作成します
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void OkBtn_Click(object sender, RoutedEventArgs e)
        {
            DialogResult dialogResult;
            //TODO:ECIファイル作成処理はECIファイル管理用クラスに移植
            string srSavePath;                                  // 保存先のファイルパス作成用
            int errorCode;

            ////////////////////////////////
            // 入力エラーチェック
            ////////////////////////////////
            // Service要素Name属性の入力チェック
            errorCode = CheckConsistencyOfElem.ServiceName(FileNameTextBox.Text);
            if (errorCode == Define.ErrorCode.NULL)
            {
                System.Windows.Forms.MessageBox.Show(EgConfig.Properties.Resources.MSGBOX_ERROR_000_MSGTEXT, EgConfig.Properties.Resources.MSGBOX_ERROR_000_CAPTION, MessageBoxButtons.OK, MessageBoxIcon.Hand);
                return;
            }
            else if (errorCode == Define.ErrorCode.EXCEPT_HWAC)
            {
                System.Windows.Forms.MessageBox.Show(EgConfig.Properties.Resources.MSGBOX_ERROR_001_MSGTEXT, EgConfig.Properties.Resources.MSGBOX_ERROR_001_CAPTION, MessageBoxButtons.OK, MessageBoxIcon.Hand);
                return;
            }
            else if (errorCode == Define.ErrorCode.OVER_BYTE)
            {
                System.Windows.Forms.MessageBox.Show(String.Format(EgConfig.Properties.Resources.MSGBOX_ERROR_002_MSGTEXT, "8"), EgConfig.Properties.Resources.MSGBOX_ERROR_002_CAPTION, MessageBoxButtons.OK, MessageBoxIcon.Hand);
                return;
            }

            // 保存先フォルダパスが空
            if (SaveFolderTextBox.Text == "")
            {
                System.Windows.Forms.MessageBox.Show(Properties.Resources.MSGBOX_ERROR_004_MSGTEXT, Properties.Resources.MSGBOX_ERROR_004_CAPTION, MessageBoxButtons.OK, MessageBoxIcon.Hand);
                return;
            }

            // ファイルに使用できない文字が含まれていないか確認する
            if (0 <= FileNameTextBox.Text.IndexOfAny(System.IO.Path.GetInvalidFileNameChars()))
            {
                System.Windows.Forms.MessageBox.Show(Properties.Resources.MSGBOX_ERROR_009_MSGTEXT, Properties.Resources.MSGBOX_ERROR_009_CAPTION, MessageBoxButtons.OK, MessageBoxIcon.Hand);
                return;
            }

            // ディレクトリが存在しない
            if (System.IO.Directory.Exists(SaveFolderTextBox.Text) == false)
            {
                System.Windows.Forms.MessageBox.Show(Properties.Resources.MSGBOX_ERROR_005_MSGTEXT, Properties.Resources.MSGBOX_ERROR_005_CAPTION, MessageBoxButtons.OK, MessageBoxIcon.Hand);
                return;
            }

            try
            {
                srSavePath = System.IO.Path.Combine(SaveFolderTextBox.Text, FileNameTextBox.Text + ".xml");         // ファイルパスを作成
            }
            catch (Exception)
            {
                // パスが不正
                return;
            }

            // 既に同名のファイルが存在している場合
            if(File.Exists(srSavePath) == true)
            {
                dialogResult = System.Windows.Forms.MessageBox.Show(Properties.Resources.MSGBOX_WARN_004_MSGTEXT, Properties.Resources.MSGBOX_WARN_004_CAPTION, MessageBoxButtons.OKCancel, MessageBoxIcon.Exclamation);
                if(dialogResult != System.Windows.Forms.DialogResult.OK)
                {
                    this.Close();
                    return;
                }
            }

            /////////////////////////////////
            // XMLファイル作成
            /////////////////////////////////
            RTedge rtedge = new RTedge();                       // RTedgeデータ構造初期化
            rtedge.Tags = new List<Tag>();                      // Tags要素(リスト)の初期化
            rtedge.Datasets = new List<Dataset>();              // DataSets要素(リスト)の初期化
            rtedge.Services = new List<Service>();              // Services要素(リスト)の初期化

            Service service = new Service();                    // サービス(コンテナ)要素の初期化
            service.TagRefs = new List<TagRef>();
            service.TagRefs_IN = new List<TagRef>();
            service.TagRefs_OUT = new List<TagRef>();
            service.Collectors = new List<Collector>();
            service.CollectorRefs = new List<CollectorRef>();
            service.MailBoxes = new List<MailBox>();
            service.Name = FileNameTextBox.Text;                // サービス名は、入力したコンテナ名を使用する
            rtedge.Services.Add(service);                       // Services要素にServiceを追加する

            MnXmlUtility.SaveToFile(srSavePath, rtedge, typeof(RTedge));                                        // XMLシリアライズ

            Path = System.IO.Path.GetFullPath(srSavePath);  
            this.DialogResult = true;   // 保存完了
            
            this.Close();   // ファイル新規作成画面を閉じる
        }

        /// <summary>
        /// CANCELボタンクリックイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void CancelBtn_Click(object sender, RoutedEventArgs e)
        {
            // ファイル新規作成画面を閉じる
            this.Close();
        }
    }
}
