﻿////////////////////////////////////////////////////////////////////////////////////////
/// 機能       :   RT-edge XML編集ツール　新規プロジェクト作成画面コードビハインド
/// 作成日     :   2022/02/21
/// 作成者     :   mizuno
/// 
/// RT-edge XML編集ツール 機能仕様書
/// file://192.168.1.240/Jobs/KA1601(RT-edge開発）/trunk/0051_Documnet(社内)/0200_egCoreService/20211021_RT-edge_XML編集ツール_機能仕様書.pptx
/// P7. 基本機能 新規プロジェクト作成画面
/// 
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;
using System.IO;
using System.Xml.Linq;                      // LINQtoXML
using System.Text.RegularExpressions;       // 正規表現クラス使用
using System.Windows.Forms;                 // FolderBrowserDialog()使用
using EgBoot;                               // XMLシリアライズ関数使用
using RTedgeXmlObj;                 // ECIファイルのデータ構造定義使用
using System.Collections.ObjectModel;

namespace EgConfig
{

    /// <summary>
    /// CreateNewProject.xaml の相互作用ロジック
    /// </summary>
    public partial class CreateNewProject : Window
    {
        public CreateNewProject()
        {
            InitializeComponent();

            ProjNameTextBox.Text = "";                                      // プロジェクト名をブランクに設定する
            SaveFolderTextBox.Text = AppDomain.CurrentDomain.BaseDirectory; // 実行ファイルパスを設定する
        }

        public CreateNewProject(string BaseDirectory = "")
        {
            InitializeComponent();

            ProjNameTextBox.Text = "";                                      // プロジェクト名をブランクに設定する
            if (BaseDirectory == "")
                SaveFolderTextBox.Text = AppDomain.CurrentDomain.BaseDirectory;     // 実行ファイルパスを設定する
            else
                SaveFolderTextBox.Text = BaseDirectory;                             // 指定パスを設定する
        }

        /// <summary>
        /// 参照ボタンクリックイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ReferenceBtn_Click(object sender, RoutedEventArgs e)
        {
            // フォルダ選択ダイアログを表示する
            using (FolderBrowserDialog fbd = new FolderBrowserDialog())
            {
                fbd.Description = "フォルダを選択してください";
                fbd.SelectedPath = SaveFolderTextBox.Text;
                fbd.ShowNewFolderButton = false;

                if(fbd.ShowDialog() != System.Windows.Forms.DialogResult.OK)
                {
                    return;
                }
                SaveFolderTextBox.Text = fbd.SelectedPath;

            }
        }

        /// <summary>
        /// CANCELボタンクリックイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void CancelBtn_Click(object sender, RoutedEventArgs e)
        {
            // プロジェクト新規作成画面を閉じる
            MainWindow.CreateNewProject.Close();
        }

        /// <summary>
        /// OKボタンクリックイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void OkBtn_Click(object sender, RoutedEventArgs e)
        {
            string srSavePath, projDir, filenames = "", egbootPath;
            ObservableCollection<TreeItem> items;
            DialogResult dialogResult;
            int errorCode;


            //////////////////////////////////////
            // プロジェクト名の入力チェック
            //////////////////////////////////////
            errorCode = CheckConsistencyOfElem.ProjectName(ProjNameTextBox.Text);
            if (errorCode == Define.ErrorCode.NULL)
            {
                System.Windows.Forms.MessageBox.Show(Properties.Resources.MSGBOX_ERROR_006_MSGTEXT, Properties.Resources.MSGBOX_ERROR_006_CAPTION, MessageBoxButtons.OK, MessageBoxIcon.Hand);
                return;
            }
            else if (errorCode == Define.ErrorCode.EXCEPT_HWAC)
            {
                System.Windows.Forms.MessageBox.Show(Properties.Resources.MSGBOX_ERROR_007_MSGTEXT, Properties.Resources.MSGBOX_ERROR_007_CAPTION, MessageBoxButtons.OK, MessageBoxIcon.Hand);
                return;
            }
            else if (errorCode == Define.ErrorCode.OVER_BYTE)
            {
                System.Windows.Forms.MessageBox.Show(Properties.Resources.MSGBOX_ERROR_008_MSGTEXT, Properties.Resources.MSGBOX_ERROR_008_CAPTION, MessageBoxButtons.OK, MessageBoxIcon.Hand);
                return;
            }

            // ファイルに使用できない文字が含まれていないか確認する
            if (0 <= ProjNameTextBox.Text.IndexOfAny(System.IO.Path.GetInvalidFileNameChars()))
            {
                System.Windows.Forms.MessageBox.Show(Properties.Resources.MSGBOX_ERROR_009_MSGTEXT, Properties.Resources.MSGBOX_ERROR_009_CAPTION, MessageBoxButtons.OK, MessageBoxIcon.Hand);
                return;
            }

            //////////////////////////////////////
            // 保存フォルダ名の入力チェック
            //////////////////////////////////////
            if(String.IsNullOrEmpty(SaveFolderTextBox.Text) == true)    // 保存フォルダ未入力
            {
                System.Windows.Forms.MessageBox.Show(Properties.Resources.MSGBOX_ERROR_004_MSGTEXT, Properties.Resources.MSGBOX_ERROR_004_CAPTION, MessageBoxButtons.OK, MessageBoxIcon.Hand);
                return;
            }
            else if (!Directory.Exists(SaveFolderTextBox.Text))         // 保存フォルダに存在しないパスが入力されている
            {
                System.Windows.Forms.MessageBox.Show(Properties.Resources.MSGBOX_ERROR_005_MSGTEXT, Properties.Resources.MSGBOX_ERROR_005_CAPTION, MessageBoxButtons.OK, MessageBoxIcon.Hand);
                return;
            }

            //////////////////////////////////////
            // プロジェクトフォルダ作成
            //////////////////////////////////////
            projDir = System.IO.Path.Combine(SaveFolderTextBox.Text, ProjNameTextBox.Text); // プロジェクトのディレクトリパス作成
            if (Directory.Exists(projDir) == true)                                          // プロジェクトが既に存在していた場合、プロジェクトを作成しない
            {
                dialogResult = System.Windows.Forms.MessageBox.Show(Properties.Resources.MSGBOX_ERROR_021_MSGTEXT, Properties.Resources.MSGBOX_ERROR_021_CAPTION, MessageBoxButtons.OK, MessageBoxIcon.Hand);
                return;
            }

            try
            {
                Directory.CreateDirectory(projDir); // プロジェクトフォルダ作成
            }
            catch (Exception)
            {
                return;
            }

            // 既に同名のプロジェクトが存在している場合、上書きするか確認
            srSavePath = System.IO.Path.Combine(projDir, ProjNameTextBox.Text + ".eciproj"); // プロジェクトのファイルパスを作成
            if (File.Exists(srSavePath) == true)
            {
                dialogResult = System.Windows.Forms.MessageBox.Show(Properties.Resources.MSGBOX_WARN_003_MSGTEXT, Properties.Resources.MSGBOX_WARN_003_CAPTION, MessageBoxButtons.OKCancel, MessageBoxIcon.Exclamation);
                if (dialogResult != System.Windows.Forms.DialogResult.OK)
                {
                    this.Close();
                    return;
                }
            }

            ///////////////////////////////////////
            // プロジェクトファイル作成処理
            ///////////////////////////////////////
            EciProjMgmt.CreateNewProject(srSavePath);

            ///////////////////////////////////////
            // EgBoot.xml作成処理
            ///////////////////////////////////////
            egbootPath = System.IO.Path.Combine(projDir, "EgBoot.xml");
            if (File.Exists(egbootPath) == true)
            {
                System.Windows.Forms.MessageBox.Show(Properties.Resources.MSGBOX_INFO_000_MSGTEXT, Properties.Resources.MSGBOX_INFO_000_CAPTION, MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
            }
            else
            {
                /////////////////////////////////
                // EgBoot.xmlファイル作成
                /////////////////////////////////
                RTedge rtedge = new RTedge();                       // RTedgeデータ構造初期化
                rtedge.Tags = new List<Tag>();                      // Tags要素(リスト)の初期化
                rtedge.Datasets = new List<Dataset>();              // DataSets要素(リスト)の初期化
                rtedge.Services = new List<Service>();              // Services要素(リスト)の初期化

                Service service = new Service();                    // サービス(コンテナ)要素の初期化
                service.TagRefs = new List<TagRef>();
                service.TagRefs_IN = new List<TagRef>();
                service.TagRefs_OUT = new List<TagRef>();
                service.Collectors = new List<Collector>();
                service.CollectorRefs = new List<CollectorRef>();
                service.MailBoxes = new List<MailBox>();
                service.Name = "EgBoot";                            // サービス名は、入力したコンテナ名を使用する
                rtedge.Services.Add(service);                       // Services要素にServiceを追加する

                MnXmlUtility.SaveToFile(egbootPath, rtedge, typeof(RTedge));
            }

            ////////////////////////////////////
            // プロジェクトファイル読み込み
            ////////////////////////////////////
            MainWindow.projMgmt = EciProjMgmt.ReadProjectFile(srSavePath, out List<string> readFailures);
            if (0 < readFailures.Count)
            {
                foreach (string readFailure in readFailures)
                {
                    try
                    {
                        filenames = filenames + System.IO.Path.GetFileName(readFailure) + "\n";
                    }
                    catch (Exception)
                    {
                        filenames = filenames + "" + "\n";
                    }
                }
                System.Windows.Forms.MessageBox.Show(String.Format(Properties.Resources.MSGBOX_ERROR_003_MSGTEXT, filenames), Properties.Resources.MSGBOX_ERROR_003_CAPTION, MessageBoxButtons.OK, MessageBoxIcon.Hand);
            }

            ////////////////////////////////////////////////
            // 読み込んだECIファイルをTreeViewに表示
            ////////////////////////////////////////////////
            items = TreeItem.EciProjMgmt2TreeItem(MainWindow.projMgmt);    // ECIプロジェクト管理データからTreeItemを作成
            MainWindow.treeItems.Clear();
            foreach (TreeItem item in items)
            {
                MainWindow.treeItems.Add(item);
            }

            // TreeItemの変更状態をなしにする
            MainWindow.isFileChanged.Clear();
            for (int i = 0; i < MainWindow.treeItems.Count; i++)
            {
                MainWindow.isFileChanged.Add(false);
            }

            //////////////////////////////////////////////////////////////////////
            // ファイル監視を有効にする
            //////////////////////////////////////////////////////////////////////
            MainWindow.fsw.Path = System.IO.Path.GetDirectoryName(MainWindow.projMgmt.eciProjName);
            MainWindow.fsw.EnableRaisingEvents = true;

            this.DialogResult = true;
            MainWindow.CreateNewProject.Close();    // 画面を閉じる
        }
    }

    
}
