﻿////////////////////////////////////////////////////////////////////////////////////////
/// 機能       :   RT-edge XML編集ツール　定義
/// 作成日     :   2022/02/21
/// 作成者     :   mizuno
/// 
/// RT-edge XML編集ツール 機能仕様書
/// file://192.168.1.240/Jobs/KA1601(RT-edge開発）/trunk/0051_Documnet(社内)/0200_egCoreService/20211021_RT-edge_XML編集ツール_機能仕様書.pptx
/// P38. プログラム仕様 エラーコード①
/// P39. プログラム仕様 エラーコード②
/// 
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace EgConfig
{
    /// <summary>
    /// XML編集ツール定義クラス
    /// </summary>
    public static class Define
    {
        public static string FILE_EXT_000 = "*.xml";
        // ファイル選択フィルター用
        public static string FILE_FILTER_000 = "ecipackファイル|*.ecipack";         // .ecipackファイル

        // 状態
        public static readonly int STATE_INI    = 0x0000;              // 初期
        public static readonly int STATE_PROJ   = 0x1000;              // プロジェクトファイル読み込み
        public static readonly int STATE_FILE   = 0x2000;              // ECIファイル読み込み

        public static readonly int MASK_STATE           = ~0x0FFF;         // 状態保持
        public static readonly int MASK_SELECTED_ECI    = 0x0001;          // TreeViewECI選択
        public static readonly int MASK_SELECTED_RTEDGE = 0x0002;          // TreeViewRTedge要素選択
        public static readonly int MASK_SELECTED_PER    = 0x0003;          // TreeView親要素選択
        public static readonly int MASK_SELECTED_CHL    = 0x0004;          // TreeView子要素選択

        // 値定義
        public static readonly int DEF_EG_MAX_SERVICENAME = 24;             // サービス名最大長

        // サービス共通設定用 サービス名
        public static string COMMON_SERVICE_NAME = "*";
        /// <summary>
        /// 状態に対するボタンの有効・無効定義辞書
        /// </summary>
        public static readonly Dictionary<int, ButtonVisibility> ButtonVisibilityDic = new Dictionary<int, ButtonVisibility>()
        {
            //  状態,                                                                                                                                    新規作成          プロジェクトの新規作成      ファイルの新規作成       開く            プロジェクトを開く     ファイルを開く      追加                新しいファイルの追加    既存のファイルの追加       保存                エクスポート       インポート        終了            追加                   削除                      名前の変更                上へ移動        下へ移動              昇順ソート          降順ソート          エラーチェック         起動順序設定          外部エディタ起動          設定               外部エディタ             バージョン
            /* 初期状態(ファイル未読み込み) */                           {   Define.STATE_INI,                                new ButtonVisibility(){    Create = true,    CreateNewProject = true,    CreateNewFile = true,    Open = true,    OpenProject = true,    OpenFile = true,    AddFile = false,    AddNewFile = false,    AddExistingFile = false,    SaveAll = false,    Export = false,    Import = true,    Exit = true,    AddElement = false,    RemoveElement = false,    RenameElement = false,    UpRow = false,    DownRow = false,    AscSort = false,    DesSort = false,    ErrorCheck = false,    BootOrder = false,    StartupEditor = false,    Setting = true,    SettingEditor = true,    Version = true }},
            /* プロジェクト読み込み状態:TreeItem未選択時 */              {   Define.STATE_PROJ,                               new ButtonVisibility(){    Create = true,    CreateNewProject = true,    CreateNewFile = true,    Open = true,    OpenProject = true,    OpenFile = true,    AddFile = true,     AddNewFile = true,     AddExistingFile = true,     SaveAll = true,     Export = true,     Import = true,    Exit = true,    AddElement = false,    RemoveElement = false,    RenameElement = false,    UpRow = false,    DownRow = false,    AscSort = false,    DesSort = false,    ErrorCheck = true,     BootOrder = false,    StartupEditor = false,    Setting = true,    SettingEditor = true,    Version = true }},
            /* プロジェクト読み込み状態:TreeItemECI要素選択時 */         {   Define.STATE_PROJ|Define.MASK_SELECTED_ECI,      new ButtonVisibility(){    Create = true,    CreateNewProject = true,    CreateNewFile = true,    Open = true,    OpenProject = true,    OpenFile = true,    AddFile = true,     AddNewFile = true,     AddExistingFile = true,     SaveAll = true,     Export = true,     Import = true,    Exit = true,    AddElement = true,     RemoveElement = true,     RenameElement = true,     UpRow = true,     DownRow = true,     AscSort = true,     DesSort = true,     ErrorCheck = true,     BootOrder = true,     StartupEditor = true,     Setting = true,    SettingEditor = true,    Version = true }},
            /* プロジェクト読み込み状態:TreeItemRTedge要素選択時 */      {   Define.STATE_PROJ|Define.MASK_SELECTED_RTEDGE,   new ButtonVisibility(){    Create = true,    CreateNewProject = true,    CreateNewFile = true,    Open = true,    OpenProject = true,    OpenFile = true,    AddFile = true,     AddNewFile = true,     AddExistingFile = true,     SaveAll = true,     Export = true,     Import = true,    Exit = true,    AddElement = true,     RemoveElement = false ,   RenameElement = false,    UpRow = false,    DownRow = false,    AscSort = false,    DesSort = false,    ErrorCheck = true,     BootOrder = true,     StartupEditor = true,     Setting = true,    SettingEditor = true,    Version = true }},
            /* プロジェクト読み込み状態:TreeItem親要素選択時 */          {   Define.STATE_PROJ|Define.MASK_SELECTED_PER,      new ButtonVisibility(){    Create = true,    CreateNewProject = true,    CreateNewFile = true,    Open = true,    OpenProject = true,    OpenFile = true,    AddFile = true,     AddNewFile = true,     AddExistingFile = true,     SaveAll = true,     Export = true,     Import = true,    Exit = true,    AddElement = true,     RemoveElement = true,     RenameElement = false,    UpRow = false,    DownRow = false,    AscSort = false,    DesSort = false,    ErrorCheck = true,     BootOrder = true,     StartupEditor = true,     Setting = true,    SettingEditor = true,    Version = true }},
            /* プロジェクト読み込み状態:TreeItem子要素選択時 */          {   Define.STATE_PROJ|Define.MASK_SELECTED_CHL,      new ButtonVisibility(){    Create = true,    CreateNewProject = true,    CreateNewFile = true,    Open = true,    OpenProject = true,    OpenFile = true,    AddFile = true,     AddNewFile = true,     AddExistingFile = true,     SaveAll = true,     Export = true,     Import = true,    Exit = true,    AddElement = true,     RemoveElement = true,     RenameElement = true,     UpRow = true,     DownRow = true,     AscSort = true,     DesSort = true,     ErrorCheck = true,     BootOrder = true,     StartupEditor = true,     Setting = true,    SettingEditor = true,    Version = true }},
            /* ECIファイル単体読み込み状態:TreeItem未選択時 */           {   Define.STATE_FILE,                               new ButtonVisibility(){    Create = true,    CreateNewProject = true,    CreateNewFile = true,    Open = true,    OpenProject = true,    OpenFile = true,    AddFile = false,    AddNewFile = false,    AddExistingFile = false,    SaveAll = true,     Export = false,    Import = true,    Exit = true,    AddElement = false,    RemoveElement = false,    RenameElement = false,    UpRow = false,    DownRow = false,    AscSort = false,    DesSort = false,    ErrorCheck = true,     BootOrder = false,    StartupEditor = false,    Setting = true,    SettingEditor = true,    Version = true }},
            /* ECIファイル単体読み込み状態:TreeItemECI要素選択時 */      {   Define.STATE_FILE|Define.MASK_SELECTED_ECI,      new ButtonVisibility(){    Create = true,    CreateNewProject = true,    CreateNewFile = true,    Open = true,    OpenProject = true,    OpenFile = true,    AddFile = false,    AddNewFile = false,    AddExistingFile = false,    SaveAll = true,     Export = false,    Import = true,    Exit = true,    AddElement = true,     RemoveElement = true,     RenameElement = true,     UpRow = true,     DownRow = true,     AscSort = true,     DesSort = true,     ErrorCheck = true,     BootOrder = true,     StartupEditor = true,     Setting = true,    SettingEditor = true,    Version = true }},
            /* ECIファイル単体読み込み状態:TreeItemRTedge要素選択時 */   {   Define.STATE_FILE|Define.MASK_SELECTED_RTEDGE,   new ButtonVisibility(){    Create = true,    CreateNewProject = true,    CreateNewFile = true,    Open = true,    OpenProject = true,    OpenFile = true,    AddFile = false,    AddNewFile = false,    AddExistingFile = false,    SaveAll = true,     Export = false,    Import = true,    Exit = true,    AddElement = true,     RemoveElement = false,    RenameElement = false,    UpRow = false,    DownRow = false,    AscSort = false,    DesSort = false,    ErrorCheck = true,     BootOrder = true,     StartupEditor = true,     Setting = true,    SettingEditor = true,    Version = true }},
            /* ECIファイル単体読み込み状態:TreeItem親要素選択時 */       {   Define.STATE_FILE|Define.MASK_SELECTED_PER,      new ButtonVisibility(){    Create = true,    CreateNewProject = true,    CreateNewFile = true,    Open = true,    OpenProject = true,    OpenFile = true,    AddFile = false,    AddNewFile = false,    AddExistingFile = false,    SaveAll = true,     Export = false,    Import = true,    Exit = true,    AddElement = true,     RemoveElement = true,     RenameElement = false,    UpRow = false,    DownRow = false,    AscSort = false,    DesSort = false,    ErrorCheck = true,     BootOrder = true,     StartupEditor = true,     Setting = true,    SettingEditor = true,    Version = true }},
            /* ECIファイル単体読み込み状態:TreeItem子要素選択時 */       {   Define.STATE_FILE|Define.MASK_SELECTED_CHL,      new ButtonVisibility(){    Create = true,    CreateNewProject = true,    CreateNewFile = true,    Open = true,    OpenProject = true,    OpenFile = true,    AddFile = false,    AddNewFile = false,    AddExistingFile = false,    SaveAll = true,     Export = false,    Import = true,    Exit = true,    AddElement = true,     RemoveElement = true,     RenameElement = true,     UpRow = true,     DownRow = true,     AscSort = true,     DesSort = true,     ErrorCheck = true,     BootOrder = true,     StartupEditor = true,     Setting = true,    SettingEditor = true,    Version = true }}
        };

        public static class TreeView
        {
            public static readonly string ITEM_NAME_002 = "Tags";
            public static readonly string ITEM_NAME_004 = "Datasets";
            public static readonly string ITEM_NAME_006 = "TagRefs";
            public static readonly string ITEM_NAME_008 = "TagRefs";
            public static readonly string ITEM_NAME_010 = "TagRefs_IN";
            public static readonly string ITEM_NAME_012 = "TagRefs_OUT";
            public static readonly string ITEM_NAME_014 = "Collectors";
            public static readonly string ITEM_NAME_016 = "CollectorRefs";
            public static readonly string ITEM_NAME_018 = "MailBoxes";
            public static readonly string ITEM_NAME_020 = "Services";
        }

        /// <summary>
        /// プロパティウインドウに関する定義
        /// </summary>
        public static class PropertyWindow
        {
            /// <summary>
            /// プロパティ表示ウインドウのタブ切り替え
            /// </summary>
            public static readonly int Tab_None = -1;
            public static readonly int Tab_Tag = 0;
            public static readonly int Tab_TagRef = 1;
            public static readonly int Tab_Container = 2;
            public static readonly int Tab_Collector = 3;
            public static readonly int Tab_CollectorRef = 4;
            public static readonly int Tab_MailBox = 5;
            public static readonly int Tab_Dataset = 6;

            public enum TagType
            {
                Boolean = 1,
                SByte,
                Byte,
                Int16,
                UInt16,
                Int32,
                UInt32,
                Int64,
                UInt64,
                Float,
                Double
            }

            public enum Enable
            {
                False   = 0,
                True    = 1
            }

            public enum Boolean
            {
                False = 0,
                True = 1
            }
        }

        /// <summary>
        /// エラーコード
        /// </summary>
        public static class ErrorCode
        {
            public static readonly int NONE = 0x0000;
            

            public static readonly int NULL = 0x0001;               // NULLである
            public static readonly int EXCEPT_HWAC = 0x0002;        // 半角英数字(half-width alphanumeric characters)以外
            public static readonly int OVER_BYTE = 0x0004;          // 設定Byteを超えた
            public static readonly int EXCEPT_NUM = 0x0003;         // 数値以外
            public static readonly int NUM_OUT_OF_RANGE = 0x0005;   // 数値が範囲外(out of range)

            public static readonly int NOT_EXIST_TAGREF_IN_TAGS         = 0x1101;       // Tags内に同一のTagRef名が存在しない
            public static readonly int NOT_EXIST_TAGREFIN_IN_TAGS       = 0x1201;       // Tags内に同一のTagRef名が存在しない
            public static readonly int NOT_EXIST_TAGREFOUT_IN_TAGS      = 0x1301;       // Tags内に同一のTagRef名が存在しない
            public static readonly int NOT_EXIST_DSETTAGREF_IN_TAGS  = 0x2001;          // Tags内に同一のTagRef名が存在しない
            public static readonly int EXIST_SAME_NAMES_IN_DSET = 0x2002;               // 同一名称が複数存在する
            public static readonly int EXIST_SAME_NAMES_IN_COLL = 0x3002;               // 同一名称が複数存在する
            public static readonly int EXIST_SAME_NAMES_IN_SRV = 0x4002;                // 同一名称が複数存在する
            public static readonly int NOT_EXIST_COLLDSET_IN_DSETS = 0x3001;            // Datasets内に同一のDataset名が存在しない
            public static readonly int NOT_EXIST_COLLREF_IN_COLLREFS = 0x3101;          // Collectors内に同一のCollectorNameが存在しない
            public static readonly int NOT_EXIST_DEP_IN_SRVS = 0x4001;                  // Services内に同一のService名が存在しない
            public static readonly int NOT_SATISFIED_SRVDEP = 0x4010;                   // 起動順番が成立しない依存関係

        }

        /// <summary>
        /// エラーコードに対応するエラーメッセージ
        /// </summary>
        public static Dictionary<int, string> ErrorMsg = new Dictionary<int, string>
        {
            {ErrorCode.NULL, "[{0}]{1}が未定義です。" },
            {ErrorCode.EXCEPT_HWAC, "[{0}]{1}に使用できない文字が含まれています。半角英数字で設定してください。" },
            {ErrorCode.OVER_BYTE, "[{0}]{1}がサイズオーバーです。文字数を調節してください。" },
            {ErrorCode.EXCEPT_NUM, "[{0}]{1}に使用できない文字が含まれています。数値を設定してください。" },
            {ErrorCode.NUM_OUT_OF_RANGE, "[{0}]{1}が範囲外です。設定範囲:{2}" },
            {ErrorCode.NOT_EXIST_TAGREF_IN_TAGS, "{0}というTagは存在しません。TagRefのNameを見直してください。" },
            {ErrorCode.NOT_EXIST_TAGREFIN_IN_TAGS, "{0}というTagは存在しません。TagRef_INのNameを見直してください。" },
            {ErrorCode.NOT_EXIST_TAGREFOUT_IN_TAGS, "{0}というTagは存在しません。TagRef_OUTのNameを見直してください。" },
            {ErrorCode.NOT_EXIST_DSETTAGREF_IN_TAGS, "{0}というTagは存在しません。TagRefのNameを見直してください。" },
            {ErrorCode.EXIST_SAME_NAMES_IN_DSET, "{0}というDatasetの名前が重複しています。Datasetの重複定義はできません。" },
            {ErrorCode.EXIST_SAME_NAMES_IN_COLL, "{0}というCollectorの名前が重複しています。Collectorの重複定義はできません。" },
            {ErrorCode.EXIST_SAME_NAMES_IN_SRV, "{0}というServiceの名前が重複しています。Serviceの重複定義はできません。" },
            {ErrorCode.NOT_EXIST_COLLDSET_IN_DSETS, "{0}というDatasetは存在しません。DatasetのNameを見直してください。" },
            {ErrorCode.NOT_EXIST_COLLREF_IN_COLLREFS, "{0}というCollectorは存在しません。CollectorRefのNameを見直してください。" },
            {ErrorCode.NOT_EXIST_DEP_IN_SRVS, "{0}というServiceは存在しません。Dependentを見直してください。" },
            {ErrorCode.NOT_SATISFIED_SRVDEP, "コンテナ起動の依存関係が不正です。Dependentを見直してください。" },
        };

        /// <summary>
        /// 編集に関する定義
        /// </summary>
        public static class Edit
        {
            /// <summary>
            /// 追加項目タイプ
            /// </summary>
            public enum AddElemType
            {
                Tag = 0,
                Dataset,
                DatasetTagRef,
                TagRef,
                TagRefIN,
                TagRefOUT,
                Collector,
                CollectorRef,
                Mailbox,
                Service
            }
        }

        /// <summary>
        /// 外部エディタに関する定義
        /// </summary>
        public static class Editor
        {
            public static readonly string CONFIG_NAME = "EgXmlEditor.xml";                                 // 設定情報ファイル名デフォルト
            public static readonly string DEFAULT_EDITOR_PATH = @"C:\WINDOWS\system32\notepad.exe";     // 外部エディタのデフォルト
            public static readonly string DEFAULT_EDITOR_ARGS = "";                                     // 外部エディタのデフォルト引数

            public static readonly string REPLACE_PATH = "$filepath";                                   // 置換するファイルパス

            /// <summary>
            /// 起動引数の特定の文字列を置換する
            /// </summary>
            /// <param name="args">起動引数</param>
            /// <param name="path">ファイルパス</param>
            /// <returns></returns>
            public static string ReplaceArguments(string args, string path)
            {
                string reArgs;

                reArgs = args.Replace(REPLACE_PATH, path);   // ファイルパスを置換する

                return reArgs;
            }
        }
    }
}
