﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;
using EgBoot;

namespace EgConfig
{
    /// <summary>
    /// EditorRegistration.xaml の相互作用ロジック
    /// </summary>
    public partial class EditorRegistration : Window
    {
        private string path;

        /// <summary>
        /// コンストラクタ
        /// </summary>
        public EditorRegistration()
        {
            EgConfigXML xml;
            string dir;

            InitializeComponent();

            ///////////////////////////////////////////////////////////////
            // 既に設定ファイルがある場合はその設定を読み込む
            ///////////////////////////////////////////////////////////////
            dir = AppDomain.CurrentDomain.BaseDirectory;                            // 実行ファイルのパス取得
            this.path = System.IO.Path.Combine(dir, Define.Editor.CONFIG_NAME);     // 実行ファイルと同ディレクトリの設定ファイルパス作成
            if (File.Exists(this.path) == true)
            {
                xml = MnXmlUtility.LoadFromFile<EgConfigXML>(this.path);
                if(xml is EgConfigXML)
                {
                    // 読み込んだ設定ファイルの値を設定
                    EditorPathTBX.Text = xml.Editor.Path;                           // 外部エディタのパス
                    StartupArgsTBX.Text = xml.Editor.Args;                          // 外部エディタの起動引数     
                }
                else
                {
                    // 形式が違う場合、デフォルトの設定を使用
                    EditorPathTBX.Text = Define.Editor.DEFAULT_EDITOR_PATH;      // デフォルトの外部エディタのパスを設定
                    StartupArgsTBX.Text = Define.Editor.DEFAULT_EDITOR_ARGS;     // デフォルトの外部エディタの起動引数を設定     
                }
            }
            else
            {
                // 設定ファイルがない場合、デフォルトの設定を使用
                EditorPathTBX.Text = Define.Editor.DEFAULT_EDITOR_PATH;      // デフォルトの外部エディタのパスを設定
                StartupArgsTBX.Text = Define.Editor.DEFAULT_EDITOR_ARGS;     // デフォルトの外部エディタの起動引数を設定     
            }
            
        }

        /// <summary>
        /// 参照ボタンクリック
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ReferenceBtn_Click(object sender, RoutedEventArgs e)
        {
            ////////////////////////////////////
            // ファイル選択を行う
            ////////////////////////////////////
            Microsoft.Win32.OpenFileDialog dialog = new Microsoft.Win32.OpenFileDialog();
            try
            {
                dialog.InitialDirectory = System.IO.Path.GetDirectoryName(EditorPathTBX.Text);
            }
            catch (Exception)
            {
                dialog.InitialDirectory = "";
            }
            dialog.FileName = "";
            dialog.Filter = "実行可能ファイル|*.exe";
            dialog.FilterIndex = 0;
            if (dialog.ShowDialog() != true)
            {
                return;
            }

            EditorPathTBX.Text = dialog.FileName;
        }

        /// <summary>
        /// OKボタンクリック
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void OkBtn_Click(object sender, RoutedEventArgs e)
        {
            EgConfigXML xml = new EgConfigXML();

            ///////////////////////////////////////////////
            // 入力値から設定ファイルのデータ作成
            ///////////////////////////////////////////////
            xml.Editor.Path = EditorPathTBX.Text;                                               // 外部エディタのパスを設定
            xml.Editor.Args = StartupArgsTBX.Text;                                              // 外部エディタの起動引数設定

            ///////////////////////////////////////////////
            // ファイルに書き出し XML形式
            ///////////////////////////////////////////////
            try
            {
                MnXmlUtility.SaveToFile(this.path, xml, typeof(EgConfigXML));
            }
            catch (Exception)
            {
                // 書き込めなかったら？
            }

            this.Close();
        }

        /// <summary>
        /// Cancelボタンクリック
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void CancelBtn_Click(object sender, RoutedEventArgs e)
        {
            this.Close();   // 画面を閉じる
        }
    }
}
