﻿////////////////////////////////////////////////////////////////////////////////////////
/// 機能       :   RT-edge XML編集ツール　エクスポート画面コードビハインド
/// 作成日     :   2022/02/24
/// 作成者     :   mizuno
/// 
/// 
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Forms;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;

namespace EgConfig
{
    /// <summary>
    /// ExportProject.xaml の相互作用ロジック
    /// </summary>
    public partial class ExportPack : Window
    {
        public ExportPack()
        {
            InitializeComponent();

            SaveFolderTextBox.Text = AppDomain.CurrentDomain.BaseDirectory;
        }

        /// <summary>
        /// 参照ボタンクリックイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ReferenceBtn_Click(object sender, RoutedEventArgs e)
        {
            // フォルダ選択ダイアログを表示する
            using (FolderBrowserDialog fbd = new FolderBrowserDialog())
            {
                fbd.Description = "フォルダを選択してください";
                fbd.SelectedPath = SaveFolderTextBox.Text;
                fbd.ShowNewFolderButton = false;

                if (fbd.ShowDialog() != System.Windows.Forms.DialogResult.OK)
                {
                    return;
                }
                SaveFolderTextBox.Text = fbd.SelectedPath;

            }
        }

        /// <summary>
        /// OKボタンクリックイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void OkBtn_Click(object sender, RoutedEventArgs e)
        {
            bool isDone;
            string path;
            DialogResult result;

            // パスの作成
            try
            {
                path = System.IO.Path.Combine(SaveFolderTextBox.Text, FileNameTextBox.Text);
                path += ".ecipack";
            }
            catch (Exception)
            {
                // パスに無効な文字が含まれている, null
                return;
            }

            // 同名のファイルが存在する場合
            if(File.Exists(path) == true)
            {
                result = System.Windows.Forms.MessageBox.Show(Properties.Resources.MSGBOX_WARN_005_MSGTEXT, Properties.Resources.MSGBOX_WARN_005_CAPTION, MessageBoxButtons.OKCancel, MessageBoxIcon.Exclamation);
                if(result == System.Windows.Forms.DialogResult.Cancel)
                {
                    this.Close();
                    return;
                }
            }

            // エクスポートする
            isDone = MainWindow.projMgmt.Export(path);
            if(isDone != true)
            {
                System.Windows.Forms.MessageBox.Show(Properties.Resources.MSGBOX_ERROR_013_MSGTEXT, Properties.Resources.MSGBOX_ERROR_013_CAPTION, MessageBoxButtons.OK, MessageBoxIcon.Hand);
            }

            this.Close();   // 画面を閉じる
        }

        /// <summary>
        /// CANCELボタンクリックイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void CancelBtn_Click(object sender, RoutedEventArgs e)
        {
            this.Close();   // 画面を閉じる
        }
    }
}
