﻿using System;
using System.Collections.Generic;
using System.IO;
using System.IO.Compression;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Forms;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;

namespace EgConfig
{
    /// <summary>
    /// InportPack.xaml の相互作用ロジック
    /// </summary>
    public partial class ImportPack : Window
    {
        public ImportPack()
        {
            InitializeComponent();
            
        }

        /// <summary>
        /// ファイル参照ボタンクリックイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ReferenceFileBtn_Click(object sender, RoutedEventArgs e)
        {
            ////////////////////////////////////
            // ファイル選択を行う
            ////////////////////////////////////
            Microsoft.Win32.OpenFileDialog dialog = new Microsoft.Win32.OpenFileDialog();
            dialog.InitialDirectory = AppDomain.CurrentDomain.BaseDirectory;
            dialog.FileName = "";
            dialog.Filter = Define.FILE_FILTER_000;
            dialog.FilterIndex = 0;
            if (dialog.ShowDialog() != true)
            {
                return;
            }

            FileNameTextBox.Text = dialog.FileName;
        }

        /// <summary>
        /// フォルダ参照ボタンクリックイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ReferenceFolderBtn_Click(object sender, RoutedEventArgs e)
        {
            // フォルダ選択ダイアログを表示する
            using (FolderBrowserDialog fbd = new FolderBrowserDialog())
            {
                fbd.Description = "フォルダを選択してください";
                fbd.SelectedPath = FolderNameTextBox.Text;
                fbd.ShowNewFolderButton = false;

                if (fbd.ShowDialog() != System.Windows.Forms.DialogResult.OK)
                {
                    return;
                }
                FolderNameTextBox.Text = fbd.SelectedPath;

            }
        }

        /// <summary>
        /// OKボタンクリックイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void OkBtn_Click(object sender, RoutedEventArgs e)
        {
            bool isDone;
            DialogResult result;
            string names = "";

            //////////////////////////////////////
            // ECIPACKファイル名の入力チェック
            //////////////////////////////////////
            if (String.IsNullOrEmpty(FileNameTextBox.Text) == true)         // ECIPACKファイル未入力
            {
                System.Windows.Forms.MessageBox.Show(Properties.Resources.MSGBOX_ERROR_019_MSGTEXT, Properties.Resources.MSGBOX_ERROR_019_CAPTION, MessageBoxButtons.OK, MessageBoxIcon.Hand);
                return;
            }
            else if (!File.Exists(FileNameTextBox.Text))               // ECIPACKファイルに存在しないパスが入力されている
            {
                System.Windows.Forms.MessageBox.Show(Properties.Resources.MSGBOX_ERROR_020_MSGTEXT, Properties.Resources.MSGBOX_ERROR_020_CAPTION, MessageBoxButtons.OK, MessageBoxIcon.Hand);
                return;
            }

            //////////////////////////////////////
            // 展開先フォルダ名の入力チェック
            //////////////////////////////////////
            if (String.IsNullOrEmpty(FolderNameTextBox.Text) == true)       // 展開先フォルダ未入力
            {
                System.Windows.Forms.MessageBox.Show(Properties.Resources.MSGBOX_ERROR_017_MSGTEXT, Properties.Resources.MSGBOX_ERROR_017_CAPTION, MessageBoxButtons.OK, MessageBoxIcon.Hand);
                return;
            }
            else if (!Directory.Exists(FolderNameTextBox.Text))             // 展開先フォルダに存在しないパスが入力されている
            {
                System.Windows.Forms.MessageBox.Show(Properties.Resources.MSGBOX_ERROR_018_MSGTEXT, Properties.Resources.MSGBOX_ERROR_018_CAPTION, MessageBoxButtons.OK, MessageBoxIcon.Hand);
                return;
            }

            // 展開先に被りあるか？(Zipの一部中身が展開先フォルダに一致)
            isDone = Check(FileNameTextBox.Text, FolderNameTextBox.Text, out List<string> dupList);
            if(isDone == true)
            {
                // 重複あり
                if(0 < dupList.Count)
                {
                    // 重複したファイル名前を作成
                    foreach(string path in dupList)
                    {
                        try
                        {
                            names = names + System.IO.Path.GetFileName(path) + "\n";
                        }
                        catch (Exception)
                        {
                            names = names + "\n";
                        }
                    }

                    result = System.Windows.Forms.MessageBox.Show(String.Format(Properties.Resources.MSGBOX_WARN_006_MSGTEXT, names), Properties.Resources.MSGBOX_WARN_006_CAPTION, MessageBoxButtons.OKCancel, MessageBoxIcon.Exclamation);
                    if (result == System.Windows.Forms.DialogResult.OK)
                    {
                        // 展開先のファイルすべて消す
                        foreach(string path in dupList)
                        {
                            File.Delete(path);
                        }
                    }
                    else
                    {
                        this.Close();
                        return;
                    }
                }
            }
            else
            {
                // チェック失敗
                return;
            }

            // インポート
            MainWindow.fsw.EnableRaisingEvents = false;     // フォルダ監視切る
            isDone = MainWindow.projMgmt.Import(FileNameTextBox.Text, FolderNameTextBox.Text);
            if(isDone == true)
            {
                DialogResult = true;
            }

            //////////////////////////////////////////////////////////////////////
            // ファイル監視を有効にする
            //////////////////////////////////////////////////////////////////////
            MainWindow.fsw.Path = System.IO.Path.GetDirectoryName(MainWindow.projMgmt.eciProjName);
            MainWindow.fsw.EnableRaisingEvents = true;

        }

        /// <summary>
        /// 展開先に被りあるかチェックする
        /// </summary>
        /// <param name="filepath">ecipackファイル</param>
        /// <param name="dirpath">展開先ディレクトリ</param>
        /// <param name="dupList">同名のファイルリスト</param>
        /// <returns>true:チェック済み, false:最後までチェック処理が行えなかった</returns>
        private bool Check(string filepath, string dirpath, out List<string> dupList)
        {
            bool isDone = false;
            string[] dirs;
            dupList = new List<string>();

            try
            {
                // ディレクトリ内のファイル名を取得
                dirs = System.IO.Directory.GetFiles(dirpath, "*", System.IO.SearchOption.TopDirectoryOnly);
            }
            catch (Exception)
            {
                // パスが不正
                return isDone;
            }

            try
            {
                using (ZipArchive zip = ZipFile.Open(filepath, ZipArchiveMode.Update))  // 指定したパスとモードでzipアーカイブを開く
                {

                    // 展開先のディレクトリに同名のファイルが存在する場合、リストにそのファイルパス追加する
                    foreach(string dir in dirs)
                    {
                        if (zip.Entries.Where(x => x.Name == System.IO.Path.GetFileName(dir)).Count() > 0)
                        {
                            dupList.Add(dir);
                        }
                    }

                    isDone = true;
                }
            }
            catch (Exception)
            {
                dupList.Clear();
                isDone = false;
            }

            return isDone;
        }

        /// <summary>
        /// CANCELボタンクリックイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void CancelBtn_Click(object sender, RoutedEventArgs e)
        {
            this.Close();   // 画面を閉じる
        }

        
    }
}
