﻿////////////////////////////////////////////////////////////////////////////////////////
/// 機能       :   RT-edge XML編集ツールのコードビハインド
/// 作成日     :   2022/02/21
/// 作成者     :   mizuno
/// 
/// RT-edge XML編集ツール 機能仕様書
/// file://192.168.1.240/Jobs/KA1601(RT-edge開発）/trunk/0051_Documnet(社内)/0200_egCoreService/20211021_RT-edge_XML編集ツール_機能仕様書.pptx
/// P6. 画面イメージ
/// P13. 機能詳細 ツリー表示エリア/プロパティ表示エリア
/// P17. 機能詳細 編集機能「上へ移動」「下へ移動」
/// P18-20. 機能詳細 編集機能 コンテキストメニュー 項目の"表示・非表示"/"有効・無効"の設定
/// 
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using RTedgeXmlObj;                 // ECIファイルのデータ構造定義使用
using System.Collections.ObjectModel;
using Microsoft.Win32;              // フォルダ選択ダイアログ
using EgBoot;                       // XMLシリアライズ関数使用
using System.Xml.Serialization;
using System.Globalization;
using System.ComponentModel;            // INotifyPropertyChanged
using System.Runtime.CompilerServices;  // CallerMemberName
using System.IO;
using System.Windows.Forms;
using System.Windows.Controls.Primitives;

namespace EgConfig
{
    /// <summary>
    /// 【プロパティウインドウ】Tag要素のType属性表示
    /// </summary>
    public class DataType
    {
        public int TypeNo { get; set; }
        public string TypeName { get; set; }

        public DataType(int TypeNo, string TypeName)
        {
            this.TypeNo = TypeNo;
            this.TypeName = TypeName;
        }
    }

    /// <summary>
    /// 【プロパティウインドウ】TagRef要素のTag選択表示
    /// </summary>
    public class TagRefName
    {
        public string Name { get; set; }

        public TagRefName(string Name)
        {
            this.Name = Name;
        }
    }

    /// <summary>
    /// 【プロパティウインドウ】Service要素のEnable選択表示
    /// </summary>
    public class ServiceEnable
    {
        public int Num { get; set; }                // コンボボックスで選択した値(数値)
        public string Enable { get; set; }          // コンボボックスに表示する値(文字列)

        public ServiceEnable(int Num, string Enable)
        {
            this.Num = Num;
            this.Enable = Enable;
        }
    }

    /// <summary>
    /// 【プロパティウインドウ】Service要素のOnlyOwner選択表示
    /// </summary>
    public class ServiceOwnerFlag
    {
        public int Num { get; set; }                // コンボボックスで選択した値(数値)
        public string Boolean { get; set; }          // コンボボックスに表示する値(文字列)

        public ServiceOwnerFlag(int Num, string Boolean)
        {
            this.Num = Num;
            this.Boolean = Boolean;
        }
    }

    /// <summary>
    /// MainWindow.xaml の相互作用ロジック
    /// </summary>
    public partial class MainWindow : Window, INotifyPropertyChanged
    {
        public static EciProjMgmt projMgmt = new EciProjMgmt();             // ECIプロジェクト管理データ
        public static CreateNewProject CreateNewProject;                    // プロジェクト作成ダイアログ

        // TreeViewに表示するデータ
        public static ObservableCollection<TreeItem> treeItems = new ObservableCollection<TreeItem>();

        /// <summary>
        /// プロパティウインドウに表示するデータ
        /// </summary>
        public static ObservableCollection<DataType> dataTypes = new ObservableCollection<DataType>();                      // TagのType選択用
        public static ObservableCollection<TagRef> tagRefNames = new ObservableCollection<TagRef>();                        // TagRef選択用
        public static ObservableCollection<ServiceEnable> ServiceEnables = new ObservableCollection<ServiceEnable>();       // Service項目のEnable選択用
        public static ObservableCollection<Dataset> datasetNames = new ObservableCollection<Dataset>();                     // Dataset選択用
        public static ObservableCollection<Collector> collectorNames = new ObservableCollection<Collector>();               // Collector選択用
        public static ObservableCollection<ServiceOwnerFlag> ServiceOwnerFlags = new ObservableCollection<ServiceOwnerFlag>();       // Service項目のEnable選択用

        private bool selectedChanged = false;    // プロパティウインドウの選択項目変更フラグ(TextChangedイベントで処理を行わないため)

        /// <summary>
        /// ファイル更新監視
        /// </summary>
        public static FileSystemWatcher fsw = new FileSystemWatcher();      // ファイル監視用
        public static List<bool> isFileChanged = new List<bool>();          // TreeItemsのファイル更新状態保持(true:変更あり, false:変更なし)

        /// <summary>
        /// コンテキストメニュー"名前変更"有効・無効リスト
        /// </summary>
        public static Dictionary<int, bool> RenameAvailability = new Dictionary<int, bool>()
        {
            //////////////////////////////////////////////////////////////////////////////////////
            // TreeViewの選択項目に対して、コンテキストメニュー"名前変更"の有効・無効を設定する
            // true:有効   false:無効
            //////////////////////////////////////////////////////////////////////////////////////
            {(int)ItemType.ECI,             true },
            {(int)ItemType.RTedge,          false },
            {(int)ItemType.Tags,            false },
            {(int)ItemType.Tag,             true },
            {(int)ItemType.Datasets,        false },
            {(int)ItemType.Dataset,         true },
            {(int)ItemType.DatasetTagRefs,  false },
            {(int)ItemType.DatasetTagRef,   true },
            {(int)ItemType.TagRefs,         false },
            {(int)ItemType.TagRef,          true },
            {(int)ItemType.TagRefsIN,       false },
            {(int)ItemType.TagRefIN,        true },
            {(int)ItemType.TagRefsOUT,      false },
            {(int)ItemType.TagRefOUT,       true },
            {(int)ItemType.Collectors,      false },
            {(int)ItemType.Collector,       true },
            {(int)ItemType.CollectorRefs,   false },
            {(int)ItemType.CollectorRef,    true },
            {(int)ItemType.MailBoxes,       false },
            {(int)ItemType.MailBox,         true },
            {(int)ItemType.Services,        false },
            {(int)ItemType.Service,         true },
        };

        /// <summary>
        /// コンテキストメニュー"削除"有効・無効リスト
        /// </summary>
        public static Dictionary<int, bool> RemoveAvailability = new Dictionary<int, bool>()
        {
            //////////////////////////////////////////////////////////////////////////////////////
            // TreeViewの選択項目に対して、コンテキストメニュー"削除"の有効・無効を設定する
            // true:有効   false:無効
            //////////////////////////////////////////////////////////////////////////////////////
            {(int)ItemType.ECI,             true },
            {(int)ItemType.RTedge,          false },
            {(int)ItemType.Tags,            true },
            {(int)ItemType.Tag,             true },
            {(int)ItemType.Datasets,        true },
            {(int)ItemType.Dataset,         true },
            {(int)ItemType.DatasetTagRefs,  true },
            {(int)ItemType.DatasetTagRef,   true },
            {(int)ItemType.TagRefs,         true },
            {(int)ItemType.TagRef,          true },
            {(int)ItemType.TagRefsIN,       true },
            {(int)ItemType.TagRefIN,        true },
            {(int)ItemType.TagRefsOUT,      true },
            {(int)ItemType.TagRefOUT,       true },
            {(int)ItemType.Collectors,      true },
            {(int)ItemType.Collector,       true },
            {(int)ItemType.CollectorRefs,   true },
            {(int)ItemType.CollectorRef,    true },
            {(int)ItemType.MailBoxes,       true },
            {(int)ItemType.MailBox,         true },
            {(int)ItemType.Services,        true },
            {(int)ItemType.Service,         true },
        };

        /// <summary>
        /// コンテキストメニュー追加項目表示・非表示リスト
        /// </summary>
        public static Dictionary<int, AddElemVis> AddElemAvailability = new Dictionary<int, AddElemVis>()
        {
            //////////////////////////////////////////////////////////////////////
            // TreeViewの選択項目に対して、各タグ追加項目の表示・非表示を設定する
            // Visible:表示   Collapsed:非表示
            //////////////////////////////////////////////////////////////////////
            //TreeViewの選択項目,                            //Tag                   //Dataset               //DatasetTagRef         //TagRef                //TagRefIN              //TagRefOUT             //Collector             //CollectorRef          //Mailbox               //Service
            {(int)ItemType.ECI,             new AddElemVis  (Visibility.Visible,      Visibility.Visible,     Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Visible)},
            {(int)ItemType.RTedge,          new AddElemVis  (Visibility.Visible,      Visibility.Visible,     Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Visible) },
            {(int)ItemType.Tags,            new AddElemVis  (Visibility.Visible,      Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Collapsed) },
            {(int)ItemType.Tag,             new AddElemVis  (Visibility.Visible,      Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Collapsed) },
            {(int)ItemType.Datasets,        new AddElemVis  (Visibility.Collapsed,    Visibility.Visible,     Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Collapsed) },
            {(int)ItemType.Dataset,         new AddElemVis  (Visibility.Collapsed,    Visibility.Collapsed,   Visibility.Visible,     Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Collapsed) },
            {(int)ItemType.DatasetTagRefs,  new AddElemVis  (Visibility.Collapsed,    Visibility.Collapsed,   Visibility.Visible,     Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Collapsed) },
            {(int)ItemType.DatasetTagRef,   new AddElemVis  (Visibility.Collapsed,    Visibility.Collapsed,   Visibility.Visible,     Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Collapsed) },
            {(int)ItemType.TagRefs,         new AddElemVis  (Visibility.Collapsed,    Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Visible,     Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Collapsed) },
            {(int)ItemType.TagRef,          new AddElemVis  (Visibility.Collapsed,    Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Visible,     Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Collapsed) },
            {(int)ItemType.TagRefsIN,       new AddElemVis  (Visibility.Collapsed,    Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Visible,     Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Collapsed) },
            {(int)ItemType.TagRefIN,        new AddElemVis  (Visibility.Collapsed,    Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Visible,     Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Collapsed) },
            {(int)ItemType.TagRefsOUT,      new AddElemVis  (Visibility.Collapsed,    Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Visible,     Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Collapsed) },
            {(int)ItemType.TagRefOUT,       new AddElemVis  (Visibility.Collapsed,    Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Visible,     Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Collapsed) },
            {(int)ItemType.Collectors,      new AddElemVis  (Visibility.Collapsed,    Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Visible,     Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Collapsed) },
            {(int)ItemType.Collector,       new AddElemVis  (Visibility.Collapsed,    Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Visible,     Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Collapsed) },
            {(int)ItemType.CollectorRefs,   new AddElemVis  (Visibility.Collapsed,    Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Visible,     Visibility.Collapsed,   Visibility.Collapsed) },
            {(int)ItemType.CollectorRef,    new AddElemVis  (Visibility.Collapsed,    Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Visible,     Visibility.Collapsed,   Visibility.Collapsed) },
            {(int)ItemType.MailBoxes,       new AddElemVis  (Visibility.Collapsed,    Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Visible,     Visibility.Collapsed) },
            {(int)ItemType.MailBox,         new AddElemVis  (Visibility.Collapsed,    Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Visible,     Visibility.Collapsed) },
            {(int)ItemType.Services,        new AddElemVis  (Visibility.Collapsed,    Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Collapsed,   Visibility.Visible) },
            {(int)ItemType.Service,         new AddElemVis  (Visibility.Visible,      Visibility.Visible,     Visibility.Collapsed,   Visibility.Visible,     Visibility.Visible,     Visibility.Visible,     Visibility.Visible,     Visibility.Visible,     Visibility.Visible,     Visibility.Collapsed) },
        };

        /// <summary>
        /// プロパティ変更通知イベント
        /// </summary>
        public event PropertyChangedEventHandler PropertyChanged;
        private void NotifyPropertyChanged([CallerMemberName] String propertyName = "")
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
            }
        }

        /// <summary>
        /// プロジェクト名
        /// </summary>
        private string projectName = "";
        public string ProjectName
        {
            get { return projectName; }
            set
            {
                if (value != projectName)
                {
                    projectName = value;
                    NotifyPropertyChanged("ProjectName");
                }
            }
        }

        /// <summary>
        /// プロジェクト名表示・非表示用
        /// </summary>
        private Visibility projectNameVis = Visibility.Collapsed;
        public Visibility ProjectNameVis
        {
            get { return projectNameVis; }
            set
            {
                if (value != projectNameVis)
                {
                    projectNameVis = value;
                    NotifyPropertyChanged("ProjectNameVis");
                }
            }
        }

        // ****を保存 ファイル名
        private string saveFilename = "ファイル";
        public string SaveFileName
        {
            get { return saveFilename; }
            set
            {
                if(value != saveFilename)
                {
                    saveFilename = value;
                    NotifyPropertyChanged("SaveFileName");
                }
            }
        }

        // ****を保存 実行可否
        private bool saveFileEnable = false;
        public bool SaveFileEnable
        {
            get { return saveFileEnable; }
            set
            {
                if (value != saveFileEnable)
                {
                    saveFileEnable = value;
                    NotifyPropertyChanged("SaveFileEnable");
                }
            }
        }

        // すべて保存 実行可否
        private bool saveAllFileEnable = false;
        public bool SaveAllFileEnable
        {
            get { return saveAllFileEnable; }
            set
            {
                if (value != saveAllFileEnable)
                {
                    saveAllFileEnable = value;
                    NotifyPropertyChanged("SaveAllFileEnable");
                }
            }
        }
        

        // プロパティ表示ウインドウの設定項目表示設定
        private int propertyIndex = -1;
        public int PropertyIndex
        {
            get { return propertyIndex; }
            set
            {
                if(value != propertyIndex)
                {
                    propertyIndex = value;
                    NotifyPropertyChanged("PropertyIndex");
                }
            }
        }

        // プロパティウインドウのTagにバインドするデータ
        private Tag _tagData;
        public Tag TagData
        {
            get { return _tagData; }
            set
            {
                if (value != _tagData)
                {
                    _tagData = value;
                    NotifyPropertyChanged("TagData");
                }
            }
        }

        // プロパティウインドウのメールボックスにバインドするデータ
        private MailBox _mailBoxData;
        public MailBox MailBoxData
        {
            get { return _mailBoxData; }
            set
            {
                if(value != _mailBoxData)
                {
                    _mailBoxData = value;
                    NotifyPropertyChanged("MailBoxData");
                }
            }
        }

        // プロパティウインドウのTagRefにバインドするデータ
        private TagRef _tagRefData;
        public TagRef TagRefData
        {
            get { return _tagRefData; }
            set
            {
                if(value != _tagRefData)
                {
                    _tagRefData = value;
                    NotifyPropertyChanged("TagRefData");
                }
            }
        }

        // プロパティウインドウのTagRefにバインドするデータ
        private CollectorRef _collectorRefData;
        public CollectorRef CollectorRefData
        {
            get { return _collectorRefData; }
            set
            {
                if (value != _collectorRefData)
                {
                    _collectorRefData = value;
                    NotifyPropertyChanged("CollectorRefData");
                }
            }
        }

        // プロパティウインドウのServiceにバインドするデータ
        private Service _serviceData;
        public Service ServiceData
        {
            get { return _serviceData; }
            set
            {
                if(value != _serviceData)
                {
                    _serviceData = value;
                    if(_serviceData.ENABLED == null)
                    {
                        _serviceData.ENABLED = "1";
                    }
                    if (_serviceData.ONLYOWNER == null)
                    {
                        _serviceData.ONLYOWNER = "0";
                    }

                    NotifyPropertyChanged("ServiceData");
                }
            }
        }

        // プロパティウインドウのCollectorにバインドするデータ
        private Collector _collectorData;
        public Collector CollectorData
        {
            get { return _collectorData; }
            set
            {
                if (value != _collectorData)
                {
                    _collectorData = value;
                    NotifyPropertyChanged("CollectorData");
                }
            }
        }

        // プロパティウインドウのDatasetにバインドするデータ
        private Dataset _datasetData;
        public Dataset DatasetData
        {
            get { return _datasetData; }
            set
            {
                if (value != _datasetData)
                {
                    _datasetData = value;
                    NotifyPropertyChanged("DatasetData");
                }
            }
        }

        // ポップアップメニューの"名前変更"表示・非表示
        private bool _canrename=false;
        public bool CanRename
        {
            get { return _canrename; }
            set
            {
                if(value != _canrename)
                {
                    _canrename = value;
                    NotifyPropertyChanged("CanRename");
                }
            }
        }

        // 追加項目の表示・非表示設定
        // (Tag,Dataset,DatasetTagRef,TagRef,TagRefIN,TagRefOUT,Collector,CollectorRef,Mailbox,Service)
        private AddElemVis _addElemVis;
        public AddElemVis AddElemVis
        {
            get { return _addElemVis; }
            set
            {
                if(value != _addElemVis)
                {
                    _addElemVis = value;
                    NotifyPropertyChanged("AddElemVis");
                }
            }
        }

        // メッセージウインドウ表示用
        private ObservableCollection<ElemCheckMsg> _elemCheckMsgs = new ObservableCollection<ElemCheckMsg>();
        public ObservableCollection<ElemCheckMsg> ElemCheckMsgs
        {
            get { return _elemCheckMsgs; }
            set
            {
                if (value != _elemCheckMsgs)
                {
                    _elemCheckMsgs = value;
                    NotifyPropertyChanged("ElemCheckMsgs");
                }
            }
        }

        /// <summary>
        /// 状態管理
        /// </summary>
        private int _state;
        public int State
        {
            get { return _state; }
            set
            {
                _state = value;

                if(Define.ButtonVisibilityDic.ContainsKey(_state) == true)
                {
                    ButtonVisibility = Define.ButtonVisibilityDic[_state];
                }
                else
                {
                    ButtonVisibility = new ButtonVisibility();
                }

                NotifyPropertyChanged("State");
                
            }
        }

        /// <summary>
        /// ボタンの有効・無効設定
        /// </summary>
        private ButtonVisibility _buttonVisibility;
        public ButtonVisibility ButtonVisibility
        {
            get { return _buttonVisibility; }
            set
            {
                if (value != _buttonVisibility)
                {
                    _buttonVisibility = value;
                    NotifyPropertyChanged("ButtonVisibility");
                }
            }
        }

        /// <summary>
        /// 指定展開先パスの取得・設定
        /// </summary>
        private string _baseDirectory = "";
        public string BaseDirectory
        {
            get { return _baseDirectory; }
            set
            {
                if (value != _baseDirectory)
                {
                    _baseDirectory = value;
                    NotifyPropertyChanged("BaseDirectory");
                }
            }
        }

        /// <summary>
        /// タイトル名
        /// </summary>
        public string TitleName
        {
            get { return m_srTitleName; }
            set { m_srTitleName = value; }
        }
        private string m_srTitleName = "";


        /// <summary>
        /// MainWindowコンストラクタ
        /// </summary>
        public MainWindow()
        {
            InitializeComponent();
            
            XmlTreeView.ItemsSource = treeItems;        // ツリービューのItemsSorceに登録

            ///////////////////////////////////////////////////////
            // プロパティウインドウTagのTypeコンボボックス設定
            ///////////////////////////////////////////////////////
            dataTypes.Add(new DataType((int)Define.PropertyWindow.TagType.Boolean,      (int)Define.PropertyWindow.TagType.Boolean     + ":" + Define.PropertyWindow.TagType.Boolean.ToString()));
            dataTypes.Add(new DataType((int)Define.PropertyWindow.TagType.SByte,        (int)Define.PropertyWindow.TagType.SByte       + ":" + Define.PropertyWindow.TagType.SByte.ToString()));
            dataTypes.Add(new DataType((int)Define.PropertyWindow.TagType.Byte,         (int)Define.PropertyWindow.TagType.Byte        + ":" + Define.PropertyWindow.TagType.Byte.ToString()));
            dataTypes.Add(new DataType((int)Define.PropertyWindow.TagType.Int16,        (int)Define.PropertyWindow.TagType.Int16       + ":" + Define.PropertyWindow.TagType.Int16.ToString()));
            dataTypes.Add(new DataType((int)Define.PropertyWindow.TagType.UInt16,       (int)Define.PropertyWindow.TagType.UInt16      + ":" + Define.PropertyWindow.TagType.UInt16.ToString()));
            dataTypes.Add(new DataType((int)Define.PropertyWindow.TagType.Int32,        (int)Define.PropertyWindow.TagType.Int32       + ":" + Define.PropertyWindow.TagType.Int32.ToString()));
            dataTypes.Add(new DataType((int)Define.PropertyWindow.TagType.UInt32,       (int)Define.PropertyWindow.TagType.UInt32      + ":" + Define.PropertyWindow.TagType.UInt32.ToString()));
            dataTypes.Add(new DataType((int)Define.PropertyWindow.TagType.Int64,        (int)Define.PropertyWindow.TagType.Int64       + ":" + Define.PropertyWindow.TagType.Int64.ToString()));
            dataTypes.Add(new DataType((int)Define.PropertyWindow.TagType.UInt64,       (int)Define.PropertyWindow.TagType.UInt64      + ":" + Define.PropertyWindow.TagType.UInt64.ToString()));
            dataTypes.Add(new DataType((int)Define.PropertyWindow.TagType.Float,        (int)Define.PropertyWindow.TagType.Float       + ":" + Define.PropertyWindow.TagType.Float.ToString()));
            dataTypes.Add(new DataType((int)Define.PropertyWindow.TagType.Double,       (int)Define.PropertyWindow.TagType.Double      + ":" + Define.PropertyWindow.TagType.Double.ToString()));
            TypeComboBox.ItemsSource = dataTypes;

            ///////////////////////////////////////////////////////
            // プロパティウインドウTagRefのTagの名称を設定
            ///////////////////////////////////////////////////////
            TagRefComboBox.ItemsSource = tagRefNames;

            ///////////////////////////////////////////////////////
            // ServiceのEnable選択をコンボボックスへ設定する
            ///////////////////////////////////////////////////////
            ServiceEnables.Add(new ServiceEnable((int)Define.PropertyWindow.Enable.False, Define.PropertyWindow.Enable.False.ToString()));
            ServiceEnables.Add(new ServiceEnable((int)Define.PropertyWindow.Enable.True, Define.PropertyWindow.Enable.True.ToString()));
            EnabledComboBox.ItemsSource = ServiceEnables;

            ///////////////////////////////////////////////////////
            // ServiceのOnlyOwner選択をコンボボックスへ設定する
            ///////////////////////////////////////////////////////
            ServiceOwnerFlags.Add(new ServiceOwnerFlag((int)Define.PropertyWindow.Boolean.False, Define.PropertyWindow.Boolean.False.ToString()));
            ServiceOwnerFlags.Add(new ServiceOwnerFlag((int)Define.PropertyWindow.Boolean.True, Define.PropertyWindow.Boolean.True.ToString()));
            OwnerFlagComboBox.ItemsSource = ServiceOwnerFlags;


            ///////////////////////////////////////////////////////
            // プロパティウインドウDatasetにデータセット名称を設定
            ///////////////////////////////////////////////////////
            DatasetComboBox.ItemsSource = datasetNames;

            //////////////////////////////////////////////////////////////
            // プロパティウインドウCollectorRefにCollector名称を設定
            //////////////////////////////////////////////////////////////
            CollectorComboBox.ItemsSource = collectorNames;

            //////////////////////////////////////////////////////////////
            // ファイル更新時のイベントの設定
            //////////////////////////////////////////////////////////////
            fsw.Path = "";
            fsw.NotifyFilter = NotifyFilters.LastWrite;
            fsw.IncludeSubdirectories = false;
            fsw.Filter = "*.xml";
            fsw.Changed += Fsw_Changed;
            fsw.EnableRaisingEvents = false;

            State = Define.STATE_INI;

            //////////////////////////////////////////////////////////////
            // タイトルの取得
            //////////////////////////////////////////////////////////////
            TitleName = this.Title;

            DataContext = this;
        }

        /// <summary>
        /// 新規プロジェクト作成ボタンクリック
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void NewProjectBtn_Click(object sender, RoutedEventArgs e)
        {
            bool canDo;
            
            // ファイル変更未保存かチェック
            canDo = CheckUnsaveFile();
            if(canDo == false)
            {
                return;
            }

            // プロジェクト作成ダイアログ表示
            CreateNewProject = new CreateNewProject(this.BaseDirectory);
            CreateNewProject.WindowStartupLocation = WindowStartupLocation.CenterScreen;
            CreateNewProject.ShowDialog();
            if(CreateNewProject.DialogResult == true)
            {
                SaveAllFileEnable = true;                                           // すべて保存ボタンを押せるようにする
                PropertyIndex = (int)Define.PropertyWindow.Tab_None;                // プロパティ画面を非表示にする
                ProjectName = System.IO.Path.GetFileName(projMgmt.eciProjName);     // プロジェクト名を設定
                ProjectNameVis = Visibility.Visible;                                // プロジェクト名を表示
                ElemCheckMsgs.Clear();                                              // メッセージ表示ウインドウクリア
                this.State = Define.STATE_PROJ;                                     // 状態を変化させる
                this.BaseDirectory = CreateNewProject.SaveFolderTextBox.Text;       // プロジェクトを生成した場所を保管

                UpdateTitlebar();                                                   // タイトルバーの更新：デフォルト値をセット
            }
        }

        /// <summary>
        /// 新規ファイル作成ボタンクリック
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void NewFileBtn_Click(object sender, RoutedEventArgs e)
        {
            bool canDo;
            ObservableCollection<TreeItem> items;   // TreeItem作成用

            // ファイル変更未保存かチェック
            canDo = CheckUnsaveFile();
            if (canDo == false)
            {
                return;
            }

            // 新規ファイル作成ダイアログ表示
            CreateNewFile CreateNewFile = new CreateNewFile();
            CreateNewFile.WindowStartupLocation = WindowStartupLocation.CenterScreen;
            CreateNewFile.ShowDialog();
            if(CreateNewFile.DialogResult == true)  // ファイル作成成功時、データを読み込む
            {
                /////////////////////////////////
                // XMLファイル読み込み
                /////////////////////////////////
                MainWindow.projMgmt = EciProjMgmt.ReadEciFile(CreateNewFile.Path);              // ECIファイル読み込み

                /////////////////////////////////
                // TreeViewにデータを出力
                /////////////////////////////////
                items = TreeItem.EciProjMgmt2TreeItem(MainWindow.projMgmt);                     // ECIプロジェクト管理データからTreeItemを作成
                MainWindow.treeItems.Clear();
                foreach (TreeItem item in items)
                {
                    MainWindow.treeItems.Add(item);
                }

                // TreeItemの変更状態をなしにする
                isFileChanged.Clear();
                for (int i = 0; i < treeItems.Count; i++)
                {
                    isFileChanged.Add(false);
                }

                SaveAllFileEnable = false;   // すべて保存ボタンを押せないようにする

                //////////////////////////////////////////////////////////////////////
                // ファイル監視を有効にする
                //////////////////////////////////////////////////////////////////////
                fsw.Path = System.IO.Path.GetDirectoryName(CreateNewFile.Path);
                fsw.EnableRaisingEvents = true;

                PropertyIndex = (int)Define.PropertyWindow.Tab_None;    // プロパティ画面を非表示にする
                ProjectName = "";                                       // プロジェクト名を設定
                ProjectNameVis = Visibility.Collapsed;                  // プロジェクト名を非表示
                ElemCheckMsgs.Clear();                                  // メッセージ表示ウインドウクリア
                this.State = Define.STATE_FILE;                         // 状態を変化させる

                UpdateTitlebar();                                       // タイトルバーの更新：デフォルト値をセット
            }
        }

        /// <summary>
        /// 既存のプロジェクトを開くクリック
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ExistingProjectBtn_Click(object sender, RoutedEventArgs e)
        {
            bool canDo;
            ObservableCollection<TreeItem> items;
            string filenames = "";

            // ファイル変更未保存かチェック
            canDo = CheckUnsaveFile();
            if (canDo == false)
            {
                return;
            }

            ////////////////////////////////////
            // ファイル選択を行う
            ////////////////////////////////////
            Microsoft.Win32.OpenFileDialog dialog = new Microsoft.Win32.OpenFileDialog();
            dialog.InitialDirectory = this.BaseDirectory=="" ? AppDomain.CurrentDomain.BaseDirectory : this.BaseDirectory;
            dialog.FileName = "";
            dialog.Filter = "ECIプロジェクトファイル|*.eciproj";
            dialog.FilterIndex = 0;
            if (dialog.ShowDialog() != true)
            {
                return;
            }

            this.BaseDirectory = System.IO.Path.GetDirectoryName(dialog.FileName);  // 既存のプロジェクトがあるパスを保管
                       
            ////////////////////////////////////
            // プロジェクトファイル読み込み
            ////////////////////////////////////
            projMgmt = EciProjMgmt.ReadProjectFile(dialog.FileName, out List<string> readFailures);
            if (0 < readFailures.Count)
            {
                foreach (string readFailure in readFailures)
                {
                    try
                    {
                        filenames = filenames + System.IO.Path.GetFileName(readFailure) + "\n";
                    }
                    catch (Exception)
                    {
                        filenames = filenames + "" + "\n";
                    }
                }
                System.Windows.Forms.MessageBox.Show(String.Format(Properties.Resources.MSGBOX_ERROR_003_MSGTEXT, filenames), Properties.Resources.MSGBOX_ERROR_003_CAPTION, MessageBoxButtons.OK, MessageBoxIcon.Hand);
            }

            ////////////////////////////////////////////////
            // 読み込んだECIファイルをTreeViewに表示
            ////////////////////////////////////////////////
            //ShowEciFile(projMgmt);
            items = TreeItem.EciProjMgmt2TreeItem(projMgmt);    // ECIプロジェクト管理データからTreeItemを作成
            treeItems.Clear();
            foreach(TreeItem item in items)
            {
                treeItems.Add(item);
            }

            // TreeItemの変更状態をなしにする
            isFileChanged.Clear();
            for(int i = 0; i < treeItems.Count; i++)
            {
                isFileChanged.Add(false);
            }

            ////////////////////////////////////////////////
            // プロパティウインドウのTagRef選択にデータ登録
            ////////////////////////////////////////////////
            UpdateTagRefData();

            //////////////////////////////////////////////////////////////////////
            // プロパティウインドウのCollector項目DatasetName選択にデータ登録
            //////////////////////////////////////////////////////////////////////
            UpdateDatasetData();

            //////////////////////////////////////////////////////////////////////
            // プロパティウインドウのCollectorRef項目Collector選択にデータ登録
            // TODO:重複しない処理を入れる
            //////////////////////////////////////////////////////////////////////
            UpdateCollectorData();

            SaveAllFileEnable = true;   // すべて保存ボタンを押せるようにする

            //////////////////////////////////////////////////////////////////////
            // ファイル監視を有効にする
            //////////////////////////////////////////////////////////////////////
            fsw.Path = System.IO.Path.GetDirectoryName(projMgmt.eciProjName);
            fsw.EnableRaisingEvents = true;

            PropertyIndex = (int)Define.PropertyWindow.Tab_None;                // プロパティ画面を非表示にする
            ProjectName = System.IO.Path.GetFileName(projMgmt.eciProjName);     // プロジェクト名を設定
            ProjectNameVis = Visibility.Visible;                                // プロジェクト名を表示
            ElemCheckMsgs.Clear();                                              // メッセージ表示ウインドウクリア
            this.State = Define.STATE_PROJ;                                     // 状態を変化させる

            UpdateTitlebar();                                                   // タイトルバーの更新：デフォルト値をセット
        }

        /// <summary>
        /// 既存のファイルを開くクリック
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ExistingFileBtn_Click(object sender, RoutedEventArgs e)
        {
            bool isDone;
            ObservableCollection<TreeItem> items;
            EciProjMgmt projMgmt;

            // ファイル変更未保存かチェック
            isDone = CheckUnsaveFile();
            if (isDone == false)
            {
                return;
            }

            ////////////////////////////////////
            // ファイル選択を行う
            ////////////////////////////////////
            Microsoft.Win32.OpenFileDialog dialog = new Microsoft.Win32.OpenFileDialog();
            dialog.InitialDirectory = this.BaseDirectory == "" ? AppDomain.CurrentDomain.BaseDirectory : this.BaseDirectory;
            dialog.FileName = "";
            dialog.Filter = "ECIファイル|*.xml";
            dialog.FilterIndex = 0;
            if (dialog.ShowDialog() != true)
            {
                return;
            }

            if( this.BaseDirectory == "" ) // 保管したディレクトリパスが無い場合に限り
                this.BaseDirectory = System.IO.Path.GetDirectoryName(dialog.FileName);  // 既存のファイルがあるパスを保管

            ////////////////////////////////////////////////
            // 選択したファイルのシリアライズ(読み込み)
            ////////////////////////////////////////////////
            projMgmt = EciProjMgmt.ReadEciFile(dialog.FileName);
            foreach(EciFileMgmt eciFileMgmt in projMgmt.eciList)
            {
                if(eciFileMgmt == null)
                {
                    // ファイル読み込み失敗メッセージ
                    System.Windows.Forms.MessageBox.Show(String.Format(Properties.Resources.MSGBOX_ERROR_016_MSGTEXT, System.IO.Path.GetFileName(dialog.FileName)), Properties.Resources.MSGBOX_ERROR_016_CAPTION, MessageBoxButtons.OK, MessageBoxIcon.Hand);
                    return;
                }
            }
            MainWindow.projMgmt = projMgmt;

            ////////////////////////////////////////////////
            // 読み込んだECIファイルをTreeViewに表示
            ////////////////////////////////////////////////
            items = TreeItem.EciProjMgmt2TreeItem(MainWindow.projMgmt);    // ECIプロジェクト管理データからTreeItemを作成
            treeItems.Clear();
            foreach (TreeItem item in items)
            {
                treeItems.Add(item);
            }

            // TreeItemの変更状態をなしにする
            isFileChanged.Clear();
            for (int i = 0; i < treeItems.Count; i++)
            {
                isFileChanged.Add(false);
            }
            ////////////////////////////////////////////////
            // プロパティウインドウのTagRef選択にデータ登録
            ////////////////////////////////////////////////
            UpdateTagRefData();

            //////////////////////////////////////////////////////////////////////
            // プロパティウインドウのCollector項目DatasetName選択にデータ登録
            //////////////////////////////////////////////////////////////////////
            UpdateDatasetData();

            //////////////////////////////////////////////////////////////////////
            // プロパティウインドウのCollectorRef項目Collector選択にデータ登録
            // TODO:重複しない処理を入れる
            //////////////////////////////////////////////////////////////////////
            UpdateCollectorData();

            SaveAllFileEnable = false;   // すべて保存ボタンを押せないようにする
            
            //////////////////////////////////////////////////////////////////////
            // ファイル監視を有効にする
            //////////////////////////////////////////////////////////////////////
            fsw.Path = System.IO.Path.GetDirectoryName(dialog.FileName);
            fsw.EnableRaisingEvents = true;

            PropertyIndex = (int)Define.PropertyWindow.Tab_None;                // プロパティ画面を非表示にする
            ProjectName = "";                                                   // プロジェクト名を設定
            ProjectNameVis = Visibility.Collapsed;                              // プロジェクト名を非表示
            ElemCheckMsgs.Clear();                                              // メッセージ表示ウインドウクリア
            this.State = Define.STATE_FILE;                                     // 状態を変化させる

            UpdateTitlebar();                                                   // タイトルバーの更新：デフォルト値をセット
        }

        /// <summary>
        /// 読み込んだXMLに登録してあるすべてのTagの名称をコンボボックスに登録する
        /// </summary>
        private void UpdateTagRefData()
        {
            tagRefNames.Clear();

            // 読み込んだデータにあるTag名を取得する
            foreach (EciFileMgmt eciFileMgmt in projMgmt.eciList)
            {
                foreach (Tag tag in eciFileMgmt.eciData.Tags)   // グローバルなタグを追加
                {
                    tagRefNames.Add(new TagRef(tag.Name));  // リストに追加
                }
                foreach (Service s in eciFileMgmt.eciData.Services) //サービス配下も追加
                {
                    foreach (Tag tag in s.Tags)
                    {
                        tagRefNames.Add(new TagRef(tag.Name));  // リストに追加
                    }
                }
            }
            
        }

        /// <summary>
        /// 読み込んだXMLに登録してあるすべてのDatasetの名称をコンボボックスに登録する
        /// </summary>
        private void UpdateDatasetData()
        {
            datasetNames.Clear();

            // 読み込んだデータにあるTag名を取得する
            foreach (EciFileMgmt eciFileMgmt in projMgmt.eciList)
            {
                foreach (Dataset dataset in eciFileMgmt.eciData.Datasets)
                {
                    datasetNames.Add(new Dataset(dataset.Name));  // リストに追加
                }
                foreach (Service s in eciFileMgmt.eciData.Services) //サービス配下も追加
                {
                    foreach (Dataset dataset in s.Datasets)
                    {
                        datasetNames.Add(new Dataset(dataset.Name));  // リストに追加
                    }
                }
            }

        }

        /// <summary>
        /// 読み込んだXMLに登録してあるすべてのCollectorの名称をコンボボックスに登録する
        /// </summary>
        private void UpdateCollectorData()
        {
            collectorNames.Clear();

            // 読み込んだデータにあるTag名を取得する
            foreach (EciFileMgmt eciFileMgmt in projMgmt.eciList)
            {
                foreach (Service service in eciFileMgmt.eciData.Services)
                {
                    foreach(Collector collector in service.Collectors)
                    {
                        collectorNames.Add(collector);  // リストに追加
                    }
                }
            }
            // TODO:重複削除されない
            // collectorNames.Distinct();  
        }


        /// <summary>
        /// ファイルメニューの保存をクリック
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void SaveFileBtn_Click(object sender, RoutedEventArgs e)
        {
            bool isDone = false;

            // TreeViewで選択しているファイル名取得
            if(XmlTreeView.SelectedItem is TreeItem selectedItem)
            {
                // ファイル変更イベントOFF
                fsw.EnableRaisingEvents = false;

                // ファイルを保存
                isDone = projMgmt.SaveFile(selectedItem.Filename);
                if(isDone == true)
                {
                    System.Windows.Forms.MessageBox.Show(String.Format(Properties.Resources.MSGBOX_INFO_001_MSGTEXT, selectedItem.Filename), Properties.Resources.MSGBOX_INFO_001_CAPTION, MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                }
                else
                {
                    System.Windows.Forms.MessageBox.Show(String.Format(Properties.Resources.MSGBOX_ERROR_015_MSGTEXT, selectedItem.Filename), Properties.Resources.MSGBOX_ERROR_015_CAPTION, MessageBoxButtons.OK, MessageBoxIcon.Hand);
                }

                // ファイル変更イベントON
                fsw.EnableRaisingEvents = true;
            }
            
        }

        /// <summary>
        /// タイトルバーを更新する
        /// </summary>
        /// <param name="srPath">ファイルパス</param>
        private void UpdateTitlebar(string srPath = "")
        {
            // タイトルバーに表示するタイトル名の設定
            string srTitle = TitleName;

            // 付加するファイルパスのチェック
            if ("" != srPath)
            {
                // ファイルパスありのため、タイトルにファイルパスを付加する
                srTitle += " - " + srPath;
                // ファイル属性の取得
                var clFileAttr = File.GetAttributes(srPath);
                // 読み取り専用かどうかのチェック
                if ((clFileAttr & FileAttributes.ReadOnly) == FileAttributes.ReadOnly)
                {
                    // 読み取り専用の内容をタイトルに付加
                    srTitle += " [readonly]";
                }
            }

            // 作成したタイトルをタイトルバーにセットする。
            this.Title = srTitle;                   
        }

        /// <summary>
        /// TreeViewの選択項目変更イベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void SelectedItemChanged(object sender, RoutedPropertyChangedEventArgs<object> e)
        {
            TreeItem selectedItem = XmlTreeView.SelectedItem as TreeItem;   // TreeViewで選択中の項目を取得
            if(selectedItem == null)                                        // 選択項目がない場合
            {
                this.SaveFileName = "ファイル";
                this.SaveFileEnable = false;
                this.State = this.State & Define.MASK_STATE;
                return;
            }

            selectedChanged = true; // プロパティウインドウ選択項目変更フラグON

            // 選択項目変更時に、参照をクリアする
            // (編集によりコンボボックスのアイテムが変更された際、アイテム選択不可によりTextを空文字にしないため)
            TagRefData = new TagRef() { Name="" };
            CollectorData = new Collector() { DatasetName = "" };
            CollectorRefData = new CollectorRef() { Name = "" };

            this.SaveFileName = System.IO.Path.GetFileName(selectedItem.Filename);  // メニューのファイル保存のファイル名変更
            this.SaveFileEnable = true;

            UpdateTitlebar(selectedItem.Filename);                                  // タイトルバーの更新：ファイル名を付加

            ////////////////////////////////////////////////////////////
            // プロパティ表示ウインドウの編集設定項目を切り替える
            ////////////////////////////////////////////////////////////
            PropertyIndex = selectedItem.ItemType;              // プロパティ編集ウインドウタブ切り替え
            
            object obj = projMgmt.GetItemData(selectedItem);    // 選択中から管理データを取得する

            /////////////////////////////////////////////////
            // 取得したデータをプロパティに設定する
            /////////////////////////////////////////////////
            if(obj is Tag)
            {
                Tag tag = (Tag)obj;
                TagData = tag;
            }
            else if(obj is Dataset dataset)
            {
                DatasetData = dataset;
            }
            else if(obj is TagRef)
            {
                TagRef tagref = (TagRef)obj;
                TagRefData = tagref;
            }
            else if(obj is MailBox)
            {
                MailBox mailBox = (MailBox)obj;
                MailBoxData = mailBox;
            }
            else if(obj is Service)
            {
                Service service = (Service)obj;
                string a = "";
                if (OwnerFlagComboBox.SelectedItem != null)
                {
                    a = OwnerFlagComboBox.SelectedItem.ToString();
                }
                ServiceData = service;
            }
            else if(obj is Collector)
            {
                Collector collector = (Collector)obj;
                CollectorData = collector;
                
            }
            else if(obj is CollectorRef)
            {
                CollectorRef collectorref = (CollectorRef)obj;
                CollectorRefData = collectorref;
            }

            // 編集項目表示・非表示設定
            SetRenameAvailability(selectedItem);                // "名前変更"の表示・非表示取得
            SetAddElement(selectedItem);                        // 追加項目の視認性変更(表示・非表示)

            // ツールのボタンの状態変更
            switch (selectedItem.ItemType)
            {
                case (int)ItemType.ECI:
                    this.State = (this.State & Define.MASK_STATE) | Define.MASK_SELECTED_ECI;
                    break;
                case (int)ItemType.RTedge:
                    this.State = (this.State & Define.MASK_STATE) | Define.MASK_SELECTED_RTEDGE;
                    break;
                case (int)ItemType.Tags:
                case (int)ItemType.Datasets:
                case (int)ItemType.DatasetTagRefs:
                case (int)ItemType.Services:
                case (int)ItemType.TagRefs:
                case (int)ItemType.TagRefsIN:
                case (int)ItemType.TagRefsOUT:
                case (int)ItemType.Collectors:
                case (int)ItemType.CollectorRefs:
                case (int)ItemType.MailBoxes:
                    this.State = (this.State & Define.MASK_STATE) | Define.MASK_SELECTED_PER;
                    break;
                case (int)ItemType.Tag:
                case (int)ItemType.Dataset:
                case (int)ItemType.DatasetTagRef:
                case (int)ItemType.Service:
                case (int)ItemType.TagRef:
                case (int)ItemType.TagRefIN:
                case (int)ItemType.TagRefOUT:
                case (int)ItemType.Collector:
                case (int)ItemType.CollectorRef:
                case (int)ItemType.MailBox:
                    this.State = (this.State & Define.MASK_STATE) | Define.MASK_SELECTED_CHL;
                    break;
                default:
                    break;
            }

            selectedChanged = false;    // プロパティウインドウ選択項目変更フラグOFF
        }


        /// <summary>
        /// ツリービューのアイテム右クリックイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void TreeViewItem_MouseRightButtonDown(object sender, MouseButtonEventArgs e)
        {
            TreeViewItem item = sender as TreeViewItem;

            if(item != null)
            {
                item.IsSelected = true;     // ツリービューのアイテムを選択状態にする
                e.Handled = true;           // 右クリックイベントは無視
            }
            
        }

        /// <summary>
        /// ツリーアイテムで選択した項目の種別から、"名前変更"の有効・無効設定
        /// </summary>
        /// <param name="selectedItem">ツリービューで選択中のアイテム</param>
        /// <returns></returns>
        private void SetRenameAvailability(TreeItem selectedItem)
        {
            bool ans = false;

            ans = RenameAvailability[selectedItem.ItemType];
            CanRename = ans;
        }
        

        /// <summary>
        /// 追加項目の要素の表示・非表示設定
        /// </summary>
        /// <param name="selectedItem"></param>
        private void SetAddElement(TreeItem selectedItem)
        {
            // 選択項目の種別から、表示・非表示設定を検索し、プロパティに設定する
            AddElemVis = AddElemAvailability[selectedItem.ItemType];
        }

        /// <summary>
        /// ポップアップメニューの名前変更選択
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void PopupRename_Click(object sender, RoutedEventArgs e)
        {
            // TreeItemで選択している項目を取得
            if (XmlTreeView.SelectedItem is TreeItem selectedItem)
            {
                RenameDialog renameDialog = new RenameDialog(this);
                renameDialog.WindowStartupLocation = WindowStartupLocation.CenterScreen;
                renameDialog.ShowDialog();
                if (renameDialog.DialogResult == true)   // 名前変更時にファイル変更済み状態にする
                {
                    if(selectedItem.ItemType == (int)ItemType.ECI)
                    {
                        projMgmt.SaveEciProject();          // プロジェクトファイルの更新
                    }
                    else
                    {
                        projMgmt.SetIsChanged2EciFile(true, selectedItem.Filename);
                        selectedItem.SearchRoot().IsChanged = true;                 // TreeItemの親を太字にする
                    }
                    UpdateTagRefData();                 // TagRefリスト更新
                    UpdateDatasetData();                // Datasetリスト更新
                    UpdateCollectorData();              // Collectorリスト更新
                }
            }
            
        }

        /// <summary>
        /// 追加→Tagクリックイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void AddTag_Click(object sender, RoutedEventArgs e)
        {
            // TreeItemで選択している項目を取得
            if (XmlTreeView.SelectedItem is TreeItem selectedItem)
            {
                //////////////////////////
                // Tagの追加画面表示
                //////////////////////////
                AddElement addElement = new AddElement(Define.Edit.AddElemType.Tag, this);
                addElement.WindowStartupLocation = WindowStartupLocation.CenterScreen;
                addElement.ShowDialog();
                if (addElement.DialogResult == true) // 追加完了時、対象のファイルを変更済み状態にする
                {
                    projMgmt.SetIsChanged2EciFile(true, selectedItem.Filename);
                    selectedItem.SearchRoot().IsChanged = true;                 // TreeItemの親を太字にする
                    UpdateTagRefData();                 // TagRefリスト更新
                }
            }

        }
        /// <summary>
        /// 追加→Datasetクリックイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void AddDataset_Click(object sender, RoutedEventArgs e)
        {
            // TreeItemで選択している項目を取得
            if (XmlTreeView.SelectedItem is TreeItem selectedItem)
            {
                //////////////////////////
                // Datasetの追加画面表示
                //////////////////////////
                AddElement addElement = new AddElement(Define.Edit.AddElemType.Dataset, this);
                addElement.WindowStartupLocation = WindowStartupLocation.CenterScreen;
                addElement.ShowDialog();
                if (addElement.DialogResult == true) // 追加完了時、対象のファイルを変更済み状態にする
                {
                    projMgmt.SetIsChanged2EciFile(true, selectedItem.Filename);
                    selectedItem.SearchRoot().IsChanged = true;                 // TreeItemの親を太字にする
                    UpdateDatasetData();                // Datasetリスト更新
                }
            }
            
        }
        /// <summary>
        /// 追加→DatasetTagRefクリックイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void AddDatasetTagRef_Click(object sender, RoutedEventArgs e)
        {
            // TreeItemで選択している項目を取得
            if (XmlTreeView.SelectedItem is TreeItem selectedItem)
            {
                ///////////////////////////////////
                // DatasetTagRefの追加画面表示
                ///////////////////////////////////
                AddElemRef addElemRef = new AddElemRef(Define.Edit.AddElemType.DatasetTagRef, this);
                addElemRef.WindowStartupLocation = WindowStartupLocation.CenterScreen;
                addElemRef.ShowDialog();
                if (addElemRef.DialogResult == true) // 追加完了時、対象のファイルを変更済み状態にする
                {
                    projMgmt.SetIsChanged2EciFile(true, selectedItem.Filename);
                    selectedItem.SearchRoot().IsChanged = true;                 // TreeItemの親を太字にする
                }
            }
            
        }
        /// <summary>
        /// 追加→TagRefクリックイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void AddTagRef_Click(object sender, RoutedEventArgs e)
        {
            // TreeItemで選択している項目を取得
            if (XmlTreeView.SelectedItem is TreeItem selectedItem)
            {
                ///////////////////////////////////
                // TagRefの追加画面表示
                ///////////////////////////////////
                AddElemRef addElemRef = new AddElemRef(Define.Edit.AddElemType.TagRef, this);
                addElemRef.WindowStartupLocation = WindowStartupLocation.CenterScreen;
                addElemRef.ShowDialog();
                if (addElemRef.DialogResult == true) // 追加完了時、対象のファイルを変更済み状態にする
                {
                    UpdateTagRefData();                 // コンボボックスのアイテム更新
                    projMgmt.SetIsChanged2EciFile(true, selectedItem.Filename);
                    selectedItem.SearchRoot().IsChanged = true;                 // TreeItemの親を太字にする
                }
            }
        }
        /// <summary>
        /// 追加→TagRefINクリックイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void AddTagRefIN_Click(object sender, RoutedEventArgs e)
        {
            // TreeItemで選択している項目を取得
            if (XmlTreeView.SelectedItem is TreeItem selectedItem)
            {
                ///////////////////////////////////
                // TagRefINの追加画面表示
                ///////////////////////////////////
                AddElemRef addElemRef = new AddElemRef(Define.Edit.AddElemType.TagRefIN, this);
                addElemRef.WindowStartupLocation = WindowStartupLocation.CenterScreen;
                addElemRef.ShowDialog();
                if (addElemRef.DialogResult == true) // 追加完了時、対象のファイルを変更済み状態にする
                {
                    UpdateTagRefData();                     // コンボボックスのアイテム更新
                    projMgmt.SetIsChanged2EciFile(true, selectedItem.Filename);
                    selectedItem.SearchRoot().IsChanged = true;                 // TreeItemの親を太字にする
                }
            }
        }
        /// <summary>
        /// 追加→TagRefOUTクリックイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void AddTagRefOUT_Click(object sender, RoutedEventArgs e)
        {
            // TreeItemで選択している項目を取得
            if (XmlTreeView.SelectedItem is TreeItem selectedItem)
            {
                ///////////////////////////////////
                // TagRefOUTの追加画面表示
                ///////////////////////////////////
                AddElemRef addElemRef = new AddElemRef(Define.Edit.AddElemType.TagRefOUT, this);
                addElemRef.WindowStartupLocation = WindowStartupLocation.CenterScreen;
                addElemRef.ShowDialog();
                if (addElemRef.DialogResult == true) // 追加完了時、対象のファイルを変更済み状態にする
                {
                    UpdateTagRefData();                 // コンボボックスのアイテム更新
                    projMgmt.SetIsChanged2EciFile(true, selectedItem.Filename);
                    selectedItem.SearchRoot().IsChanged = true;                 // TreeItemの親を太字にする
                }
            }
        }
        /// <summary>
        /// 追加→Collectorクリックイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void AddCollector_Click(object sender, RoutedEventArgs e)
        {
            // TreeItemで選択している項目を取得
            if (XmlTreeView.SelectedItem is TreeItem selectedItem)
            {
                ///////////////////////////////////
                // Collectorの追加画面表示
                ///////////////////////////////////
                AddElement addElement = new AddElement(Define.Edit.AddElemType.Collector, this);
                addElement.WindowStartupLocation = WindowStartupLocation.CenterScreen;
                addElement.ShowDialog();
                if (addElement.DialogResult == true) // 追加完了時、対象のファイルを変更済み状態にする
                {
                    projMgmt.SetIsChanged2EciFile(true, selectedItem.Filename);
                    selectedItem.SearchRoot().IsChanged = true;                 // TreeItemの親を太字にする
                    UpdateCollectorData();              // Collectorリスト更新
                }
            }
            
        }
        /// <summary>
        /// 追加→Mailboxクリックイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void AddMailbox_Click(object sender, RoutedEventArgs e)
        {
            // TreeItemで選択している項目を取得
            if (XmlTreeView.SelectedItem is TreeItem selectedItem)
            {
                ///////////////////////////////////
                // Mailboxの追加画面表示
                ///////////////////////////////////
                AddElement addElement = new AddElement(Define.Edit.AddElemType.Mailbox, this);
                addElement.WindowStartupLocation = WindowStartupLocation.CenterScreen;
                addElement.ShowDialog();
                if (addElement.DialogResult == true) // 追加完了時、対象のファイルを変更済み状態にする
                {
                    projMgmt.SetIsChanged2EciFile(true, selectedItem.Filename);
                    selectedItem.SearchRoot().IsChanged = true;                 // TreeItemの親を太字にする
                }
            }
        }
        /// <summary>
        /// 追加→CollectorRefクリックイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void AddCollectorRef_Click(object sender, RoutedEventArgs e)
        {
            // TreeItemで選択している項目を取得
            if (XmlTreeView.SelectedItem is TreeItem selectedItem)
            {
                ///////////////////////////////////
                // CollectorRefの追加画面表示
                ///////////////////////////////////
                AddElemRef addElemRef = new AddElemRef(Define.Edit.AddElemType.CollectorRef, this);
                addElemRef.WindowStartupLocation = WindowStartupLocation.CenterScreen;
                addElemRef.ShowDialog();
                if (addElemRef.DialogResult == true) // 追加完了時、対象のファイルを変更済み状態にする
                {
                    projMgmt.SetIsChanged2EciFile(true, selectedItem.Filename);
                    selectedItem.SearchRoot().IsChanged = true;                 // TreeItemの親を太字にする
                }
            }
        }
        /// <summary>
        /// 追加→Serviceクリックイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void AddService_Click(object sender, RoutedEventArgs e)
        {
            // TreeItemで選択している項目を取得
            if (XmlTreeView.SelectedItem is TreeItem selectedItem)
            {
                ///////////////////////////////////
                // Serviceの追加画面表示
                ///////////////////////////////////
                AddElement addElement = new AddElement(Define.Edit.AddElemType.Service, this);
                addElement.WindowStartupLocation = WindowStartupLocation.CenterScreen;
                addElement.ShowDialog();
                if (addElement.DialogResult == true) // 追加完了時、対象のファイルを変更済み状態にする
                {
                    projMgmt.SetIsChanged2EciFile(true, selectedItem.Filename);
                    selectedItem.SearchRoot().IsChanged = true;                 // TreeItemの親を太字にする
                }
            }
            
        }

        /// <summary>
        /// ポップアップメニューの"削除"選択
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void PopupRemove_Click(object sender, RoutedEventArgs e)
        {
            RemoveElement removeElement;                                // 削除確認ダイアログ

            // TreeItemで選択している項目を取得
            if (XmlTreeView.SelectedItem is TreeItem selectedItem)
            {
                ///////////////////////////////////////////////////////////////////////////
                // TreeViewで選択している項目によって表示する削除確認ダイアログを変更
                ///////////////////////////////////////////////////////////////////////////
                removeElement = new RemoveElement((EgConfig.ItemType)selectedItem.ItemType, this);
                removeElement.WindowStartupLocation = WindowStartupLocation.CenterScreen;
                removeElement.ShowDialog(); // ダイアログ表示
                if (removeElement.DialogResult == true)  // 削除処理完了時、対象のファイルを変更済み状態にする
                {
                    projMgmt.SetIsChanged2EciFile(true, selectedItem.Filename);
                    selectedItem.SearchRoot().IsChanged = true;                 // TreeItemの親を太字にする
                    UpdateTagRefData();         // TagRefリスト更新
                    UpdateDatasetData();        // Datasetリスト更新
                    UpdateCollectorData();      // Collectorリスト更新
                    projMgmt.SaveEciProject();  // プロジェクトファイルの更新
                }
            }
            
        }

        /// <summary>
        /// コンボボックスのデータ変更イベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void TagRefComboBox_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if(TagRefComboBox.SelectedItem is TagRef tagRef)        // 変更した名前をバインドデータに反映する
            {
                TagRefData.Name = tagRef.NAME;
            }
        }

        /// <summary>
        /// エラーチェックボタンクリック
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ErrorCheckBtn_Click(object sender, RoutedEventArgs e)
        {
            // プロジェクト管理データのチェックを行う
            ElemCheckMsgs = CheckConsistencyOfElem.CheckProjConsistency(treeItems, projMgmt);

        }

        /// <summary>
        /// ListViewをダブルクリックする
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ListView_MouseDoubleClick(object sender, MouseButtonEventArgs e)
        {
            TreeItem parent;

            /////////////////////////////////////////////////////////////
            // 選択したアイテムに対応するTreeItemを選択する
            /////////////////////////////////////////////////////////////
            if (listView.SelectedItem is ElemCheckMsg item)
            {
                if(item.TreeItem is TreeItem treeItem)
                {
                    treeItem.IsSelected = true; // 選択状態にする

                    /////////////////////////////////////
                    // TreeViewのItemを展開する
                    /////////////////////////////////////
                    parent = treeItem.Parent;
                    while(parent != null)
                    {
                        parent.IsExpanded = true;
                        parent = parent.Parent;
                    }
                }
            }
        }

        /// <summary>
        /// 項目を上に移動する
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void UpRow_Click(object sender, RoutedEventArgs e)
        {
            if(XmlTreeView.SelectedItem is TreeItem selectedItem)
            {
                if(selectedItem.ItemType == (int)ItemType.ECI)
                {
                    UpRowFile(selectedItem);
                    projMgmt.SaveEciProject();  // ECIプロジェクト更新
                }
                else
                {
                    selectedItem.UpRow();
                }

                
            }
        }

        /// <summary>
        /// 項目を下に移動する
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void DownRow_Click(object sender, RoutedEventArgs e)
        {
            if (XmlTreeView.SelectedItem is TreeItem selectedItem)
            {
                if (selectedItem.ItemType == (int)ItemType.ECI)
                {
                    DownRowFile(selectedItem);
                    projMgmt.SaveEciProject();  // ECIプロジェクト更新
                }
                else
                {
                    selectedItem.DownRow();
                }

                
            }
        }

        /// <summary>
        /// 項目を昇順ソートする
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void AscSort_Click(object sender, RoutedEventArgs e)
        {
            int inN = 0;

            if (XmlTreeView.SelectedItem is TreeItem selectedItem)
            {
                if (selectedItem.ItemType == (int)ItemType.ECI)
                {
                    ////////////////////////////////////////////////////////////////////
                    // 昇順ソート
                    // 最後の要素から探索し、1回の探索で最初の要素に小さい結果を格納する
                    inN = treeItems.Count();                                    // 選択したリストの要素数を取得
                    for (int i = 0; i < inN - 1; i++)
                    {
                        for (int j = inN - 1; j > i; j--)
                        {
                            if (0 < string.Compare(treeItems[j - 1].Name, treeItems[j].Name))
                            {
                                UpRowFile(treeItems[j]);            // 交換する
                                projMgmt.SaveEciProject();          // ECIプロジェクト更新
                            }
                        }
                    }
                }
                else
                {
                    if(selectedItem.Parent is TreeItem parent)
                    {
                        ////////////////////////////////////////////////////////////////////
                        // 昇順ソート
                        // 最後の要素から探索し、1回の探索で最初の要素に小さい結果を格納する
                        inN = parent.Children.Count();                          // 選択したリストの要素数を取得
                        for (int i = 0; i < inN - 1; i++)
                        {
                            for (int j = inN - 1; j > i; j--)
                            {
                                if(0 < string.Compare(parent.Children[j-1].Name, parent.Children[j].Name))
                                {
                                    parent.Children[j].UpRow();                 // 交換する
                                }
                            }
                        }
                    }
                    
                }
            }
        }

        /// <summary>
        /// 項目を降順ソートする
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void DesSort_Click(object sender, RoutedEventArgs e)
        {
            int inN = 0;

            if (XmlTreeView.SelectedItem is TreeItem selectedItem)
            {
                if (selectedItem.ItemType == (int)ItemType.ECI)
                {
                    ////////////////////////////////////////////////////////////////////
                    // 降順ソート
                    // 最後の要素から探索し、1回の探索で最初の要素に大きい結果を格納する
                    inN = treeItems.Count();                                    // 選択したリストの要素数を取得
                    for (int i = 0; i < inN - 1; i++)
                    {
                        for (int j = inN - 1; j > i; j--)
                        {
                            if (string.Compare(treeItems[j - 1].Name, treeItems[j].Name) < 0)
                            {
                                UpRowFile(treeItems[j]);            // 交換する
                                projMgmt.SaveEciProject();          // ECIプロジェクト更新
                            }
                        }
                    }
                }
                else
                {
                    if (selectedItem.Parent is TreeItem parent)
                    {
                        ////////////////////////////////////////////////////////////////////
                        // 降順ソート
                        // 最後の要素から探索し、1回の探索で最初の要素に大きい結果を格納する
                        inN = parent.Children.Count();                          // 選択したリストの要素数を取得
                        for (int i = 0; i < inN - 1; i++)
                        {
                            for (int j = inN - 1; j > i; j--)
                            {
                                if (string.Compare(parent.Children[j - 1].Name, parent.Children[j].Name) < 0)
                                {
                                    parent.Children[j].UpRow();                 // 交換する
                                }
                            }
                        }
                    }

                }
            }
        }

        /// <summary>
        /// ECIファイルを上に移動する
        /// </summary>
        /// <param name="treeItem"></param>
        /// <returns></returns>
        private bool UpRowFile(TreeItem treeItem)
        {
            bool isDone = false;
            int index;
            EciFileMgmt eciFileMgmt;

            index = MainWindow.treeItems.IndexOf(treeItem);
            if (index < 0)       // 見つからなかった
            {
                return isDone;
            }
            else if (index == 0) // 最初の要素
            {
                isDone = true;
                return isDone;
            }

            // ECIファイルのデータを移動する
            eciFileMgmt = MainWindow.projMgmt.eciList[index];
            MainWindow.projMgmt.eciList.RemoveAt(index);
            MainWindow.projMgmt.eciList.Insert(index - 1, eciFileMgmt);

            // TreeItemの更新
            MainWindow.treeItems.Move(index, index - 1);            // TreeItemのデータを移動する
            for (int i = 0; i < MainWindow.treeItems.Count; i++)    // ツリーアイテムのインデックス番号更新
            {
                MainWindow.treeItems[i].Index = i;
            }

            isDone = true;

            return isDone;
        }

        /// <summary>
        /// ECIファイルを下に移動する
        /// </summary>
        /// <param name="treeItem"></param>
        /// <returns></returns>
        public bool DownRowFile(TreeItem treeItem)
        {
            bool isDone = false;
            int index;
            EciFileMgmt eciFileMgmt;

            index = MainWindow.treeItems.IndexOf(treeItem);
            if (index < 0)       // 見つからなかった
            {
                return isDone;
            }
            else if (MainWindow.treeItems.Count - 1 <= index) // 最後の要素
            {
                isDone = true;
                return isDone;
            }

            // ECIファイルのデータを移動する
            eciFileMgmt = MainWindow.projMgmt.eciList[index + 1];
            MainWindow.projMgmt.eciList.RemoveAt(index + 1);
            MainWindow.projMgmt.eciList.Insert(index, eciFileMgmt);

            // TreeItemの更新
            MainWindow.treeItems.Move(index, index + 1);            // TreeItemのデータを移動する
            for (int i = 0; i < MainWindow.treeItems.Count; i++)    // ツリーアイテムのインデックス番号更新
            {
                MainWindow.treeItems[i].Index = i;
            }

            isDone = true;

            return isDone;
        }

        #region プロパティウインドウ内容変更イベント

        /// <summary>
        /// テキストボックスの内容変更
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void TextBox_TextChanged(object sender, TextChangedEventArgs e)
        {
            // TreeView選択項目のプロパティ変更の際、ファイル変更状態をONに設定
            SetEciFileChanged();

            /////////////////////////////////////////////////////////////////////////////
            // プロパティウインドウのName属性の値を選択中のTreeItemの名前に反映する
            /////////////////////////////////////////////////////////////////////////////
            if (this.XmlTreeView.SelectedItem is TreeItem selectedItem)
            {
                switch (selectedItem.ItemType)
                {
                    case (int)ItemType.Tag:
                        selectedItem.Name = TagData?.NAME;
                        UpdateTagRefData();
                        break;
                    case (int)ItemType.Dataset:
                        selectedItem.Name = DatasetData?.NAME;
                        UpdateDatasetData();
                        break;
                    case (int)ItemType.Service:
                        selectedItem.Name = ServiceData?.NAME;
                        break;
                    case (int)ItemType.DatasetTagRef:
                    case (int)ItemType.TagRef:
                    case (int)ItemType.TagRefIN:
                    case (int)ItemType.TagRefOUT:
                        selectedItem.Name = TagRefData?.NAME;
                        break;
                    case (int)ItemType.Collector:
                        selectedItem.Name = CollectorData?.NAME;
                        UpdateCollectorData();
                        break;
                    case (int)ItemType.CollectorRef:
                        selectedItem.Name = CollectorRefData?.NAME;
                        break;
                    case (int)ItemType.MailBox:
                        selectedItem.Name = MailBoxData?.NAME;
                        break;
                    default:
                        break;
                }
            }
        }

        /// <summary>
        /// コンボボックスの選択変更
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ComboBox_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            // TreeView選択項目のプロパティ変更の際、ファイル変更状態をONに設定
            SetEciFileChanged();
        }

        #endregion



        /// <summary>
        /// 新しいファイルの追加
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void AddNewFile_Click(object sender, RoutedEventArgs e)
        {
            TreeItem treeItem;
            int index;
            string dir;

            dir = System.IO.Path.GetDirectoryName(projMgmt.eciProjName);

            // 新規ファイル作成ダイアログ表示
            CreateNewFile CreateNewFile = new CreateNewFile(dir);
            CreateNewFile.ShowDialog();
            if (CreateNewFile.DialogResult == true)
            {
                ////////////////////////////////////////////////////////////
                // 既にプロジェクトでECIファイルを管理済みの場合は更新、
                // ない場合は追加する
                ////////////////////////////////////////////////////////////

                index = projMgmt.UpdateEciFile(CreateNewFile.Path); // ECIファイル更新する
                if (!(index < 0))
                {
                    // ツリー更新
                    UpdateTreeItems(index, projMgmt.eciList[index]);
                }
                else
                {
                    ////////////////////////////////////////////
                    // プロジェクトにECIファイルを追加
                    ////////////////////////////////////////////
                    projMgmt.AddEciFile(CreateNewFile.Path);

                    ////////////////////////////////////////////
                    // ツリーにECIファイルを追加
                    ////////////////////////////////////////////
                    index = projMgmt.eciList.Count - 1;
                    treeItem = TreeItem.EciFileMgmt2TreeItem(projMgmt.eciList[index]);
                    treeItems.Add(treeItem);

                    isFileChanged.Add(false);           // ファイル更新状態を追加
                }

                ////////////////////////////////////////////////
                // プロパティウインドウのTagRef選択にデータ登録
                ////////////////////////////////////////////////
                UpdateTagRefData();

                //////////////////////////////////////////////////////////////////////
                // プロパティウインドウのCollector項目DatasetName選択にデータ登録
                //////////////////////////////////////////////////////////////////////
                UpdateDatasetData();

                //////////////////////////////////////////////////////////////////////
                // プロパティウインドウのCollectorRef項目Collector選択にデータ登録
                // TODO:重複しない処理を入れる
                //////////////////////////////////////////////////////////////////////
                UpdateCollectorData();

                // プロジェクトファイルを更新
                projMgmt.SaveEciProject();
            }
        }

        /// <summary>
        /// 既存のファイルを追加
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void AddExistingFile_Click(object sender, RoutedEventArgs e)
        {
            DialogResult dialogResult;
            TreeItem treeItem;
            int index;
            string path, dir, name;
            bool isDone;
            EciFileMgmt eciFileMgmt;    // ファイルの形式正しいかチェック用

            ////////////////////////////////////
            // ファイル選択を行う
            ////////////////////////////////////
            Microsoft.Win32.OpenFileDialog dialog = new Microsoft.Win32.OpenFileDialog();
            dialog.InitialDirectory = this.BaseDirectory == "" ? AppDomain.CurrentDomain.BaseDirectory : this.BaseDirectory;
            dialog.FileName = "";
            dialog.Filter = "ECIファイル|*.xml";
            dialog.FilterIndex = 0;
            if (dialog.ShowDialog() != true)
            {
                return;
            }

            if (this.BaseDirectory == "") // 保管したディレクトリパスが無い場合に限り
                this.BaseDirectory = System.IO.Path.GetDirectoryName(dialog.FileName);  // 既存のファイルがあるパスを保管


            ////////////////////////////////////
            // ECIファイルの形式かチェック
            ////////////////////////////////////
            eciFileMgmt = EciFileMgmt.ReadEciFile(dialog.FileName);
            if(eciFileMgmt is null)
            {
                // ファイル読み込み失敗メッセージ
                System.Windows.Forms.MessageBox.Show(String.Format(Properties.Resources.MSGBOX_ERROR_016_MSGTEXT, System.IO.Path.GetFileName(dialog.FileName)), Properties.Resources.MSGBOX_ERROR_016_CAPTION, MessageBoxButtons.OK, MessageBoxIcon.Hand);
                return;
            }

            ////////////////////////////////////////////////////////////////////
            // 同名のファイルが、既にプロジェクトで管理しているかチェック
            ////////////////////////////////////////////////////////////////////
            dir = System.IO.Path.GetDirectoryName(projMgmt.eciProjName);    // プロジェクトファイルのディレクトリ取得
            name = System.IO.Path.GetFileName(dialog.FileName);             // 追加するファイル名から名前を取得
            path = System.IO.Path.Combine(dir, name);                       // プロジェクトの同ディレクトリのファイルパスを作成
            isDone = projMgmt.SearchListForSameName(name);
            if (isDone == true)  // ファイルをプロジェクトで管理中
            {
                ////////////////////////////////////////////////////////////////////////////
                // プロジェクトディレクトリ外からファイルを追加する際、
                // プロジェクトディレクトリ内のファイルを上書きする
                ////////////////////////////////////////////////////////////////////////////
                dialogResult = System.Windows.Forms.MessageBox.Show(Properties.Resources.MSGBOX_WARN_007_MSGTEXT, Properties.Resources.MSGBOX_WARN_007_CAPTION, MessageBoxButtons.OKCancel, MessageBoxIcon.Exclamation);
                if (dialogResult != System.Windows.Forms.DialogResult.OK)
                {
                    return; // 上書きしない場合、ファイル追加処理は行わない
                }

                if (dir != System.IO.Path.GetDirectoryName(dialog.FileName))
                {
                    System.IO.File.Copy(dialog.FileName, path, true);   // 上書き
                }

            }
            else    // ファイルをプロジェクトで未管理
            {
                //////////////////////////////////////////////////////////////////////////////////////
                // プロジェクトファイルと同ディレクトリに指定したファイルが存在しない場合、コピーする
                //////////////////////////////////////////////////////////////////////////////////////
                if (File.Exists(path) == false)
                {
                    try
                    {
                        System.IO.File.Copy(dialog.FileName, path); // プロジェクトのディレクトリにECIファイルをコピーする
                    }
                    catch (Exception)
                    {
                        return;
                    }
                }
                else
                {
                    /////////////////////////////////////////////////////////////////
                    // プロジェクトのディレクトリ外にあるファイルをプロジェクトの
                    // ディレクトリに追加
                    // (プロジェクトディレクトリにあるファイルの追加ではない)
                    /////////////////////////////////////////////////////////////////
                    if (path != dialog.FileName)
                    {
                        dialogResult = System.Windows.Forms.MessageBox.Show(Properties.Resources.MSGBOX_WARN_009_MSGTEXT, Properties.Resources.MSGBOX_WARN_009_CAPTION, MessageBoxButtons.OKCancel, MessageBoxIcon.Exclamation);
                        if (dialogResult != System.Windows.Forms.DialogResult.OK)
                        {
                            return; // 上書きしない場合、ファイル追加処理は行わない
                        }
                        
                        // 上書き
                        System.IO.File.Copy(dialog.FileName, path, true);
                    }
                }
            }

            /////////////////////////////////////////////////////////
            // データ・TreeView更新処理
            // 既にプロジェクトでファイルを管理している場合は更新
            // 管理していない場合は追加する
            /////////////////////////////////////////////////////////
            index = projMgmt.SearchEciIndex(path); 
            if (!(index < 0))
            {
                index = projMgmt.UpdateEciFile(path);               // ECIファイル更新する
                UpdateTreeItems(index, projMgmt.eciList[index]);    // ツリー更新
            }
            else
            {
                projMgmt.AddEciFile(path);                          // プロジェクトにECIファイルを追加

                // ツリーにECIファイルを追加
                index = projMgmt.eciList.Count - 1;
                treeItem = TreeItem.EciFileMgmt2TreeItem(projMgmt.eciList[index]);
                treeItems.Add(treeItem);

                isFileChanged.Add(false);                           // ファイル更新状態を追加
            }

            ////////////////////////////////////////////////
            // プロパティウインドウのTagRef選択にデータ登録
            ////////////////////////////////////////////////
            UpdateTagRefData();

            //////////////////////////////////////////////////////////////////////
            // プロパティウインドウのCollector項目DatasetName選択にデータ登録
            //////////////////////////////////////////////////////////////////////
            UpdateDatasetData();

            //////////////////////////////////////////////////////////////////////
            // プロパティウインドウのCollectorRef項目Collector選択にデータ登録
            // TODO:重複しない処理を入れる
            //////////////////////////////////////////////////////////////////////
            UpdateCollectorData();

            // プロジェクトファイルを更新
            projMgmt.SaveEciProject();
        }

        /// <summary>
        /// すべて保存クリック
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void SaveAllBtn_Click(object sender, RoutedEventArgs e)
        {
            bool isDone;

            // ファイル変更イベントOFF
            fsw.EnableRaisingEvents = false;

            if(Define.STATE_PROJ == (this.State & Define.MASK_STATE))       // プロジェクトの保存
            {
                isDone = projMgmt.SaveAll();
                if (isDone == true)
                {
                    // 太字をクリアする
                    foreach(TreeItem treeItem in treeItems)
                    {
                        treeItem.IsChanged = false;
                    }
                    System.Windows.Forms.MessageBox.Show(String.Format(Properties.Resources.MSGBOX_INFO_002_MSGTEXT, projMgmt.eciProjName), Properties.Resources.MSGBOX_INFO_002_CAPTION, MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                }
                else
                {
                    System.Windows.Forms.MessageBox.Show(String.Format(Properties.Resources.MSGBOX_ERROR_014_MSGTEXT, projMgmt.eciProjName), Properties.Resources.MSGBOX_ERROR_014_CAPTION, MessageBoxButtons.OK, MessageBoxIcon.Hand);
                }
            }
            else if(Define.STATE_FILE == (this.State & Define.MASK_STATE))  // ファイルの保存
            {
                // ファイルを保存
                if (1 <= projMgmt.eciList?.Count)
                {
                    isDone = projMgmt.SaveFile(projMgmt.eciList[0].eciFileName);
                    if (isDone == true)
                    {
                        // 太字をクリアする
                        foreach (TreeItem treeItem in treeItems)
                        {
                            treeItem.IsChanged = false;
                        }
                        System.Windows.Forms.MessageBox.Show(String.Format(Properties.Resources.MSGBOX_INFO_001_MSGTEXT, projMgmt.eciList[0].eciFileName), Properties.Resources.MSGBOX_INFO_001_CAPTION, MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                    }
                    else
                    {
                        System.Windows.Forms.MessageBox.Show(String.Format(Properties.Resources.MSGBOX_ERROR_015_MSGTEXT, projMgmt.eciList[0].eciFileName), Properties.Resources.MSGBOX_ERROR_015_CAPTION, MessageBoxButtons.OK, MessageBoxIcon.Hand);
                    }
                }
                
            }

            // ファイル変更イベントON
            fsw.EnableRaisingEvents = true;
        }

        /// <summary>
        /// TreeItemを更新する
        /// </summary>
        /// <param name="index">更新するTreeItemsのインデックス</param>
        /// <param name="eciFileMgmt">更新するECIファイル</param>
        private static void UpdateTreeItems(int index, EciFileMgmt eciFileMgmt)
        {
            TreeItem treeItem = TreeItem.EciFileMgmt2TreeItem(eciFileMgmt);
            int i = 0;

            // 更新するインデックスが存在するか確認
            if(0 <= index && index < treeItems.Count)
            {
                // ECIファイルを更新する
                treeItems.RemoveAt(index);
                if (index < treeItems.Count)
                {
                    treeItems.Insert(index, treeItem);
                }
                else
                {
                    treeItems.Add(treeItem);
                }

                // TreeItemのインデックスを更新する
                foreach (TreeItem item in treeItems)
                {
                    item.Index = i++;
                }
            }
            
        }

        /// <summary>
        /// 外部エディタ設定
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void SettingEditor(object sender, RoutedEventArgs e)
        {
            EditorRegistration editorRegistration = new EditorRegistration();
            editorRegistration.WindowStartupLocation = WindowStartupLocation.CenterScreen;
            editorRegistration.ShowDialog();
        }

        

        /// <summary>
        /// 外部エディタ起動
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void RunEditor(object sender, RoutedEventArgs e)
        {
            string path, dir;                           // 読み込み先の設定ファイルパス作成
            EgConfigXML xml;                            // 設定ファイルデシリアライズ用
            System.Diagnostics.ProcessStartInfo pInfo;  // 外部エディタ実行用
            string args;                                // 外部エディタ実行時の起動引数
            EciFileMgmt eciFileMgmt;
            string filename;
            DialogResult result;

            ///////////////////////////////////////////////////////////////////
            // 選択しているツリーのアイテムのファイルを外部エディタで開く
            ///////////////////////////////////////////////////////////////////
            if (XmlTreeView.SelectedItem is TreeItem selectedItem)
            {
                // ファイル変更未保存かチェック
                eciFileMgmt = projMgmt.SearchEciFileMgmt(selectedItem.Filename);
                if(eciFileMgmt is EciFileMgmt)
                {
                    if(eciFileMgmt.isChanged == true)
                    {
                        try
                        {
                            filename = System.IO.Path.GetFileName(selectedItem.Filename);
                        }
                        catch (Exception)
                        {
                            filename = "";
                        }

                        result = System.Windows.Forms.MessageBox.Show(String.Format(Properties.Resources.MSGBOX_WARN_002_MSGTEXT, filename), Properties.Resources.MSGBOX_WARN_002_CAPTION, MessageBoxButtons.YesNo, MessageBoxIcon.Exclamation);
                        if(result == System.Windows.Forms.DialogResult.Yes)
                        {
                            fsw.EnableRaisingEvents = false;
                            eciFileMgmt.Save();
                            fsw.EnableRaisingEvents = true;
                        }
                    }
                }

                // ツールと同じディレクトリにあるConfigファイルを読み込む
                dir = AppDomain.CurrentDomain.BaseDirectory;                    // ツールのディレクトリパス取得
                path = System.IO.Path.Combine(dir, Define.Editor.CONFIG_NAME);  // Configファイルパス作成
                if (File.Exists(path) == false)                                 // Configファイルが存在しない場合作成
                {
                    xml = new EgConfigXML()
                    {
                        Editor = new Editor()
                        {
                            Path = Define.Editor.DEFAULT_EDITOR_PATH ,
                            Args = Define.Editor.REPLACE_PATH
                        }
                    };
                    MnXmlUtility.SaveToFile(path, xml, typeof(EgConfigXML));
                }

                xml = MnXmlUtility.LoadFromFile<EgConfigXML>(path);             // Configファイル読み込み
                if (xml is EgConfigXML)
                {
                    if (File.Exists(xml.Editor.Path) == false)
                    {
                        System.Windows.Forms.MessageBox.Show(Properties.Resources.MSGBOX_ERROR_010_MSGTEXT, Properties.Resources.MSGBOX_ERROR_010_CAPTION, MessageBoxButtons.OK, MessageBoxIcon.Hand);
                        return;
                    }

                    // 起動引数ファイルパスを置換する
                    args = Define.Editor.ReplaceArguments(xml.Editor.Args, selectedItem.Filename);

                    // Configの実行ファイルを起動する
                    pInfo = new System.Diagnostics.ProcessStartInfo();
                    pInfo.FileName = xml.Editor.Path;                       // 実行ファイルパスを設定する
                    pInfo.Arguments = args;                                 // 引数を設定する
                    try
                    {
                        System.Diagnostics.Process.Start(pInfo);            // 外部エディタを実行する

                    }
                    catch (Exception)
                    {
                        System.Windows.Forms.MessageBox.Show(Properties.Resources.MSGBOX_ERROR_011_MSGTEXT, Properties.Resources.MSGBOX_ERROR_011_CAPTION, MessageBoxButtons.OK, MessageBoxIcon.Hand);
                    }
                    
                }
                else
                {
                    System.Windows.Forms.MessageBox.Show(Properties.Resources.MSGBOX_ERROR_012_MSGTEXT, Properties.Resources.MSGBOX_ERROR_012_CAPTION, MessageBoxButtons.OK, MessageBoxIcon.Hand);
                    pInfo = new System.Diagnostics.ProcessStartInfo();
                    pInfo.FileName = Define.Editor.DEFAULT_EDITOR_PATH;     // 実行ファイルパスを設定する
                    pInfo.Arguments = selectedItem.Filename;                // 引数を設定する
                    try
                    {
                        System.Diagnostics.Process.Start(pInfo);            // 外部エディタを実行する

                    }
                    catch (Exception)
                    {
                        System.Windows.Forms.MessageBox.Show(Properties.Resources.MSGBOX_ERROR_011_MSGTEXT, Properties.Resources.MSGBOX_ERROR_011_CAPTION, MessageBoxButtons.OK, MessageBoxIcon.Hand);
                    }
                }
            }
        }

        /// <summary>
        /// ファイルの変更監視
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Fsw_Changed(object sender, FileSystemEventArgs e)
        {
            int i = 0;
            ObservableCollection<TreeItem> copy = new ObservableCollection<TreeItem>(treeItems);    // 監視イベント実行時のTreeItemをコピーする(スレッド間のデータ変更による例外を避けるため)

            // TreeItemsのfilename検索
            foreach(TreeItem item in copy)
            {
                // TreeItemのパスと変更されたファイルパスが一致していた時、変更フラグをONにする
                if(item.Filename == e.FullPath)
                {
                    isFileChanged[i] = true;
                    break;
                }

                i++;
            }

        }
        
        /// <summary>
        /// 画面フォーカスイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Window_Activated(object sender, EventArgs e)
        {
            int i, index;
            DialogResult dialogResult;

            // 変更ファイルがある場合、変更しますか？　データ・TreeView更新
            for (i = 0; i < isFileChanged.Count; i++)
            {
                // 変更ファイルがある
                if (isFileChanged[i] == true)
                {
                    dialogResult = System.Windows.Forms.MessageBox.Show(String.Format(Properties.Resources.MSGBOX_WARN_008_MSGTEXT, System.IO.Path.GetFileName(treeItems[i].Filename)), Properties.Resources.MSGBOX_WARN_008_CAPTION, MessageBoxButtons.OKCancel, MessageBoxIcon.Exclamation);
                    if (dialogResult == System.Windows.Forms.DialogResult.OK)
                    {
                        // ECIファイルおよびツリーアイテムを更新する
                        index = projMgmt.UpdateEciFile(treeItems[i].Filename); // ECIファイル更新する
                        if (!(index < 0))
                        {
                            UpdateTreeItems(index, projMgmt.eciList[index]);    // ツリー更新
                            
                        }
                        else    // ECIファイル読み込み失敗
                        {
                            System.Windows.Forms.MessageBox.Show(String.Format(Properties.Resources.MSGBOX_ERROR_016_MSGTEXT, System.IO.Path.GetFileName(treeItems[i].Filename)), Properties.Resources.MSGBOX_ERROR_016_CAPTION, MessageBoxButtons.OK, MessageBoxIcon.Hand);
                        }
                    }

                    isFileChanged[i] = false;       // 変更状態解除
                }
            }
        }

        /// <summary>
        /// エクスポートクリック
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Export_Click(object sender, RoutedEventArgs e)
        {
            bool canDo;

            // ファイル変更未保存かチェック
            canDo = CheckUnsaveFile();
            if (canDo == false)
            {
                return;
            }

            ExportPack exportProject = new ExportPack();
            exportProject.WindowStartupLocation = WindowStartupLocation.CenterScreen;
            exportProject.ShowDialog();
            
        }

        /// <summary>
        /// インポートクリック
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Import_Click(object sender, RoutedEventArgs e)
        {
            bool canDo;
            ObservableCollection<TreeItem> items;

            // ファイル変更未保存かチェック
            canDo = CheckUnsaveFile();
            if (canDo == false)
            {
                return;
            }

            ImportPack importProject = new ImportPack();
            importProject.WindowStartupLocation = WindowStartupLocation.CenterScreen;
            importProject.ShowDialog();

            if(importProject.DialogResult == true)
            {
                ////////////////////////////////////////////////
                // 読み込んだECIファイルをTreeViewに表示
                ////////////////////////////////////////////////
                items = TreeItem.EciProjMgmt2TreeItem(projMgmt);    // ECIプロジェクト管理データからTreeItemを作成
                treeItems.Clear();
                foreach (TreeItem item in items)
                {
                    treeItems.Add(item);
                }

                // TreeItemの変更状態をなしにする
                isFileChanged.Clear();
                for (int i = 0; i < treeItems.Count; i++)
                {
                    isFileChanged.Add(false);
                }

                ////////////////////////////////////////////////
                // プロパティウインドウのTagRef選択にデータ登録
                ////////////////////////////////////////////////
                UpdateTagRefData();

                //////////////////////////////////////////////////////////////////////
                // プロパティウインドウのCollector項目DatasetName選択にデータ登録
                //////////////////////////////////////////////////////////////////////
                UpdateDatasetData();

                //////////////////////////////////////////////////////////////////////
                // プロパティウインドウのCollectorRef項目Collector選択にデータ登録
                // TODO:重複しない処理を入れる
                //////////////////////////////////////////////////////////////////////
                UpdateCollectorData();

                SaveAllFileEnable = true;   // すべて保存ボタンを押せるようにする

                //////////////////////////////////////////////////////////////////////
                // ファイル監視を有効にする
                //////////////////////////////////////////////////////////////////////
                fsw.Path = System.IO.Path.GetDirectoryName(projMgmt.eciProjName);
                fsw.EnableRaisingEvents = true;

                ProjectName = System.IO.Path.GetFileName(projMgmt.eciProjName);     // プロジェクト名を設定
                ProjectNameVis = Visibility.Visible;                                // プロジェクト名を表示
                ElemCheckMsgs.Clear();                                              // メッセージ表示ウインドウクリア
                this.State = Define.STATE_PROJ;                                     // 状態を変化させる
            }

        }

        /// <summary>
        /// 起動順序設定クリック
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void BootOrderSetting_Click(object sender, RoutedEventArgs e)
        {
            // ツリービュー選択時のみ実行
            if(XmlTreeView.SelectedItem is TreeItem)
            {
                BootOrderSetting bootOrderSetting = new BootOrderSetting(this);
                bootOrderSetting.WindowStartupLocation = WindowStartupLocation.CenterScreen;
                bootOrderSetting.ShowDialog();
            }
        }

        /// <summary>
        /// TreeView選択項目のプロパティ変更の際、ファイル変更状態をONに設定
        /// </summary>
        private void SetEciFileChanged()
        {
            if (selectedChanged == false)
            {
                if (XmlTreeView.SelectedItem is TreeItem selectedItem)
                {
                    projMgmt.SetIsChanged2EciFile(true, selectedItem.Filename);
                    selectedItem.SearchRoot().IsChanged = true;                 // TreeItemの親を太字にする

                }
            }
        }
        

        /// <summary>
        /// バージョン情報クリック
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Version_Click(object sender, RoutedEventArgs e)
        {
            VersionInformation versionInformation = new VersionInformation();
            versionInformation.WindowStartupLocation = WindowStartupLocation.CenterScreen;
            versionInformation.ShowDialog();
        }

        /// <summary>
        /// ファイル変更未保存チェック
        /// </summary>
        /// <returns>true:この後の処理を実行可能にする, false:この後の処理を実行不可</returns>
        private bool CheckUnsaveFile()
        {
            bool isChanged, canDo = true;
            DialogResult result;
            string filename = "", msg;

            // ファイル変更未保存かチェック
            isChanged = projMgmt.GetChangedFilename(out List<string> paths);
            if (isChanged == true)
            {
                foreach (string path in paths)
                {
                    try
                    {
                        filename = filename + System.IO.Path.GetFileName(path) + "\n";
                    }
                    catch (Exception)
                    {
                        filename = "";
                    }
                }

                msg = String.Format(Properties.Resources.MSGBOX_WARN_000_MSGTEXT, filename);
                result = System.Windows.Forms.MessageBox.Show(msg, Properties.Resources.MSGBOX_WARN_000_CAPTION, MessageBoxButtons.OKCancel, MessageBoxIcon.Exclamation);
                if (result == System.Windows.Forms.DialogResult.Cancel)
                {
                    canDo = false;
                }
            }

            return canDo;
        }

        /// <summary>
        /// XMLツール終了時
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Window_Closing(object sender, CancelEventArgs e)
        {
            bool isChanged;
            DialogResult result;
            string filename = "", msg;

            // ファイル変更未保存かチェック
            isChanged = projMgmt.GetChangedFilename(out List<string> paths);
            if (isChanged == true)
            {
                foreach (string path in paths)
                {
                    try
                    {
                        filename = filename + System.IO.Path.GetFileName(path) + "\n";
                    }
                    catch (Exception)
                    {
                        filename = "";
                    }
                }

                msg = String.Format(Properties.Resources.MSGBOX_WARN_000_MSGTEXT, filename);
                result = System.Windows.Forms.MessageBox.Show(msg, Properties.Resources.MSGBOX_WARN_000_CAPTION, MessageBoxButtons.OKCancel, MessageBoxIcon.Exclamation);
                if (result == System.Windows.Forms.DialogResult.Cancel)
                {
                    e.Cancel = true;
                    return;
                }
            }
        }

        private void XmlTreeView_Selected(object sender, RoutedEventArgs e)
        {
            FrameworkElement element = e.OriginalSource as FrameworkElement;
            element.BringIntoView();
        }

        /// <summary>
        /// 終了クリック
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Close_Click(object sender, RoutedEventArgs e)
        {
            this.Close();
        }

    }

    #region コンバーター

    /// <summary>
    /// 選択項目タイプからプロパティ表示ウインドウに対応したSelectedIndexへ変換する
    /// </summary>
    public class TabIndexConverter : IValueConverter
    {
        public object Convert(object value, Type targetType, object parameter, CultureInfo culture)
        {
            int Index = Define.PropertyWindow.Tab_None;

            
            switch (value)
            {
                case (int)ItemType.Tag:
                    Index = Define.PropertyWindow.Tab_Tag;
                    break;
                case (int)ItemType.Dataset:
                    Index = Define.PropertyWindow.Tab_Dataset;
                    break;
                case (int)ItemType.DatasetTagRef:
                case (int)ItemType.TagRef:
                case (int)ItemType.TagRefIN:
                case (int)ItemType.TagRefOUT:
                    Index = Define.PropertyWindow.Tab_TagRef;
                    break;
                case (int)ItemType.Service:
                    Index = Define.PropertyWindow.Tab_Container;
                    break;
                case (int)ItemType.Collector:
                    Index = Define.PropertyWindow.Tab_Collector;
                    break;
                case (int)ItemType.CollectorRef:
                    Index = Define.PropertyWindow.Tab_CollectorRef;
                    break;
                case (int)ItemType.MailBox:
                    Index = Define.PropertyWindow.Tab_MailBox;
                    break;
                default:
                    Index = Define.PropertyWindow.Tab_None;
                    break;
            }
            return Index;
        }

        // TODO:逆コンバータ不要か？
        public object ConvertBack(object value, Type targetType, object parameter, CultureInfo culture)
        {
            return -1;
        }
    }

    /// <summary>
    /// データ型(数字)を数値に変換
    /// </summary>
    public class DataTypeConverter : IValueConverter
    {
        public object Convert(object value, Type targetType, object parameter, CultureInfo culture)
        {
            int Index = 0, min = -1, max = -1;
            bool isFirst = true;
            string str;

            str = value.ToString();
            if (Int32.TryParse(str, out int num) == true)
            {
                // タイプの最大・最小値を求める
                Array array = System.Enum.GetValues(typeof(Define.PropertyWindow.TagType));
                foreach(Define.PropertyWindow.TagType tagType in array)
                {
                    if(isFirst == true)
                    {
                        min = (int)tagType;
                        max = (int)tagType;
                        isFirst = false;
                    }

                    if((int)tagType < min)
                    {
                        min = (int)tagType;
                    }
                    if (max < (int)tagType)
                    {
                        max = (int)tagType;
                    }
                }

                // 最大・最小の範囲内かチェック
                if(min <= num && num <= max)
                {
                    Index = num-1;      // 範囲内の場合、コンボボックスのIndexに合わせて値を調節する
                }
                else
                {
                    Index = -1;         // 範囲外の場合、コンボボックス選択なし
                }
            }
            else
            {
                Index = -1;             // 数値以外の場合、コンボボックス選択なし
            }

            return Index;
        }

        public object ConvertBack(object value, Type targetType, object parameter, CultureInfo culture)
        {
            int num = 0;
            if(value is int)
            {
                num = (int)value+1;
            }
            else
            {
                num = 0;
            }
            return num.ToString();
        }
    }

    /// <summary>
    /// Service項目のEnable属性の値と、コンボボックスのSelectedIndexを相互変換する
    /// </summary>
    public class EnableIndexConverter : IValueConverter
    {
        public object Convert(object value, Type targetType, object parameter, CultureInfo culture)
        {
            //TODO:注意)属性にnullが設定されていた場合、コンボボックスの表示はTrueだが、保持値はnullのまま
            int index = (int)Define.PropertyWindow.Enable.True;
            string str = ((int)Define.PropertyWindow.Enable.True).ToString();

            if(value != null)
            {
                // TODO:文字列かチェック必要
                str = value.ToString();
            }

            
            if(Int32.TryParse(str, out index) == true)
            {
                if(index != (int)Define.PropertyWindow.Enable.False && index != (int)Define.PropertyWindow.Enable.True)
                {
                    // 初期値を設定
                    index = (int)Define.PropertyWindow.Enable.True;
                }
            }
            else
            {
                // 初期値を設定
                index = (int)Define.PropertyWindow.Enable.True;
            }

            return index;
        }

        public object ConvertBack(object value, Type targetType, object parameter, CultureInfo culture)
        {
            int index;
            string str;

            // TODO:数値かチェック
            index = (int)value;
            str = index.ToString();

            return str;
        }
    }
    /// <summary>
    /// Boolean属性の値と、コンボボックスのSelectedIndexを相互変換する
    /// </summary>
    public class BooleanIndexConverter : IValueConverter
    {
        public object Convert(object value, Type targetType, object parameter, CultureInfo culture)
        {
            //TODO:注意)属性にnullが設定されていた場合、コンボボックスの表示はFalseだが、保持値はnullのまま
            int index = (int)Define.PropertyWindow.Boolean.False;
            string str = ((int)Define.PropertyWindow.Boolean.False).ToString();

            if (value != null)
            {
                // TODO:文字列かチェック必要
                str = value.ToString();
            }


            if (Int32.TryParse(str, out index) == true)
            {
                if (index != (int)Define.PropertyWindow.Boolean.True && index != (int)Define.PropertyWindow.Boolean.False)
                {
                    // 初期値を設定
                    index = (int)Define.PropertyWindow.Boolean.False;
                }
            }
            else
            {
                // 初期値を設定
                index = (int)Define.PropertyWindow.Boolean.False;
            }

            return index;
        }

        public object ConvertBack(object value, Type targetType, object parameter, CultureInfo culture)
        {
            int index;
            string str;

            // TODO:数値かチェック
            index = (int)value;
            str = index.ToString();

            return str;
        }
    }

    /// <summary>
    /// bool型をFontWeight型に変換する
    /// </summary>
    public class FontWeightConverter : IValueConverter
    {
        public object Convert(object value, Type targetType, object parameter, CultureInfo culture)
        {
            FontWeight fontWeight = FontWeights.Normal;

            if (value is bool isChanged)
            {
                if(isChanged == true)
                {
                    fontWeight = FontWeights.Bold;      // bool型trueの時,太くする
                }
                else
                {
                    fontWeight = FontWeights.Normal;
                }
            }
            else
            {
                fontWeight = FontWeights.Normal;
            }

            return fontWeight;
        }

        public object ConvertBack(object value, Type targetType, object parameter, CultureInfo culture)
        {
            bool isChanged = false;

            if(value is FontWeight fontWeight)
            {
                if(fontWeight == FontWeights.Bold)
                {
                    isChanged = true;
                }
                else
                {
                    isChanged = false;
                }
            }
            else
            {
                isChanged = false;
            }

            return isChanged;
        }
    }
    #endregion コンバーター

}
