﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Xml;
using System.Xml.Serialization;
using System.ComponentModel;            // INotifyPropertyChanged
using System.Runtime.CompilerServices;  // CallerMemberName

namespace RTedgeXmlObj
{
    /// <summary>
    /// xml Tagの定義
    /// </summary>
    [Serializable]
    public class Tag:INotifyPropertyChanged
    {
        [XmlAttribute("Name")]
        public string Name;                         //Tag名
        [XmlAttribute("Comment")]
        public string Comment;                      //コメント
        [XmlAttribute("Address")]
        public string Address;                      //アドレス・ソース
        [XmlAttribute("Type")]
        public string Type;                         //データタイプ(数値)
        //[XmlAttribute("Size")]
        //public string Size;                         //データサイズ
        [XmlAttribute("Value")]
        public string Value;                        //データ初期値
        public Tag() { Name = ""; Comment = ""; Address = ""; Type = "0"; /*Size = "0";*/ Value = ""; }

        public event PropertyChangedEventHandler PropertyChanged;
        private void NotifyPropertyChanged([CallerMemberName] String propertyName = "")
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
            }
        }

        [System.Xml.Serialization.XmlIgnore]
        public string NAME
        {
            get { return Name; }
            set { if (value != Name) { Name = value; NotifyPropertyChanged("NAME"); } }
        }
        [System.Xml.Serialization.XmlIgnore]
        public string COMMENT
        {
            get { return Comment; }
            set { if (value != Comment) { Comment = value; NotifyPropertyChanged("COMMENT"); } }
        }
        [System.Xml.Serialization.XmlIgnore]
        public string ADDRESS
        {
            get { return Address; }
            set { if (value != Address) { Address = value; NotifyPropertyChanged("ADDRESS"); } }
        }
        [System.Xml.Serialization.XmlIgnore]
        public string TYPE
        {
            get { return Type; }
            set { if (Type != value) { Type = value; NotifyPropertyChanged("TYPE"); } }
        }
        //[System.Xml.Serialization.XmlIgnore]
        //public string SIZE
        //{
        //    get { return Size; }
        //    set { if (Size != value) { Size = value; NotifyPropertyChanged("SIZE"); } }
        //}
        [System.Xml.Serialization.XmlIgnore]
        public string VALUE
        {
            get { return Value; }
            set { if (Value != value) { Value = value; NotifyPropertyChanged("VALUE"); } }
        }

        
    }
    /// <summary>
    /// xml Tag参照用(TagRef)の定義
    /// </summary>
    [Serializable]
    public class TagRef : INotifyPropertyChanged
    {
        [XmlAttribute("Name")]
        public string Name;                         //Tag名
        public TagRef() { }
        public TagRef(string i1) { Name = i1; }

        // プロパティ変更通知
        public event PropertyChangedEventHandler PropertyChanged;
        private void NotifyPropertyChanged([CallerMemberName] String propertyName = "")
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
            }
        }

        [System.Xml.Serialization.XmlIgnore]
        public string NAME
        {
            get { return Name; }
            set
            {
                if (value != Name)
                {
                    Name = value;
                    NotifyPropertyChanged("NAME");
                }
            }
        }
    }
    /// <summary>
    /// xml CollectorRefの定義(サービスが参照するCollector)
    /// </summary>
    [Serializable]
    public class CollectorRef : INotifyPropertyChanged
    {
        [XmlAttribute("Name")]
        public string Name;                         //Collector名
        public CollectorRef() { }
        public CollectorRef(string i1) { Name = i1; }

        // プロパティ変更通知
        public event PropertyChangedEventHandler PropertyChanged;
        private void NotifyPropertyChanged([CallerMemberName] String propertyName = "")
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
            }
        }

        [System.Xml.Serialization.XmlIgnore]
        public string NAME
        {
            get { return Name; }
            set
            {
                if (value != Name)
                {
                    Name = value;
                    NotifyPropertyChanged("NAME");
                }
            }
        }
    }
    /// <summary>
    /// xml Mailboxの定義
    /// </summary>
    [Serializable]
    public class MailBox : INotifyPropertyChanged
    {
        [XmlAttribute("Name")]
        public string Name;                         //Mailbox名
        [XmlAttribute("ColSize")]
        public string ColSize;                      //Mailbox 1レコードのサイズ
        [XmlAttribute("RowSize")]
        public string RowSize;                      //Mailbox レコード数
    
        public MailBox() { Name = ""; ColSize = ""; RowSize = "";}

        public event PropertyChangedEventHandler PropertyChanged;
        private void NotifyPropertyChanged([CallerMemberName] String propertyName = "")
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
            }
        }

        [System.Xml.Serialization.XmlIgnore]
        public string NAME
        {
            get {   return Name; }
            set {   if (value != Name){ Name = value;   NotifyPropertyChanged("NAME");}}
        }
        [System.Xml.Serialization.XmlIgnore]
        public string COLSIZE
        {
            get {   return ColSize; }
            set {   if (value != ColSize){  ColSize = value;    NotifyPropertyChanged("COLSIZE");}}
        }
        [System.Xml.Serialization.XmlIgnore]
        public string ROWSIZE
        {
            get {   return RowSize; }
            set {   if (value != RowSize){  RowSize = value;    NotifyPropertyChanged("ROWSIZE");}}
        }
    }
    /// <summary>
    /// xml Datasetの定義
    /// </summary>
    [Serializable]
    public class Dataset:INotifyPropertyChanged
    {
        [XmlAttribute("Name")]
        public string Name;                         //Dataset名
        public List<TagRef> TagRefs;                //Dataset内TagRef
        public Dataset() { }
        public Dataset(string i1) { Name = i1;}

        // プロパティ変更通知
        public event PropertyChangedEventHandler PropertyChanged;
        private void NotifyPropertyChanged([CallerMemberName] String propertyName = "")
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
            }
        }

        [System.Xml.Serialization.XmlIgnore]
        public string NAME
        {
            get { return Name; }
            set { if (value != Name) { Name = value; NotifyPropertyChanged("NAME"); } }
        }
    }
    /// <summary>
    /// xml Collectorの定義
    /// </summary>
    [Serializable]
    public class Collector : INotifyPropertyChanged
    {
        [XmlAttribute("Name")]
        public string Name;                         //Collector名 = メールボックス名
        [XmlAttribute("Interval")]
        public string Interval;                     //ポーリング周期(ms)
        [XmlAttribute("Priority")]
        public string Priority;                     //Thread Priority
        [XmlAttribute("DatasetName")]
        public string DatasetName;                  //Dataset名
        [XmlAttribute("ColSize")]
        public string ColSize;                      //1テーブル長
        [XmlAttribute("RowSize")]
        public string RowSize;                      //メールボックステーブル数
        [XmlAttribute("Comment")]
        public string Comment;                      //コメント
        public Collector() { }
        public Collector(string i1, string i2 = "", string i3 = "", string i4 = "", string i5 = "", string i6 = "", string i7 = "")
                        { Name = i1; Interval = i2; Priority = i3;  DatasetName = i4; ColSize = i5; RowSize = i6; Comment = i7; }

        // プロパティ変更通知
        public event PropertyChangedEventHandler PropertyChanged;
        private void NotifyPropertyChanged([CallerMemberName] String propertyName = "")
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
            }
        }

        [System.Xml.Serialization.XmlIgnore]
        public string NAME
        {
            get { return Name; }
            set { if (value != Name) { Name = value; NotifyPropertyChanged("NAME"); } }
        }
        [System.Xml.Serialization.XmlIgnore]
        public string INTERVAL
        {
            get {   return Interval; }
            set {   if (value != Interval){ Interval = value;   NotifyPropertyChanged("INTERVAL");}}
        }
        [System.Xml.Serialization.XmlIgnore]
        public string PRIORITY
        {
            get {   return Priority; }
            set {   if (value != Priority){ Priority = value;   NotifyPropertyChanged("PRIORITY");}}
        }
        [System.Xml.Serialization.XmlIgnore]
        public string DATASETNAME
        {
            get {   return DatasetName; }
            set {   if (value != DatasetName){  DatasetName = value;    NotifyPropertyChanged("DATASETNAME");}}
        }
        [System.Xml.Serialization.XmlIgnore]
        public string COLSIZE
        {
            get { return ColSize; }
            set { if (value != ColSize) { ColSize = value; NotifyPropertyChanged("COLSIZE"); } }
        }
        [System.Xml.Serialization.XmlIgnore]
        public string ROWSIZE
        {
            get { return RowSize; }
            set { if (value != RowSize) { RowSize = value; NotifyPropertyChanged("ROWSIZE"); } }
        }
        [System.Xml.Serialization.XmlIgnore]
        public string COMMENT
        {
            get { return Comment; }
            set { if (value != Comment) { Comment = value; NotifyPropertyChanged("COMMENT"); } }
        }
    }
    /// <summary>
    /// xml Serviceの定義
    /// </summary>
    [Serializable]
    public class Service : INotifyPropertyChanged
    {
        [XmlAttribute("Name")]
        public string Name;                         //サービス名
        [XmlAttribute("Path")]
        public string Path;                         //サービス(プログラム)のパス
        [XmlAttribute("Priority")]
        public string Priority;                     //サービス間通信用EgMailboxのPriority
        [XmlAttribute("Argument")]
        public string Argument;                     //サービス(プログラム)の起動引数
        [XmlAttribute("Comment")]
        public string Comment;                      //サービスのコメント
        [XmlAttribute("Timeout")]
        public string Timeout;                      //サービス起動時にサービス側から立ち上げ完了メッセージを受け取るのに待つ時間(過ぎたら作れないと判断)
        [XmlAttribute("Node")]
        public string Node;                         //起動ノード
        [XmlAttribute("Dependent")]
        public string Dependent;                    //依存コンテナ名
        [XmlAttribute("Enabled")]
        public string Enabled;                      //起動有効・無効(0:起動しない、1:起動する)
        [XmlAttribute("ColSize")]
        public string ColSize;                      //1テーブル長
        [XmlAttribute("RowSize")]
        public string RowSize;                      //メールボックステーブル数
        [XmlAttribute("Class")]
        public string Class;                         //サービスクラス
        [XmlAttribute("OnlyOwner")]
        public string OnlyOwner;                    // 呼び出し元のみ読み込むフラグ(TRUE以外常に読み込む Default)
        [XmlAttribute("ECI")]
        public string ECIPath;                      // ECI Path指定

        public List<Tag> Tags;                      //Tagの定義(グローバルと同じ扱い ただしリンクタグの優先度高とする)
        public List<Dataset> Datasets;              //EgDataset(グローバルと同じ扱い)
        public List<TagRef> TagRefs;                //サービス内参照用Tag(IN/OUT共通)
        public List<TagRef> TagRefs_OUT;            //サービス内参照用Tag(OUT用)
        public List<TagRef> TagRefs_IN;             //サービス内参照用Tag(IN用)
        public List<Collector> Collectors;          //サービス内実行(メールボックスにwrite)用Collector
        public List<CollectorRef> CollectorRefs;    //サービス内参照用(メールボックスからRead)用Collector
        public List<MailBox> MailBoxes;             //サービス内参照用メールボックス
        public Service() { Name = ""; Path = ""; Priority = ""; Argument = ""; Comment = ""; Timeout = ""; Node = ""; Dependent = ""; ColSize = ""; RowSize = ""; Class = ""; OnlyOwner = ""; ECIPath = ""; }     //コンストラクタ
        public Service(string i1, string i2 = "", string i3 = "", string i4 = "", string i5 = "", string i6 = "", string i7 = "", string i8 = "", string i9 = "", string i10 = "", string i11 = "", string i12 = "", string i13 = "")
                        { Name = i1; Path = i2; Priority = i3; Comment = i4; Argument = i5; Timeout = i6; Node = i7; Dependent = i8; ColSize = i9; RowSize = i10; Class = i11; OnlyOwner = i12; ECIPath = i13; }

        // プロパティ変更通知
        public event PropertyChangedEventHandler PropertyChanged;
        private void NotifyPropertyChanged([CallerMemberName] String propertyName = "")
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
            }
        }

        [System.Xml.Serialization.XmlIgnore]
        public string NAME
        {
            get { return Name; }
            set { if (value != Name) { Name = value; NotifyPropertyChanged("NAME"); } }
        }
        [System.Xml.Serialization.XmlIgnore]
        public string PATH
        {
            get {   return Path; }
            set {   if (value != Path){ Path = value;   NotifyPropertyChanged("PATH");}}
        }
        [System.Xml.Serialization.XmlIgnore]
        public string PRIORITY
        {
            get {   return Priority; }
            set {   if (value != Priority){ Priority = value;   NotifyPropertyChanged("PRIORITY");}}
        }
        [System.Xml.Serialization.XmlIgnore]
        public string ARGUMENT
        {
            get {   return Argument; }
            set {   if (value != Argument){ Argument = value;   NotifyPropertyChanged("ARGUMENT");}}
        }
        [System.Xml.Serialization.XmlIgnore]
        public string COMMENT
        {
            get {   return Comment; }
            set {   if (value != Comment){  Comment = value;    NotifyPropertyChanged("COMMENT");}}
        }
        [System.Xml.Serialization.XmlIgnore]
        public string TIMEOUT
        {
            get {   return Timeout; }
            set {   if (value != Timeout){  Timeout = value;    NotifyPropertyChanged("TIMEOUT");}}
        }
        [System.Xml.Serialization.XmlIgnore]
        public string NODE
        {
            get {   return Node; }
            set {   if (value != Node){ Node = value;   NotifyPropertyChanged("NODE");}}
        }
        [System.Xml.Serialization.XmlIgnore]
        public string DEPENDENT
        {
            get {   return Dependent; }
            set {   if (value != Dependent){    Dependent = value;  NotifyPropertyChanged("DEPENDENT");}}
        }
        [System.Xml.Serialization.XmlIgnore]

        public string COLSIZE
        {
            get { return ColSize; }
            set { if (value != ColSize) { ColSize = value; NotifyPropertyChanged("COLSIZE"); } }
        }
        [System.Xml.Serialization.XmlIgnore]
        public string ROWSIZE
        {
            get { return RowSize; }
            set { if (value != RowSize) { RowSize = value; NotifyPropertyChanged("ROWSIZE"); } }
        }

        [System.Xml.Serialization.XmlIgnore]
        public string ENABLED
        {
            get {   return Enabled; }
            set {   if (value != Enabled){  Enabled = value;    NotifyPropertyChanged("ENABLED");}}
        }

        [System.Xml.Serialization.XmlIgnore]
        public string CLASS
        {
            get { return Class; }
            set { if (value != Class) { Class = value; NotifyPropertyChanged("CLASS"); } }
        }

        [System.Xml.Serialization.XmlIgnore]
        public string ONLYOWNER
        {
            get { return OnlyOwner; }
            set { if (value != OnlyOwner) { OnlyOwner = value; NotifyPropertyChanged("ONLYOWNER"); } }
        }

        [System.Xml.Serialization.XmlIgnore]
        public string ECI
        {
            get { return ECIPath; }
            set { if (value != ECIPath) { ECIPath = value; NotifyPropertyChanged("ECI"); } }
        }
    }
    /// <summary>
    /// xmlの全体
    /// </summary>
    public class RTedge
    {
        public List<Tag> Tags;          //EgTagデータ
        public List<Dataset> Datasets;  //EgDatasetデータ
        public List<Service> Services;  //EgServiceデータ
    }
}
