﻿////////////////////////////////////////////////////////////////////////////////////////
/// 機能       :   RT-edge XML編集ツール　編集機能 「削除」画面コードビハインド
/// 作成日     :   2022/02/21
/// 作成者     :   mizuno
/// 
/// RT-edge XML編集ツール 機能仕様書
/// file://192.168.1.240/Jobs/KA1601(RT-edge開発）/trunk/0051_Documnet(社内)/0200_egCoreService/20211021_RT-edge_XML編集ツール_機能仕様書.pptx
/// P15. 機能詳細 編集機能 「削除」
///
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;

namespace EgConfig
{
    /// <summary>
    /// RemoveElement.xaml の相互作用ロジック
    /// </summary>
    public partial class RemoveElement : Window
    {
        private EgConfig.ItemType type;         // 項目タイプ
        private MainWindow mainWindow;          // メインウインドウの参照

        private string _elemType;               // 項目タイプ名
        private string _explanation;            // 説明文

        /// <summary>
        /// プロパティ
        /// </summary>
        public string ElemType
        {
            get { return _elemType; }
            set
            {
                _elemType = value;
            }
        }
        public string Explanation
        {
            get { return _explanation; }
            set
            {
                _explanation = value;
            }
        }

        /// <summary>
        /// コンストラクタ
        /// </summary>
        /// <param name="type">削除する項目タイプ</param>
        /// <param name="mainWindow">メインウインドウ</param>
        public RemoveElement(EgConfig.ItemType type, MainWindow mainWindow)
        {
            string itemName;

            InitializeComponent();

            this.type = type;
            this.mainWindow = mainWindow;
            DataContext = this;                 // DataContextを登録

            //////////////////////////////////////////////////////
            // 項目タイプによって表示するメッセージを設定
            //////////////////////////////////////////////////////
            if(this.mainWindow.XmlTreeView.SelectedItem is TreeItem selectedItem)
            {
                ElemType = this.type.ToString();                        // 削除する項目タイプ名を設定する
                itemName = selectedItem.Name;                           // TreeItemで選択しているアイテム名を取得
                switch (this.type)
                {
                    // ファイルの削除
                    case EgConfig.ItemType.ECI:
                        Explanation = "ツリーから" + itemName + "を除外します。よろしいですか?\nファイルを削除する場合は、エクスプローラから直接削除してください。";
                        break;
                    // 子要素の削除
                    case EgConfig.ItemType.Tag:
                    case EgConfig.ItemType.DatasetTagRef:
                    case EgConfig.ItemType.TagRef:
                    case EgConfig.ItemType.TagRefIN:
                    case EgConfig.ItemType.TagRefOUT:
                    case EgConfig.ItemType.Collector:
                    case EgConfig.ItemType.CollectorRef:
                    case EgConfig.ItemType.MailBox:
                        Explanation = itemName + "を削除します。よろしいですか?";
                        break;
                    // 親要素の削除
                    case EgConfig.ItemType.Tags:
                    case EgConfig.ItemType.Datasets:
                    case EgConfig.ItemType.DatasetTagRefs:
                    case EgConfig.ItemType.TagRefs:
                    case EgConfig.ItemType.TagRefsIN:
                    case EgConfig.ItemType.TagRefsOUT:
                    case EgConfig.ItemType.Collectors:
                    case EgConfig.ItemType.CollectorRefs:
                    case EgConfig.ItemType.MailBoxes:
                    case EgConfig.ItemType.Services:
                        Explanation = ElemType + "に定義されているすべての項目を削除します。\nよろしいですか?";
                        break;
                    case EgConfig.ItemType.Dataset:
                    case EgConfig.ItemType.Service:
                        Explanation = itemName + "に定義されているすべての項目を削除します。\nよろしいですか?";
                        break;
                    default:
                        break;
                }
            }
            
        }

        /// <summary>
        /// OKボタンクリック
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void OkBtn_Click(object sender, RoutedEventArgs e)
        {
            ////////////////////////////////////////
            // 項目別に削除処理を行う
            ////////////////////////////////////////
            switch (this.type)
            {
                case ItemType.ECI:
                    RemoveECI();
                    break;
                case ItemType.Tags:
                    RemoveTags();
                    break;
                case ItemType.Tag:
                    RemoveTag();
                    break;
                case ItemType.Datasets:
                    RemoveDatasets();
                    break;
                case ItemType.Dataset:
                    RemoveDataset();
                    break;
                case ItemType.DatasetTagRefs:
                    RemoveDatasetTagRefs();
                    break;
                case ItemType.DatasetTagRef:
                    RemoveDatasetTagRef();
                    break;
                case ItemType.TagRefs:
                    RemoveTagRefs();
                    break;
                case ItemType.TagRef:
                    RemoveTagRef();
                    break;
                case ItemType.TagRefsIN:
                    RemoveTagRefsIN();
                    break;
                case ItemType.TagRefIN:
                    RemoveTagRefIN();
                    break;
                case ItemType.TagRefsOUT:
                    RemoveTagRefsOUT();
                    break;
                case ItemType.TagRefOUT:
                    RemoveTagRefOUT();
                    break;
                case ItemType.Collectors:
                    RemoveCollectors();
                    break;
                case ItemType.Collector:
                    RemoveCollector();
                    break;
                case ItemType.CollectorRefs:
                    RemoveCollectorRefs();
                    break;
                case ItemType.CollectorRef:
                    RemoveCollectorRef();
                    break;
                case ItemType.MailBoxes:
                    RemoveMailboxes();
                    break;
                case ItemType.MailBox:
                    RemoveMailbox();
                    break;
                case ItemType.Services:
                    RemoveServices();
                    break;
                case ItemType.Service:
                    RemoveService();
                    break;
                default:
                    break;
            }

            this.Close();
        }

        /// <summary>
        /// キャンセルボタンクリック
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void CancelBtn_Click(object sender, RoutedEventArgs e)
        {
            this.Close();   // 画面を閉じる
        }

        /// <summary>
        /// ECIファイルを除外
        /// </summary>
        private void RemoveECI()
        {
            int index = 0, i = 0;

            // TreeItemで選択している項目を取得
            if (mainWindow.XmlTreeView.SelectedItem is TreeItem selectedItem)
            {
                index = MainWindow.projMgmt.RemoveEciFile(selectedItem.Filename);   // データ削除

                MainWindow.treeItems.RemoveAt(index);                               // TreeViewに反映
                foreach (TreeItem treeItem in MainWindow.treeItems)                  // インデックスを更新
                {
                    treeItem.Index = i++;
                }

                MainWindow.isFileChanged.RemoveAt(index);                           // ファイル変更状態の管理削除

                this.DialogResult = true;                   // 削除完了
            }

        }

        /// <summary>
        /// Tagsを削除
        /// </summary>
        private void RemoveTags()
        {
            // TreeItemで選択している項目を取得
            if (mainWindow.XmlTreeView.SelectedItem is TreeItem selectedItem)
            {
                MainWindow.projMgmt.RemoveTags(selectedItem);   // データ削除
                selectedItem.Remove();                          // TreeViewに反映

                this.DialogResult = true;   // 削除完了
            }

        }

        /// <summary>
        /// Tagを削除
        /// </summary>
        private void RemoveTag()
        {
            // TreeItemで選択している項目を取得
            if (mainWindow.XmlTreeView.SelectedItem is TreeItem selectedItem)
            {
                MainWindow.projMgmt.RemoveTag(selectedItem);       // データ削除
                selectedItem.Remove();                          // TreeViewに反映

                this.DialogResult = true;   // 削除完了
            }
        }

        /// <summary>
        /// Datasetsを削除
        /// </summary>
        private void RemoveDatasets()
        {
            // TreeItemで選択している項目を取得
            if (mainWindow.XmlTreeView.SelectedItem is TreeItem selectedItem)
            {
                MainWindow.projMgmt.RemoveDatasets(selectedItem);               // データ削除
                selectedItem.Remove();                                          // TreeViewに反映

                this.DialogResult = true;   // 削除完了
            }
        }

        /// <summary>
        /// Datasetを削除
        /// </summary>
        private void RemoveDataset()
        {
            // TreeItemで選択している項目を取得
            if (mainWindow.XmlTreeView.SelectedItem is TreeItem selectedItem)
            {
                MainWindow.projMgmt.RemoveDataset(selectedItem);               // データ削除
                selectedItem.Remove();                                          // TreeViewに反映

                this.DialogResult = true;   // 削除完了
            }
        }

        /// <summary>
        /// DatasetTagRefsを削除
        /// </summary>
        private void RemoveDatasetTagRefs()
        {
            // TreeItemで選択している項目を取得
            if (mainWindow.XmlTreeView.SelectedItem is TreeItem selectedItem)
            {
                MainWindow.projMgmt.RemoveDatasetTagRefs(selectedItem);         // データ削除
                selectedItem.Remove();                                          // TreeViewに反映

                this.DialogResult = true;   // 削除完了
            }

        }

        /// <summary>
        /// DatasetTagRefを削除
        /// </summary>
        private void RemoveDatasetTagRef()
        {
            // TreeItemで選択している項目を取得
            if (mainWindow.XmlTreeView.SelectedItem is TreeItem selectedItem)
            {
                MainWindow.projMgmt.RemoveDatasetTagRef(selectedItem);          // データ削除
                selectedItem.Remove();                                          // TreeViewに反映

                this.DialogResult = true;   // 削除完了
            }

        }

        /// <summary>
        /// TagRefsを削除
        /// </summary>
        private void RemoveTagRefs()
        {
            // TreeItemで選択している項目を取得
            if (mainWindow.XmlTreeView.SelectedItem is TreeItem selectedItem)
            {
                MainWindow.projMgmt.RemoveTagRefs(selectedItem);        // データ削除
                selectedItem.Remove();                                  // TreeViewに反映

                this.DialogResult = true;   // 削除完了
            }

        }

        /// <summary>
        /// TagRefを削除
        /// </summary>
        private void RemoveTagRef()
        {
            // TreeItemで選択している項目を取得
            if (mainWindow.XmlTreeView.SelectedItem is TreeItem selectedItem)
            {
                MainWindow.projMgmt.RemoveTagRef(selectedItem);         // データ削除
                selectedItem.Remove();                                  // TreeViewに反映

                this.DialogResult = true;   // 削除完了
            }

        }

        /// <summary>
        /// TagRefsINを削除
        /// </summary>
        private void RemoveTagRefsIN()
        {
            // TreeItemで選択している項目を取得
            if (mainWindow.XmlTreeView.SelectedItem is TreeItem selectedItem)
            {
                MainWindow.projMgmt.RemoveTagRefsIN(selectedItem);        // データ削除
                selectedItem.Remove();                                  // TreeViewに反映

                this.DialogResult = true;   // 削除完了
            }

        }

        /// <summary>
        /// TagRefINを削除
        /// </summary>
        private void RemoveTagRefIN()
        {
            // TreeItemで選択している項目を取得
            if (mainWindow.XmlTreeView.SelectedItem is TreeItem selectedItem)
            {
                MainWindow.projMgmt.RemoveTagRefIN(selectedItem);   // データ削除
                selectedItem.Remove();                              // TreeViewに反映

                this.DialogResult = true;   // 削除完了
            }
        }

        /// <summary>
        /// TagRefsOUTを削除
        /// </summary>
        private void RemoveTagRefsOUT()
        {
            // TreeItemで選択している項目を取得
            if (mainWindow.XmlTreeView.SelectedItem is TreeItem selectedItem)
            {
                MainWindow.projMgmt.RemoveTagRefsOUT(selectedItem);        // データ削除
                selectedItem.Remove();                                  // TreeViewに反映

                this.DialogResult = true;   // 削除完了
            }
        }

        /// <summary>
        /// TagRefOUTを削除
        /// </summary>
        private void RemoveTagRefOUT()
        {
            // TreeItemで選択している項目を取得
            if (mainWindow.XmlTreeView.SelectedItem is TreeItem selectedItem)
            {
                MainWindow.projMgmt.RemoveTagRefOUT(selectedItem);  // データ削除
                selectedItem.Remove();                              // TreeViewに反映

                this.DialogResult = true;   // 削除完了
            }

        }

        /// <summary>
        /// Collectorsを削除
        /// </summary>
        private void RemoveCollectors()
        {
            // TreeItemで選択している項目を取得
            if (mainWindow.XmlTreeView.SelectedItem is TreeItem selectedItem)
            {
                MainWindow.projMgmt.RemoveCollectors(selectedItem);  // データ削除
                selectedItem.Remove();                              // TreeViewに反映

                this.DialogResult = true;   // 削除完了
            }

        }

        /// <summary>
        /// Collectorを削除
        /// </summary>
        private void RemoveCollector()
        {
            // TreeItemで選択している項目を取得
            if (mainWindow.XmlTreeView.SelectedItem is TreeItem selectedItem)
            {
                MainWindow.projMgmt.RemoveCollector(selectedItem);  // データ削除
                selectedItem.Remove();                              // TreeViewに反映

                this.DialogResult = true;   // 削除完了
            }

        }

        /// <summary>
        /// CollectorRefsを削除
        /// </summary>
        private void RemoveCollectorRefs()
        {
            // TreeItemで選択している項目を取得
            if (mainWindow.XmlTreeView.SelectedItem is TreeItem selectedItem)
            {
                MainWindow.projMgmt.RemoveCollectorRefs(selectedItem);  // データ削除
                selectedItem.Remove();                                  // TreeViewに反映

                this.DialogResult = true;   // 削除完了
            }

        }

        /// <summary>
        /// CollectorRefを削除
        /// </summary>
        private void RemoveCollectorRef()
        {
            // TreeItemで選択している項目を取得
            if (mainWindow.XmlTreeView.SelectedItem is TreeItem selectedItem)
            {
                MainWindow.projMgmt.RemoveCollectorRef(selectedItem);   // データ削除
                selectedItem.Remove();                                  // TreeViewに反映

                this.DialogResult = true;   // 削除完了
            }
        }

        /// <summary>
        /// Mailboxesを削除
        /// </summary>
        private void RemoveMailboxes()
        {
            // TreeItemで選択している項目を取得
            if (mainWindow.XmlTreeView.SelectedItem is TreeItem selectedItem)
            {
                MainWindow.projMgmt.RemoveMailboxes(selectedItem);        // データ削除
                selectedItem.Remove();                                  // TreeViewに反映

                this.DialogResult = true;   // 削除完了
            }

        }

        /// <summary>
        /// Mailboxを削除
        /// </summary>
        private void RemoveMailbox()
        {
            // TreeItemで選択している項目を取得
            if (mainWindow.XmlTreeView.SelectedItem is TreeItem selectedItem)
            {
                MainWindow.projMgmt.RemoveMailbox(selectedItem);        // データ削除
                selectedItem.Remove();                                  // TreeViewに反映

                this.DialogResult = true;   // 削除完了
            }

        }

        /// <summary>
        /// Servicesを削除
        /// </summary>
        private void RemoveServices()
        {
            // TreeItemで選択している項目を取得
            if (mainWindow.XmlTreeView.SelectedItem is TreeItem selectedItem)
            {
                MainWindow.projMgmt.RemoveServices(selectedItem);       // データ削除
                selectedItem.Remove();                                  // TreeViewに反映

                this.DialogResult = true;   // 削除完了
            }
        }

        /// <summary>
        /// Serviceを削除
        /// </summary>
        private void RemoveService()
        {
            // TreeItemで選択している項目を取得
            if (mainWindow.XmlTreeView.SelectedItem is TreeItem selectedItem)
            {
                MainWindow.projMgmt.RemoveService(selectedItem);        // データ削除
                selectedItem.Remove();                                  // TreeViewに反映

                this.DialogResult = true;   // 削除完了
            }

        }
    }
}
