﻿////////////////////////////////////////////////////////////////////////////////////////
/// 機能       :   RT-edge XML編集ツール　編集機能 「名前の変更」画面コードビハインド
/// 作成日     :   2022/02/21
/// 作成者     :   mizuno
/// 
/// RT-edge XML編集ツール 機能仕様書
/// file://192.168.1.240/Jobs/KA1601(RT-edge開発）/trunk/0051_Documnet(社内)/0200_egCoreService/20211021_RT-edge_XML編集ツール_機能仕様書.pptx
/// P16. 機能詳細 編集機能 「名前の変更」
///
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;
using RTedgeXmlObj;
using System.IO;
using System.Windows.Forms;

namespace EgConfig
{
    /// <summary>
    /// RenameDialog.xaml の相互作用ロジック
    /// </summary>
    public partial class RenameDialog : Window
    {
        public MainWindow mainWindow;   // メインウインドウの参照
        //public TreeItem treeItem;

        private string _currentName;
        public string CurrentName
        {
            get { return _currentName; }
            set
            {
                _currentName = CurrentName;
            }
        }

        public RenameDialog(MainWindow mainWindow)
        {
            InitializeComponent();

            this.mainWindow = mainWindow;   // メインウインドウの参照を設定

            ///////////////////////////////////////////////////
            // 変更前の名前をテキストボックスに設定
            ///////////////////////////////////////////////////
            if (mainWindow.XmlTreeView.SelectedItem is TreeItem selectedItem)
            {
                if (selectedItem.ItemType == (int)ItemType.ECI)  // ECIファイル名が選択されていた場合、拡張子は除いて出力
                {
                    CurrentNameTBX.Text = selectedItem.Name.Replace(".xml", "");
                }
                else
                {
                    CurrentNameTBX.Text = selectedItem.Name;
                }
            }
              
            //this.Title 
        }

        /// <summary>
        /// OKボタンクリック
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void OkBtn_Click(object sender, RoutedEventArgs e)
        {
            int errorCode;
            TreeItem selectedItem;

            // TreeItemで選択している項目を取得
            selectedItem = mainWindow.XmlTreeView.SelectedItem as TreeItem;
            if (selectedItem is null)
            {
                return;
            }

            // XMLデータ取得
            object obj = MainWindow.projMgmt.GetItemData(selectedItem);

            /////////////////////////////////////////////////
            // 名前変更を行う
            // TODO:プロジェクト管理クラスの中に移行
            /////////////////////////////////////////////////
            if(obj is RTedgeECI)
            {
                RTedgeECI rtedgeECI = (RTedgeECI)obj;
                string directoryName;
                string newFileName;

                /////////////////////////////
                // ECIファイル名変更
                /////////////////////////////
                directoryName = System.IO.Path.GetDirectoryName(MainWindow.projMgmt.eciList[rtedgeECI.index].eciFileName);
                newFileName = System.IO.Path.Combine(directoryName, RenameTBX.Text + ".xml");
                File.Move(MainWindow.projMgmt.eciList[rtedgeECI.index].eciFileName, newFileName);                   // 管理先の名前変更

                MainWindow.projMgmt.eciList[rtedgeECI.index].eciFileName = newFileName;                             // 管理データのECIファイル名変更
                selectedItem.Name = System.IO.Path.GetFileName(newFileName);                                        // TreeViewに表示するデータ変更

                
                UpdateFilename(newFileName, selectedItem);                                                          // TreeViewのリスト更新(File名を変更した名前にする)
            }
            if (obj is Tag tag)
            {
                // Tag要素Name属性の入力チェック
                errorCode = CheckConsistencyOfElem.TagName(RenameTBX.Text);
                if(errorCode == Define.ErrorCode.NULL)
                {
                    System.Windows.Forms.MessageBox.Show(EgConfig.Properties.Resources.MSGBOX_ERROR_000_MSGTEXT, EgConfig.Properties.Resources.MSGBOX_ERROR_000_CAPTION, MessageBoxButtons.OK, MessageBoxIcon.Hand);
                    return;
                }
                else if(errorCode == Define.ErrorCode.EXCEPT_HWAC)
                {
                    System.Windows.Forms.MessageBox.Show(EgConfig.Properties.Resources.MSGBOX_ERROR_001_MSGTEXT, EgConfig.Properties.Resources.MSGBOX_ERROR_001_CAPTION, MessageBoxButtons.OK, MessageBoxIcon.Hand);
                    return;
                }
                else if(errorCode == Define.ErrorCode.OVER_BYTE)
                {
                    System.Windows.Forms.MessageBox.Show(String.Format(EgConfig.Properties.Resources.MSGBOX_ERROR_002_MSGTEXT, "48"), EgConfig.Properties.Resources.MSGBOX_ERROR_002_CAPTION, MessageBoxButtons.OK, MessageBoxIcon.Hand);
                    return;
                }

                tag.NAME = RenameTBX.Text;              // 管理データ変更
                selectedItem.Name = RenameTBX.Text;     // TreeViewに表示するデータ変更
            }
            else if(obj is Dataset dataset)
            {
                // Dataset要素Name属性の入力チェック
                errorCode = CheckConsistencyOfElem.DatasetName(RenameTBX.Text);
                if (errorCode == Define.ErrorCode.NULL)
                {
                    System.Windows.Forms.MessageBox.Show(EgConfig.Properties.Resources.MSGBOX_ERROR_000_MSGTEXT, EgConfig.Properties.Resources.MSGBOX_ERROR_000_CAPTION, MessageBoxButtons.OK, MessageBoxIcon.Hand);
                    return;
                }
                else if (errorCode == Define.ErrorCode.EXCEPT_HWAC)
                {
                    System.Windows.Forms.MessageBox.Show(EgConfig.Properties.Resources.MSGBOX_ERROR_001_MSGTEXT, EgConfig.Properties.Resources.MSGBOX_ERROR_001_CAPTION, MessageBoxButtons.OK, MessageBoxIcon.Hand);
                    return;
                }
                else if (errorCode == Define.ErrorCode.OVER_BYTE)
                {
                    System.Windows.Forms.MessageBox.Show(String.Format(EgConfig.Properties.Resources.MSGBOX_ERROR_002_MSGTEXT, "32"), EgConfig.Properties.Resources.MSGBOX_ERROR_002_CAPTION, MessageBoxButtons.OK, MessageBoxIcon.Hand);
                    return;
                }

                dataset.NAME = RenameTBX.Text;              // 管理データ変更
                selectedItem.Name = RenameTBX.Text;         // TreeViewに表示するデータ変更
            }
            else if (obj is TagRef tagref)
            {
                int index = 0;
                System.Windows.Controls.ComboBox TagRefComboBox = mainWindow.TagRefComboBox;

                tagref.NAME = RenameTBX.Text;           // 管理データ変更
                selectedItem.Name = RenameTBX.Text;     // TreeViewに表示するデータ変更
                for (index = 0; index < MainWindow.tagRefNames.Count; index++)
                {
                    if (MainWindow.tagRefNames[index].Name == tagref.Name)
                    {
                        break;
                    }
                }

                if (MainWindow.tagRefNames.Count <= index) { index = -1; }
                // テキストボックスのインデックス番号変更
                TagRefComboBox.SelectedIndex = index;
            }
            else if (obj is MailBox mailBox)
            {
                // MailBox要素Name属性の入力チェック
                errorCode = CheckConsistencyOfElem.MailboxName(RenameTBX.Text);
                if (errorCode == Define.ErrorCode.NULL)
                {
                    System.Windows.Forms.MessageBox.Show(EgConfig.Properties.Resources.MSGBOX_ERROR_000_MSGTEXT, EgConfig.Properties.Resources.MSGBOX_ERROR_000_CAPTION, MessageBoxButtons.OK, MessageBoxIcon.Hand);
                    return;
                }
                else if (errorCode == Define.ErrorCode.EXCEPT_HWAC)
                {
                    System.Windows.Forms.MessageBox.Show(EgConfig.Properties.Resources.MSGBOX_ERROR_001_MSGTEXT, EgConfig.Properties.Resources.MSGBOX_ERROR_001_CAPTION, MessageBoxButtons.OK, MessageBoxIcon.Hand);
                    return;
                }
                else if (errorCode == Define.ErrorCode.OVER_BYTE)
                {
                    System.Windows.Forms.MessageBox.Show(String.Format(EgConfig.Properties.Resources.MSGBOX_ERROR_002_MSGTEXT, "8"), EgConfig.Properties.Resources.MSGBOX_ERROR_002_CAPTION, MessageBoxButtons.OK, MessageBoxIcon.Hand);
                    return;
                }

                mailBox.NAME = RenameTBX.Text;          // 管理データ変更
                selectedItem.Name = RenameTBX.Text;     // TreeViewに表示するデータ変更
            }
            else if (obj is Service service)
            {
                // Service要素Name属性の入力チェック
                errorCode = CheckConsistencyOfElem.ServiceName(RenameTBX.Text);
                if (errorCode == Define.ErrorCode.NULL)
                {
                    System.Windows.Forms.MessageBox.Show(EgConfig.Properties.Resources.MSGBOX_ERROR_000_MSGTEXT, EgConfig.Properties.Resources.MSGBOX_ERROR_000_CAPTION, MessageBoxButtons.OK, MessageBoxIcon.Hand);
                    return;
                }
                else if (errorCode == Define.ErrorCode.EXCEPT_HWAC)
                {
                    System.Windows.Forms.MessageBox.Show(EgConfig.Properties.Resources.MSGBOX_ERROR_001_MSGTEXT, EgConfig.Properties.Resources.MSGBOX_ERROR_001_CAPTION, MessageBoxButtons.OK, MessageBoxIcon.Hand);
                    return;
                }
                else if (errorCode == Define.ErrorCode.OVER_BYTE)
                {
                    System.Windows.Forms.MessageBox.Show(String.Format(EgConfig.Properties.Resources.MSGBOX_ERROR_002_MSGTEXT, "8"), EgConfig.Properties.Resources.MSGBOX_ERROR_002_CAPTION, MessageBoxButtons.OK, MessageBoxIcon.Hand);
                    return;
                }

                service.NAME = RenameTBX.Text;          // 管理データ変更
                selectedItem.Name = RenameTBX.Text;     // TreeViewに表示するデータ変更
            }
            else if (obj is Collector collector)
            {
                // Collector要素Name属性の入力チェック
                errorCode = CheckConsistencyOfElem.CollectorName(RenameTBX.Text);
                if (errorCode == Define.ErrorCode.NULL)
                {
                    System.Windows.Forms.MessageBox.Show(EgConfig.Properties.Resources.MSGBOX_ERROR_000_MSGTEXT, EgConfig.Properties.Resources.MSGBOX_ERROR_000_CAPTION, MessageBoxButtons.OK, MessageBoxIcon.Hand);
                    return;
                }
                else if (errorCode == Define.ErrorCode.EXCEPT_HWAC)
                {
                    System.Windows.Forms.MessageBox.Show(EgConfig.Properties.Resources.MSGBOX_ERROR_001_MSGTEXT, EgConfig.Properties.Resources.MSGBOX_ERROR_001_CAPTION, MessageBoxButtons.OK, MessageBoxIcon.Hand);
                    return;
                }
                else if (errorCode == Define.ErrorCode.OVER_BYTE)
                {
                    System.Windows.Forms.MessageBox.Show(String.Format(EgConfig.Properties.Resources.MSGBOX_ERROR_002_MSGTEXT, "8"), EgConfig.Properties.Resources.MSGBOX_ERROR_002_CAPTION, MessageBoxButtons.OK, MessageBoxIcon.Hand);
                    return;
                }

                collector.NAME = RenameTBX.Text;        // 管理データ変更
                selectedItem.Name = RenameTBX.Text;     // TreeViewに表示するデータ変更
            }
            else if (obj is CollectorRef collectorref)
            {
                System.Windows.Controls.ComboBox CollectorComboBox = mainWindow.CollectorComboBox;
                int index = 0;

                collectorref.NAME = RenameTBX.Text;         // 管理データ変更
                selectedItem.Name = RenameTBX.Text;         // TreeViewに表示するデータ変更

                ///////////////////////////////////////////
                // Collectorの選択を行う
                ///////////////////////////////////////////
                for (index = 0; index < MainWindow.collectorNames.Count; index++)
                {
                    if (MainWindow.collectorNames[index].Name == collectorref.Name)
                    {
                        break;
                    }
                }
                if (MainWindow.collectorNames.Count <= index) { index = -1; }
                CollectorComboBox.SelectedIndex = index;
            }

            this.DialogResult = true;   // 名前変更完了
            this.Close();
        }

        /// <summary>
        /// TreeViewのファイル名を変更した名前にする
        /// -----------------------------------------
        /// リストの根まで子要素を検索し、名前を変更する
        /// </summary>
        /// <param name="newFileName">新しいファイル名</param>
        /// <param name="treeItem">ECIファイル選択</param>
        private void UpdateFilename(string newFileName, TreeItem treeItem)
        {
            // 子要素がなくなるまで検索
            if(treeItem.Children != null)
            {
                foreach(TreeItem chItem in treeItem.Children)
                {
                    UpdateFilename(newFileName, chItem);
                }
                
            }

            treeItem.Filename = newFileName;    // ファイル名変更
        }

        /// <summary>
        /// 入力した名前がファイル名に使用できるか
        /// </summary>
        private bool CheckEciFile()
        {
            bool ans = false;
            int byteCount;

            // TODO:共通の関数を用意する
            /////////////////////////////////////
            // 入力した文字が8文字以内半角か
            /////////////////////////////////////
            byteCount = System.Text.Encoding.GetEncoding("Shift_JIS").GetByteCount(RenameTBX.Text); // 入力した文字数とバイト数が等しいなら半角
            if (byteCount == RenameTBX.Text.Length && RenameTBX.Text.Length <= 8)
            {
                ans = true;
            }

            return ans;
        }

        /// <summary>
        /// キャンセルボタンクリック
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void CancelBtn_Click(object sender, RoutedEventArgs e)
        {
            this.Close();
        }
    }
}
