﻿////////////////////////////////////////////////////////////////////////////////////////
/// 機能       :   RT-edge XML編集ツール　メイン画面のTreeViewに表示する項目定義
/// 作成日     :   2022/02/21
/// 作成者     :   mizuno
/// 
/// RT-edge XML編集ツール 機能仕様書
/// file://192.168.1.240/Jobs/KA1601(RT-edge開発）/trunk/0051_Documnet(社内)/0200_egCoreService/20211021_RT-edge_XML編集ツール_機能仕様書.pptx
/// P34. プログラム仕様 ツリー表示項目
/// P35. プログラム仕様 項目タイプEnum定義
/// 
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Collections.ObjectModel;
using System.ComponentModel;            // INotifyPropertyChanged
using System.Runtime.CompilerServices;  // CallerMemberName
using System.Windows;                   // Visibility 
using RTedgeXmlObj;

namespace EgConfig
{
    /// <summary>
    /// TreeViewの編集　追加項目の視認性設定
    /// </summary>
    public class AddElemVis
    {
        public Visibility Tag { get; set; }
        public Visibility Dataset { get; set; }
        public Visibility DatasetTagRef { get; set; }
        public Visibility TagRef { get; set; }
        public Visibility TagRefIN { get; set; }
        public Visibility TagRefOUT { get; set; }
        public Visibility Collector { get; set; }
        public Visibility CollectorRef { get; set; }
        public Visibility Mailbox { get; set; }
        public Visibility Service { get; set; }

        public AddElemVis
            (Visibility Tag,
            Visibility Dataset,
            Visibility DatasetTagRef,
            Visibility TagRef,
            Visibility TagRefIN, 
            Visibility TagRefOUT,
            Visibility Collector,
            Visibility CollectorRef,
            Visibility Mailbox,
            Visibility Service)
        {
            this.Tag = Tag;
            this.Dataset = Dataset;
            this.DatasetTagRef = DatasetTagRef;
            this.TagRef = TagRef;
            this.TagRefIN = TagRefIN;
            this.TagRefOUT = TagRefOUT;
            this.Collector = Collector;
            this.CollectorRef = CollectorRef;
            this.Mailbox = Mailbox;
            this.Service = Service;
        }
    }

    /// <summary>
    /// 項目タイプの定義
    /// </summary>
    public enum ItemType
    {
        ECI = 0,
        RTedge,
        Tags,
        Tag,
        Datasets,
        Dataset,
        DatasetTagRefs,
        DatasetTagRef,
        TagRefs,
        TagRef,
        TagRefsIN,
        TagRefIN,
        TagRefsOUT,
        TagRefOUT,
        Collectors,
        Collector,
        CollectorRefs,
        CollectorRef,
        MailBoxes,
        MailBox,
        Services,
        Service,
    }

    /// <summary>
    /// ツリー項目クラス定義
    /// </summary>
    public class TreeItem : INotifyPropertyChanged
    {
        /// <summary>
        /// メンバー変数
        /// </summary>
        private string _name;                                           // 項目名
        private TreeItem _parent;                                       // 親
        private ObservableCollection<TreeItem> _children;               // 子
        private string _filename;                                       // ECIファイル名
        private int _itemType;                                          // 項目タイプ
        private int _index;                                             // 項目別インデックス
        private bool _isSelected;                                       // 項目の選択状態(true:選択中, false:未選択)
        private bool _isExpanded;                                       // ツリーの展開状態(true:展開中, false:未展開)
        private bool _isChanged;                                        // 要素の設定変更状態(true:変更中, false:未変更)

        /// <summary>
        /// コンストラクタ
        /// </summary>
        public TreeItem()
        {
            Name = "";
            Children = new ObservableCollection<TreeItem>();
        }

        public TreeItem(string name, TreeItem parent, ObservableCollection<TreeItem> children, string filename, int itemType, int index)
        {
            this._name = name;
            this._parent = parent;
            this._children = children;
            this._filename = filename;
            this._itemType = itemType;
            this._index = index;
        }

        /// <summary>
        /// プロパティ
        /// </summary>
        public string Name                                              // 項目名
        {
            get { return _name; }
            set
            {
                if (_name != value)
                {
                    _name = value;
                    NotifyPropertyChanged("Name");
                }
            }
        }
        public TreeItem Parent                                          // 親
        {
            get { return _parent; }
            set
            {
                if (_parent != value)
                {
                    _parent = value;
                    NotifyPropertyChanged("Parent");
                }
            }
        }                            
        public ObservableCollection<TreeItem> Children                  // 子
        {
            get { return _children; }
            set
            {
                if (_children != value)
                {
                    _children = value;
                    NotifyPropertyChanged("Children");
                }
            }
        }
        public string Filename                                          // ECIファイル名
        {
            get { return _filename; }
            set
            {
                if (_filename != value)
                {
                    _filename = value;
                    NotifyPropertyChanged("Filename");
                }
            }
        }
        public int ItemType                                             // 項目タイプ
        {
            get { return _itemType; }
            set
            {
                if (_itemType != value)
                {
                    _itemType = value;
                    NotifyPropertyChanged("ItemType");
                }
            }
        }
        public int Index                                                // 項目別インデックス
        {
            get { return _index; }
            set
            {
                if (_index != value)
                {
                    _index = value;
                    NotifyPropertyChanged("Index");
                }
            }
        }
        public bool IsSelected                                          // 項目の選択状態(true:選択中, false:未選択)
        {
            get { return _isSelected; }
            set
            {
                if (_isSelected != value)
                {
                    _isSelected = value;
                    NotifyPropertyChanged("IsSelected");
                }
            }
        }                            
        public bool IsExpanded                                          // ツリーの展開状態(true:展開中, false:未展開)
        {
            get { return _isExpanded; }
            set
            {
                if (_isExpanded != value)
                {
                    _isExpanded = value;
                    NotifyPropertyChanged("IsExpanded");
                }
            }
        }
        public bool IsChanged                                           // 要素の設定変更状態(true:変更中, false:未変更)
        {
            get { return _isChanged; }
            set
            {
                if (_isChanged != value)
                {
                    _isChanged = value;
                    NotifyPropertyChanged("IsChanged");
                }
            }
        }
        /// <summary>
        /// プロパティ変更通知
        /// </summary>
        public event PropertyChangedEventHandler PropertyChanged;
        private void NotifyPropertyChanged([CallerMemberName] String propertyName = "")
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
            }
        }

        //TODO:ECIファイル管理リストからTreeItemのリストを返却する処理をここに移行

        /// <summary>
        /// ECIプロジェクト管理データをTreeItemに変換
        /// </summary>
        /// <param name="projMgmt">ECIプロジェクト管理データ</param>
        /// <returns>TreeItemのリスト</returns>
        public static ObservableCollection<TreeItem> EciProjMgmt2TreeItem(EciProjMgmt eciProjMgmt)
        {
            ObservableCollection<TreeItem> treeItems = new ObservableCollection<TreeItem>();
            TreeItem treeItem;
            int index = 0;

            ////////////////////////////////////////////////////
            // 管理中のECIファイルをTreeItemに変換
            ////////////////////////////////////////////////////
            foreach (EciFileMgmt eciFileMgmt in eciProjMgmt.eciList)
            {
                treeItem = new TreeItem();
                treeItem = EciFileMgmt2TreeItem(eciFileMgmt);
                treeItem.Index = index++;
                treeItems.Add(treeItem);
            }

            return treeItems;
        }


        /// <summary>
        /// TreeViewに表示するECIファイルリストを作成する
        /// </summary>
        /// <param name="eciFileList">ECIファイル管理データリスト</param>
        /// <returns>TreeView管理用ECIファイルデータ</returns>
        public static TreeItem EciFileMgmt2TreeItem(EciFileMgmt eciFileMgmt)
        {
            TreeItem treeItem = new TreeItem();
            treeItem.Name = System.IO.Path.GetFileName(eciFileMgmt.eciFileName);
            treeItem.Filename = eciFileMgmt.eciFileName;
            treeItem.Children = new ObservableCollection<TreeItem>();

            //////////////////////////////////////
            // RTedge要素
            //////////////////////////////////////
            TreeItem RTedge = new TreeItem();
            RTedge = GetListRTedge(eciFileMgmt, treeItem);
            RTedge.Parent = treeItem;

            //////////////////////////////////////
            // Tags
            //////////////////////////////////////
            TreeItem Tags = new TreeItem();
            Tags = GetListTags(eciFileMgmt);
            Tags.Parent = RTedge;

            //////////////////////////////////////
            // Datasets
            //////////////////////////////////////
            TreeItem Datasets = new TreeItem();
            Datasets = GetListDatasets(eciFileMgmt);
            Datasets.Parent = RTedge;

            //////////////////////////////////////
            // Services
            //////////////////////////////////////
            TreeItem Services = new TreeItem();
            Services = GetListServices(eciFileMgmt, RTedge);
            Services.Parent = RTedge;

            //////////////////////////////////////
            // 組み立て
            //////////////////////////////////////
            RTedge.Children.Add(Tags);
            RTedge.Children.Add(Datasets);
            RTedge.Children.Add(Services);
            treeItem.Children.Add(RTedge);


            return treeItem;

        }

        /// <summary>
        /// RTedge用のTreeView表示リストを作成
        /// </summary>
        /// <param name="eciFile"></param>
        /// <param name="parentItem">親</param>
        /// <returns></returns>
        public static TreeItem GetListRTedge(EciFileMgmt eciFile, TreeItem parentItem)
        {
            TreeItem RTedge = new TreeItem();
            RTedge.Children = new ObservableCollection<TreeItem>();
            RTedge.Name = "RTedge";
            RTedge.Parent = parentItem;
            RTedge.Filename = eciFile.eciFileName;
            RTedge.ItemType = (int)EgConfig.ItemType.RTedge;
            RTedge.Index = 0;
            return RTedge;
        }

        public static TreeItem GetListTags(EciFileMgmt eciFile)
        {
            int index_Tag = 0;

            TreeItem Tags = new TreeItem();
            Tags.Name = "Tags";
            Tags.Filename = eciFile.eciFileName;
            Tags.ItemType = (int)EgConfig.ItemType.Tags;
            Tags.Index = 0;
            if (0 < eciFile.eciData.Tags.Count) { Tags.Children = new ObservableCollection<TreeItem>(); }// TODO:ファイル名がないとnullExceptionになる対策
            foreach (Tag TagData in eciFile.eciData.Tags)
            {
                TreeItem Tag = new TreeItem();
                Tag.Name = TagData.Name;
                Tag.Parent = Tags;
                Tag.Filename = eciFile.eciFileName;
                Tag.ItemType = (int)EgConfig.ItemType.Tag;
                Tag.Index = index_Tag++;
                Tags.Children.Add(Tag);
            }
            return Tags;
        }

        /// <summary>
        /// Datasets用のTreeView表示リストを作成
        /// </summary>
        /// <param name="eciFile"></param>
        /// <returns></returns>
        public static TreeItem GetListDatasets(EciFileMgmt eciFile)
        {
            int index_Dataset = 0;
            int index_TagRef = 0;

            TreeItem Datasets = new TreeItem();
            Datasets.Name = "Datasets";
            Datasets.Filename = eciFile.eciFileName;
            Datasets.ItemType = (int)EgConfig.ItemType.Datasets;
            Datasets.Index = 0;
            Datasets.Children = new ObservableCollection<TreeItem>();
            foreach (Dataset DatasetData in eciFile.eciData.Datasets)
            {
                index_TagRef = 0;   // indexNo初期化

                TreeItem Dataset = new TreeItem();
                Dataset.Children = new ObservableCollection<TreeItem>();
                Dataset.Parent = Datasets;
                Dataset.Name = DatasetData.Name;
                Dataset.Filename = eciFile.eciFileName;
                Dataset.ItemType = (int)EgConfig.ItemType.Dataset;
                Dataset.Index = index_Dataset++;

                TreeItem TagRefs = new TreeItem();
                if (0 < DatasetData.TagRefs.Count) { TagRefs.Children = new ObservableCollection<TreeItem>(); }
                TagRefs.Parent = Dataset;
                TagRefs.Name = "TagRefs";
                TagRefs.Filename = eciFile.eciFileName;
                TagRefs.ItemType = (int)EgConfig.ItemType.DatasetTagRefs;
                foreach (TagRef TagRefData in DatasetData.TagRefs)
                {
                    TreeItem TagRef = new TreeItem();
                    TagRef.Name = TagRefData.Name;
                    TagRef.Parent = TagRefs;
                    TagRef.Filename = eciFile.eciFileName;
                    TagRef.ItemType = (int)EgConfig.ItemType.DatasetTagRef;
                    TagRef.Index = index_TagRef++;
                    TagRefs.Children.Add(TagRef);
                }

                Dataset.Children.Add(TagRefs);
                Datasets.Children.Add(Dataset);
            }
            return Datasets;
        }

        /// <summary>
        /// 管理ファイルからサービスのリストを作成
        /// </summary>
        /// <param name="eciFile"></param>
        /// <param name="parentItem">親</param>
        /// <returns></returns>
        public static TreeItem GetListServices(EciFileMgmt eciFile, TreeItem parentItem)
        {
            int index_Service = 0, index_Tag = 0, index_Dataset = 0, index_DatasetTagRef = 0, index_TagRef_OUT = 0, index_TagRef_IN = 0, index_TagRef = 0, index_Collector = 0, index_CollectorRef = 0, index_MailBox = 0;

            TreeItem Services = new TreeItem();
            Services.Name = "Services";
            Services.Parent = parentItem;
            Services.Filename = eciFile.eciFileName;
            Services.ItemType = (int)EgConfig.ItemType.Services;
            Services.Index = 0;
            if (0 < eciFile.eciData.Services.Count) { Services.Children = new ObservableCollection<TreeItem>(); }
            foreach (Service ServiceData in eciFile.eciData.Services)
            {
                TreeItem Service = new TreeItem();
                Service.Name = ServiceData.Name;
                Service.Parent = Services;
                Service.Filename = eciFile.eciFileName;
                Service.ItemType = (int)EgConfig.ItemType.Service;
                Service.Index = index_Service++;

                // indexNo初期化
                index_Tag = 0;
                index_Dataset = 0;
                index_DatasetTagRef = 0;
                index_TagRef_OUT = 0;
                index_TagRef_IN = 0;
                index_TagRef = 0;
                index_Collector = 0;
                index_CollectorRef = 0;
                index_MailBox = 0;

                //
                // Tagsの追加
                //
                TreeItem Tags = new TreeItem();
                Tags.Name = "Tags";
                Tags.Parent = Service;
                Tags.Filename = eciFile.eciFileName;
                Tags.ItemType = (int)EgConfig.ItemType.Tags;
                Tags.Index = 0;
                foreach (Tag TagData in ServiceData.Tags) //Serviceが保持するTagsの数だけループ
                {
                    TreeItem Tag = new TreeItem();
                    Tag.Filename = eciFile.eciFileName;
                    Tag.Parent = Tags;
                    Tag.Name = TagData.Name;
                    Tag.ItemType = (int)EgConfig.ItemType.Tag;
                    Tag.Index = index_Tag++;
                    Tags.Children.Add(Tag);//Tagを追加
                }
                //
                // Datasetsの追加
                //
                TreeItem Datasets = new TreeItem();
                Datasets.Name = "Datasets";
                Datasets.Parent = Service;
                Datasets.Filename = eciFile.eciFileName;
                Datasets.ItemType = (int)EgConfig.ItemType.Datasets;
                Datasets.Index = 1;
                Datasets.Children = new ObservableCollection<TreeItem>();
                foreach (Dataset DatasetData in ServiceData.Datasets)//Serviceが保持するDatasetsの数だけループ
                {
                    index_DatasetTagRef = 0;   // indexNo初期化

                    TreeItem Dataset = new TreeItem();
                    Dataset.Children = new ObservableCollection<TreeItem>();
                    Dataset.Parent = Datasets;
                    Dataset.Name = DatasetData.Name;
                    Dataset.Filename = eciFile.eciFileName;
                    Dataset.ItemType = (int)EgConfig.ItemType.Dataset;
                    Dataset.Index = index_Dataset++;

                    //
                    // Dataset内のTags追加
                    //
                    TreeItem DSetTagRefs = new TreeItem();
                    if (0 < DatasetData.TagRefs.Count) { DSetTagRefs.Children = new ObservableCollection<TreeItem>(); }
                    DSetTagRefs.Parent = Dataset;
                    DSetTagRefs.Name = "TagRefs";
                    DSetTagRefs.Filename = eciFile.eciFileName;
                    DSetTagRefs.ItemType = (int)EgConfig.ItemType.DatasetTagRefs;
                    foreach (TagRef TagRefData in DatasetData.TagRefs)//Datasetが保持するTagsの数だけループ
                    {
                        TreeItem DSetTagRef = new TreeItem();
                        DSetTagRef.Name = TagRefData.Name;
                        DSetTagRef.Parent = DSetTagRefs;
                        DSetTagRef.Filename = eciFile.eciFileName;
                        DSetTagRef.ItemType = (int)EgConfig.ItemType.DatasetTagRef;
                        DSetTagRef.Index = index_DatasetTagRef++;
                        DSetTagRefs.Children.Add(DSetTagRef);
                    }

                    Dataset.Children.Add(DSetTagRefs);
                    Datasets.Children.Add(Dataset);
                }
                //
                // TagRefsの追加
                //
                TreeItem TagRefs = new TreeItem();
                TagRefs.Name = "TagRefs";
                TagRefs.Parent = Service;
                TagRefs.Filename = eciFile.eciFileName;
                TagRefs.ItemType = (int)EgConfig.ItemType.TagRefs;
                TagRefs.Index = 2;
                foreach (TagRef TagRefData in ServiceData.TagRefs)
                {
                    TreeItem TagRef = new TreeItem();
                    TagRef.Name = TagRefData.Name;
                    TagRef.Parent = TagRefs;
                    TagRef.Filename = eciFile.eciFileName;
                    TagRef.ItemType = (int)EgConfig.ItemType.TagRef;
                    TagRef.Index = index_TagRef++;
                    TagRefs.Children.Add(TagRef);
                }

                //
                // TagRefs_OUTの追加
                //
                TreeItem TagRefs_OUT = new TreeItem();
                TagRefs_OUT.Name = "TagRefs_OUT";
                TagRefs_OUT.Parent = Service;
                TagRefs_OUT.Filename = eciFile.eciFileName;
                TagRefs_OUT.ItemType = (int)EgConfig.ItemType.TagRefsOUT;
                TagRefs_OUT.Index = 3;
                foreach (TagRef TagRefOUTData in ServiceData.TagRefs_OUT)
                {
                    TreeItem TagRef_OUT = new TreeItem();
                    TagRef_OUT.Filename = eciFile.eciFileName;
                    TagRef_OUT.Parent = TagRefs_OUT;
                    TagRef_OUT.Name = TagRefOUTData.Name;
                    TagRef_OUT.ItemType = (int)EgConfig.ItemType.TagRefOUT;
                    TagRef_OUT.Index = index_TagRef_OUT++;
                    TagRefs_OUT.Children.Add(TagRef_OUT);
                }
                //
                // TagRefs_INの追加
                //
                TreeItem TagRefs_IN = new TreeItem();
                TagRefs_IN.Name = "TagRefs_IN";
                TagRefs_IN.Parent = Service;
                TagRefs_IN.Filename = eciFile.eciFileName;
                TagRefs_IN.ItemType = (int)EgConfig.ItemType.TagRefsIN;
                TagRefs_IN.Index = 4;
                foreach (TagRef TagRefINData in ServiceData.TagRefs_IN)
                {
                    TreeItem TagRef_IN = new TreeItem();
                    TagRef_IN.Name = TagRefINData.Name;
                    TagRef_IN.Parent = TagRefs_IN;
                    TagRef_IN.Filename = eciFile.eciFileName;
                    TagRef_IN.ItemType = (int)EgConfig.ItemType.TagRefIN;
                    TagRef_IN.Index = index_TagRef_IN++;
                    TagRefs_IN.Children.Add(TagRef_IN);
                }
                //
                // Collectorsの追加
                //
                TreeItem Collectors = new TreeItem();
                Collectors.Name = "Collectors";
                Collectors.Parent = Service;
                Collectors.Filename = eciFile.eciFileName;
                Collectors.ItemType = (int)EgConfig.ItemType.Collectors;
                Collectors.Index = 5;
                foreach (Collector CollectorData in ServiceData.Collectors)
                {
                    TreeItem Collector = new TreeItem();
                    Collector.Name = CollectorData.Name;
                    Collector.Parent = Collectors;
                    Collector.Filename = eciFile.eciFileName;
                    Collector.ItemType = (int)EgConfig.ItemType.Collector;
                    Collector.Index = index_Collector++;
                    Collectors.Children.Add(Collector);
                }
                //
                // CollectorRefsの追加
                //
                TreeItem CollectorRefs = new TreeItem();
                CollectorRefs.Name = "CollectorRefs";
                CollectorRefs.Parent = Service;
                CollectorRefs.Filename = eciFile.eciFileName;
                CollectorRefs.ItemType = (int)EgConfig.ItemType.CollectorRefs;
                CollectorRefs.Index = 6;
                foreach (CollectorRef CollectorRefData in ServiceData.CollectorRefs)
                {
                    TreeItem CollectorRef = new TreeItem();
                    CollectorRef.Name = CollectorRefData.Name;
                    CollectorRef.Parent = CollectorRefs;
                    CollectorRef.Filename = eciFile.eciFileName;
                    CollectorRef.ItemType = (int)EgConfig.ItemType.CollectorRef;
                    CollectorRef.Index = index_CollectorRef++;
                    CollectorRefs.Children.Add(CollectorRef);
                }
                //
                // MailBoxesの追加
                //
                TreeItem MailBoxes = new TreeItem();
                MailBoxes.Name = "MailBoxes";
                MailBoxes.Parent = Service;
                MailBoxes.Filename = eciFile.eciFileName;
                MailBoxes.ItemType = (int)EgConfig.ItemType.MailBoxes;
                MailBoxes.Index = 7;
                foreach (MailBox MailBoxData in ServiceData.MailBoxes)
                {
                    TreeItem MailBox = new TreeItem();
                    MailBox.Name = MailBoxData.Name;
                    MailBox.Parent = MailBoxes;
                    MailBox.Filename = eciFile.eciFileName;
                    MailBox.ItemType = (int)EgConfig.ItemType.MailBox;
                    MailBox.Index = index_MailBox++;
                    MailBoxes.Children.Add(MailBox);
                }
                //
                //それぞれをサービスは以下に追加
                //
                Service.Children.Add(Tags);
                Service.Children.Add(Datasets);
                Service.Children.Add(TagRefs);
                Service.Children.Add(TagRefs_OUT);
                Service.Children.Add(TagRefs_IN);
                Service.Children.Add(Collectors);
                Service.Children.Add(CollectorRefs);
                Service.Children.Add(MailBoxes);
                Services.Children.Add(Service);
            }
            return Services;
        }

        /// <summary>
        /// TreeItemのTagsを作成する
        /// </summary>
        /// <returns></returns>
        public static TreeItem CreateTags()
        {
            TreeItem Tags = new TreeItem();

            return Tags;
        }

        public static TreeItem CreateDataset()
        {
            TreeItem Dataset = new TreeItem();

            return Dataset;
        }

        public static TreeItem CreateDataset(Dataset dataset)
        {
            TreeItem Dataset = new TreeItem();
            int index_TagRef = 0;

            Dataset.Children = new ObservableCollection<TreeItem>();
            //Dataset.Parent = Datasets;
            Dataset.Name = dataset.Name;
            //Dataset.Filename = eciFile.eciFileName;
            Dataset.ItemType = (int)EgConfig.ItemType.Dataset;
            //Dataset.Index = index_Dataset++;

            TreeItem TagRefs = new TreeItem();
            if (0 < dataset.TagRefs.Count) { TagRefs.Children = new ObservableCollection<TreeItem>(); }
            TagRefs.Parent = Dataset;
            TagRefs.Name = "TagRefs";
            //TagRefs.Filename = eciFile.eciFileName;
            TagRefs.ItemType = (int)EgConfig.ItemType.DatasetTagRefs;
            foreach (TagRef TagRefData in dataset.TagRefs)
            {
                TreeItem TagRef = new TreeItem();
                TagRef.Name = TagRefData.Name;
                TagRef.Parent = TagRefs;
                //TagRef.Filename = eciFile.eciFileName;
                TagRef.ItemType = (int)EgConfig.ItemType.DatasetTagRef;
                TagRef.Index = index_TagRef++;
                TagRefs.Children.Add(TagRef);
            }

            Dataset.Children.Add(TagRefs);

            return Dataset;
        }

        public void AddDataset(TreeItem Dataset)
        {

            // Datasets検索
            TreeItem treeItem = SearchTreeItem(EgConfig.ItemType.Datasets);// このツリーはメンバーのツリー？

            treeItem.Children.Add(Dataset);
        }

        /// <summary>
        /// データセットを追加する
        /// </summary>
        /// <param name="data"></param>
        public void AddDataset(Dataset data)
        {
            TreeItem Dataset = new TreeItem();
            TreeItem Datasets = new TreeItem();

            // DatasetのTreeItem作成
            Dataset = CreateDataset(data);

            // DatasetsのTreeItem取得
            Datasets = SearchDatasets();

            // Datasetの親・ファイル名・Indexを設定
            Dataset.Parent = Datasets;
            Dataset.Filename = Datasets.Filename;
            Dataset.Index = Datasets.Children.Count;

            // DatasetTagRefsの親・ファイル名設定
            foreach(TreeItem TagRefs in Dataset.Children)
            {
                // DatasetTagRefの親・ファイル名設定
                foreach (TreeItem TagRef in TagRefs.Children)
                {
                    TagRef.Parent = TagRefs;
                    TagRef.Filename = Datasets.Filename;
                }

                TagRefs.Parent = Dataset;
                TagRefs.Filename = Datasets.Filename;
            }

            Datasets.Children.Add(Dataset);
        }

        /// <summary>
        /// DatasetTagRefを追加する
        /// </summary>
        /// <param name="tagRefs"></param>
        public void AddDatasetTagRef(int index, List<TagRef> tagRefs)
        {
            TreeItem DatasetTagRef = new TreeItem();
            TreeItem TagRef;

            DatasetTagRef = SearchDatasetTagRefs();    // DatasetTagRefsのツリーアイテムを検索

            foreach(TagRef tagRef in tagRefs)
            {
                TagRef = new TreeItem();
                TagRef = CreateDatasetTagRef(tagRef);
                TagRef.Parent = DatasetTagRef;
                TagRef.Filename = DatasetTagRef.Filename;
                TagRef.Index = DatasetTagRef.Children.Count();
                DatasetTagRef.Children.Add(TagRef);
            }
        }

        /// <summary>
        /// DatasetTagRefの作成
        /// </summary>
        /// <returns></returns>
        public static TreeItem CreateDatasetTagRef(TagRef tagRef)
        {
            TreeItem TagRef = new TreeItem();

            TagRef.Name = tagRef.Name;
            // TagRef.Parent = TagRefs;
            // TagRef.Filename = eciFile.eciFileName;
            TagRef.ItemType = (int)EgConfig.ItemType.DatasetTagRef;
            // TagRef.Index = index_TagRef++;
            // TagRefs.Children.Add(TagRef);

            return TagRef;
        }

        /// <summary>
        /// TagRefを追加する
        /// </summary>
        /// <param name="tagRefs"></param>
        public void AddTagRef(int index, List<TagRef> tagRefs)
        {
            TreeItem TagRefs = new TreeItem();
            TreeItem TagRef;

            TagRefs = SearchTagRefs();    // TagRefのツリーアイテムを検索

            foreach (TagRef tagRef in tagRefs)
            {
                TagRef = new TreeItem();
                TagRef = CreateTagRef(tagRef);
                TagRef.Parent = TagRefs;
                TagRef.Filename = TagRefs.Filename;
                TagRef.Index = TagRefs.Children.Count();
                TagRefs.Children.Add(TagRef);
            }
        }

        /// <summary>
        /// TagRefの作成
        /// </summary>
        /// <returns></returns>
        public static TreeItem CreateTagRef(TagRef tagRef)
        {
            TreeItem TagRef = new TreeItem();

            TagRef.Name = tagRef.Name;
            // TagRef.Parent = TagRefs;
            // TagRef.Filename = eciFile.eciFileName;
            TagRef.ItemType = (int)EgConfig.ItemType.TagRef;
            // TagRef.Index = index_TagRef++;
            // TagRefs.Children.Add(TagRef);

            return TagRef;
        }

        /// <summary>
        /// TagRefINを追加する
        /// </summary>
        /// <param name="tagRefs"></param>
        public void AddTagRefIN(int index, List<TagRef> tagRefs)
        {
            TreeItem TagRefs = new TreeItem();
            TreeItem TagRefIN;

            TagRefs = SearchTagRefsIN();    // TagRefINのツリーアイテムを検索

            foreach (TagRef tagRef in tagRefs)
            {
                TagRefIN = new TreeItem();
                TagRefIN = CreateTagRefIN(tagRef);
                TagRefIN.Parent = TagRefs;
                TagRefIN.Filename = TagRefs.Filename;
                TagRefIN.Index = TagRefs.Children.Count();
                TagRefs.Children.Add(TagRefIN);
            }
        }

        /// <summary>
        /// TagRefINの作成
        /// </summary>
        /// <returns></returns>
        public static TreeItem CreateTagRefIN(TagRef tagRef)
        {
            TreeItem TagRef = new TreeItem();

            TagRef.Name = tagRef.Name;
            // TagRef.Parent = TagRefs;
            // TagRef.Filename = eciFile.eciFileName;
            TagRef.ItemType = (int)EgConfig.ItemType.TagRefIN;
            // TagRef.Index = index_TagRef++;
            // TagRefs.Children.Add(TagRef);

            return TagRef;
        }

        /// <summary>
        /// TagRefOUTを追加する
        /// </summary>
        /// <param name="tagRefs"></param>
        public void AddTagRefOUT(int index, List<TagRef> tagRefs)
        {
            TreeItem TagRefsOUT = new TreeItem();
            TreeItem TagRefOUT;

            TagRefsOUT = SearchTagRefsOUT();        // TagRefsOUTのツリーアイテムを検索

            foreach (TagRef tagRef in tagRefs)
            {
                TagRefOUT = new TreeItem();
                TagRefOUT = CreateTagRefOUT(tagRef);
                TagRefOUT.Parent = TagRefsOUT;
                TagRefOUT.Filename = TagRefsOUT.Filename;
                TagRefOUT.Index = TagRefsOUT.Children.Count();
                TagRefsOUT.Children.Add(TagRefOUT);
            }
        }

        /// <summary>
        /// TagRefOUTの作成
        /// </summary>
        /// <returns></returns>
        public static TreeItem CreateTagRefOUT(TagRef tagRef)
        {
            TreeItem TagRef = new TreeItem();

            TagRef.Name = tagRef.Name;
            // TagRef.Parent = TagRefs;
            // TagRef.Filename = eciFile.eciFileName;
            TagRef.ItemType = (int)EgConfig.ItemType.TagRefOUT;
            // TagRef.Index = index_TagRef++;
            // TagRefs.Children.Add(TagRef);

            return TagRef;
        }

        /// <summary>
        /// Collectorを追加する
        /// </summary>
        public void AddCollector(int index, Collector collector)
        {
            TreeItem Collectors;
            Collectors = SearchCollectors();    // Collectorのツリーアイテム検索

            // CollectorデータからTreeItemを作成する
            TreeItem Collector = CreateCollector(collector);
            Collector.Parent = Collectors;
            Collector.Filename = Collectors.Filename;
            Collector.Index = Collectors.Children.Count;

            Collectors.Children.Add(Collector);
        }

        /// <summary>
        /// CollectorデータからTreeItemを作成する
        /// </summary>
        /// <param name="collector"></param>
        /// <returns></returns>
        public static TreeItem CreateCollector(Collector collector)
        {
            TreeItem Collector = new TreeItem();

            Collector.Name = collector.Name;
            Collector.ItemType = (int)EgConfig.ItemType.Collector;

            return Collector;
        }

        /// <summary>
        /// CollectorRefを追加する
        /// </summary>
        /// <param name="index"></param>
        /// <param name="collector"></param>
        public void AddCollectorRef(int index, List<CollectorRef> collectorRefs)
        {
            TreeItem CollectorRefs;
            CollectorRefs = SearchCollectorRefs();   // CollectorRefsのツリーアイテム検索

            // CollectorRefデータからTreeItemを作成する
            foreach(CollectorRef collectorRef in collectorRefs)
            {
                TreeItem CollectorRef = CreateCollectorRef(collectorRef);
                CollectorRef.Parent = CollectorRefs;
                CollectorRef.Filename = CollectorRefs.Filename;
                CollectorRef.Index = CollectorRefs.Children.Count;
                CollectorRefs.Children.Add(CollectorRef);
            }
            
        }

        /// <summary>
        /// CollectorRefを追加するデータからTreeItemを作成する
        /// </summary>
        /// <param name="collector"></param>
        /// <returns></returns>
        public static TreeItem CreateCollectorRef(CollectorRef collectorref)
        {
            TreeItem Collector = new TreeItem();

            Collector.Name = collectorref.Name;
            Collector.ItemType = (int)EgConfig.ItemType.CollectorRef;

            return Collector;
        }

        /// <summary>
        /// Mailboxを追加する
        /// </summary>
        public void AddMailbox(int index, MailBox mailbox)
        {
            TreeItem Mailboxes;
            Mailboxes = SearchMailboxes(index);   // Collectorのツリーアイテム検索

            // CollectorデータからTreeItemを作成する
            TreeItem Mailbox = CreateMailbox(mailbox);
            Mailbox.Parent = Mailboxes;
            Mailbox.Filename = Mailboxes.Filename;
            Mailbox.Index = Mailboxes.Children.Count;

            Mailboxes.Children.Add(Mailbox);
        }

        /// <summary>
        /// MailboxデータからTreeItemを作成する
        /// </summary>
        /// <param name="mailbox"></param>
        /// <returns></returns>
        public static TreeItem CreateMailbox(MailBox mailbox)
        {
            TreeItem Mailbox = new TreeItem();

            Mailbox.Name = mailbox.Name;
            Mailbox.ItemType = (int)EgConfig.ItemType.MailBox;

            return Mailbox;
        }

        public TreeItem SearchTreeItem(ItemType type)
        {
            TreeItem treeItem = new TreeItem();

            switch (type)
            {
                case EgConfig.ItemType.Datasets:
                    treeItem = SearchDatasets();
                    break;
                default:
                    break;
            }

            return treeItem;
        }


        /// <summary>
        /// Mailboxesツリーアイテムを検索
        /// </summary>
        /// <param name="index">サービスのindex</param>
        /// <returns></returns>
        private TreeItem SearchMailboxes(int index)
        {
            TreeItem treeItem = new TreeItem();

            treeItem = this._children[0]._children[2]._children[index]._children[5];  // RTedge-Services-Service[]-Mailboxes

            return treeItem;
        }

        /// <summary>
        /// TagsにTagデータを追加する
        /// </summary>
        /// <param name="tag">Tagデータ</param>
        public void AddTag2Tags(Tag tag)
        {
            TreeItem Tags;
            
            Tags = SearchTags();    // Tagsを検索する
            if (Tags != null)
            {
                Tags.AddTag(tag);   // Tagを追加する
            }
        }

        /// <summary>
        /// ServicesにServiceデータを追加する
        /// </summary>
        /// <param name="service">Serviceデータ</param>
        public void AddService2Services(Service service)
        {
            TreeItem Services;
            // Servicesを検索する
            Services = SearchServices();

            // Serviceを追加する
            Services.AddService(service);
        }
        /// <summary>
        /// MailboxesにMailboxデータを追加する
        /// </summary>
        /// <param name="mailBox"></param>
        public void AddMailbox2Mailboxes(MailBox mailBox)
        {
            TreeItem Mailboxes;
            // Mailboxesを検索する
            Mailboxes = SearchMailboxes();

            // Mailboxを追加する
            Mailboxes.AddMailbox(mailBox);
        }

        /// <summary>
        /// 現在のTreeItemからTagsを検索する
        /// (RTedge内にない場合は追加する)
        /// </summary>
        /// <returns></returns>
        public TreeItem SearchTags()
        {
            int i;
            TreeItem Tags = null;

            switch (ItemType)
            {
                case (int)EgConfig.ItemType.ECI:
                    for(i = 0; i < this.Children.Count; i++)
                    {
                        if(this.Children[i].ItemType == (int)EgConfig.ItemType.RTedge)
                        {
                            Tags = this.Children[i].SearchTags();
                            break;
                        }
                    }
                    break;
                case (int)EgConfig.ItemType.RTedge:
                case (int)EgConfig.ItemType.Service:
                    for (i = 0; i < this.Children.Count; i++)
                    {
                        if (this.Children[i].ItemType == (int)EgConfig.ItemType.Tags)
                        {
                            Tags = this.Children[i];
                            break;
                        }
                    }
                    // Tagsがない作成
                    if(Tags == null)
                    {
                        Tags = new TreeItem(Define.TreeView.ITEM_NAME_002, this, new ObservableCollection<TreeItem>(), this.Filename, (int)EgConfig.ItemType.Tags, i);
                        this.AddTags(Tags);
                    }
                    break;
                case (int)EgConfig.ItemType.Tags:
                    Tags = this;
                    break;
                case (int)EgConfig.ItemType.Tag:
                    if(this.Parent != null)
                    {
                        Tags = Parent.SearchTags();
                    }
                    break;
                default:
                    break;
            }

            return Tags;
        }

        /// <summary>
        /// 現在のTreeItemからDatasetsを検索する
        /// </summary>
        /// <returns></returns>
        public TreeItem SearchDatasets()
        {
            int i;
            TreeItem Datasets = null;

            switch (ItemType)
            {
                case (int)EgConfig.ItemType.ECI:
                    for (i = 0; i < this.Children.Count; i++)
                    {
                        if (this.Children[i].ItemType == (int)EgConfig.ItemType.RTedge)
                        {
                            Datasets = this.Children[i].SearchDatasets();
                            break;
                        }
                    }
                    break;
                case (int)EgConfig.ItemType.RTedge:
                case (int)EgConfig.ItemType.Service:
                    for (i = 0; i < this.Children.Count; i++)
                    {
                        if (this.Children[i].ItemType == (int)EgConfig.ItemType.Datasets)
                        {
                            Datasets = this.Children[i].SearchDatasets();
                            break;
                        }
                    }
                    // Datasetsがない場合作成
                    if (Datasets == null)
                    {
                        Datasets = new TreeItem(Define.TreeView.ITEM_NAME_004, this, new ObservableCollection<TreeItem>(), this.Filename, (int)EgConfig.ItemType.Datasets, i);
                        this.AddDatasets(Datasets);
                    }
                    break;
                case (int)EgConfig.ItemType.Dataset:
                case (int)EgConfig.ItemType.DatasetTagRefs:
                case (int)EgConfig.ItemType.DatasetTagRef:
                    if (this.Parent != null)
                    {
                        Datasets = Parent.SearchDataset();
                    }
                    break;
                case (int)EgConfig.ItemType.Datasets:
                    Datasets = this;
                    break;
                default:
                    break;
            }

            return Datasets;
        }

        /// <summary>
        /// 現在のTreeItemからDatasetを検索する
        /// </summary>
        /// <returns></returns>
        public TreeItem SearchDataset()
        {
            TreeItem treeItem = null;

            switch (ItemType)
            {
                case (int)EgConfig.ItemType.DatasetTagRefs:
                case (int)EgConfig.ItemType.DatasetTagRef:
                    treeItem = Parent.SearchDataset();
                    break;
                case (int)EgConfig.ItemType.Dataset:
                    treeItem = this;
                    break;
                default:
                    break;
            }

            return treeItem;
        }

        /// <summary>
        /// 現在のTreeItemからDatasetTagRefsを検索する
        /// </summary>
        /// <returns></returns>
        private TreeItem SearchDatasetTagRefs()
        {
            int i;
            TreeItem DatasetTagRefs = null;

            switch (ItemType)
            {
                case (int)EgConfig.ItemType.Dataset:
                    for (i = 0; i < this.Children.Count; i++)
                    {
                        if (this.Children[i].ItemType == (int)EgConfig.ItemType.DatasetTagRefs)
                        {
                            DatasetTagRefs = this.Children[i];
                            break;
                        }
                    }
                    // DatasetTagRefsがない場合作成
                    if (DatasetTagRefs == null)
                    {
                        DatasetTagRefs = new TreeItem(Define.TreeView.ITEM_NAME_006, this, new ObservableCollection<TreeItem>(), this.Filename, (int)EgConfig.ItemType.DatasetTagRefs, i);
                        this.AddDatasetTagRefs(DatasetTagRefs);
                    }
                    break;
                
                case (int)EgConfig.ItemType.DatasetTagRefs:
                    DatasetTagRefs = this;
                    break;
                case (int)EgConfig.ItemType.DatasetTagRef:
                    if (this.Parent != null)
                    {
                        DatasetTagRefs = Parent.SearchDatasetTagRefs();
                    }
                    break;
                default:
                    break;
            }

            return DatasetTagRefs;
        }

        /// <summary>
        /// DatasetTagRefツリーアイテムを検索
        /// </summary>
        /// <param name="index"></param>
        /// <returns></returns>
        private TreeItem SearchDatasetTagRef(int index)
        {
            TreeItem treeItem = new TreeItem();

            treeItem = this._children[0]._children[1]._children[index]._children[0];  // RTedge-Datasets-Dataset[]-TagRef

            return treeItem;
        }

        /// <summary>
        /// 現在のTreeItemからServicesを検索する
        /// </summary>
        /// <returns></returns>
        public TreeItem SearchServices()
        {
            int i;
            TreeItem Services = null;

            switch (ItemType)
            {
                case (int)EgConfig.ItemType.ECI:
                    for (i = 0; i < this.Children.Count; i++)
                    {
                        if (this.Children[i].ItemType == (int)EgConfig.ItemType.RTedge)
                        {
                            Services = this.Children[i].SearchServices();
                            break;
                        }
                    }
                    break;
                case (int)EgConfig.ItemType.RTedge:
                    for (i = 0; i < this.Children.Count; i++)
                    {
                        if (this.Children[i].ItemType == (int)EgConfig.ItemType.Services)
                        {
                            Services = this.Children[i];
                            break;
                        }
                    }
                    // Servicesがない場合作成する
                    if (Services == null)
                    {
                        Services = new TreeItem(Define.TreeView.ITEM_NAME_020, this, new ObservableCollection<TreeItem>(), this.Filename, (int)EgConfig.ItemType.Services, i);
                        this.AddServices(Services);
                    }
                    break;
                case (int)EgConfig.ItemType.Services:
                    Services = this;
                    break;
                default:
                    break;
            }

            return Services;
        }

        /// <summary>
        /// ルート要素から一致するItemTypeのTreeItemを検索(深さ優先検索)
        /// </summary>
        /// <param name="type">検索するItemType</param>
        /// <returns>ヒットしたTreeItem</returns>
        public TreeItem SearchForItemFromRoot(ItemType type)
        {
            TreeItem root = null, searchItem = null;

            root = SearchRoot();
            if(root is TreeItem)
            {
                searchItem = root.SearchItemType(type);
            }

            return searchItem;
        }

        /// <summary>
        /// TreeItemのルート要素を検索
        /// Parentを遡ります
        /// </summary>
        /// <returns>TreeItemのルート</returns>
        public TreeItem SearchRoot()
        {
            TreeItem root = this;

            if(this.Parent is TreeItem parent)
            {
                root = parent.SearchRoot();
            }

            return root;
        }

        /// <summary>
        /// 自分・子孫ノードに該当するItemTypeのTreeItemを検索します
        /// </summary>
        /// <param name="type"></param>
        /// <returns>ヒットしたTreeITem</returns>
        public TreeItem SearchItemType(ItemType type)
        {
            TreeItem searchItem = null;

            // TreeItemのItemTypeが検索対象と一致
            if(this.ItemType == (int)type)
            {
                return this;
            }

            // 子ノードを検索
            foreach (TreeItem item in this.Children)
            {
                searchItem = item.SearchItemType(type);
                if(searchItem is TreeItem)
                {
                    break;
                }
            }

            return searchItem;
        }

        /// <summary>
        /// 現在のTreeItemからServiceを検索する
        /// </summary>
        /// <returns></returns>
        public TreeItem SearchService()
        {
            TreeItem treeItem = null;

            switch (ItemType)
            {
                case (int)EgConfig.ItemType.Tags:
                case (int)EgConfig.ItemType.Tag:
                case (int)EgConfig.ItemType.Datasets:
                case (int)EgConfig.ItemType.Dataset:
                case (int)EgConfig.ItemType.DatasetTagRefs:
                case (int)EgConfig.ItemType.DatasetTagRef:
                case (int)EgConfig.ItemType.TagRefs:
                case (int)EgConfig.ItemType.TagRef:
                case (int)EgConfig.ItemType.TagRefsIN:
                case (int)EgConfig.ItemType.TagRefIN:
                case (int)EgConfig.ItemType.TagRefsOUT:
                case (int)EgConfig.ItemType.TagRefOUT:
                case (int)EgConfig.ItemType.Collectors:
                case (int)EgConfig.ItemType.Collector:
                case (int)EgConfig.ItemType.CollectorRefs:
                case (int)EgConfig.ItemType.CollectorRef:
                case (int)EgConfig.ItemType.MailBoxes:
                case (int)EgConfig.ItemType.MailBox:
                    treeItem = Parent.SearchService();
                    break;
                case (int)EgConfig.ItemType.Service:
                    treeItem = this;
                    break;
                default:
                    break;
            }

            return treeItem;
        }

        /// <summary>
        /// 現在のTreeItemからTagRefsを検索する
        /// </summary>
        /// <returns></returns>
        private TreeItem SearchTagRefs()
        {
            int i;
            TreeItem TagRefs = null;

            switch (ItemType)
            {
                case (int)EgConfig.ItemType.Service:
                    for (i = 0; i < this.Children.Count; i++)
                    {
                        if (this.Children[i].ItemType == (int)EgConfig.ItemType.TagRefs)
                        {
                            TagRefs = this.Children[i];
                            break;
                        }
                    }
                    // TagRefsがない場合作成
                    if (TagRefs == null)
                    {
                        TagRefs = new TreeItem(Define.TreeView.ITEM_NAME_008, this, new ObservableCollection<TreeItem>(), this.Filename, (int)EgConfig.ItemType.TagRefs, i);
                        this.AddTagRefs(TagRefs);
                    }
                    break;
                case (int)EgConfig.ItemType.TagRefs:
                    TagRefs = this;
                    break;
                case (int)EgConfig.ItemType.TagRef:
                    if (this.Parent != null)
                    {
                        TagRefs = Parent.SearchTagRefs();
                    }
                    break;
                default:
                    break;
            }

            return TagRefs;
        }

        /// <summary>
        /// 現在のTreeItemからTagRefsINを検索する
        /// </summary>
        /// <returns></returns>
        private TreeItem SearchTagRefsIN()
        {
            int i;
            TreeItem TagRefsIN = null;

            switch (ItemType)
            {
                case (int)EgConfig.ItemType.Service:
                    for (i = 0; i < this.Children.Count; i++)
                    {
                        if (this.Children[i].ItemType == (int)EgConfig.ItemType.TagRefsIN)
                        {
                            TagRefsIN = this.Children[i];
                            break;
                        }
                    }
                    // TagRefsINがない場合作成
                    if (TagRefsIN == null)
                    {
                        TagRefsIN = new TreeItem(Define.TreeView.ITEM_NAME_010, this, new ObservableCollection<TreeItem>(), this.Filename, (int)EgConfig.ItemType.TagRefsIN, i);
                        this.AddTagRefsIN(TagRefsIN);
                    }
                    break;
                case (int)EgConfig.ItemType.TagRefsIN:
                    TagRefsIN = this;
                    break;
                case (int)EgConfig.ItemType.TagRefIN:
                    if (this.Parent != null)
                    {
                        TagRefsIN = Parent.SearchTagRefsIN();
                    }
                    break;
                default:
                    break;
            }

            return TagRefsIN;
        }

        /// <summary>
        /// 現在のTreeItemからTagRefsOUTを検索する
        /// </summary>
        /// <returns></returns>
        private TreeItem SearchTagRefsOUT()
        {
            int i;
            TreeItem TagRefsOUT = null;

            switch (ItemType)
            {
                case (int)EgConfig.ItemType.Service:
                    for (i = 0; i < this.Children.Count; i++)
                    {
                        if (this.Children[i].ItemType == (int)EgConfig.ItemType.TagRefsOUT)
                        {
                            TagRefsOUT = this.Children[i];
                            break;
                        }
                    }
                    // TagRefsがない場合作成
                    if (TagRefsOUT == null)
                    {
                        TagRefsOUT = new TreeItem(Define.TreeView.ITEM_NAME_012, this, new ObservableCollection<TreeItem>(), this.Filename, (int)EgConfig.ItemType.TagRefsOUT, i);
                        this.AddTagRefsOUT(TagRefsOUT);
                    }
                    break;
                case (int)EgConfig.ItemType.TagRefsOUT:
                    TagRefsOUT = this;
                    break;
                case (int)EgConfig.ItemType.TagRefOUT:
                    if (this.Parent != null)
                    {
                        TagRefsOUT = Parent.SearchTagRefsOUT();
                    }
                    break;
                default:
                    break;
            }

            return TagRefsOUT;
        }

        /// <summary>
        /// 現在のTreeItemからCollectorsを検索する
        /// </summary>
        /// <returns></returns>
        private TreeItem SearchCollectors()
        {
            int i;
            TreeItem Collectors = null;

            switch (ItemType)
            {
                case (int)EgConfig.ItemType.Service:
                    for (i = 0; i < this.Children.Count; i++)
                    {
                        if (this.Children[i].ItemType == (int)EgConfig.ItemType.Collectors)
                        {
                            Collectors = this.Children[i];
                            break;
                        }
                    }
                    // Collectorsがない場合作成
                    if (Collectors == null)
                    {
                        Collectors = new TreeItem(Define.TreeView.ITEM_NAME_014, this, new ObservableCollection<TreeItem>(), this.Filename, (int)EgConfig.ItemType.Collectors, i);
                        this.AddCollectors(Collectors);
                    }
                    break;
                case (int)EgConfig.ItemType.Collectors:
                    Collectors = this;
                    break;
                case (int)EgConfig.ItemType.Collector:
                    if (this.Parent != null)
                    {
                        Collectors = Parent.SearchCollectors();
                    }
                    break;
                default:
                    break;
            }

            return Collectors;
        }

        /// <summary>
        /// 現在のTreeItemからCollectorRefsを検索する
        /// </summary>
        /// <returns></returns>
        private TreeItem SearchCollectorRefs()
        {
            int i;
            TreeItem CollectorRefs = null;

            switch (ItemType)
            {
                case (int)EgConfig.ItemType.Service:
                    for (i = 0; i < this.Children.Count; i++)
                    {
                        if (this.Children[i].ItemType == (int)EgConfig.ItemType.CollectorRefs)
                        {
                            CollectorRefs = this.Children[i];
                            break;
                        }
                    }
                    // CollectorRefsがない場合作成
                    if (CollectorRefs == null)
                    {
                        CollectorRefs = new TreeItem(Define.TreeView.ITEM_NAME_016, this, new ObservableCollection<TreeItem>(), this.Filename, (int)EgConfig.ItemType.CollectorRefs, i);
                        this.AddCollectorRefs(CollectorRefs);
                    }
                    break;
                case (int)EgConfig.ItemType.CollectorRefs:
                    CollectorRefs = this;
                    break;
                case (int)EgConfig.ItemType.CollectorRef:
                    if (this.Parent != null)
                    {
                        CollectorRefs = Parent.SearchCollectorRefs();
                    }
                    break;
                default:
                    break;
            }

            return CollectorRefs;
        }

        /// <summary>
        /// 現在のTreeItemからMailboxesを検索する
        /// </summary>
        /// <returns></returns>
        public TreeItem SearchMailboxes()
        {
            int i;
            TreeItem Mailboxes = null;

            switch (ItemType)
            {
                case (int)EgConfig.ItemType.Service:
                    for (i = 0; i < this.Children.Count; i++)
                    {
                        if (this.Children[i].ItemType == (int)EgConfig.ItemType.MailBoxes)
                        {
                            Mailboxes = this.Children[i];
                            break;
                        }
                    }
                    // Mailboxesがない場合作成
                    if (Mailboxes == null)
                    {
                        Mailboxes = new TreeItem(Define.TreeView.ITEM_NAME_018, this, new ObservableCollection<TreeItem>(), this.Filename, (int)EgConfig.ItemType.MailBoxes, i);
                        this.AddMailboxes(Mailboxes);
                    }
                    break;
                case (int)EgConfig.ItemType.MailBoxes:
                    Mailboxes = this;
                    break;
                case (int)EgConfig.ItemType.MailBox:
                    if (this.Parent != null)
                    {
                        Mailboxes = Parent.SearchMailboxes();
                    }
                    break;
                default:
                    break;
            }

            return Mailboxes;
        }

        /// <summary>
        /// ツリーアイテムの子要素にTagsを追加する
        /// </summary>
        public void AddTags(TreeItem Tags)
        {
            int index = -1, i = 0;

            ////////////////////////////////////////////////
            // Datasetsの前にTagsを追加する
            ////////////////////////////////////////////////
            foreach (TreeItem bro in Tags.Parent.Children)
            {
                if (bro.ItemType == (int)EgConfig.ItemType.Datasets) // Datasetsの要素Indexを検索
                {
                    index = i;
                    break;
                }
                i++;
            }
            // Datasetsが存在しているか
            if (index < 0)
            {
                if(0 <= this.Children.Count)
                {
                    this.Children.Insert(0, Tags);      // Datasetsを追加する
                }
                else
                {
                    this.Children.Add(Tags);                // 存在しない場合、リストの最後尾にDatasetsを追加する
                }
            }
            else
            {
                this.Children.Insert(index, Tags);      // 存在する場合、Servicesの前にDatasetsを追加する
            }
            this.UpdateIndexOfChildren();               // 子要素のIndex更新
        }

        /// <summary>
        /// ツリーアイテムの子要素にDatasetsを追加する
        /// </summary>
        public void AddDatasets(TreeItem Datasets)
        {
            int index = -1, i = 0;

            ////////////////////////////////////////////////
            // Servicesの前にDatasetsを追加する
            ////////////////////////////////////////////////
            foreach (TreeItem bro in Datasets.Parent.Children)
            {
                if(bro.ItemType == (int)EgConfig.ItemType.Services) // Servicesの要素Indexを検索
                {
                    index = i;
                    break;
                }
                i++;
            }
            // Servicesが存在しているか
            if(index < 0)
            {
                this.Children.Add(Datasets);            // 存在しない場合、リストの最後尾にDatasetsを追加する
            }
            else
            {
                this.Children.Insert(index, Datasets);  // 存在する場合、Servicesの前にDatasetsを追加する
            }
            this.UpdateIndexOfChildren();               // 子要素のIndex更新
        }

        /// <summary>
        /// ツリーアイテムの子要素にDatasetTagRefsを追加する
        /// </summary>
        /// <param name="DatasetTagRefs"></param>
        public void AddDatasetTagRefs(TreeItem DatasetTagRefs)
        {
            this.Children.Add(DatasetTagRefs);                // リストの最後尾にDatasetTagRefsを追加する
        }

        /// <summary>
        /// ツリーアイテムの子要素にServicesを追加する
        /// </summary>
        /// <param name="Services"></param>
        public void AddServices(TreeItem Services)
        {
            this.Children.Add(Services);                // リストの最後尾にServicesを追加する
        }

        /// <summary>
        /// ツリーアイテムの子要素にTagRefsを追加する
        /// </summary>
        /// <param name="TagRefs"></param>
        public void AddTagRefs(TreeItem TagRefs)
        {
            int index = -1, i = 0;

            ////////////////////////////////////////////////
            // TagRefsINの前にTagRefsを追加する
            ////////////////////////////////////////////////
            foreach (TreeItem bro in TagRefs.Parent.Children)
            {
                if (bro.ItemType == (int)EgConfig.ItemType.TagRefsIN) // TagRefsINの要素Indexを検索
                {
                    index = i;
                    break;
                }
                i++;
            }
            // TagRefsINが存在しているか
            if (index < 0)
            {
                if (0 <= this.Children.Count)
                {
                    this.Children.Insert(0, TagRefs);           // TagRefsを追加する
                }
                else
                {
                    this.Children.Add(TagRefs);                 // 存在しない場合、リストの最後尾にTagRefsを追加する
                }
            }
            else
            {
                this.Children.Insert(index, TagRefs);           // 存在する場合、TagRefsINの前にTagRefsを追加する
            }
            this.UpdateIndexOfChildren();                       // 子要素のIndex更新
        }

        /// <summary>
        /// ツリーアイテムの子要素にTagRefsINを追加する
        /// </summary>
        /// <param name="TagRefsIN"></param>
        public void AddTagRefsIN(TreeItem TagRefsIN)
        {
            int index = -1, i = 0;

            ////////////////////////////////////////////////
            // TagRefsOUTの前にTagRefsINを追加する
            ////////////////////////////////////////////////
            foreach (TreeItem bro in TagRefsIN.Parent.Children)
            {
                // Collectors/CollectorRefs/Mailboxesの要素Indexを検索
                if (bro.ItemType == (int)EgConfig.ItemType.TagRefsOUT ||
                    bro.ItemType == (int)EgConfig.ItemType.Collectors ||
                    bro.ItemType == (int)EgConfig.ItemType.CollectorRefs ||
                    bro.ItemType == (int)EgConfig.ItemType.MailBoxes
                    )
                {
                    index = i;
                    break;
                }
                i++;
            }
            // TagRefsOUTが存在しているか
            if (index < 0)
            {
                this.Children.Add(TagRefsIN);                       // 存在しない場合、リストの最後尾にTagRefsINを追加する

            }
            else
            {
                this.Children.Insert(index, TagRefsIN);             // 存在する場合、TagRefsOUTの前にTagRefsINを追加する
            }
            this.UpdateIndexOfChildren();                           // 子要素のIndex更新
        }

        /// <summary>
        /// ツリーアイテムの子要素にTagRefsOUTを追加する
        /// </summary>
        /// <param name="TagRefs"></param>
        public void AddTagRefsOUT(TreeItem TagRefsOUT)
        {
            int index = -1, i = 0;

            ////////////////////////////////////////////////
            // Collectorsの前にTagRefsOUTを追加する
            ////////////////////////////////////////////////
            foreach (TreeItem bro in TagRefsOUT.Parent.Children)
            {
                // Collectors/CollectorRefs/Mailboxesの要素Indexを検索
                if (bro.ItemType == (int)EgConfig.ItemType.Collectors ||
                    bro.ItemType == (int)EgConfig.ItemType.CollectorRefs ||
                    bro.ItemType == (int)EgConfig.ItemType.MailBoxes
                    )
                {
                    index = i;
                    break;
                }
                i++;
            }
            // Collectorsが存在しているか
            if (index < 0)
            {
                this.Children.Add(TagRefsOUT);                      // 存在しない場合、リストの最後尾にTagRefsOUTを追加する
                
            }
            else
            {
                this.Children.Insert(index, TagRefsOUT);            // 存在する場合、Collectorsの前にTagRefsOUTを追加する
            }
            this.UpdateIndexOfChildren();                           // 子要素のIndex更新
        }

        /// <summary>
        /// ツリーアイテムの子要素にCollectorsを追加する
        /// </summary>
        /// <param name="Collectors"></param>
        public void AddCollectors(TreeItem Collectors)
        {
            int index = -1, i = 0;

            ////////////////////////////////////////////////
            // CollectorRefsの前にCollectorsを追加する
            ////////////////////////////////////////////////
            foreach (TreeItem bro in Collectors.Parent.Children)
            {
                // CollectorRefs/Mailboxesの要素Indexを検索
                if (bro.ItemType == (int)EgConfig.ItemType.CollectorRefs ||
                    bro.ItemType == (int)EgConfig.ItemType.MailBoxes
                    )
                {
                    index = i;
                    break;
                }
                i++;
            }
            // CollectorRefsが存在しているか
            if (index < 0)
            {
                this.Children.Add(Collectors);                      // 存在しない場合、リストの最後尾にCollectorsを追加する

            }
            else
            {
                this.Children.Insert(index, Collectors);            // 存在する場合、CollectorRefsの前にCollectorsを追加する
            }
            this.UpdateIndexOfChildren();                           // 子要素のIndex更新
        }

        /// <summary>
        /// ツリーアイテムの子要素にCollectorRefsを追加する
        /// </summary>
        /// <param name="CollectorRefs"></param>
        public void AddCollectorRefs(TreeItem CollectorRefs)
        {
            int index = -1, i = 0;

            ////////////////////////////////////////////////
            // Mailboxesの前にCollectorRefsを追加する
            ////////////////////////////////////////////////
            foreach (TreeItem bro in CollectorRefs.Parent.Children)
            {
                // Mailboxesの要素Indexを検索
                if (bro.ItemType == (int)EgConfig.ItemType.MailBoxes)
                {
                    index = i;
                    break;
                }
                i++;
            }
            // CollectorRefsが存在しているか
            if (index < 0)
            {
                this.Children.Add(CollectorRefs);                   // 存在しない場合、リストの最後尾にCollectorRefsを追加する

            }
            else
            {
                this.Children.Insert(index, CollectorRefs);         // 存在する場合、Mailboxesの前にCollectorRefsを追加する
            }
            this.UpdateIndexOfChildren();                           // 子要素のIndex更新
        }

        /// <summary>
        /// ツリーアイテムの子要素にMailboxesを追加する
        /// </summary>
        /// <param name="Mailboxes"></param>
        public void AddMailboxes(TreeItem Mailboxes)
        {
            this.Children.Add(Mailboxes);                           // リストの最後尾にMailboxesを追加する
            this.UpdateIndexOfChildren();                           // 子要素のIndex更新
        }

        /// <summary>
        /// ツリーアイテムの子要素にTagを追加する
        /// </summary>
        public void AddTag(Tag tag)
        {
            TreeItem treeItem;

            // TagデータをTreeItemに変換
            treeItem = Tag2TreeItem(tag);

            // 現在のTreeItemのFilename・Index・親を子要素に設定
            treeItem.Filename = this.Filename;
            treeItem.Index = this.Children.Count();
            treeItem.Parent = this;

            Children.Add(treeItem);
        }

        /// <summary>
        /// ツリーアイテムの子要素にServiceを追加する
        /// </summary>
        public void AddService(Service service)
        {
            TreeItem treeItem;

            // ServiceデータをTreeItemに変換
            treeItem = Service2TreeItem(service);

            // 現在のTreeItemのFilename・Index・親を子要素に設定
            treeItem.Filename = this.Filename;
            treeItem.Index = this.Children.Count();
            treeItem.Parent = this;

            Children.Add(treeItem);
        }

        /// <summary>
        /// ツリーアイテムの子要素にMailboxを追加する
        /// </summary>
        /// <param name="mailBox"></param>
        public void AddMailbox(MailBox mailBox)
        {
            TreeItem treeItem;

            // MailboxデータをTreeItemに変換
            treeItem = Mailbox2TreeItem(mailBox);

            // 現在のTreeItemのFilename・Index・親を子要素に設定
            treeItem.Filename = this.Filename;
            treeItem.Index = this.Children.Count();
            treeItem.Parent = this;

            Children.Add(treeItem);
        }

        /// <summary>
        /// TagをTreeItemに変換
        /// </summary>
        /// <param name="tag">Tagデータ</param>
        /// <returns>TreeItem</returns>
        public TreeItem Tag2TreeItem(Tag tag)
        {
            TreeItem treeItem = new TreeItem();

            treeItem.Name = tag.Name;
            treeItem.ItemType = (int)EgConfig.ItemType.Tag;

            return treeItem;
        }

        /// <summary>
        /// ServiceをTreeItemに変換
        /// </summary>
        /// <param name="service"></param>
        /// <returns></returns>
        public TreeItem Service2TreeItem(Service service)
        {
            TreeItem treeItem = new TreeItem();

            treeItem.Name = service.Name;
            treeItem.ItemType = (int)EgConfig.ItemType.Service;

            return treeItem;
        }

        /// <summary>
        /// MailboxをTreeItemに変換
        /// </summary>
        /// <param name="mailBox"></param>
        /// <returns></returns>
        public TreeItem Mailbox2TreeItem(MailBox mailBox)
        {
            TreeItem treeItem = new TreeItem();

            treeItem.Name = mailBox.Name;
            treeItem.ItemType = (int)EgConfig.ItemType.MailBox;

            return treeItem;
        }

        /// <summary>
        /// TagsにTagデータを追加する
        /// </summary>
        /// <param name="tag">Tagデータ</param>
        public void DeleteTag(TreeItem tag)
        {
            TreeItem Tags;
            // Tagsを検索する
            Tags = SearchTags();

            // Tagを追加する
            //Tags.AddTag(tag);
        }

        /// <summary>
        /// TreeItemを削除する
        /// </summary>
        /// <returns></returns>
        public bool Remove()
        {
            bool Remove_OK = false;
            int index = 0;

            // 親のTreeItemを取得
            TreeItem parent = this.Parent;
            if(parent == null) { return false; }// 親がいない

            // 検索
            index = parent.IndexOf(this);
            if(index < 0) { return false; }     // 親に含まれていなかった

            parent.Children.RemoveAt(index);    // 削除

            // index更新
            parent.UpdateIndexOfChildren();

            return Remove_OK;
        }

        /// <summary>
        /// 子要素のIndexを検索する
        /// </summary>
        /// <param name="item">検索するTreeItem</param>
        /// <returns>子要素のIndex番号</returns>
        public int IndexOf(TreeItem item)
        {
            int index = -1, i = 0;

            foreach(TreeItem ch in this.Children)
            {
                // 一致した
                if(ch == item)
                {
                    index = i;
                    break;
                }

                i++;
            }

            return index;
        }

        /// <summary>
        /// 子要素のIndexを更新する
        /// </summary>
        public void UpdateIndexOfChildren()
        {
            int i = 0;

            foreach(TreeItem ch in this.Children)
            {
                ch.Index = i++;
            }
        }

        /// <summary>
        /// 子要素のファイル名を書き換える
        /// </summary>
        /// <param name="filename">ファイル名</param>
        public void UpdateFilename(string filename)
        {
            this.Filename = filename;

            foreach(TreeItem ch in this.Children)
            {
                ch.UpdateFilename(filename);
            }
        }

        /// <summary>
        /// TreeItemの項目を上に移動する
        /// </summary>
        /// <returns>true:移動成功, false:移動失敗</returns>
        public bool UpRow()
        {
            bool isDone = true;
            int index;

            if (this.Parent is TreeItem parentItem)
            {
                index = parentItem.Children.IndexOf(this);

                if (index == 0)  //項目Indexが最初の場合は何もしない
                {
                    return isDone;
                }

                switch (MainWindow.projMgmt.GetItemData(parentItem))
                {
                    case List<Tag> tags:
                        Tag tag = tags[index];
                        tags.RemoveAt(index);
                        tags.Insert(index - 1, tag);
                        break;
                    case List<Dataset> datasets:
                        Dataset dataset = datasets[index];
                        datasets.RemoveAt(index);
                        datasets.Insert(index - 1, dataset);
                        break;
                    case List<Service> services:
                        Service service = services[index];
                        services.RemoveAt(index);
                        services.Insert(index - 1, service);
                        break;
                    case List<TagRef> tagRefs:
                        TagRef tagRef = tagRefs[index];
                        tagRefs.RemoveAt(index);
                        tagRefs.Insert(index - 1, tagRef);
                        break;
                    case List<Collector> collectors:
                        Collector collector = collectors[index];
                        collectors.RemoveAt(index);
                        collectors.Insert(index - 1, collector);
                        break;
                    case List<CollectorRef> collectorRefs:
                        CollectorRef collectorRef = collectorRefs[index];
                        collectorRefs.RemoveAt(index);
                        collectorRefs.Insert(index - 1, collectorRef);
                        break;
                    case List<MailBox> mailBoxes:
                        MailBox mailBox = mailBoxes[index];
                        mailBoxes.RemoveAt(index);
                        mailBoxes.Insert(index - 1, mailBox);
                        break;
                    default:
                        isDone = false;
                        break;
                }

                parentItem.Children.Move(index, index - 1); // 現在の選択項目のItemのIndexをひとつ前に移動する
                parentItem.UpdateIndexOfChildren();         // 子要素のインデックス更新
                MainWindow.projMgmt.SetIsChanged2EciFile(true, this.Filename);

                // TreeItemの変更フラグをONにする
                foreach(TreeItem treeItem in MainWindow.treeItems)
                {
                    if(treeItem.Filename == this.Filename)
                    {
                        treeItem.IsChanged = true;
                    }
                }
            }
            else
            {
                isDone = false;
            }

            return isDone;
        }

        /// <summary>
        /// TreeItemの項目を下に移動する
        /// </summary>
        /// <returns>true:移動成功, false:移動失敗</returns>
        public bool DownRow()
        {
            bool isDone = true;
            int index;

            if (this.Parent is TreeItem parentItem)
            {
                index = parentItem.Children.IndexOf(this);

                if (parentItem.Children.Count - 1 <= index)  //項目Indexが最後の場合は何もしない
                {
                    return isDone;
                }

                switch (MainWindow.projMgmt.GetItemData(parentItem))
                {
                    case List<Tag> tags:
                        Tag tag = tags[index + 1];
                        tags.RemoveAt(index + 1);
                        tags.Insert(index, tag);
                        break;
                    case List<Dataset> datasets:
                        Dataset dataset = datasets[index + 1];
                        datasets.RemoveAt(index + 1);
                        datasets.Insert(index, dataset);
                        break;
                    case List<Service> services:
                        Service service = services[index + 1];
                        services.RemoveAt(index + 1);
                        services.Insert(index, service);
                        break;
                    case List<TagRef> tagRefs:
                        TagRef tagRef = tagRefs[index + 1];
                        tagRefs.RemoveAt(index + 1);
                        tagRefs.Insert(index, tagRef);
                        break;
                    case List<Collector> collectors:
                        Collector collector = collectors[index + 1];
                        collectors.RemoveAt(index + 1);
                        collectors.Insert(index, collector);
                        break;
                    case List<CollectorRef> collectorRefs:
                        CollectorRef collectorRef = collectorRefs[index + 1];
                        collectorRefs.RemoveAt(index + 1);
                        collectorRefs.Insert(index, collectorRef);
                        break;
                    case List<MailBox> mailBoxes:
                        MailBox mailBox = mailBoxes[index + 1];
                        mailBoxes.RemoveAt(index + 1);
                        mailBoxes.Insert(index, mailBox);
                        break;
                    default:
                        isDone = false;
                        break;
                }

                parentItem.Children.Move(index, index + 1); // 現在の選択項目のItemのIndexを次に移動する
                parentItem.UpdateIndexOfChildren();         // 子要素のインデックス更新
                MainWindow.projMgmt.SetIsChanged2EciFile(true, this.Filename);

                // TreeItemの変更フラグをONにする
                foreach (TreeItem treeItem in MainWindow.treeItems)
                {
                    if (treeItem.Filename == this.Filename)
                    {
                        treeItem.IsChanged = true;
                    }
                }
            }
            else
            {
                isDone = false;
            }

            return isDone;
        }
        
    }


}
