﻿////
/// Copyright(C) 2020 Micronet Co. All Rights Reserved.
//
using System;
using System.Collections.Concurrent;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Diagnostics;           // Debug.WriteLine出力用
using System.Runtime.InteropServices;
using RTedge;
using System.IO;
using System.Windows.Forms;
using System.Collections;
using RTedgeXmlObj;

namespace EgBoot
{
    /// <summary>
    /// タグトリガー登録情報
    /// </summary>
    public struct TagTriggerInfo
    {
        public UInt16 mgrNo;            // サービスの管理番号
        public string ServiceName;      // サービス名
        public List<Tuple<string, bool>> TagTriggers;   // タグトリガーリスト
        public TagTriggerInfo(int dummy)
        {
            mgrNo = 0xFFFF;
            ServiceName = "";
            TagTriggers = new List<Tuple<string, bool>>();
        }
    }
    /// <summary>
    /// 名前Set　 スレッドセーフ
    /// </summary>
    public class StrHashSet
    {
        //名前Set
        private HashSet<string> _Set;
        //コンストラクタ
        public StrHashSet() { _Set = new HashSet<string>(); }
        //
        private object _lock = new object(); //HashSetのロック
        /// <summary>
        /// 文字列の追加
        /// </summary>
        /// <param name="str"></param>
        /// <returns></returns>
        public bool Add(string str)
        {
            bool ret = false;
            //lock (_lock)
            //{
                ret = _Set.Add(str);
            //}
            return ret;
        }
        /// <summary>
        /// 要素の削除
        /// </summary>
        /// <param name="str"></param>
        /// <returns></returns>
        public bool Remove(string str)
        {
            bool ret = false;
            //lock (_lock)
            //{
                ret = _Set.Remove(str);
            //}
            return ret;
        }
        /// <summary>
        /// 要素が含まれているか否か
        /// </summary>
        /// <param name="str"></param>
        /// <returns></returns>
        public bool Contains(string str)
        {
            bool ret = false;
            //lock (_lock)
           // {
                ret = _Set.Contains(str);
            //}
            return ret;
        }
    }
    /// <summary>
    /// オブジェクトのリスト スレッドセーフ
    /// </summary>
    /// <typeparam name="T"></typeparam>
    public class ObjList<T> : IEnumerable<T>
    {
        private readonly List<T> _list = new List<T>();
        private readonly object _lock = new object();
        /// <summary>
        /// 要素を追加する（スレッドセーフ）
        /// </summary>
        public void Add(T item)
        {
            //lock (_lock)
            //{
                _list.Add(item);
            //}
        }

        /// <summary>
        /// 要素を削除する（スレッドセーフ）
        /// </summary>
        public void Remove(T item)
        {
            //lock (_lock)
            //{
                _list.Remove(item);
            //}
        }

        /// <summary>
        /// リストの要素数を取得（スレッドセーフ）
        /// </summary>
        public int Count
        {
            get
            {
                //lock (_lock)
                //{
                    return _list.Count;
                //}
            }
        }

        /// <summary>
        /// 現在のリストのスナップショットを取得（スレッドセーフ）
        /// </summary>
        public List<T> ToList()
        {
            //lock (_lock)
            //{
                return new List<T>(_list); // 内部リストのコピーを返す
            //}
        }

        /// <summary>
        /// 条件に一致する最初の要素を取得（スレッドセーフ）
        /// </summary>
        public T Find(Predicate<T> match)
        {
            //lock (_lock)
            //{
                return _list.Find(match);
            //}
        }

        /// <summary>
        /// 条件に一致するすべての要素を取得（スレッドセーフ）
        /// </summary>
        public List<T> FindAll(Predicate<T> match)
        {
            //lock (_lock)
            //{
                return _list.FindAll(match);
            //}
        }
        /// <summary>
        /// インデクサー（インデックス指定で要素を取得・設定）（スレッドセーフ）
        /// </summary>
        public T this[int index]
        {
            get
            {
                //lock (_lock)
                //{
                    if (index < 0 || index >= _list.Count)
                    {
                        throw new ArgumentOutOfRangeException(nameof(index), "インデックスが範囲外です。");
                    }
                    return _list[index];
                //}
            }
            set
            {
                //lock (_lock)
                //{
                    if (index < 0 || index >= _list.Count)
                    {
                        throw new ArgumentOutOfRangeException(nameof(index), "インデックスが範囲外です。");
                    }
                    _list[index] = value;
                //}
            }
        }
        /// <summary>
        /// foreach で安全に使うため、スナップショットを取得
        /// </summary>
        public IEnumerator<T> GetEnumerator()
        {
            List<T> snapshot;
            //lock (_lock)
            //{
                snapshot = new List<T>(_list); // 内部リストのコピーを作成
            //}
            return snapshot.GetEnumerator(); // コピーの Enumerator を返す
        }

        /// <summary>
        /// 非ジェネリック IEnumerable の実装（foreach 用）
        /// </summary>
        IEnumerator IEnumerable.GetEnumerator()
        {
            return GetEnumerator(); // ジェネリック版の GetEnumerator を利用
        }

    }
    /// <summary>
    /// サービス定義用xmlファイル変換クラス
    /// </summary>
    public class MnConvertDefFile
    {
        edge_API EGAPI = new edge_API();                                //edge APIのインスタンス
        edge_API.RTCDMEMINFO m_LogMBoxInfo;
        public static readonly string CORE_SERVICE_NAME = "EgBoot";    //固定名
        public static readonly string EGSYSDEF_FILE_NAME = "EgSysDef.xml";    //固定名
        public static readonly uint SERVICE_INIT_WAIT_TIME = 30;        //コンテナ起動完了待機時間[sec]
        public static readonly byte NOT_USE_TRIGGER = 0xFF;             //TagTrigger未使用
        public ObjList<TagVal> TagVals;                                 //Tagの値リスト
        public StrHashSet TagExistNames;                                //存在タグ管理用Set
        public StrHashSet BootingSet;                                   //起動中コンテナ管理用Set
        public ObjList<TagTriggerInfo> ListTagTriggers;                 //タグトリガー設定情報
        public LinkTag LinkTagTbl = new LinkTag();                      //リンクタグテーブル
        public object LinkTagDicLock = new object();                    //Dictionalyのロック
        public int _waitTime_ms = (int)(SERVICE_INIT_WAIT_TIME * 1000); //待ち時間デフォルト
        public int _sleepTime = 50;                                     //待ち周期
        public uint ServiceWaitFlag = 0;                                //コンテナ起動時に
        public static readonly byte SERVICE_STATE_2_DEPEND_INITEND = 16; //Depend用初期化完了状態の値]
        /// <summary>
        /// TagValの定義
        /// </summary>
        public class TagVal
        {
            public string Name;                         //Tag名
            public string Type;                         //Type値
            public string Value;                        //データ初期値
            public string InitGroup;                    //初期値反映対象グループ
            public TagVal() { Name = ""; Type = ""; Value = ""; InitGroup = ""; }
            public TagVal(string _name, string _type, string _value, string _initGroup) { Name = _name; Type= _type; Value = _value; InitGroup = _initGroup;  }
        }

        /// <summary>
        /// TagからTagValへ変換するヘルパークラス
        /// </summary>
        public static TagVal ToTagVal(RTedgeXmlObj.Tag tag)
        {
            return new TagVal { Name = tag.Name, Value = tag.Value };
        }        

        /// <summary>
        /// リンクタグ詳細情報
        /// 所属情報保持(所属サービス優先)
        /// </summary>
        public class LinkTagDetail
        {
            public string TagName;          //リンクタグ名
            public string ServiceName;      //所属サービス名
            /// <summary>
            /// コンストラクタ(引数なし)
            /// </summary>
            public LinkTagDetail()
            {
                TagName = string.Empty;
                ServiceName = string.Empty;
            }
            /// <summary>
            /// コンストラクタ(引数あり)
            /// </summary>
            /// <param name="stTagName"></param>
            /// <param name="stServiceName"></param>
            public LinkTagDetail(string stTagName, string stServiceName)
            {
                TagName = stTagName;
                ServiceName = stServiceName;
            }
        }
        /// <summary>
        /// リンクタグ保持クラス
        /// </summary>
        public class LinkTag
        {         
            public ConcurrentDictionary<ulong, List<LinkTagDetail>> dic;// スレッドセーフな辞書
            private readonly object _lock = new object(); // List操作用のロック
            public LinkTag() { dic = new ConcurrentDictionary<ulong, List<LinkTagDetail>>(); }
            /// <summary>
            ///Tag名を取得 (後勝ちにするため後ろから検索) 
            /// </summary>
            /// <param name="hex"></param>
            /// <param name="stServiceName"></param>
            /// <returns></returns>
            public string GetLinkTagName(ulong hex, string stServiceName)
            {
                try
                {
                    string emptyServiceTagName = "";
                    if (dic.TryGetValue(hex, out var tagList)) //タグ重複登録時のHash値と一致する場合取得
                    {
                        //lock (_lock) // Listのスレッド安全を確保
                        //{
                            var reverseList = tagList.ToList(); // コピーを作成
                            reverseList.Reverse();              //後勝ちを採用したいため逆順にする

                            //採用するタグを検索
                            foreach (var item in reverseList)
                            {
                                if (emptyServiceTagName.Length == 0 && item.ServiceName.Length == 0)
                                    emptyServiceTagName = item.TagName; // グローバルタグの保存

                                if (item.ServiceName == stServiceName)
                                    return item.TagName; // サービス名が一致した場合
                            }
                        //}
                    }
                    return emptyServiceTagName;//グローバルなリンクタグを返す
                }
                catch (Exception ex)
                {
                    string mess = System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message;
                    Debug.WriteLine(mess);
                    edge_FW._PrintSysLog(CORE_SERVICE_NAME, "FER", "{0}", mess);
                    return "";
                }
            }
            /// <summary>
            /// リンクタグを追加（スレッドセーフ）
            /// </summary>
            public void AddLinkTag(ulong hex, LinkTagDetail detail)
            {
                //lock (_lock) // List操作のためロック
                //{
                    if (!dic.ContainsKey(hex))
                    {
                        dic[hex] = new List<LinkTagDetail>();
                    }
                    dic[hex].Add(detail);
                //}
            }

            /// <summary>
            /// リンクタグを削除（スレッドセーフ）
            /// </summary>
            public bool RemoveLinkTag(ulong hex, LinkTagDetail detail)
            {
                //lock (_lock)
                //{
                    if (dic.TryGetValue(hex, out var tagList))
                    {
                        return tagList.Remove(detail);
                    }
                //}
                return false;
            }

            /// <summary>
            /// 指定したキーを削除（スレッドセーフ）
            /// </summary>
            public bool RemoveKey(ulong hex)
            {
                //lock (_lock)
                //{
                    return dic.TryRemove(hex, out _);
                //}
            }
        }

        /// <summary>
        /// 起動待機中コンテナ管理クラス
        /// </summary>
        public class InitWait
        {
            /// <summary>
            /// 起動待機中のコンテナ名
            /// </summary>
            public string Name;

            /// <summary>
            /// 依存するコンテナ名
            /// </summary>
            public List<string> Dependents;

            /// <summary>
            /// コンストラクタ
            /// </summary>
            public InitWait()
            {
                Name = string.Empty;
                Dependents = new List<string>();
            }
            /// <summary>
            /// コンストラクタ
            /// </summary>
            /// <param name="serv"></param>
            public InitWait(RTedgeXmlObj.Service serv)
            {
                Name = serv.Name;
                Dependents = new List<string>();
                Dependents.AddRange(serv.Dependent.Split(';').ToList());
            }
        }

        /// <summary>
        /// コンストラクタ
        /// </summary>
        public MnConvertDefFile(ref edge_API egEGAPI, ref edge_API.RTCDMEMINFO LogMBoxInfo)
        {
            EGAPI = egEGAPI;             //edge APIのインスタンス
            m_LogMBoxInfo = LogMBoxInfo;

            TagVals = new ObjList<TagVal>();
            TagExistNames = new StrHashSet();
            BootingSet = new StrHashSet();
            ListTagTriggers = new ObjList<TagTriggerInfo>();
        }

        object _Lock_TagAdd = new object();
        /// <summary>
        /// EgTagの追加
        /// </summary>
        /// <param name="Tag"></param>
        /// <returns></returns>
        public int AttachTag(string stSendServiceName, RTedgeXmlObj.Tag Tag)
        {
            int rtn = -1;
            int rtnProp = -1;
            try
            {
                // EgTagCreate
                object regName = string.Empty;
                ushort isLink = 0;
                // TypeによってAPIを代える
                switch (CUtility.fStringToEnum<EGDEFINE.egTagDataType>(Tag.Type))
                {
                    case EGDEFINE.egTagDataType.String:                 // 文字列
                    case EGDEFINE.egTagDataType.Segment:              // バイト配列
                        rtn = EGAPI.EgTagCreateSegment(Tag.Name, (ushort)CUtility.fStringToEnum<EGDEFINE.egTagDataType>(Tag.Type), CUtility.fStringToUshort(Tag.Size), Tag.Address, Tag.Comment, ref regName, ref isLink);
                        break;
                    case EGDEFINE.egTagDataType.FIFO:
                        rtn = EGAPI.EgTagCreateFIFO(Tag.Name, CUtility.fStringToUshort(Tag.Size), CUtility.fStringToUshort(Tag.RecordCount), false/* 準備工事 */, Tag.Comment, ref regName, ref isLink);
                        break;
                    default:                                                    // それ以外
                        rtn = EGAPI.EgTagCreateSp(Tag.Name, (ushort)CUtility.fStringToEnum<EGDEFINE.egTagDataType>(Tag.Type), Tag.Address, Tag.Comment, ref regName, ref isLink);
                        break;
                }
                if (rtn == 0)
                {
                    //Valueの定義がある場合はValueを設定する
                    if (Tag.Value != null && Tag.Value.Length > 0)
                    {
                        //型に応じて値の変更
                        object objValue = egCommonFunc.GetTagAnyDataFromString(Tag.Value, CUtility.fStringToEnum<EGDEFINE.egTagDataType>(Tag.Type));
                        if (objValue != null)
                        {
                            int ret = 0;
                            // TypeによってAPIを代える
                            switch (CUtility.fStringToEnum<EGDEFINE.egTagDataType>(Tag.Type))
                            {
                                //初期値書き込み
                                case EGDEFINE.egTagDataType.String:                 // 文字列
                                    ret = EGAPI.EgTagWriteString(Tag.Name, (string)objValue);
                                    break;
                                default:                                                    // それ以外
                                    ret = EGAPI.EgTagWrite(Tag.Name, objValue);
                                    break;
                            }
                            if (ret != 0)
                                edge_FW._PrintSysLog(CORE_SERVICE_NAME, "ERR", "EgTagCreate -> EgTagWrite Error. Name : {0}, state : {1}", Tag.Name, ret.ToString("X8"));

                        }
                        else //正しく変換できない場合はエラー
                            edge_FW._PrintSysLog(CORE_SERVICE_NAME, "ERR", "EgTagCreateSp -> EgTagWrite Value is Invalid Data. Name : {0}, Value : {1}", Tag.Name, Tag.Value);
                    }
                    //Hash値を取得
                    object rtnObj = null;
                    rtnProp = EGAPI.EgTagGetProperty(Tag.Name, "Hash", ref rtnObj);
                    if (rtnProp == 0)
                    {
                        edge_API.HASHCODE hcode = (edge_API.HASHCODE)rtnObj;
                        ulong hex = hcode.code;

                        // タグ名重複していた場合の処理(リンクタグ対応)
                        if (isLink != 0)
                        {
                            // 取得したHash値と別名登録したタグ名(リンクタグ名)を保持
                            LinkTagTbl.AddLinkTag(hex, new LinkTagDetail((string)regName, stSendServiceName));
                        }
                    }
                    else
                        edge_FW._PrintSysLog(CORE_SERVICE_NAME, "ERR", "EgTagCreateSp -> EgTagGetProperty ERROR. Name : {0}, State : {1}", Tag.Name, rtn.ToString("X8"));
                }
                else
                    edge_FW._PrintSysLog(CORE_SERVICE_NAME, "ERR", "EgTagCreateSp ERROR. Name : {0}, State : {1}", Tag.Name, rtn.ToString("X8"));

                return rtn;
            }
            catch (Exception ex)
            {
                string mess = System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message;
                Debug.WriteLine(mess);
                edge_FW._PrintSysLog(CORE_SERVICE_NAME, "FER", "{0}", mess);
                return rtn;
            }
        }
        /// <summary>
        /// EgTagの追加
        /// </summary>
        /// <param name="Tags"></param>
        /// <returns></returns>
        public int AttachTags(string stSendServiceName, List<RTedgeXmlObj.Tag> Tags)
        {
            int rtn = 0;
            int cnt = 0;    // 登録処理数カウンタ
            try
            {
                //EgTagの定義だけループ
                foreach (RTedgeXmlObj.Tag t in Tags)
                {
                    //TagTableへ登録
                    rtn = AttachTag(stSendServiceName, t);
                    if(rtn != 0)
                    {
                        //中でログ
                        Debug.WriteLine("ERROR : AttachTags = " + rtn.ToString());
                        if ((uint)rtn == EGDEFINE.EDGE_TTABLE_NOEMPTY)
                        {
                            // タグテーブルの空きがない場合のエラーは処理を止める
                            break;
                        }
                        else
                        {
                            // その他は処理継続
                            rtn = 0;
                        }
                    }
                    cnt++;
                    if (cnt % 1000 == 0)
                    {
                        System.Threading.Thread.Sleep(100); //ログ欠損抑止のためWait
                    }
                }
                System.Threading.Thread.Sleep(100); //ログ欠損抑止のためWait
                return rtn;
            }
            catch (Exception ex)
            {
                string mess = System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message;
                Debug.WriteLine(mess);
                edge_FW._PrintSysLog(CORE_SERVICE_NAME, "FER", "{0}", mess);
                return -1;
            }
        }
        /// <summary>
        /// DatasetにEgTag情報を追加
        /// </summary>
        /// <param name="DSetName"></param>
        /// <param name="TagRefs"></param>
        /// <returns></returns>
        public int AttachDSetTagRefs(string stSendServiceName, string DSetName, List<RTedgeXmlObj.TagRef> TagRefs)
        {
            int rtn = 0;
            try
            {
                string tagName = string.Empty;
                object rtnObj = null;

                //xml EgDataset内 EgTagRefの数だけ処理
                foreach (RTedgeXmlObj.TagRef t in TagRefs)
                {
                    tagName = t.Name;
                    // Hash値取得
                    rtn = EGAPI.EgTagGetProperty(tagName, "Hash", ref rtnObj);
                    if (rtn == 0)
                    {
                        edge_API.HASHCODE hcode = (edge_API.HASHCODE)rtnObj;
                        //lock (LinkTagDicLock)
                        //{
                            if (LinkTagTbl.dic.ContainsKey(hcode.code)) // タグ重複登録時のHash値と一致する場合
                            {
                                tagName = LinkTagTbl.GetLinkTagName(hcode.code, stSendServiceName);   // 保持していたリンクタグ名を取得
                                if (tagName.Length == 0)//からの場合は無いはずだが一応からの場合は元に戻す
                                {
                                    tagName = t.Name;
                                    edge_FW._PrintSysLog(CORE_SERVICE_NAME, "ERR", "EgDatasetAddTag GetLinkTagName ERROR. DatasetName : {0}, TagName : {1}", DSetName, t.Name);
                                }
                            }
                        //}
                    }

                    //EgDatasetに追加
                    rtn = EGAPI.EgDatasetAddTag(DSetName, tagName);
                    if(rtn != 0)
                        edge_FW._PrintSysLog(CORE_SERVICE_NAME, "ERR", "EgDatasetAddTag ERROR. DatasetName : {0}, TagName : {1}, LinkTag : {2}, State : 0x{3}", DSetName, t.Name, tagName, rtn.ToString("X8"));
                    else
                        edge_FW._PrintSysLog(CORE_SERVICE_NAME, "OPE", "EgDatasetAddTag SUCCESS. DatasetName : {0}, TagName : {1}, LinkTag : {2}, State : 0x{3}", DSetName, t.Name, tagName, rtn.ToString("X8"));
                }
                return rtn;
            }
            catch (Exception ex)
            {
                string mess = System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message;
                Debug.WriteLine(mess);
                edge_FW._PrintSysLog(CORE_SERVICE_NAME, "FER", "{0}", mess);
                return -1;
            }
        }
        /// <summary>
        /// Datasetの追加
        /// </summary>
        /// <param name="DSet"></param>
        /// <returns></returns>
        public int AttachDataset(string stSendServiceName, RTedgeXmlObj.Dataset DSet)
        {
            int rtn = 0;
            try
            {
                // Dataset
                rtn = EGAPI.EgDatasetCreate(DSet.Name);
                if (rtn != 0)
                {
                    edge_FW._PrintSysLog(CORE_SERVICE_NAME, "ERR", "EgDatasetCreate ERROR. Name : {0}, State : 0x{1}", DSet.Name, rtn.ToString("X8"));
                    return rtn;
                }

                // tagRefの登録
                rtn = AttachDSetTagRefs(stSendServiceName, DSet.Name, DSet.TagRefs);
                return rtn;
            }
            catch (Exception ex)
            {
                string mess = System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message;
                Debug.WriteLine(mess);
                edge_FW._PrintSysLog(CORE_SERVICE_NAME, "FER", "{0}", mess);
                return -1;
            }
        }
        /// <summary>
        /// Datasetの追加
        /// </summary>
        /// <param name="DSets"></param>
        /// <returns></returns>
        public void AttachDatasets(string stSendServiceName, List<RTedgeXmlObj.Dataset> DSets)
        {
            try
            {
                int cnt = 0;    // 登録処理数カウンタ

                //xml Datasetの数だけ追加
                foreach (RTedgeXmlObj.Dataset d in DSets)
                {
                    //DatasetTableへ登録
                    AttachDataset(stSendServiceName, d);

                    cnt++;
                    if (cnt % 10 == 0)
                    {
                        System.Threading.Thread.Sleep(100); //ログ欠損抑止のためWait
                    }
                }
            }
            catch (Exception ex)
            {
                string mess = System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message;
                Debug.WriteLine(mess);
                edge_FW._PrintSysLog(CORE_SERVICE_NAME, "FER", "{0}", mess);
            }
        }
        /// <summary>
        /// コレクタの追加
        /// </summary>
        /// <param name="stSendServiceName"></param>
        /// <param name="Collectors"></param>
        /// <returns></returns>
        public int AttachCollectors(string stSendServiceName, List<RTedgeXmlObj.Collector> Collectors)
        {
            int rtn = 0;
            try
            {
                //コレクタの作成
                foreach (RTedgeXmlObj.Collector c in Collectors)
                {
                    uint interval = 100; //収集周期
                    if (uint.TryParse(c.Interval, out interval) == false)
                        interval = 100;

                    int priority = 150;
                    if (c.Priority != null && c.Priority.Length > 0)
                        priority = chgStrPriority(c.Priority);

                    byte newPriority = 150;
                    if (priority < 256)   //255より大きい数字はデフォルト150とする
                        newPriority = (byte)priority;

                    uint colSize = 2048; //メールボックステーブル長
                    if (uint.TryParse(c.ColSize, out colSize) == false)
                        colSize = 2048;

                    uint rowSize = 1024; //メールボックステーブル数
                    if (uint.TryParse(c.RowSize, out rowSize) == false)
                        rowSize = 1024;

                    string stDataset = ""; //データセット名
                    if (c.DatasetName != null)
                        stDataset = c.DatasetName;

                    // コレクタテーブルにコレクタを追加
                    rtn = EGAPI.EgCollectorCreate(c.Name, stSendServiceName, interval, newPriority, colSize, rowSize, stDataset);
                    if (rtn == 0)
                    {
                        byte[] byArgs = Encoding.ASCII.GetBytes(c.Name); 
                        // コレクタを実行するサービスにメッセージ送信
                        rtn = EGAPI.EgMailboxSendSC(stSendServiceName, CORE_SERVICE_NAME, (int)EGDEFINE.EGMSG.EM_COLLECTOR_CREATE, byArgs);
                        if (rtn != 0)
                        {
                            Debug.WriteLine("ERROR : AttachCollectors EgMailboxSend = 0x" + rtn.ToString("X8"));
                            edge_FW._PrintSysLog(CORE_SERVICE_NAME, "ERR", "EgMailboxSend ERROR. Name : {0}, State : 0x{1}", stSendServiceName, rtn.ToString("X8"));
                        }
                    }
                    else
                    {
                        edge_FW._PrintSysLog(CORE_SERVICE_NAME, "ERR", "EgCollectorCreate ERROR. Name : {0}, State : 0x{1}", c.Name, rtn.ToString("X8"));
                        Debug.WriteLine("ERROR : AttachCollectors EgCollectorCreate = 0x" + rtn.ToString("X8"));
                    }
                }
                return rtn;
            }
            catch (Exception ex)
            {
                string mess = System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message;
                Debug.WriteLine(mess);
                edge_FW._PrintSysLog(CORE_SERVICE_NAME, "FER", "{0}", mess);
                return -1;
            }
        }
        /// <summary>
        /// サービス内参照用CollectorRefの登録
        /// </summary>
        /// <param name="stSendServiceName"></param>
        /// <param name="clctRefs"></param>
        /// <returns></returns>
        public int AttachServiceClctRefs(string stSendServiceName, List<RTedgeXmlObj.CollectorRef> clctRefs)
        {
            int rtn = 0;
            try
            {
                //サービス内のCollectorRefの追加
                foreach (RTedgeXmlObj.CollectorRef clctRef in clctRefs)
                {
                    //AddCollectorRef Service Message
                    byte[] byArgs = Encoding.ASCII.GetBytes(clctRef.Name);
                    // コレクタを実行するサービスにメッセージ送信
                    rtn = EGAPI.EgMailboxSendSC(stSendServiceName, CORE_SERVICE_NAME, (int)EGDEFINE.EGMSG.EM_SERVICE_ADD_CLCTREF, byArgs);
                    if (rtn != 0)
                    {
                        edge_FW._PrintSysLog(CORE_SERVICE_NAME, "ERR", "AttachServiceClctRefs ERROR. Name : {0}, State : 0x{1}", clctRef.Name, rtn.ToString("X8"));
                        break;
                    }
                }
                return rtn;
            }
            catch (Exception ex)
            {
                string mess = System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message;
                Debug.WriteLine(mess);
                edge_FW._PrintSysLog(CORE_SERVICE_NAME, "FER", "{0}", mess);
                return -1;
            }
        }
        
        /// <summary>
        /// サービス内参照用TagRef登録
        /// </summary>
        /// <param name="stSendServiceName"></param>
        /// <param name="tagRefs"></param>
        /// <returns></returns>
        public int AttachServiceTagRefs(string stSendServiceName, List<RTedgeXmlObj.TagRef> tagRefs)
        {
            int rtn = 0;
            try
            {
                string tagName = string.Empty;
                object rtnObj = null;
                int cnt = 0;    // 登録処理数カウンタ

                //サービス内のTagRefの追加
                foreach (RTedgeXmlObj.TagRef tagRef in tagRefs)
                {
                    tagName = tagRef.Name;
                    // Hash値取得
                    rtn = EGAPI.EgTagGetProperty(tagName, "Hash", ref rtnObj);
                    if (rtn == 0)
                    {
                        edge_API.HASHCODE hcode = (edge_API.HASHCODE)rtnObj;
                        tagName = LinkTagTbl.GetLinkTagName(hcode.code, stSendServiceName);   // 保持していたリンクタグ名を取得
                        if (tagName.Length == 0)//からの場合は元に戻す
                        {
                            tagName = tagRef.Name;
                            edge_FW._PrintSysLog(CORE_SERVICE_NAME, "ERR", "AttachServiceTagRefs GetLinkTagName ERROR. TagName : {1}", tagRef.Name);
                        }
                    }

                    //AddTagRef Service Message
                    byte[] byArgs = Encoding.ASCII.GetBytes(tagName);   //string をbyte配列へ変換
                    edge_FW._PrintSysLog(CORE_SERVICE_NAME, "DBG", "AttachServiceTagRefs. ServeName : {0}, TagRefName : {1}", stSendServiceName, tagName);
                    rtn = EGAPI.EgMailboxSendSC(stSendServiceName, CORE_SERVICE_NAME, (int)EGDEFINE.EGMSG.EM_SERVICE_ADD_TAG, byArgs);
                    if (rtn != 0)
                    {
                        edge_FW._PrintSysLog(CORE_SERVICE_NAME, "ERR", "AttachServiceTagRefs ERROR. Name : {0}, State : 0x{1}", tagName, rtn.ToString("X8"));
                        break;
                    }
                    cnt++;  // 登録処理数カウンタ更新
                    if (cnt % 100 == 0)
                    {
                        System.Threading.Thread.Sleep(100); //ログ欠損抑止のためWait
                    }
                }
                return rtn;
            }
            catch (Exception ex)
            {
                string mess = System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message;
                Debug.WriteLine(mess);
                edge_FW._PrintSysLog(CORE_SERVICE_NAME, "FER", "{0}", mess);
                return -1;
            }
        }
        /// <summary>
        /// サービス内参照用TagRef_OUT登録
        /// </summary>
        /// <param name="stSendServiceName"></param>
        /// <param name="tagRefs"></param>
        /// <returns></returns>
        public int AttachServiceTagRefs_OUT(string stSendServiceName, List<RTedgeXmlObj.TagRef> tagRefs)
        {
            int rtn = 0;
            try
            {
                string tagName = string.Empty;
                object rtnObj = null;
                int cnt = 0;    // 登録処理数カウンタ

                //サービス内のTagRefの追加
                foreach (RTedgeXmlObj.TagRef tagRef in tagRefs)
                {
                    tagName = tagRef.Name;
                    // Hash値取得
                    rtn = EGAPI.EgTagGetProperty(tagName, "Hash", ref rtnObj);
                    if (rtn == 0)
                    {
                        edge_API.HASHCODE hcode = (edge_API.HASHCODE)rtnObj;
                        tagName = LinkTagTbl.GetLinkTagName(hcode.code, stSendServiceName);   // 保持していたリンクタグ名を取得
                        if (tagName.Length == 0)//からの場合は元に戻す
                        {
                            tagName = tagRef.Name;
                            edge_FW._PrintSysLog(CORE_SERVICE_NAME, "ERR", "AttachServiceTagRefs_OUT GetLinkTagName ERROR. TagName : {1}", tagRef.Name);
                        }
                    }

                    //AddTagRef Service Message
                    byte[] byArgs = Encoding.ASCII.GetBytes(tagName);   //string をbyte配列へ変換
                    rtn = EGAPI.EgMailboxSendSC(stSendServiceName, CORE_SERVICE_NAME, (int)EGDEFINE.EGMSG.EM_SERVICE_ADD_OUT_TAG, byArgs);
                    if (rtn != 0)
                    {
                        edge_FW._PrintSysLog(CORE_SERVICE_NAME, "ERR", "AttachServiceTagRefs_OUT ERROR. Name : {0}, State : 0x{1}", tagName, rtn.ToString("X8"));
                        break;
                    }
                    cnt++;  // 登録処理数カウンタ更新
                    if (cnt % 100 == 0)
                    {
                        System.Threading.Thread.Sleep(100); //ログ欠損抑止のためWait
                    }
                }
                return rtn;
            }
            catch (Exception ex)
            {
                string mess = System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message;
                Debug.WriteLine(mess);
                edge_FW._PrintSysLog(CORE_SERVICE_NAME, "FER", "{0}", mess);
                return -1;
            }
        }
        /// <summary>
        /// サービス内参照用TagRef_IN登録
        /// </summary>
        /// <param name="stSendServiceName"></param>
        /// <param name="tagRefs"></param>
        /// <returns></returns>
        public int AttachServiceTagRefs_IN(string stSendServiceName, List<RTedgeXmlObj.TagRef> tagRefs)
        {
            int rtn = 0;
            try
            {
                string tagName = string.Empty;
                object rtnObj = null;
                int cnt = 0;    // 登録処理数カウンタ

                //サービス内のTagRefの追加
                foreach (RTedgeXmlObj.TagRef tagRef in tagRefs)
                {
                    tagName = tagRef.Name;
                    // Hash値取得
                    rtn = EGAPI.EgTagGetProperty(tagName, "Hash", ref rtnObj);
                    if (rtn == 0)
                    {
                        edge_API.HASHCODE hcode = (edge_API.HASHCODE)rtnObj;
                        tagName = LinkTagTbl.GetLinkTagName(hcode.code, stSendServiceName);   // 保持していたリンクタグ名を取得
                        if (tagName.Length == 0)//からの場合は元に戻す
                        {
                            tagName = tagRef.Name;
                            edge_FW._PrintSysLog(CORE_SERVICE_NAME, "ERR", "AttachServiceTagRefs_IN GetLinkTagName ERROR. TagName : {1}", tagRef.Name);
                        }
                    }
                    //AddTagRef Service Message
                    byte[] byArgs = Encoding.ASCII.GetBytes(tagName);   //string をbyte配列へ変換
                    rtn = EGAPI.EgMailboxSendSC(stSendServiceName, CORE_SERVICE_NAME, (int)EGDEFINE.EGMSG.EM_SERVICE_ADD_IN_TAG, byArgs);
                    if (rtn != 0)
                    {
                        edge_FW._PrintSysLog(CORE_SERVICE_NAME, "ERR", "AttachServiceTagRefs_IN ERROR. Name : {0}, Mbox{1} : State : 0x{2}", tagName, stSendServiceName, rtn.ToString("X8"));
                        break;
                    }
                    cnt++;  // 登録処理数カウンタ更新
                    if (cnt % 100 == 0)
                    {
                        System.Threading.Thread.Sleep(100); //ログ欠損抑止のためWait
                    }
                }
                return rtn;
            }
            catch (Exception ex)
            {
                string mess = System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message;
                Debug.WriteLine(mess);
                edge_FW._PrintSysLog(CORE_SERVICE_NAME, "FER", "{0}", mess);
                return -1;
            }
        }
        /// <summary>
        /// サービス内参照用メールボックスの登録
        /// </summary>
        /// <param name="stSendServiceName"></param>
        /// <param name="mailBoxes"></param>
        /// <returns></returns>
        public int AttachServiceMailBoxes(string stSendServiceName, List<RTedgeXmlObj.MailBox> mailBoxes)
        {
            int rtn = 0;
            try
            {
                //サービス内のMailBoxの追加
                foreach (RTedgeXmlObj.MailBox mailBox in mailBoxes)
                {
                    //AddTagRef Service Message
                    EGDEFINE.EGWMAILBOXINFO ST_MBOXINFO = new EGDEFINE.EGWMAILBOXINFO();
                    ST_MBOXINFO.MailBoxName = mailBox.Name;
                    ST_MBOXINFO.dwColSize = CUtility.fStringToUint(mailBox.ColSize);
                    ST_MBOXINFO.dwRowSize = CUtility.fStringToUint(mailBox.RowSize);
                    byte[] byArgs = new byte[Marshal.SizeOf(typeof(EGDEFINE.EGWMAILBOXINFO))];
                    rtn = egCommonFunc.fnObjToBy(ST_MBOXINFO, ref byArgs);
                    if (rtn != 0)
                    {
                        edge_FW._PrintSysLog(CORE_SERVICE_NAME, "ERR", "AttachServiceMailBoxes ARG ERROR. Name : {0}, State : 0x{1}", mailBox.Name, rtn.ToString("X8"));
                        continue;
                    }
                    rtn = EGAPI.EgMailboxSendSC(stSendServiceName, CORE_SERVICE_NAME, (int)EGDEFINE.EGMSG.EM_SERVICE_ADD_MBOX, byArgs);
                    if (rtn != 0)
                    {
                        edge_FW._PrintSysLog(CORE_SERVICE_NAME, "ERR", "AttachServiceMailBoxes ERROR. Name : {0}, State : 0x{1}", mailBox.Name, rtn.ToString("X8"));
                        continue;
                    }
                }
                return rtn;
            }
            catch (Exception ex)
            {
                string mess = System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message;
                Debug.WriteLine(mess);
                edge_FW._PrintSysLog(CORE_SERVICE_NAME, "FER", "{0}", mess);
                return -1;
            }
        }
        /// <summary>
        /// サービス内のタグトリガー情報をリストに登録する
        /// ※ タグトリガーは後で一機に登録を行う
        /// </summary>
        /// <param name="stSendServiceName">サービス名</param>
        /// <param name="tagTriggers">タグトリガー設定(xmlより)</param>
        /// <param name="tagTrigInfo">返却するタグトリガ―情報</param>
        /// <returns>0:エラーなし</returns>
        public int EntryTagTriggerInfos(string stSendServiceName, List<RTedgeXmlObj.TagTrigger> tagTriggers, ref TagTriggerInfo tagTrigInfo)
        {
            int rtn = 0;
            try
            {
                string tagName = string.Empty;
                bool flgAlready = false;    // TagTrigger重複フラグ
                int cntRead = 0;    // 読み込んだTagTriggerカウンタ

                tagTrigInfo.ServiceName = stSendServiceName;

                //サービス内のTagTriggerの追加
                foreach (RTedgeXmlObj.TagTrigger tagTrigger in tagTriggers)
                {
                    tagName = tagTrigger.Name;
                    // TagTriggerの重複チェック
                    for (int i = 0; i < cntRead; i++)    // 登録済みTagTrigger分だけ繰り返す
                    {
                        if (tagName.Equals(tagTriggers[i].Name)) // Tag名が同じ場合
                        {
                            flgAlready = true;                  // 重複フラグをON
                            break;
                        }
                    }
                    if (flgAlready)                     // 重複していた場合
                    {
                        flgAlready = false;             // 重複フラグをOFF
                        continue;                       // 次のTagTrigger読み込みへ
                    }

                    tagTrigInfo.TagTriggers.Add(Tuple.Create(tagName, false));  // 対象のタグ名を登録
                }
                return rtn;                
            }
            catch (Exception ex)
            {
                string mess = System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message;
                Debug.WriteLine(mess);
                edge_FW._PrintSysLog(CORE_SERVICE_NAME, "FER", "{0}", mess);
                return -1;
            }
        }
        /// <summary>
        /// サービス内参照用TagTrigger登録
        /// </summary>
        /// <param name="stSendServiceName"></param>
        /// <param name="tagTriggers"></param>
        /// <returns></returns>
        public int AttachServiceTagTrigger(bool terminal)
        {
            int rtn = 0;
            try
            {
                string tagName = string.Empty;
                object rtnObj = null;
                bool flgTagTrigger = false; // TagTrigger通知フラグ
                int cnt = 0;    // 登録処理数カウンタ

                //サービスごとのタグトリガー情報を取得して登録
                for (int i = 0; i < ListTagTriggers.Count; i++)
                {
                    // タグトリガーを登録したいサービスが既に管理対象か？
                    TagTriggerInfo tmp = ListTagTriggers[i];    // i番目のタグトリガー情報を作業用変数にセット
                    for (int j = 0;j < tmp.TagTriggers.Count;j++)
                    {
                        if (!tmp.TagTriggers[j].Item2)           // 未登録のタグの場合
                        {
                            tagName = tmp.TagTriggers[j].Item1;
                            rtn = EGAPI.EgTagGetProperty(tagName, "Hash", ref rtnObj);
                            if (rtn != 0)   // Tag名でHash値が存在しない(=未登録のTag)
                            {
                                if(terminal) // 最後のタグトリガー登録処理の場合
                                {
                                    // タグトリガーに登録できなかったことをログに出力
                                    edge_FW._PrintSysLog(CORE_SERVICE_NAME, "ERR", "AttachServiceTagTriggers ERROR. Tag is not exist. ServiceName : {0}, TagName : {1}", tmp.ServiceName, tagName);
                                }
                                continue;
                            }

                            //AddTagTriggers Service Message
                            byte[] byArgs = Encoding.ASCII.GetBytes(tagName);   //string をbyte配列へ変換
                            edge_FW._PrintSysLog(CORE_SERVICE_NAME, "DBG", "AttachServiceTagTriggers. ServeName : {0}, TagTriggerName : {1}", tmp.ServiceName, tagName);
                            rtn = EGAPI.EgMailboxSendSC(tmp.ServiceName, CORE_SERVICE_NAME, (int)EGDEFINE.EGMSG.EM_SERVICE_ADD_TAGTRIGGER, byArgs);
                            if (rtn != 0)
                            {
                                edge_FW._PrintSysLog(CORE_SERVICE_NAME, "ERR", "AttachServiceTagTriggers ERROR. Name : {0}, State : 0x{1}", tagName, rtn.ToString("X8"));
                                break;
                            }
                            //TagTrigger Ref更新(C#用メッセージ)
                            EGAPI.EgMailboxSendSC(tmp.ServiceName, CORE_SERVICE_NAME, (int)EGDEFINE.EGMSG.EM_WRAPONLY_UPDATE_TAGTRIGREF);
                            cnt++;  // 登録処理数カウンタ更新
                            if (cnt % 100 == 0)
                            {
                                System.Threading.Thread.Sleep(100); //ログ欠損抑止のためWait
                            }
                            tmp.TagTriggers[j] = Tuple.Create(tagName, true);   // タグトリガーを登録済みで上書き

                            if (!flgTagTrigger)
                                flgTagTrigger = true;   // TagTriggerが存在して、Serviceに通知された
                        }
                    }
                    ListTagTriggers[i] = tmp;   // タグトリガー情報更新
                }

                if (terminal) // 最後のタグトリガー登録処理の場合
                    edge_FW._PrintSysLog(CORE_SERVICE_NAME, "DBG", "AttachServiceTagTriggers Finished.");

                return rtn;
            }
            catch (Exception ex)
            {
                string mess = System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message;
                Debug.WriteLine(mess);
                edge_FW._PrintSysLog(CORE_SERVICE_NAME, "FER", "{0}", mess);
                return -1;
            }
        }
        /// <summary>
        /// サービス内の参照オブジェクト登録
        /// </summary>
        /// <param name="Services"></param>
        public void AttachService(string senderName, RTedgeXmlObj.Service serv)
        {
            bool ServiceAddFlag = false;
            try
            {
                object chkvalue = new object();     //チェック用値オブジェクト
                object value = new object();        //値オブジェクト

                //待機時間の計算
                uint uinTimeOut = SERVICE_INIT_WAIT_TIME;
                //指定がある場合はその値を使用
                if (serv.Timeout != null && serv.Timeout.Length > 0)
                {
                    if (uint.TryParse(serv.Timeout, out uinTimeOut) == false)
                        uinTimeOut = 5;
                }
                //ミリ秒に変換
                int waitTime_ms = (int)(uinTimeOut * 1000);
                //リトライ回数計算
                int retryMaxCount = waitTime_ms == 0 ? 0 : (int)(waitTime_ms / _sleepTime);
                //サービスステータス タグ名の作成
                string ServiceStateName = "SERVICE." + serv.Name + ".Status";
                //サービスの作成
                int rtn = 0;
                bool createF = true;
                //パスがある場合の処理
                if ( serv.Path != null && serv.Path.Length > 0)
                {
                    
                    
                    //存在しているか？
                    if (EGAPI.EgTagGetProperty(ServiceStateName, "Name", ref chkvalue) == 0)
                    {
                        if (EGAPI.EgTagRead(ServiceStateName, ref value) == 0)
                        {
                            if ((byte)value != 0) //0 == 終了している 
                                createF = false;
                        }
                    }
                    //createF == trueの場合は起動して良い
                    if (createF)
                    {
                        BootingSet.Add(serv.Name);  //起動中コンテナに追加
                        ServiceAddFlag = true;      //起動フラグ設定

                        //CreateService　API
                        long pid = 0;
                        string newArg = "";  //priority・colsize・rowsizeを引数に追加する
                        // priorityの判定
                        int priority = -1;
                        // 正常値のみ反映
                        if (serv.Priority != null && serv.Priority.Length > 0)
                            priority = chgStrPriority(serv.Priority);

                        // colsizeの判定
                        int colsize = 0;
                        // 正常値のみ反映
                        if (serv.ColSize != null && serv.ColSize.Length > 0)
                            colsize = CUtility.fStringToInt(serv.ColSize);

                        // rowsizeの判定
                        int rowsize = 0;
                        // 正常値のみ反映
                        if (serv.RowSize != null && serv.RowSize.Length > 0)
                            rowsize = CUtility.fStringToInt(serv.RowSize);

                        // eciの判定
                        string ecipath = "";
                        // 正常値のみ反映
                        if (serv.ECIPath != null && serv.ECIPath.Length > 0 && serv.ECIPath.Length < 257)   //257文字以上は設定しない
                            ecipath = serv.ECIPath;

                        if (priority > -1)   //設定がある場合は引数で渡す
                            newArg = "-priority " + priority.ToString();

                        if (colsize > 0)   //設定がある場合は引数で渡す
                            newArg = newArg + " -colsize " + colsize.ToString();

                        if (rowsize > 0)   //設定がある場合は引数で渡す
                            newArg = newArg + " -rowsize " + rowsize.ToString();

                        if (serv.Name.Length > 0)
                            newArg = newArg + " -name " + serv.Name;

                        if (ecipath.Length > 0) //ECIの指定
                            newArg = newArg + " -eci " + ecipath;

                        newArg = newArg + " " + serv.Argument;

                        var rootPath = Path.GetPathRoot(serv.Path);        // フルパス指定されていないときはEgBootのカレントパスとする (2025/03/12修正 miya)
                        if (rootPath == "")
                            serv.Path = System.Environment.CurrentDirectory + "\\" + serv.Path;
                        
                        edge_API.EDGESERVICECONFIG config = new edge_API.EDGESERVICECONFIG();
                        // 起動ノード取得
                        if (serv.Node != null && serv.Node.Length > 0)
                            config.NodeName = serv.Node;
                        // サービスクラス取得
                        if (serv.Class != null && serv.Class.Length > 0)
                            config.Class = CUtility.fStringToUshort(serv.Class);

                        // サービス起動
                        rtn = EGAPI.EgServiceCreateEx(serv.Name, serv.Path, newArg, ref pid, config);

                        // rtn が0以外は作成失敗
                        if (rtn != 0)
                        {
                            edge_FW._PrintSysLog(CORE_SERVICE_NAME, "ERR", "EgServiceCreate ERROR. Path : {0}, State : 0x{1}", serv.Path, rtn.ToString("X8"));
                            return;
                        }

                        if (retryMaxCount == 0)    //サービス起動待機処理 実施
                            edge_FW._PrintSysLog(CORE_SERVICE_NAME, "OPE", "EgServiceCreate SUCCESS No Wait Created. Path : {0}, State : {1}", serv.Path, rtn.ToString());

                    }
                }
                else
                    createF = false;

                if (retryMaxCount != 0)    //サービス起動待機処理 実施
                {
                    bool initEndF = false;
                    while (true)
                    {
                        //Tag名が登録されているか確認
                        if (EGAPI.EgTagGetProperty(ServiceStateName, "Name", ref chkvalue) == 0)
                        {
                            //登録されている場合は値の読み込み
                            if (EGAPI.EgTagRead(ServiceStateName, ref value) == 0)
                            {
                                if ((byte)value == 1 || (byte)value == 11) //Initialize終了 == 1, メールボックス作成完了 == 11
                                {
                                    initEndF = true;
                                    break;
                                }
                            }
                        }
                        if (retryMaxCount == 0)    //タイムアウトが0になったらおわり
                            break;
                        System.Threading.Thread.Sleep(_sleepTime);//待つ

                        retryMaxCount--;
                    }
                    if (createF)
                    {
                        if (initEndF)
                            edge_FW._PrintSysLog(CORE_SERVICE_NAME, "OPE", "EgServiceCreate SUCCESS. Path : {0}, State : {1}", serv.Path, rtn.ToString("x8"));
                        else
                            edge_FW._PrintSysLog(CORE_SERVICE_NAME, "ERR", "EgServiceCreate Time Out ERROR . Path : {0}, State : {1}", serv.Path, rtn.ToString("x8"));
                    }
                    else
                    {
                        if (!initEndF)
                            edge_FW._PrintSysLog(CORE_SERVICE_NAME, "ERR", "EgService not exist. Name : {0}", serv.Name);
                    
                    }
                }

                //Tagの登録
                rtn = AttachTags(serv.Name, serv.Tags);
                if ((uint)rtn == EGDEFINE.EDGE_TTABLE_NOEMPTY)
                {
                    // タグテーブルの空きがない場合のエラーは処理を止める
                    System.Windows.Forms.MessageBox.Show("Failed to TAG create." + Environment.NewLine + "Please check the config." + Environment.NewLine + "EgTagCreate error: 0x" + rtn.ToString("X8"), "EgBoot error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return;
                }

                //Datasetの登録
                AttachDatasets(serv.Name, serv.Datasets);

                //サービスが参照するTagRefの追加
                AttachServiceTagRefs(serv.Name, serv.TagRefs);
                //サービスが参照するTagRef_OUTの追加
                AttachServiceTagRefs_OUT(serv.Name, serv.TagRefs_OUT);
                //サービスが参照するTagRef_INの追加
                AttachServiceTagRefs_IN(serv.Name, serv.TagRefs_IN);
                //サービスが実行するCollectorの登録
                AttachCollectors(serv.Name, serv.Collectors);
                //サービスが参照するCollectorRefの追加
                AttachServiceClctRefs(serv.Name, serv.CollectorRefs);
                //サービスが参照するMailBoxの追加
                AttachServiceMailBoxes(serv.Name, serv.MailBoxes);
                //サービスが参照するTagTriggerの追加
                if(serv.TagTriggers.Count != 0)
                {
                    TagTriggerInfo newTagTrig = new TagTriggerInfo(0);
                    EntryTagTriggerInfos(serv.Name, serv.TagTriggers, ref newTagTrig);
                    ListTagTriggers.Add(newTagTrig);   // タグトリガー設定情報を登録
                }
                //読み終わったらサービスをcreateした時のみcreate終了メッセージを送る
                //createしたサービス側では、Initialize処理でこのメッセージが来るのを待つ
                if (createF)
                    EGAPI.EgMailboxSendSC(serv.Name, CORE_SERVICE_NAME, (int)EGDEFINE.EGMSG.EM_CORE_CREATE_SERVICE_END, null);

                //読み終わったから初期化お願いメッセージを送信(C#ラッパー用)
                //EM_READ_XML_SERVICE_END
                EGAPI.EgMailboxSendSC(serv.Name, CORE_SERVICE_NAME, (int)EGDEFINE.EGMSG.EM_READ_XML_SERVICE_END, null);
            }
            catch (Exception ex)
            {
                string mess = System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message;
                Debug.WriteLine(mess);
                edge_FW._PrintSysLog(CORE_SERVICE_NAME, "FER", "{0}", mess);
            }
            finally
            {
                //起動中フラグを落とす
                if(ServiceAddFlag == true)
                    BootingSet.Remove(serv.Name);
            }
        }
        /// <summary>
        /// 共通設定 前半 起動はしない
        /// </summary>
        /// <param name="senderName"></param>
        /// <param name="serv"></param>
        public void AttachServiceCommonBefore(string senderName, RTedgeXmlObj.Service serv)
        {
            try
            {
                //Tagの登録
                int rtn = AttachTags(serv.Name, serv.Tags);
                if ((uint)rtn == EGDEFINE.EDGE_TTABLE_NOEMPTY)
                {
                    // タグテーブルの空きがない場合のエラーは処理を止める
                    System.Windows.Forms.MessageBox.Show("Failed to TAG create." + Environment.NewLine + "Please check the config." + Environment.NewLine + "EgTagCreate error: 0x" + rtn.ToString("X8"), "EgBoot error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return;
                }
                //Datasetの登録
                AttachDatasets(serv.Name, serv.Datasets);
            }
            catch (Exception ex)
            {
                string mess = System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message;
                Debug.WriteLine(mess);
                edge_FW._PrintSysLog(CORE_SERVICE_NAME, "FER", "{0}", mess);
            }
        }
        /// <summary>
        /// 共通設定 後半 起動はしない
        /// </summary>
        /// <param name="senderName"></param>
        /// <param name="serv"></param>
        public void AttachServiceCommonAfter(string senderName, RTedgeXmlObj.Service serv)
        {
            try
            {
                string stServiceName;
                //名前が*または空のものは、共通設定とする(送信サービス名を設定する
                if (serv.Name == null || ((serv.Name == "*" || serv.Name.Length == 0) && (serv.Path == null || serv.Path.Length == 0)))
                    stServiceName = senderName;
                else
                    stServiceName = serv.Name;

                //サービスが参照するTagRefの追加
                AttachServiceTagRefs(stServiceName, serv.TagRefs);
                //サービスが参照するTagRef_OUTの追加
                AttachServiceTagRefs_OUT(stServiceName, serv.TagRefs_OUT);
                //サービスが参照するTagRef_INの追加
                AttachServiceTagRefs_IN(stServiceName, serv.TagRefs_IN);
                //サービスが実行するCollectorの登録
                AttachCollectors(stServiceName, serv.Collectors);
                //サービスが参照するCollectorRefの追加
                AttachServiceClctRefs(stServiceName, serv.CollectorRefs);
                //サービスが参照するMailBoxの追加
                AttachServiceMailBoxes(stServiceName, serv.MailBoxes);

                
                //読み終わったから初期化お願いメッセージを送信(C#ラッパー用)
                //EM_READ_XML_SERVICE_END
                EGAPI.EgMailboxSendSC(stServiceName, CORE_SERVICE_NAME, (int)EGDEFINE.EGMSG.EM_READ_XML_SERVICE_END, null);
            }
            catch (Exception ex)
            {
                string mess = System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message;
                Debug.WriteLine(mess);
                edge_FW._PrintSysLog(CORE_SERVICE_NAME, "FER", "{0}", mess);
            }
        }

        /// <summary>
        /// サービス内の参照オブジェクト登録
        /// </summary>
        /// <param name="Services"></param>
        public void AttachServices(string senderName, List<RTedgeXmlObj.Service> Services)
        {
            try
            {
                RTedgeXmlObj.Service CommonSrv = null;
                //サービスの作成
                foreach (RTedgeXmlObj.Service s in Services)
                {
                    if (s.Name.Equals("EgLog")) { continue; }   // EgLogは先行して起動済みのため処理しない

                    if (!string.IsNullOrEmpty(s.Enabled)        // 起動設定が無効に指定されている場合
                        && 0 == CUtility.fStringToUshort(s.Enabled))
                    {
                        continue;                               // 起動無効の場合は次へ
                    }
                    // 2022/08/31 XML構成変更 同じXMLを複数のプロセスから呼び出し可能にするため
                    //            2重登録させない仕組み
                    // OnlyOwner プロパティが設定されていて TRUEの場合は呼び出し元と同じ名前以外は呼び出さない
                    if (false == IsContinueService(s, senderName))
                        continue;

                    //名前が*または無名でパスが無いものは、共通設定とする(送信サービス名を設定する)
                    if ((s.Name == null || s.Name.Length == 0 || s.Name=="*") && (s.Path == null || s.Path.Length == 0))
                    {
                        //Tag / Datasetの登録のみ最初 あとでTagRef登録 
                        AttachServiceCommonBefore(senderName, s);
                        //後で行うものは送信元の名前で実施
                        CommonSrv = s;
                        CommonSrv.Name = senderName;
                    }
                    else
                    {
                        // 名前があるもの or Pathがあるものは通常通り処理 サービス内参照オブジェクト登録
                        if(ServiceWaitFlag != 0)//起動するまで待機 同期処理
                            AttachService(senderName, s);
                        else                       //起動を待機しない 非同期処理
                            Task.Run(() => AttachService(senderName, s));

                        System.Threading.Thread.Sleep(10);    // 10ms待機
                    }
                }
                if(CommonSrv != null)//共通設定後処理
                    AttachServiceCommonAfter(senderName, CommonSrv);
            }
            catch (Exception ex)
            {
                string mess = System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message;
                Debug.WriteLine(mess);
                edge_FW._PrintSysLog(CORE_SERVICE_NAME, "FER", "{0}", mess);
            }
        }
        /// <summary>
        /// サービス内の参照オブジェクト登録
        /// </summary>
        /// <param name="Services"></param>
        public void AttachServices(string senderName, List<RTedgeXmlObj.Service> Services, List<InitWait> initWaits)
        {
            try
            {
                RTedgeXmlObj.Service CommonSrv = null;
                int xmlReadIndex = 0;      // xml定義のコンテナ情報読込インデックス
                while (true)
                {
                    // 起動対象コンテナ判定
                    RTedgeXmlObj.Service serv = GetTargetContainer(Services, ref xmlReadIndex, initWaits);
                    if (serv == null)
                    {
                        break;      // 起動対象なしのため処理終了
                    }

                    // 2022/08/31 XML構成変更 同じXMLを複数のプロセスから呼び出し可能にするため
                    //            2重登録させない仕組み (EgBootは関係ないが一応
                    // OnlyOwner プロパティが設定されていて TRUEの場合は呼び出し元と同じ名前以外は呼び出さない
                    if (false == IsContinueService(serv, senderName))
                        continue;

                    //名前が*または空のものは、共通設定とする(送信サービス名を設定する)
                    if ((serv.Name == null || serv.Name.Length == 0 || serv.Name == "*") && (serv.Path == null || serv.Path.Length == 0))
                    {
                        CommonSrv = serv;
                        CommonSrv.Name = senderName;
                        //Tag / Datasetの登録のみ最初 あとでTagRef登録
                        AttachServiceCommonBefore(senderName, CommonSrv);
                    }
                    else
                    {
                        // サービス内参照オブジェクト登録
                        if (ServiceWaitFlag != 0)//起動するまで待機 同期処理
                            AttachService(senderName, serv);
                        else                        //起動するまで待機しない 非同期処理
                            Task.Run(() => AttachService(senderName, serv));

                        System.Threading.Thread.Sleep(10);    // 10ms待機
                    }
                }
                if (CommonSrv != null)//共通設定後処理
                    AttachServiceCommonAfter(senderName, CommonSrv);
            }
            catch (Exception ex)
            {
                string mess = System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message;
                Debug.WriteLine(mess);
                edge_FW._PrintSysLog(CORE_SERVICE_NAME, "FER", "{0}", mess);
            }
        }

        /// <summary>
        /// EgBootサービス定義の固有初期化処理
        /// </summary>
        /// <param name="stXmlPath"></param>
        public int InitEgBootXml(RTedgeXmlObj.RTedge edgeXmlData)
        {
            try
            {
                //edge_FW._PrintSysLog(CORE_SERVICE_NAME, "OPE", "EgTagInit() start!!");
                int ret = 0;
                int ret_tag = 0;
                // xmlデータがなければ無処理で抜ける。エラーとしない。
                if (edgeXmlData == null) { return ret; }

                // EgBootサービス定義を取得
                RTedgeXmlObj.Service startDef = edgeXmlData.Services.FirstOrDefault(s => s.Name.Equals("EgBoot"));

                // EgBootサービス定義がなければ無処理で抜ける。エラーとしない。
                if (startDef == null) { return ret; }

                // -------------------------
                // 起動パラメータの取得
                // -------------------------
                string RTCDnode = string.Empty;
                string EgSysDefPath = System.Windows.Forms.Application.StartupPath + "\\" + EGSYSDEF_FILE_NAME;
                uint maxNum = 0;
                string[] args = startDef.Argument.Split(';');
                uint sysTagF = 0;
                foreach (var item in args)
                {
                    string[] sub = item.Split('=');
                    if(sub[0].Trim() == "RTCD")
                    {
                        // RTCDノードの取得
                        RTCDnode = sub[1].Trim();
                    }
                    else if(sub[0].Trim() == "TagMaxNum")
                    {
                        // タグ最大数の取得
                        uint.TryParse(sub[1].Trim(), out maxNum);
                    }
                    else if (sub[0].Trim() == "EgSysDef")   //オブジェクト最大値設定ファイル
                    {
                        // EgSysDefのパスを取得
                        EgSysDefPath = sub[1].Trim();
                    }
                    else if (sub[0].Trim() == "CreateSysTagF")   //システム値格納タグ作成フラグ
                    {
                        // SysTagFlagの値を取得
                        uint.TryParse(sub[1].Trim(), out sysTagF);
                    }
                    else if (sub[0].Trim() == "WaitF")          // サービス起動 待機使用フラグ
                    {
                        uint.TryParse(sub[1].Trim(), out ServiceWaitFlag);
                    }
                    else { }
                }

                if (!string.IsNullOrEmpty(RTCDnode))
                {
                    ret = EGAPI.EgSetRTCD(RTCDnode);    // RTCD初期化
                }
                // タグのヒープ領域作成
                if (maxNum > 0)//タグの最大値指定がある場合
                {
                    //Thread.Sleep(20000);
                    ret_tag = EGAPI.EgTagInit(maxNum);      // タグ登録最大数を指定して初期化
                }
                // xmlファイルを読み込み
                int sysdeferr = 0;
                RTedgeXmlObj.RTedgeSysDef SysDefData = ReadSysDefFile(EgSysDefPath);
                if (SysDefData != null)
                {
                    // オブジェクト最大値設定
                    sysdeferr = SetSysDefData2API(SysDefData);  //エラーは後で
                }
                else
                    sysdeferr = -1;

                //ログ出力用 Mailbox作成
                int rtn = EGAPI.EgMailboxCreate(EGDEFINE.EG_MBOX_NAME_LOG, ref m_LogMBoxInfo);
                if (rtn != 0)
                {
                    System.Windows.Forms.MessageBox.Show("Cannot Create Mailbox. Status=[" + rtn.ToString("X8") + "]", "EgBoot error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    Debug.WriteLine("EgMailboxCreate[EGSYSLOG] Error 0x" + rtn.ToString("X8"));
                }
                edge_FW._PrintSysLog(CORE_SERVICE_NAME, "OPE", "EgSetRTCD() RTCD={0} ret={1}", RTCDnode, ret.ToString());

                if (maxNum > 0)//ここで出力(メールボックスができていない為)
                    edge_FW._PrintSysLog(CORE_SERVICE_NAME, "OPE", "EgTagInit() TagMaxNum={0} ret_tag={1}", maxNum.ToString(), ret_tag.ToString());

                if (sysdeferr == -1)//ここで出力(メールボックスができていない為)
                    edge_FW._PrintSysLog(CORE_SERVICE_NAME, "WRN", "Cannot Read SysDef File EgSysDefPath={0}", EgSysDefPath);
                else if(sysdeferr != 0)
                    edge_FW._PrintSysLog(CORE_SERVICE_NAME, "WRN", "Cannot Set SysDef File Data. EgSysDefPath={0}", EgSysDefPath);

                if (sysTagF > 0)//システムタグを作成するか否か 引数で指定がある場合作成
                    edge_API._C__EgSysTagCreate();  //システムタグ作成

                if (ret_tag == 0)
                {
                    // タグポインタ用ヒープ領域の生成
                    ret_tag = EGAPI.EgTagInitSegment();
                    edge_FW._PrintSysLog(CORE_SERVICE_NAME, "OPE", "EgTagPtrInit() ret_tag={0}", ret_tag.ToString());
                }
                if (ret_tag != 0)
                    return ret_tag;

                return ret;
            }
            catch (Exception ex)
            {
                string mess = System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message;
                Debug.WriteLine(mess);
                edge_FW._PrintSysLog(CORE_SERVICE_NAME, "FER", "{0}", mess);
                return -1;
            }
        }
        /// <summary>
        /// オブジェクト最大値設定APIコール
        /// </summary>
        /// <param name="SysDefData"></param>
        /// <returns></returns>
        public int SetSysDefData2API(RTedgeXmlObj.RTedgeSysDef SysDefData)
        {
            try
            {
                uint val = 0;
                edge_API.EDGESYSTEMCONFIG config = EGAPI.EDGE_SYSTEMCONFIG_DEFAULT;
                if (SysDefData != null)
                {
                    val = CUtility.fStringToUint(SysDefData.DSETENTRY); // データセットエントリ最大数（テーブルのインデックス数制限）
                    if (val > 0) config.MaxDsetEntry = val; //値0は許容しない

                    val = CUtility.fStringToUint(SysDefData.DSETLINK); // データセット データリンクタグ最大数(TagRef数)
                    if (val > 0) config.MaxDsetLink = val; //値0は許容しない

                    val = CUtility.fStringToUint(SysDefData.CLCTENTRY); // コレクタ エントリ最大数
                    if (val > 0) config.MaxClctEntry = val; //値0は許容しない

                    val = CUtility.fStringToUint(SysDefData.SERVICEENTRY); // サービスコンテナエントリ最大数
                    if (val > 0) config.MaxServiceEntry = val; //値0は許容しない

                    val = CUtility.fStringToUint(SysDefData.MAILBOXENTRY); // メールボックス エントリ最大数
                    if (val > 0) config.MaxMailboxEntry = val; //値0は許容しない

                    val = CUtility.fStringToUint(SysDefData.FW_C2C_MAILBOXCOL); // RT-edgeフレームワーク コンテナ間通信メールボックス最大データサイズ(ColSize) 
                    if (val > 0) config.MaxC2CMailboxCol = val; //値0は許容しない

                    val = CUtility.fStringToUint(SysDefData.FW_C2C_MAILBOXROW); // RT-edgeフレームワーク コンテナ間通信メールボックス 最大保持件数(RowSize)
                    if (val > 0) config.MaxC2CMailboxRow = val; //値0は許容しない

                    val = CUtility.fStringToUint(SysDefData.FW_CLCTENTRY); // RT-edgeフレームワーク  コレクタオブジェクト参照 エントリ最大数
                    if (val > 0) config.MaxFwClctEntry = val; //値0は許容しない

                    val = CUtility.fStringToUint(SysDefData.FW_MAILBOXENTRY); // RT-edgeフレームワーク メールボックスオブジェクト エントリ最大数
                    if (val > 0) config.MaxFwMailboxEntry = val; //値0は許容しない

                    val = CUtility.fStringToUint(SysDefData.FW_TAGREFENTRY); // RT-edgeフレームワーク TagRefオブジェクト エントリ最大数
                    if (val > 0) config.MaxFwTagRefEntry = val; //値0は許容しない

                    val = CUtility.fStringToUint(SysDefData.MAXTAGPTRNUM);      // タグポインタ最大数
                    if (val > 0) config.MaxTagPtrNum = val;         //値0は許容しない

                    val = CUtility.fStringToUint(SysDefData.MAXTAGPTRSIZE);     // タグポインタ最大データサイズ(バイト単位)
                    if (val > 0) config.MaxTagPtrSize = val;        //値0は許容しない

                    val = CUtility.fStringToUint(SysDefData.MAXTAGPTRRETRY);    // タグポインタロックを試みる回数
                    if (val > 0) config.MaxTagPtrRetry = val;       //値0は許容しない

                    val = CUtility.fStringToUint(SysDefData.FW_TAGTRIGGERENTRY); // タグトリガー最大数
                    if (val > 0) config.MaxTagTriggerNum = val;

                }
                int ret = EGAPI._EgSysDefSetting(config);
                return ret;
            }
            catch (Exception ex)
            {
                string mess = System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message;
                Debug.WriteLine(mess);
                edge_FW._PrintSysLog(CORE_SERVICE_NAME, "FER", "{0}", mess);
                return -1;
            }
        }
        /// <summary>
        /// EgLogサービスの開始処理
        /// </summary>
        /// <param name="stXmlPath"></param>
        public int StartEgLog(RTedgeXmlObj.RTedge edgeXmlData)
        {
            try
            {
                int ret = 0;
                // xmlデータがなければ無処理で抜ける。エラーとしない。
                if (edgeXmlData == null) { return ret; }

                // EgLogサービス定義を取得
                RTedgeXmlObj.Service logDef = edgeXmlData.Services.FirstOrDefault(s => s.Name.Equals("EgLog"));

                // EgLogサービス定義がなければ無処理で抜ける。エラーとしない。
                if (logDef == null) { return ret; }

                //Serviceの登録 //ログは起動するまで待機
                AttachService("", logDef);
                
                
                return ret;
            }
            catch (Exception ex)
            {
                string mess = System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message;
                Debug.WriteLine(mess);
                edge_FW._PrintSysLog(CORE_SERVICE_NAME, "FER", "{0}", mess);
                return -1;
            }
        }

        /// <summary>
        /// xmlファイルの読み込みと分解
        /// </summary>
        /// <param name="paramArray"></param>
        public void ConvertXml2Obj(string senderName, byte[] paramArray)
        {
            try
            {
                string[] strList = new string[0];

                object obj = new EGDEFINE.RGMSPRAMTSTRING1();
                //引数の変換
                //バイト配列から構造体へ内容のコピーを行う
                //文字列1つの構造体
                int ret = egCommonFunc.fnByToObj(paramArray, ref obj);
                if (ret != 0)
                    return;

                EGDEFINE.RGMSPRAMTSTRING1 st1 = (EGDEFINE.RGMSPRAMTSTRING1)obj;
                //配列のサイズ変更
                Array.Resize(ref strList, strList.Length + 1);
                strList[strList.Length - 1] = st1.stParam;

                string stXmlPath = strList[0];
                //xmlではない場合は拡張子をxmlに変換して試してみる(exeを想定)
                if(Path.GetFileNameWithoutExtension(stXmlPath) != "xml")
                    stXmlPath = System.IO.Path.ChangeExtension(stXmlPath, "xml");

                // xml読み込み処理
                RTedgeXmlObj.RTedge xmlData = ReadXmlFile(stXmlPath);
                
                // Egオブジェクト生成
                ConvertXml2Obj(senderName, xmlData);
            }
            catch (Exception ex)
            {
                string mess = System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message;
                Debug.WriteLine(mess);
                edge_FW._PrintSysLog(CORE_SERVICE_NAME, "FER", "{0}", mess);
            }
        }

        /// <summary>
        /// XMLの読み込み
        /// </summary>
        /// <param name="stXmlPath"></param>
        /// <returns></returns>
        public RTedgeXmlObj.RTedge ReadXmlFile(string stXmlPath)
        {
            // ファイル存在チェック
            if (!System.IO.File.Exists(stXmlPath)) { return null; }

            // XMLのロード
            RTedgeXmlObj.RTedge edgeXmlData = MnXmlUtility.LoadFromFile<RTedgeXmlObj.RTedge>(stXmlPath);

            // フォーマットチェック
            if (edgeXmlData == null || edgeXmlData.Tags == null || edgeXmlData.Datasets == null || edgeXmlData.Services == null)
            {
                edge_FW._PrintSysLog(CORE_SERVICE_NAME, "ERR", "Reading failed because the xml file format is incorrect. [{0}]", stXmlPath);
                return null;
            }

            // 読み取ったタグ一覧から、NameとTypeとValueを保管しておく(初期値リセットに利用)
            foreach(RTedgeXmlObj.Tag tmpTag in edgeXmlData.Tags)
            {
                if (tmpTag == null) // タグの実体があるか ?
                {  
                    continue;       // 実体無いのでスキップ
                }

                if (TagExistNames.Add(tmpTag.Name)) // タグ名が重複していないか ?
                {   // 重複してない
                    if (tmpTag.Value == null) {     // 値がnull ?
                        // 値がnullの場合は、文字列無しとしてセット
                        TagVals.Add(new TagVal(tmpTag.Name, tmpTag.Type, "", tmpTag.InitGroup));
                    } else {
                        // 値がある場合は、そのままセット
                        TagVals.Add(new TagVal(tmpTag.Name, tmpTag.Type, tmpTag.Value, tmpTag.InitGroup));
                    }
                }
                else
                {   // 重複している
                    TagVal  findTarget = TagVals.Find(tv => tv.Name == tmpTag.Name);
                    if(!findTarget.Equals(default(TagVal))) {
                        findTarget.Value = tmpTag.Value;    // Valueだけ上書き
                    }
                }
                
            }

            return edgeXmlData;
        }

        /// <summary>
        /// XML定義データからEgオブジェクト生成(EgBoot専用)
        /// </summary>
        /// <param name="stXmlPath"></param>
        public void ConvertEgBootXml2Obj(RTedgeXmlObj.RTedge edgeXmlData)
        {
            try
            {
                if (edgeXmlData != null)
                {
                    

                    //Tagの登録 グローバルなリンクタグは名前無しのため第一引数を空とする
                    int rtn = AttachTags("", edgeXmlData.Tags);
                    if ((uint)rtn == EGDEFINE.EDGE_TTABLE_NOEMPTY)
                    {
                        // タグテーブルの空きがない場合のエラーは処理を止める
                        System.Windows.Forms.MessageBox.Show("Failed to TAG create." + Environment.NewLine + "Please check the config." + Environment.NewLine + "EgTagCreate error: 0x" + rtn.ToString("X8"), "EgBoot error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                        return;
                    }
                    //Datasetの登録
                    AttachDatasets("", edgeXmlData.Datasets);

                    // NOTE: EgBoot.xmlのみ、依存関係の設定を有効とする
                    //       依存設定により起動処理を待機する際に下記の起動待機リストを使用します
                    List<InitWait> waitList = new List<InitWait>();     // 起動待機リスト初期化
                    
                    // Serviceの登録
                    AttachServices(EGAPI.EGFW.ServiceRealName, edgeXmlData.Services, waitList);

                    
                }
            }
            catch (Exception ex)
            {
                string mess = System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message;
                Debug.WriteLine(mess);
                edge_FW._PrintSysLog(CORE_SERVICE_NAME, "FER", "{0}", mess);
            }
        }

        /// <summary>
        /// XML定義データからEgオブジェクト生成
        /// </summary>
        /// <param name="stXmlPath"></param>
        public void ConvertXml2Obj(string senderName, RTedgeXmlObj.RTedge edgeXmlData)
        {
            try
            {
                if (edgeXmlData != null)
                {
                    //Tagの登録
                    int rtn = AttachTags(senderName, edgeXmlData.Tags);
                    if ((uint)rtn == EGDEFINE.EDGE_TTABLE_NOEMPTY)
                    {
                        // タグテーブルの空きがない場合のエラーは処理を止める
                        System.Windows.Forms.MessageBox.Show("Failed to TAG create." + Environment.NewLine + "Please check the config." + Environment.NewLine + "EgTagCreate error: 0x" + rtn.ToString("X8"), "EgBoot error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                        return;
                    }
                    //Datasetの登録
                    AttachDatasets(senderName, edgeXmlData.Datasets);
                    //Serviceの登録
                    AttachServices(senderName, edgeXmlData.Services);
                }
            }
            catch (Exception ex)
            {
                string mess = System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message;
                Debug.WriteLine(mess);
                edge_FW._PrintSysLog(CORE_SERVICE_NAME, "FER", "{0}", mess);
            }
        }
        /// <summary>
        /// Priorityの文字列を数値に変換
        /// </summary>
        /// <param name="stPriority"></param>
        /// <returns></returns>
        public int chgStrPriority(string stPriority)
        {
            try
            {
                //
                // 定義された文字列の場合は数値に変換
                //
                if (stPriority.Equals("EG_PRIORITY_240_255", StringComparison.OrdinalIgnoreCase) == true)
                    return (int)EGDEFINE.egPriority.EG_PRIORITY_240_255;
                else if (stPriority.Equals("EG_PRIORITY_232_239", StringComparison.OrdinalIgnoreCase) == true)
                    return (int)EGDEFINE.egPriority.EG_PRIORITY_232_239;
                else if (stPriority.Equals("EG_PRIORITY_224_231", StringComparison.OrdinalIgnoreCase) == true)
                    return (int)EGDEFINE.egPriority.EG_PRIORITY_224_231;
                else if (stPriority.Equals("EG_PRIORITY_216_223", StringComparison.OrdinalIgnoreCase) == true)
                    return (int)EGDEFINE.egPriority.EG_PRIORITY_216_223;
                else if (stPriority.Equals("EG_PRIORITY_208_215", StringComparison.OrdinalIgnoreCase) == true)
                    return (int)EGDEFINE.egPriority.EG_PRIORITY_208_215;
                else if (stPriority.Equals("EG_PRIORITY_200_207", StringComparison.OrdinalIgnoreCase) == true)
                    return (int)EGDEFINE.egPriority.EG_PRIORITY_200_207;
                else if (stPriority.Equals("EG_PRIORITY_192_199", StringComparison.OrdinalIgnoreCase) == true)
                    return (int)EGDEFINE.egPriority.EG_PRIORITY_192_199;
                else if (stPriority.Equals("EG_PRIORITY_184_191", StringComparison.OrdinalIgnoreCase) == true)
                    return (int)EGDEFINE.egPriority.EG_PRIORITY_184_191;
                else if (stPriority.Equals("EG_PRIORITY_176_183", StringComparison.OrdinalIgnoreCase) == true)
                    return (int)EGDEFINE.egPriority.EG_PRIORITY_176_183;
                else if (stPriority.Equals("EG_PRIORITY_168_175", StringComparison.OrdinalIgnoreCase) == true)
                    return (int)EGDEFINE.egPriority.EG_PRIORITY_168_175;
                else if (stPriority.Equals("EG_PRIORITY_160_167", StringComparison.OrdinalIgnoreCase) == true)
                    return (int)EGDEFINE.egPriority.EG_PRIORITY_160_167;
                else if (stPriority.Equals("EG_PRIORITY_152_159", StringComparison.OrdinalIgnoreCase) == true)
                    return (int)EGDEFINE.egPriority.EG_PRIORITY_152_159;
                else if (stPriority.Equals("EG_PRIORITY_144_151", StringComparison.OrdinalIgnoreCase) == true)
                    return (int)EGDEFINE.egPriority.EG_PRIORITY_144_151;
                else if (stPriority.Equals("EG_PRIORITY_136_143", StringComparison.OrdinalIgnoreCase) == true)
                    return (int)EGDEFINE.egPriority.EG_PRIORITY_136_143;
                else if (stPriority.Equals("EG_PRIORITY_128_135", StringComparison.OrdinalIgnoreCase) == true)
                    return (int)EGDEFINE.egPriority.EG_PRIORITY_128_135;
                else if (stPriority.Equals("EG_PRIORITY_80_127", StringComparison.OrdinalIgnoreCase) == true)
                    return (int)EGDEFINE.egPriority.EG_PRIORITY_80_127;
                else if (stPriority.Equals("EG_PRIORITY_72_79", StringComparison.OrdinalIgnoreCase) == true)
                    return (int)EGDEFINE.egPriority.EG_PRIORITY_72_79;
                else if (stPriority.Equals("EG_PRIORITY_64_71", StringComparison.OrdinalIgnoreCase) == true)
                    return (int)EGDEFINE.egPriority.EG_PRIORITY_64_71;
                else if (stPriority.Equals("EG_PRIORITY_56_63", StringComparison.OrdinalIgnoreCase) == true)
                    return (int)EGDEFINE.egPriority.EG_PRIORITY_56_63;
                else if (stPriority.Equals("EG_PRIORITY_48_55", StringComparison.OrdinalIgnoreCase) == true)
                    return (int)EGDEFINE.egPriority.EG_PRIORITY_48_55;
                else if (stPriority.Equals("EG_PRIORITY_8_47", StringComparison.OrdinalIgnoreCase) == true)
                    return (int)EGDEFINE.egPriority.EG_PRIORITY_8_47;
                else if (stPriority.Equals("EG_PRIORITY_0_7", StringComparison.OrdinalIgnoreCase) == true)
                    return (int)EGDEFINE.egPriority.EG_PRIORITY_0_7;

                int iPrio = 0;
                bool result = int.TryParse(stPriority, out iPrio);  //文字列が数値に変換できる場合はその値を返す
                if (result)
                    return iPrio;
                return -1;
            }
            catch(Exception ex)
            {
                string mess = System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message;
                Debug.WriteLine(mess);
                edge_FW._PrintSysLog(CORE_SERVICE_NAME, "FER", "{0}", mess);
                return -1;
            }
        }


        /// <summary>
        /// 起動対象コンテナ取得
        /// </summary>
        /// <param name="Services"></param>
        /// <param name="waitList"></param>
        /// <returns></returns>
        public RTedgeXmlObj.Service GetTargetContainer(List<RTedgeXmlObj.Service> Services, ref int index, List<InitWait> waitList)
        {
            try
            {
                RTedgeXmlObj.Service serv;

                // 起動待機リスト内のコンテナ起動判定
                serv = GetTargetFromWaitList(Services, waitList);
                if (serv != null)               // 起動待機リストに起動対象コンテナあり
                    return serv;                // 起動対象コンテナを返却

                // xml定義内のコンテナ起動判定
                serv = GetTargetFromXml(Services, ref index, waitList);
                if (serv != null)               // xml定義内に起動対象コンテナあり
                    return serv;                // 起動対象コンテナを返却

                // -----------------------------------------------------------------
                // NOTE: xml定義内のコンテナ起動処理終了後、起動待機リストに
                //       待機中コンテナが残っている場合は、一定時間リトライを行う
                // -----------------------------------------------------------------
                if (waitList.Count > 0)         // 待機中コンテナあり
                {
                    int retryMaxCount = (int)(_waitTime_ms / _sleepTime); 
                    for (int retry = 0; retry < retryMaxCount; retry++)
                    {
                        // 起動待機リスト内のコンテナ起動判定
                        serv = GetTargetFromWaitList(Services, waitList);
                        if (serv != null)               // 起動待機リストに起動対象コンテナあり
                            return serv;                // 起動対象コンテナを返却

                        System.Threading.Thread.Sleep(_sleepTime);    // 1秒周期でリトライ
                    }
                    // リトライ後もコンテナ起動出来なかった場合はエラー出力
                    foreach (var item in waitList)
                    {
                        edge_FW._PrintSysLog(CORE_SERVICE_NAME, "ERR", "EgServiceCreate Time Out ERROR. Name : {0}", item.Name);
                    }
                }
                return null;                    // 起動対象コンテナなし
            }
            catch (Exception ex)
            {
                string mess = System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message;
                Debug.WriteLine(mess);
                edge_FW._PrintSysLog(CORE_SERVICE_NAME, "FER", "{0}", mess);
                return null;                    // 起動対象コンテナなし
            }
        }

        /// <summary>
        /// xml定義内から起動対象コンテナ取得
        /// </summary>
        /// <param name="Services"></param>
        /// <param name="index"></param>
        /// <param name="waitList"></param>
        /// <returns></returns>
        public RTedgeXmlObj.Service GetTargetFromXml(List<RTedgeXmlObj.Service> Services, ref int index, List<InitWait> waitList)
        {
            try
            {
                RTedgeXmlObj.Service target;        // xml内のコンテナ定義
                bool isStatus = false;              // 依存コンテナ起動状態
                int tmpIndex = index;               // 判定対象インデックス

                while (Services.Count > index)      // 起動対象が見つかるか、xml定義終端まで判定処理をループ
                {
                    tmpIndex = index++;                             // 今回の判定対象を保持してからインデックス更新
                    target = Services[(int)tmpIndex];               // 判定対象のxml定義情報取得
                    // 起動有効・無効判定
                    if (!string.IsNullOrEmpty(target.Enabled)       // 起動設定が無効に指定されている場合
                        && 0 == CUtility.fStringToUshort(target.Enabled))
                    {
                        continue;                                   // 起動無効の場合は次へ
                    }
                    // 依存関係の判定
                    if (string.IsNullOrEmpty(target.Dependent))     // 依存コンテナの設定なし
                    {
                        return target;                              // 起動対象として返却
                    }
                    else                                            // 依存コンテナの設定あり
                    {
                        string[] deps = target.Dependent.Split(';');// 依存コンテナ名の取得
                        foreach (var dep in deps)                   // 設定されている依存コンテナをすべてチェック
                        {
                            isStatus = IsExistContainer(dep);    // 依存コンテナ起動状態取得
                            if (!isStatus) { break; }               // 未起動が1つでもある場合は起動対象にならない
                        }
                        if (isStatus)   // 全て起動済み
                        {
                            return target;                          // 起動対象として返却
                        }
                        else            // 未起動あり
                        {
                            // 起動実行中以外の場合
                            if (BootingSet.Contains(target.Name) == false)
                            {
                                // 起動できない場合は、起動待機リストへ追加
                                waitList.Add(new InitWait(target));
                                edge_FW._PrintSysLog(CORE_SERVICE_NAME, "DBG", "Add {0} to waiting list.", target.Name);
                            }
                        }
                    }
                }
                return null;
            }
            catch (Exception ex)
            {
                string mess = System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message;
                Debug.WriteLine(mess);
                edge_FW._PrintSysLog(CORE_SERVICE_NAME, "FER", "{0}", mess);
                return null;
            }
        }

        /// <summary>
        /// 起動待機リストから起動対象コンテナ取得
        /// </summary>
        /// <param name="waitList"></param>.na
        /// <returns></returns>
        public RTedgeXmlObj.Service GetTargetFromWaitList(List<RTedgeXmlObj.Service> Services, List<InitWait> waitList)
        {
            try
            {
                RTedgeXmlObj.Service serv;
                bool isExist = false;
                if (waitList.Count != 0)    // 起動待機リストに登録があればリスト項目を順番に判定する
                {
                    // 起動待機リスト項目を順番に判定
                    foreach (var target in waitList)
                    {
                        // 依存コンテナの起動状態判定
                        foreach (var dep in target.Dependents)
                        {
                            isExist = IsExistContainer(dep); // コンテナ起動状態取得
                            if (!isExist)                       // 未起動が1つでもあれば次のリスト項目へ
                                break;
                            // 現在起動実行中の場合
                            if (BootingSet.Contains(dep) == true)
                            {
                                //未起動とする
                                isExist = false;
                            }
                        }
                        if (isExist)                            // 依存コンテナ全てが起動済み
                        {
                            serv = Services.Find(x => x.Name == target.Name);   // コンテナ情報取得
                            waitList.Remove(target);            // 起動対象リストから削除
                            edge_FW._PrintSysLog(CORE_SERVICE_NAME, "DBG", "Remove {0} from waiting list.", target.Name);
                            return serv;                        // 起動可能なコンテナを返却
                        }
                    }
                }
                return null;                                    // 起動可能なコンテナが見つからなかった
            }
            catch (Exception ex)
            {
                string mess = System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message;
                Debug.WriteLine(mess);
                edge_FW._PrintSysLog(CORE_SERVICE_NAME, "FER", "{0}", mess);
                return null;
            }
        }

        /// <summary>
        /// コンテナ起動状態取得
        /// </summary>
        /// <param name="name"></param>
        /// <param name="timeout"></param>
        /// <returns>true:起動済み, false:未起動</returns>
        public bool IsExistContainer(string name)
        {
            object chkvalue = new object();
            object value = new object();



            // サービス起動状態ステータスタグの名称取得
            string tag = string.Format("SERVICE.{0}.State2Depend", name);

            // Tag名が登録されているか確認
            if (EGAPI.EgTagGetProperty(tag, "Name", ref chkvalue) == 0)
            {
                //登録されている場合は値の読み込み
                if (EGAPI.EgTagRead(tag, ref value) == 0)
                {
                    if ((byte)value == SERVICE_STATE_2_DEPEND_INITEND)   // Initialize終了 == 16
                    {
                        //起動中フラグが立っていて、起動完了フラグが立っていたら、Removeする
                        if (BootingSet.Contains(name) == true)
                            BootingSet.Remove(name);

                        return true;        // 起動済み
                    }
                }
            }
            return false;                   // 未起動
        }
        /// <summary>
        /// Serviceの処理を続行するか否か
        /// </summary>
        /// <param name="s"></param>
        /// <param name="senderName"></param>
        /// <returns>
        /// TRUE : 続行する (デフォルト)
        /// FALSE : 読み飛ばす
        /// </returns>
        public bool IsContinueService(RTedgeXmlObj.Service s, string senderName)
        {
            bool IsContinueFlag = true; //処理続行フラグ
            try
            {
                // OnlyOwner プロパティが設定されていて TRUEの場合は呼び出し元と同じ名前以外は呼び出さない
                if (!string.IsNullOrEmpty(s.OnlyOwner) && (String.Compare(s.OnlyOwner, "TRUE", true) == 0 || String.Compare(s.OnlyOwner, "1", true) == 0))
                {
                    //OnlyOwnerフラグがある＝＝同じ名前の物以外処理は続行しないので、ここでフラグを変更
                    IsContinueFlag = false;
                    
                    //呼び出し元の名前と比較 名前がある場合のみチェック 名前が無い場合は読みださない
                    if (s.Name != null && s.Name.Length > 0)
                    {
                        if (String.Compare(senderName, s.Name, true) == 0) //Name名が違う場合は次のサービスへ
                            return true;
                    }
                }
                return IsContinueFlag;
            }
            catch
            {
                return IsContinueFlag;
            }
        }
        /// <summary>
        /// SysDefファイルの読込
        /// </summary>
        /// <param name="stXmlPath"></param>
        /// <returns></returns>
        public RTedgeXmlObj.RTedgeSysDef ReadSysDefFile(string stXmlPath)
        {
            // ファイル存在チェック
            if (!System.IO.File.Exists(stXmlPath)) { return null; }
            // XMLのロード
            RTedgeXmlObj.RTedgeSysDef edgeSysDefData = MnXmlUtility.LoadFromFile<RTedgeXmlObj.RTedgeSysDef>(stXmlPath);

            // フォーマットチェック
            if (edgeSysDefData == null)
            {
                // タグテーブルの空きがない場合のエラーは処理を止める
                System.Windows.Forms.MessageBox.Show("Reading failed because the EgSysDef xml file format is incorrect. [" + stXmlPath + "]", "EgBoot error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return null;
            }
            return edgeSysDefData;
        }

    }
}
