﻿////
/// Copyright(C) 2020 Micronet Co. All Rights Reserved.
//
using System;
using System.Collections.Generic;
using System.Xml;
using System.Xml.Serialization;

namespace RTedgeXmlObj
{
    /// <summary>
    /// xml Tagの定義
    /// </summary>
    [Serializable]
    public class Tag
    {
        [XmlAttribute("Name")]
        public string Name;                         //Tag名
        [XmlAttribute("Comment")]
        public string Comment;                      //コメント
        [XmlAttribute("Address")]
        public string Address;                      //アドレス・ソース
        [XmlAttribute("Type")]
        public string Type;                         //データタイプ(数値)
        [XmlAttribute("Size")]
        public string Size;                         //データサイズ
        [XmlAttribute("Value")]
        public string Value;                        //データ初期値

        /* ﾃﾞｰﾀﾀｲﾌﾟ固有ﾌﾟﾛﾊﾟﾃｨ */
        [XmlAttribute("RecordCount")]
        public string RecordCount;                  //(FIFO)レコード数

        [XmlAttribute("InitGroup")]                 //初期値更新対象グループ bit指定
        public string InitGroup;
        public Tag() { Name = ""; Comment = ""; Address = ""; Type = "0"; Size = "0"; Value = ""; RecordCount = "0"; InitGroup = "0"; }
    }
    /// <summary>
    /// xml Tag参照用(TagRef)の定義
    /// </summary>
    [Serializable]
    public class TagRef
    {
        [XmlAttribute("Name")]
        public string Name;                         //Tag名
        public TagRef() { }
        public TagRef(string i1) { Name = i1; }
    }
    /// <summary>
    /// xml CollectorRefの定義(サービスが参照するCollector)
    /// </summary>
    [Serializable]
    public class CollectorRef
    {
        [XmlAttribute("Name")]
        public string Name;                         //Collector名
        public CollectorRef() { }
        public CollectorRef(string i1) { Name = i1; }
    }
    /// <summary>
    /// xml Mailboxの定義
    /// </summary>
    [Serializable]
    public class MailBox
    {
        [XmlAttribute("Name")]
        public string Name;                         //Mailbox名
        [XmlAttribute("ColSize")]
        public string ColSize;                      //Mailbox 1レコードのサイズ
        [XmlAttribute("RowSize")]
        public string RowSize;                      //Mailbox レコード数
    
        public MailBox() { Name = ""; ColSize = ""; RowSize = "";}
    }
    /// <summary>
    /// xml TagTriggerの定義
    /// </summary>
    [Serializable]
    public class TagTrigger
    {
        [XmlAttribute("Name")]
        public string Name;                         //タグ名

        public TagTrigger() { Name = ""; }
    }

    /// <summary>
    /// xml Datasetの定義
    /// </summary>
    [Serializable]
    public class Dataset
    {
        [XmlAttribute("Name")]
        public string Name;                         //Dataset名
        public List<TagRef> TagRefs;                //Dataset内TagRef
        public Dataset() { }
        public Dataset(string i1) { Name = i1;}
    }
    /// <summary>
    /// xml Collectorの定義
    /// </summary>
    [Serializable]
    public class Collector
    {
        [XmlAttribute("Name")]
        public string Name;                         //Collector名 = メールボックス名
        [XmlAttribute("Interval")]
        public string Interval;                     //ポーリング周期(ms)
        [XmlAttribute("Priority")]
        public string Priority;                     //Thread Priority
        [XmlAttribute("DatasetName")]
        public string DatasetName;                  //Dataset名
        [XmlAttribute("ColSize")]
        public string ColSize;                      //1テーブル長
        [XmlAttribute("RowSize")]
        public string RowSize;                      //メールボックステーブル数
        [XmlAttribute("Comment")]
        public string Comment;                      //コメント
        public Collector() { }
        public Collector(string i1, string i2 = "", string i3 = "", string i4 = "", string i5 = "", string i6 = "", string i7 = "")
                        { Name = i1; Interval = i2; Priority = i3;  DatasetName = i4; ColSize = i5; RowSize = i6; Comment = i7; }
    }
    /// <summary>
    /// xml Serviceの定義
    /// </summary>
    [Serializable]
    public class Service
    {
        [XmlAttribute("Name")]
        public string Name;                         //サービス名
        [XmlAttribute("Path")]
        public string Path;                         //サービス(プログラム)のパス
        [XmlAttribute("Priority")]
        public string Priority;                     //サービス間通信用EgMailboxのPriority
        [XmlAttribute("Argument")]
        public string Argument;                     //サービス(プログラム)の起動引数
        [XmlAttribute("Comment")]
        public string Comment;                      //サービスのコメント
        [XmlAttribute("Timeout")]
        public string Timeout;                      //サービス起動時にサービス側から立ち上げ完了メッセージを受け取るのに待つ時間(過ぎたら作れないと判断)
        [XmlAttribute("Node")]
        public string Node;                         //起動ノード
        [XmlAttribute("Dependent")]
        public string Dependent;                    //依存コンテナ名
        [XmlAttribute("Enabled")]
        public string Enabled;                      //起動有効・無効(0:起動しない、1:起動する)
        [XmlAttribute("ColSize")]
        public string ColSize;                      //1テーブル長
        [XmlAttribute("RowSize")]
        public string RowSize;                      //メールボックステーブル数
        [XmlAttribute("Class")]
        public string Class;                         //サービスクラス
        [XmlAttribute("OnlyOwner")]
        public string OnlyOwner;                    // 呼び出し元のみ読み込むフラグ(TRUE以外常に読み込む Default)
        [XmlAttribute("ECI")]
        public string ECIPath;                      // ECI Path指定

        public List<Tag>    Tags;                   //Tagの定義(グローバルと同じ扱い ただしリンクタグの優先度高とする)
        public List<Dataset> Datasets;              //EgDataset(グローバルと同じ扱い)
        public List<TagRef> TagRefs;                //サービス内参照用Tag(IN/OUT共通)
        public List<TagRef> TagRefs_OUT;            //サービス内参照用Tag(OUT用)
        public List<TagRef> TagRefs_IN;             //サービス内参照用Tag(IN用)
        public List<Collector> Collectors;          //サービス内実行(メールボックスにwrite)用Collector
        public List<CollectorRef> CollectorRefs;    //サービス内参照用(メールボックスからRead)用Collector
        public List<MailBox> MailBoxes;             //サービス内参照用メールボックス
        public List<TagTrigger> TagTriggers;        //サービス内タグトリガーリスト
        public Service() { Name = ""; Path = ""; Priority = ""; Argument = ""; Comment = ""; Timeout = ""; Node = ""; Dependent = ""; ColSize = ""; RowSize = ""; Class = ""; OnlyOwner = ""; ECIPath = ""; }     //コンストラクタ
        public Service(string i1, string i2 = "", string i3 = "", string i4 = "", string i5 = "", string i6 = "", string i7 = "", string i8 = "", string i9 = "", string i10 = "", string i11 = "", string i12 = "", string i13="")
                     { Name = i1; Path = i2; Priority = i3;  Comment = i4; Argument = i5; Timeout = i6; Node = i7; Dependent = i8; ColSize = i9; RowSize = i10; Class = i11; OnlyOwner = i12; ECIPath = i13; }
    }
    /// <summary>
    /// xmlの全体
    /// </summary>
    public class RTedge
    {
        public List<Tag> Tags;          //EgTagデータ
        public List<Dataset> Datasets;  //EgDatasetデータ
        public List<Service> Services;  //EgServiceデータ
    }

   

    /// <summary>
    /// EgSysDefファイルの定義
    /// </summary>
    [System.Xml.Serialization.XmlRoot("RTedgeSysDef")]
    public class RTedgeSysDef
    {
        /// <summary>
        /// データセット エントリ最大数
        /// </summary>
        [System.Xml.Serialization.XmlElement("DSETENTRY")]
        public string DSETENTRY;
        /// <summary>
        /// データセット データリンクタグ最大数(TagRef数)
        /// </summary>
        [System.Xml.Serialization.XmlElement("DSETLINK")]
        public string DSETLINK;
        /// <summary>
        /// コレクタ エントリ最大数
        /// </summary>
        [System.Xml.Serialization.XmlElement("CLCTENTRY")]
        public string CLCTENTRY;
        /// <summary>
        /// サービスコンテナ エントリ最大数
        /// </summary>
        [System.Xml.Serialization.XmlElement("SERVICEENTRY")]
        public string SERVICEENTRY;
        /// <summary>
        /// メールボックス エントリ最大数
        /// </summary>
        [System.Xml.Serialization.XmlElement("MAILBOXENTRY")]
        public string MAILBOXENTRY;
        /// <summary>
        /// RT-edgeフレームワーク コンテナ間通信メールボックス最大データサイズ(ColSize) 
        /// </summary>
        [System.Xml.Serialization.XmlElement("FW_C2C_MAILBOXCOL")]
        public string FW_C2C_MAILBOXCOL;
        /// <summary>
        /// RT-edgeフレームワーク コンテナ間通信メールボックス 最大保持件数(RowSize)
        /// </summary>
        [System.Xml.Serialization.XmlElement("FW_C2C_MAILBOXROW")]
        public string FW_C2C_MAILBOXROW;
        /// <summary>
        /// RT-edgeフレームワーク  コレクタオブジェクト参照 エントリ最大数
        /// </summary>
        [System.Xml.Serialization.XmlElement("FW_CLCTENTRY")]
        public string FW_CLCTENTRY;
        /// <summary>
        /// RT-edgeフレームワーク メールボックスオブジェクト エントリ最大数
        /// </summary>
        [System.Xml.Serialization.XmlElement("FW_MAILBOXENTRY")]
        public string FW_MAILBOXENTRY;
        /// <summary>
        /// RT-edgeフレームワーク TagRefオブジェクト エントリ最大数
        /// </summary>
        [System.Xml.Serialization.XmlElement("FW_TAGREFENTRY")]
        public string FW_TAGREFENTRY;
        /// <summary>
        /// タグポインタ最大数
        /// </summary>
        [System.Xml.Serialization.XmlElement("MAXTAGPTRNUM")]
        public string MAXTAGPTRNUM;
        /// <summary>
        /// タグポインタ最大データサイズ(バイト単位)
        /// </summary>
        [System.Xml.Serialization.XmlElement("MAXTAGPTRSIZE")]
        public string MAXTAGPTRSIZE;
        /// <summary>
        /// タグポインタロックを試みる回数
        /// </summary>
        [System.Xml.Serialization.XmlElement("MAXTAGPTRRETRY")]
        public string MAXTAGPTRRETRY;
        /// <summary>
        /// RT-edgeフレームワーク TagTriggerオブジェクト エントリ最大数
        /// </summary>
        [System.Xml.Serialization.XmlElement("FW_TAGTRIGGERENTRY")]
        public string FW_TAGTRIGGERENTRY;
    }

}
