﻿////
/// Copyright(C) 2020 Micronet Co. All Rights Reserved.
//
using System;
using System.Threading;
using System.Windows.Forms;
using System.Diagnostics;           // Debug.WriteLine出力用
using RTedge;

namespace EgTime
{
    public partial class EgTimeForm : Form
    {
        edge_API EGAPI = new edge_API();      //edge APIのインスタンス
        edge_FW EGFW = null;                 //edge FrameWorkのインスタンス
        CancellationTokenSource m_tokenSource = null; //時間表示用Timer
        bool bWindowF = false;                  //(debug用)Form画面を表示するか否か

        // xmlで指定したサービスコンテナ名 EgInit後設定
        public string ServiceRealName = "";
        // サービスコンテナ名デフォルト値 EgInit前で使用
        public string DefServiceName = "EgTime";

        /// <summary>
        /// サービスプロパティタグ名：[.Live]
        /// </summary>
        private string TAG_LIVE
        {
            get { return String.Format("SERVICE.{0}.Live", GetServiceName()); }
        }
        /// <summary>
        /// サービスプロパティタグ名：[.Run]
        /// </summary>
        private string TAG_RUN
        {
            get { return String.Format("SERVICE.{0}.Run", GetServiceName()); }
        }
        /// <summary>
        /// サービスプロパティタグ名：[.Error]
        /// </summary>
        private string TAG_ERROR
        {
            get { return String.Format("SERVICE.{0}.Error", GetServiceName()); }
        }

        /// <summary>
        /// 自己のサービス名を返す
        /// </summary>
        /// <returns></returns>
        public String GetServiceName()
        {
            if (ServiceRealName.Length > 0) //XMLで指定したサービス名 EgInit後に設定 設定がない場合はデフォルト値を返す
                return ServiceRealName;
            return DefServiceName; //"EgTime"
        }

        public EgTimeForm()
        {
            InitializeComponent();
            //フレームワーク初期化
            edge_API.EDGECONFIG config = new edge_API.EDGECONFIG();
            config = EGAPI.EDGE_CONFIG_DEFAULT;         //初期値設定
            config.mInstMessFunc = UserMessageHandler;  //ユーザーメッセージハンドラ追加
            int ret = EGAPI.EgInit(config);                    //イニシャライズ実行
            if (ret != 0)
                edge_FW._PrintSysLog(DefServiceName, "ERR", "Failed to initialize. Please check the environment.");

            // サービス名の設定
            ServiceRealName = EGAPI.EGFW.ServiceRealName;

            EGFW = EGAPI.EGFW;
            //時間のTagを更新
            m_tokenSource = new CancellationTokenSource();
            CancellationToken cToken = m_tokenSource.Token;
            fnTimer_task(cToken);
        }

        /// <summary>
        /// Load 処理
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Window_Load(object sender, EventArgs e)
        {
            try
            {
                bWindowF = true;

            }
            catch (Exception ex)
            {
                edge_FW._PrintSysLog(GetServiceName(), "ERR", "FATAL ERROR : {0}", System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
            }
        }

        /// <summary>
        /// Close処理
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Form_Closed(object sender, FormClosedEventArgs e)
        {
            try
            {
                CloseProcess();
            }
            catch (Exception ex)
            {
                edge_FW._PrintSysLog(GetServiceName(), "ERR", "FATAL ERROR : {0}", System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
            }

        }
        /// <summary>
        /// ユーザーメッセージハンドラ
        /// </summary>
        /// <param name="senderName"></param>
        /// <param name="messNo"></param>
        /// <param name="paramArray"></param>
        /// <returns></returns>
        public int UserMessageHandler(string senderName, int messNo, byte[] paramArray)
        {
            try
            {
                switch (messNo)
                {
                    case (int)EGDEFINE.EGMSG.EM_SERVICE_STOP:
                        {
                            //終了関数呼び出し
                            fnStop();
                        }
                        break;
                    default:
                        return -1;
                }
                return 0;
            }
            catch (Exception ex)
            {
                edge_FW._PrintSysLog(GetServiceName(), "ERR", "FATAL ERROR : {0}", System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return -1;
            }
        }
        /// <summary>
        /// ユーザーストップメッセージ用関数
        /// </summary>
        private void fnStop()
        {
            try
            {
                Close();
            }
            catch (Exception ex)
            {
                edge_FW._PrintSysLog(GetServiceName(), "ERR", "FATAL ERROR : {0}", System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
            }
            
        }

        /// <summary>
        /// 終了処理
        /// </summary>
        private void CloseProcess()
        {
            try
            {

                if (m_tokenSource != null)
                {
                    m_tokenSource.Cancel();
                    m_tokenSource = null;
                }
                EGAPI.EgFinalize();
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
            }
        }
    }
}
