﻿////
/// Copyright(C) 2020 Micronet Co. All Rights Reserved.
//
using System;
using System.Threading;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.Diagnostics;           // Debug.WriteLine出力用
using RTedge;

namespace EgTime
{
    public partial class EgTimeForm : Form
    {
        /// <summary>
        /// RT-edge 時間更新
        /// </summary>
        /// <param name="cancelToken"></param>
        public async void fnTimer_task(CancellationToken cancelToken)
        {
            try
            {
                System.Threading.Thread.Sleep(100);
                //タスク処理を開始する
                await Task.Run(() =>
                {
                    try
                    {
                        edge_FW._PrintSysLog(GetServiceName(), "DBG", "Time Task Start");                     
                        edge_API.EG_TIME_STAMP stTimeStamp = new edge_API.EG_TIME_STAMP();

                        EGAPI.EgGetTimeStamp(ref stTimeStamp);

                        ulong startTime = stTimeStamp.time;
                        ulong tempTime = 0;
                        ushort tempTime_ms = 0;
                        ulong tempOpeTime = 0;
                        int state = 0;
                        object readObj = new object();

                        int tryCount = 0;
                        bool initF = false;
                        while (true)
                        {
                            if (cancelToken.IsCancellationRequested)
                            {
                                edge_FW._PrintSysLog(GetServiceName(), "DBG", "Timer_task IsCancellationRequested");
                                break;
                            }

                            if (EGFW != null && EGFW.tagRef_IN_List.Count == 4)
                            {
                                if (initF == false)
                                {
                                    EGFW.PrintLog("DBG", "Try Count : {0}", tryCount.ToString());
                                    initF = true;
                                }
                                //時間取得
                                EGAPI.EgGetTimeStamp(ref stTimeStamp);

                                //1970.01.01からの経過秒
                                EGAPI.EgTagWrite(EGFW.tagRef_IN_List[0].Name, stTimeStamp.time);
                                //ミリ秒
                                EGAPI.EgTagWrite(EGFW.tagRef_IN_List[1].Name, stTimeStamp.millisecond);
                                //マイクロ秒
                                EGAPI.EgTagWrite(EGFW.tagRef_IN_List[2].Name, stTimeStamp.microsecond);

                                //経過時間
                                EGAPI.EgTagWrite(EGFW.tagRef_IN_List[3].Name, stTimeStamp.time - startTime);

                                //Readして表示 test用なのでINのまま使用
                                state = EGAPI.EgTagRead(EGFW.tagRef_IN_List[0].Name, ref readObj);
                                tempTime = (ulong)readObj;

                                state = EGAPI.EgTagRead(EGFW.tagRef_IN_List[3].Name, ref readObj);
                                tempOpeTime = (ulong)readObj;

                                state = EGAPI.EgTagRead(EGFW.tagRef_IN_List[1].Name, ref readObj);
                                tempTime_ms = (ushort)readObj;

                                if (bWindowF)
                                {
                                    try
                                    {
                                        TimeSpan ts = new TimeSpan(0, 0, (int)tempOpeTime);

                                        this.Invoke((MethodInvoker)(() =>
                                        {
                                            timeBox.Text = tempTime.ToString();
                                            timeFormatBox.Text = DateTimeOffset.FromUnixTimeSeconds((long)tempTime).LocalDateTime.ToString();
                                            timeBox_ms.Text = tempTime_ms.ToString();
                                            opetimeBox.Text = tempOpeTime.ToString();
                                            opetimeBox_span.Text = ts.ToString();
                                        }));
                                    }
                                    catch (Exception ex)
                                    {
                                        // 終了時のタスクキャンセル処理のタイムラグにより本例外が発生するケースがあるため、エラー扱いとはしない。ログ出力のみ。
                                        edge_FW._PrintSysLog(GetServiceName(), "DBG", "{0}", System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                                    }
                                }
                            }
                            else
                            {
                                if(tryCount > 100)
                                {
                                    edge_FW._PrintSysLog(GetServiceName(), "DBG", "Cannot Start Time Task. Check TagRefs_IN Count {0}. try = {1}", EGFW.tagRef_IN_List.Count.ToString(), tryCount.ToString());
                                    break;
                                }
                                tryCount++;
                            }

                            //100で固定 数値に特に大きな意味はない
                            System.Threading.Thread.Sleep(100);
                        }
                    }
                    catch (Exception ex)
                    {
                        Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                        edge_FW._PrintSysLog(GetServiceName(), "ERR", "FATAL ERROR : {0}", System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                    }
                });
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                edge_FW._PrintSysLog(GetServiceName(), "ERR", "FATAL ERROR : {0}", System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
            }
        }
    }
}
