/*****************************************************************************
*
* FILE NAME:		EgTimeRt.c
*
* DESCRIPTION:		RT-edgae system time service for INtime
*					edgeVXe֌ݎ(msx)Aғ(b)񋟂܂
*
* Indicator:		SERVICE.EgTime.Status			supported
*					SERVICE.EgTime.Run				supported
*					SERVICE.EgTime.Error			supported
*					SERVICE.EgTime.Live				supported
*
* Property:			SERVICE.EgTime.AutoRun			supported
*					SERVICE.EgTime.Mode				2 (fixed)
*					SERVICE.EgTime.Cycle			1~1000
*					SERVICE.EgTime.InPriority		1~254
*					SERVICE.EgTime.OutPriority		1~254
*
* Message:			EM_SERVICE_RUN					supported
*					EM_SERVICE_PAUSE				supported
*					EM_SERVICE_STOP					supported
*					EM_SERVICE_UPDATE				supported
*					EM_SERVICE_TIMEREFRESH			19999
*
* Input:			EDGESYSTEM.CurrentTime
*					EDGESYSTEM.CurrentTime_ms
*					EDGESYSTEM.CurrentTime_us
*					EDGESYSTEM.OperatingTime
*
* Output:			none
*
*	Copyright (c) Micronet 2020 
*
*****************************************************************************/
#include <stdio.h>
#include <rt.h>
#include <egAPI.h>			// RT-edge system API
#include <stdint.h>
#include <stdbool.h>
#include <stdlib.h>

/// <summary>
/// T[rX
/// </summary>
#define	SERVICENAME			"EgTime"						// pp8܂

/// <summary>
/// T[rXCWP[^
/// </summary>
#define	IND_MYSERV_STATUS	"SERVICE."SERVICENAME".Status"	// T[rXN
#define		SERVICE_UP		1								// N
#define		SERVICE_DOWN	0								// ~
#define	IND_MYSERV_ERROR	"SERVICE."SERVICENAME".Error"	// T[rXG[
#define	IND_MYSERV_RUN		"SERVICE."SERVICENAME".Run"		// T[rXs
#define	IND_MYSERV_LIVE		"SERVICE."SERVICENAME".Live"	// T[rXsJE^

/// <summary>
/// T[rXvpeB^O
/// </summary>
#define PROP_MYSERV_MODE	"SERVICE."SERVICENAME".Mode"		// T[rX샂[h
#define		MODE_AUTO		1									// oTCN
#define		MODE_SEMIAUTO	2									// ̓TCN{o̓f}h
#define		MODE_MANUAL		3									// o̓f}h
#define		DEFAULT_MODE		MODE_SEMIAUTO	/*EgTimê͓͕݂Ȃ̂*/
#define PROP_MYSERV_CYCLE	"SERVICE."SERVICENAME".Cycle"		// T[rXTCN(ms)
#define		DEFAULT_CYCLE		1
#define PROP_MYSERV_INPRIO	"SERVICE."SERVICENAME".InPriority"	// EdgeVXeւ̓̓T[rXXbhvCIeB
#define		DEFAULT_INPRIO		148
#define PROP_MYSERV_OUTPRIO	"SERVICE."SERVICENAME".OutPriority"	// EdgeVXȅo̓T[rXXbhvCIeB	
#define		DEFAULT_OUTPRIO		149
#define PROP_MYSERV_TIMESRC	"SERVICE."SERVICENAME".TimeSource"	// ṽNbN\[XI(0=RTC,1=KTick)	
#define		DEFAULT_TIMESRC		0

/// <summary>
/// T[rX񋟃^O
/// </summary>
#define	TAG_MYSERV_TIME		"EDGESYSTEM.CurrentTime"		// VXe(b)
#define	TAG_MYSERV_TIMEMS	"EDGESYSTEM.CurrentTime_ms"		// VXe(ms)
#define	TAG_MYSERV_TIMEUS	"EDGESYSTEM.CurrentTime_us"		// VXe(us)
#define	TAG_MYSERV_OPTIME	"EDGESYSTEM.OperatingTime"		// VXesoߎ(b)

/// <summary>
/// T[rXŗLbZ[W
/// </summary>
#define	EM_SERVICE_TIMEREFRESH			19999				// ݎ̃tbV(RTCĎ擾)

/// <summary>
/// T[rX
/// </summary>
static struct MYSTATE
{
	bool		State;			// CWP[^.GbWVXeڑ
	bool		Run;			// CWP[^.T[rXs
	bool		Error;			// CWP[^.T[rXُ
	uint32_t	Live;			// CWP[^.T[rXsJE^
	uint64_t	StartTime;		// CWP[^.T[rXJñ^CX^v
	uint8_t		Mode;			// vpeB.T[rXs[h
	uint32_t	Cycle;			// vpeB.T[rXsTCN(ms)
	uint8_t		InPriority;		// vpeB.T[rX()vCIeB
	uint8_t		OutPriority;	// vpeB.T[rX(o)vCIeB
	RTHANDLE	hInTrigSem;		// ̓T[rXgKZ}tHnh
	RTHANDLE	hOutTrigSem;	// o̓T[rXgKZ}tHnh
	BYTE		TimeSource;		// ṽNbN\[XI(0=RTC,1=KTick)
	bool		ForceReset;		// X^[gĐݒ
} MyState;

/// <summary>
/// T[rX֐
/// </summary>
extern void ThreadAutoCycle(struct MYSTATE *state);
extern void ThreadInService(struct MYSTATE *state);
extern void ThreadOutService(struct MYSTATE *state);
extern void KickInput(struct MYSTATE *state);
extern void KickOutput(struct MYSTATE *state);
extern int32_t CreateServiceTags(struct MYSTATE *state);
extern int32_t UpdateIndicator(struct MYSTATE *state);
extern int32_t GetProperty(struct MYSTATE *state);
extern int32_t StartService(struct MYSTATE *state);
extern int32_t PauseService(struct MYSTATE *state);
extern int32_t InitService(struct MYSTATE *state);
extern int32_t KillService(struct MYSTATE *state);
extern void WarningMsg(int32_t position, int32_t reason);
#define WARN(reason)	WarningMsg(__LINE__,(reason))

/// <summary>
/// x[X^CXV
/// </summary>
/// <param name="state"></param>
void UpdateBaseTime(struct MYSTATE *state)
{
	state->ForceReset = true;
}

/// <summary>
/// EdgebZ[Wnh
/// </summary>
/// <param name="sender">bZ[WM</param>
/// <param name="msgno">bZ[Wԍ</param>
/// <param name="param">bZ[Wp[^</param>
/// <returns>펞O</returns>
int32_t MsgHandler(const char* sender, int32_t msgno, void *param)
{
	int32_t	result;

	switch (msgno)
	{
	case EM_SERVICE_RUN:			// T[rXJnw
		result = StartService(&MyState);
		break;
	case EM_SERVICE_STOP:			// T[rXIw
		result = KillService(&MyState);
		break;
	case EM_SERVICE_PAUSE:			// T[rXꎞ~w
		result = PauseService(&MyState);
		break;
	case EM_SERVICE_UPDATE:			// T[rXAbvf[gw
		switch (MyState.Mode)
		{
		case MODE_AUTO:					// TCN[hiTCNXbĥŖj
			break;
		case MODE_SEMIAUTO:				// TCN[hio͂̍XVsj
			KickOutput(&MyState);
			break;
		case MODE_MANUAL:				// 蓮TCN[hio͋ɍXVsj
			KickInput(&MyState);
			KickOutput(&MyState);
			break;
		}
		break;
	case EM_SERVICE_TIMEREFRESH:	// x[XtbVw
		UpdateBaseTime(&MyState);
		break;
	}
	return 0;
}

/// <summary>
/// T[rX̏
/// </summary>
/// <returns>펞O</returns>
int32_t InitService(struct MYSTATE *state)
{
	int32_t		result;
	EDGECONFIG	config = EDGE_CONFIG_DEFAULT;
	RTHANDLE	hThread;

	config.fpUserMessHdlFunc = MsgHandler;						// EdgebZ[W[Unh̎w

	result = EgInit(&config);									// Edget[[N̏
	if (result != EDGE_SUCCESS)
		WARN(result);

	result = GetProperty(state);								// T[rXvpeB̃[h
	if (result != EDGE_SUCCESS)
		WARN(result);

	result = CreateServiceTags(state);							// T[rXCWP[^̐
	if (result != EDGE_SUCCESS)
		WARN(result);

	UpdateBaseTime(state);										// VXeN

	state->hInTrigSem = CreateRtSemaphore(0, 10, FIFO_QUEUING);		// ̓T[rXXbhpgKZ}tH
	state->hOutTrigSem = CreateRtSemaphore(0, 10, FIFO_QUEUING);	// o̓T[rXXbhpgKZ}tH

	SetRtProcessMaxPriority(0, 0);														// vZXŏʃvCIeB

	hThread = CreateRtThread(state->OutPriority, ThreadOutService, 0x2000, state);		// o̓T[rXXbh̐
	CatalogRtHandle(0, hThread, "OutServ");

	hThread = CreateRtThread(state->InPriority, ThreadInService, 0x2000, state);		// ̓T[rXXbh̐
	CatalogRtHandle(0, hThread, "InServ");

	uint8_t trigprio = (state->OutPriority < state->InPriority) ? state->OutPriority : state->InPriority;	// ق̃vCIeB擾
	hThread = CreateRtThread(trigprio, ThreadAutoCycle, 0x2000, state);					// T[rXTCNXbh̐
	CatalogRtHandle(0, hThread, "AutoCycle");

	RTHANDLE hRoot = GetRtThreadHandles(ROOT_PROCESS);			// vZXJ^O
	RTHANDLE hThisProc = GetRtThreadHandles(THIS_PROCESS);
	UncatalogRtHandle(hRoot, SERVICENAME);
	CatalogRtHandle(hRoot, hThisProc, SERVICENAME);
	SetLastRtError(0);

	state->State = SERVICE_UP;									// T[rXAbv
	UpdateIndicator(state);										// T[rXCWP[^̍XV

	return result;
}

/// <summary>
/// T[rX폜ivZX~j
/// </summary>
/// <returns>̊֐߂邱Ƃ͂܂</returns>
int32_t KillService(struct MYSTATE *state)
{
	PauseService(state);			// T[rXꎞ~

	state->State = SERVICE_DOWN;	// T[rX_E
	UpdateIndicator(state);			// T[rXCWP[^XV

	TerminateRtProcess(0, 1, 1000);		// vZXɏIv
													// łEgFinalizełȂB߂ĂȂȂB
	return 0;
}

/// <summary>
/// T[rXJn o͉\ɂȂ܂
/// </summary>
/// <returns>펞O</returns>
int32_t StartService(struct MYSTATE *state)
{
	state->Run = true;				// T[rXꎞ~
	UpdateIndicator(state);			// T[rXCWP[^XV
	return 0;
}

/// <summary>
/// T[rXꎞ~ o͂ȂȂ܂
/// </summary>
/// <returns>펞O</returns>
int32_t PauseService(struct MYSTATE *state)
{
	state->Run = false;				// T[rXs
	UpdateIndicator(state);			// T[rXCWP[^XV
	return 0;
}

/// <summary>
/// ̓T[rX1LbN
/// </summary>
/// <param name="state"></param>
void KickInput(struct MYSTATE *state)
{
	ReleaseRtSemaphore(state->hInTrigSem, 1);
}

/// <summary>
/// o̓T[rX1LbN
/// </summary>
/// <param name="state"></param>
void KickOutput(struct MYSTATE *state)
{
	ReleaseRtSemaphore(state->hOutTrigSem, 1);
}

/// <summary>
/// T[rX^O łɐĂꍇ͖Čp܂
/// </summary>
/// <param name="state"></param>
/// <returns>펞O</returns>
int32_t CreateServiceTags(struct MYSTATE *state)
{
	int32_t		result = 0;

	// T[rX^Oނ̐
	result |= EgTagCreate(TAG_MYSERV_TIME, EgVtUInt64, 8 ,"" ,"edge system current time");
	result |= EgTagCreate(TAG_MYSERV_TIMEMS, EgVtUInt16, 2, "", "edge system current time(ms)");
	result |= EgTagCreate(TAG_MYSERV_TIMEUS, EgVtUInt16, 2, "", "edge system current time(us)");
	result |= EgTagCreate(TAG_MYSERV_OPTIME, EgVtUInt64, 8, "", "edge system current operation time");
	return EDGE_SUCCESS;
}

/// <summary>
/// T[rXCWP[^̍XV XVɎsĂČp܂
/// </summary>
/// <param name="state"></param>
/// <returns>펞O</returns>
int32_t UpdateIndicator(struct MYSTATE *state)
{
	// CWP[^^O̓t[[NɂĎ
	int32_t		result = 0;
	result |= EgTagWrite(IND_MYSERV_STATUS, &state->State, 1);		// T[rXN
	result |= EgTagWrite(IND_MYSERV_RUN, &state->Run, 1);			// T[rXs
	result |= EgTagWrite(IND_MYSERV_LIVE, &state->Live, 4);			// T[rXsJE^
	result |= EgTagWrite(IND_MYSERV_ERROR, &state->Error, 1);		// T[rXُ
	return result;
}

/// <summary>
/// T[rXvpeB̎擾 擾łȂ̓ftHgݒl̗p܂
/// </summary>
/// <param name="state"></param>
/// <returns>펞O</returns>
int32_t GetProperty(struct MYSTATE *state)
{
	// vpeB^OXML`\@Ƃ̓ftHgݒl̗p܂
	int32_t		result;
	result = EgTagRead(PROP_MYSERV_MODE, &state->Mode, 1);				// T[rX샂[h̎擾
	if (result != EDGE_SUCCESS)	state->Mode = DEFAULT_MODE;				// ftHg

	result = EgTagRead(PROP_MYSERV_CYCLE, &state->Cycle, 4);			// T[rXTCN̎擾
	if (result != EDGE_SUCCESS)	state->Cycle = DEFAULT_CYCLE;			// ftHg

	result = EgTagRead(PROP_MYSERV_INPRIO, &state->InPriority, 1);		// ̓T[rXXbh̃vCIeBw
	if (result != EDGE_SUCCESS)	state->InPriority = DEFAULT_INPRIO;		// ftHg

	result = EgTagRead(PROP_MYSERV_OUTPRIO, &state->OutPriority, 1);	// o̓T[rXXbh̃vCIeBw
	if (result != EDGE_SUCCESS)	state->OutPriority = DEFAULT_OUTPRIO;	// ftHg

	result = EgTagRead(PROP_MYSERV_TIMESRC, &state->TimeSource, 1);		// o̓T[rXXbh̃vCIeBw
	if (result != EDGE_SUCCESS)	state->TimeSource = DEFAULT_TIMESRC;	// ftHg

	return EDGE_SUCCESS;
}

//*high performance type gettimeofday
//	(in)
//		*tv ... timeval structure to output
//		force ... true=IɃx[X^CĐݒ肵܂
//	(ret)
//		0=success 
int highperformancegettimeofday(EDGE_TIME_STAMP *tv,bool force)
{
	EDGE_TIME_STAMP stv;
	static uint64_t stvtime;		/* initial timeval */
	static uint64_t stvmicrosec;	/* initial timeval */
	static int64_t sct, tick;
	static int initialized = -1;
	uint64_t	tmptime;
	uint64_t	tmpmicrosecond;
	int64_t c;

	if ((force==true)||(initialized == -1)) {
		EgGetTime(&stv);
		stvmicrosec = stv.millisecond * 1000 + stv.microsecond;
		stvtime = stv.time;
		if (QueryRtPerformanceFrequency((QWORD*)&tick) == 0)
			return -1;	// No high-reso counter
		if (QueryRtPerformanceCounter((QWORD*)&sct) == 0)
			return -1;	// No high-reso counter
		initialized = 0;
	}

	if (!QueryRtPerformanceCounter((QWORD*)&c))
		return -1;
	tmptime = stvtime + (long)((c - sct) / tick);
	tmpmicrosecond = stvmicrosec + (long)(((c - sct) % tick) * 1000000 / tick);
	if (tmpmicrosecond >= 1000000) {
		tmptime++;
		tmpmicrosecond -= 1000000;
	}
	tv->time = tmptime;
	tv->millisecond = (unsigned short)(tmpmicrosecond / 1000);
	tv->microsecond = (unsigned short)(tmpmicrosecond % 1000);
	return 0;
}


/// <summary>
/// EdgeVXeւ̃f[^
/// </summary>
/// <param name="state"></param>
void ThreadInService(struct MYSTATE *state)
{
	int32_t				result;
	EDGE_TIME_STAMP		ts;
	uint64_t			optime;

	while (1)
	{
		WaitForRtSemaphore(state->hInTrigSem, 1, WAIT_FOREVER);			// ̓^C~OgK҂

		// {T[rX͌ݎ^OEdgeVXeɒ񋟂܂
		result = EDGE_SUCCESS;
		if (state->TimeSource == 0)
			result |= EgGetTime(&ts);										// VXe擾(RTC)
		else if (state->TimeSource == 1) {
			result |= highperformancegettimeofday(&ts, state->ForceReset);	// VXe擾(KTick)
			state->ForceReset = false;
		}
		else
			state->Error = true;

		optime = ts.time - state->StartTime;							// ғ(b)Zo(ݎ-N)

		result |= EgTagWrite(TAG_MYSERV_TIME, &ts.time, 8);				// VXei[(b)
		result |= EgTagWrite(TAG_MYSERV_TIMEMS, &ts.millisecond, 2);	// VXei[(ms)
		result |= EgTagWrite(TAG_MYSERV_TIMEUS, &ts.microsecond, 2);	// VXei[(us)
		result |= EgTagWrite(TAG_MYSERV_OPTIME, &optime, 8);			// ғԊi[(b)

		state->Error = (result != EDGE_SUCCESS);	// G[CWP[^XV
	}
}

/// <summary>
/// EdgeVXẽf[^o
/// </summary>
/// <param name="state"></param>
void ThreadOutService(struct MYSTATE *state)
{
	while (1)
	{
		WaitForRtSemaphore(state->hOutTrigSem, 1, WAIT_FOREVER);		// o̓^C~OgK҂

		/* {T[rXɂEdgeVXȅo͂͂܂ */
	}
}

/// <summary>
/// EdgeVXeo͎TCNXbh
/// </summary>
/// <param name="state"></param>
void ThreadAutoCycle(struct MYSTATE *state)
{
	RtSleepEx(1);						// ̃gK^C~OJ[leBbNɂ낦

	while (1)
	{
		++state->Live;					// T[rXsJE^
		GetProperty(state);				// T[rXvpeB̃[h

		if (state->Run)					// T[rXsԂȂ珈
		{
			switch (state->Mode)		// T[rX샂[hɉĕ
			{
			case MODE_AUTO:				// TCN[hio͋ɎXVj
				KickInput(state);
				KickOutput(state);
				break;
			case MODE_SEMIAUTO:			// TCNi͂̂ݎXVAo͂EM_SERVICE_UPDATEŎw߁j
				KickInput(state);
				break;
			case MODE_MANUAL:			// 蓮TCNio͂EM_SERVICE_UPDATEŎw߂̂Ŗj
				break;
			}
		}

		UpdateIndicator(state);			// T[rXCWP[^XV
		RtSleepEx( state->Cycle );		// TCN҂
	}
}

/// <summary>
/// T[rXC
/// </summary>
/// <param name="argc"></param>
/// <param name="argv"></param>
/// <returns></returns>
int main(int argc, char* argv[])
{
	EVENTINFO		ei;

	InitService(&MyState);				// T[rX̏

	SynchronizeRtLoader();				// T[rX̒ʒm

	while(1)
	{
		RtNotifyEvent(ALL_NOTIFICATIONS, WAIT_FOREVER, &ei);	// INtimeVXeCxg̏
		switch (ei.dwNotifyType)
		{
		case TERMINATE:											// Intex̍폜w
			KillService(&MyState);
			EgFinalize();					// EdgeVXe
			exit(0);						// T[rXvZX폜
			break;
		case KERNEL_SHUTDOWN_PENDING:							// INtimeJ[l̒~w
			KillService(&MyState);
			EgFinalize();					// EdgeVXe
			exit(0);						// T[rXvZX폜
			break;
		case NT_BLUESCREEN:										// WindowsNbV
			break;
		}
	}
	return 0;
}

/// <summary>
/// [jObZ[W
/// </summary>
/// <param name="position">蔭 __LINE__w</param>
/// <param name="reason">sRR[h</param>
void WarningMsg(int32_t position, int32_t reason)
{
	printf(SERVICENAME" service warning at line(%d) ,reason 0x%08x\n", position, reason);
}
